
/*
 *
 * JSMSX - MSX Emulator in Javascript
 * Copyright (c) 2006 Marcus Granado <mrc.gran(@)gmail.com> 
 *
 * Portions of the initial code was inspired by the work of
 * Arnon Cardoso's Java MSX Emulator and
 * Adam Davidson & Andrew Pollard's Z80 class of the Spectrum Java Emulator 
 * after reading this thread: http://www.msx.org/forumtopic4176.html
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * The full license is available at http://www.gnu.org/licenses/gpl.html
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Completely revamped to support the hardware of the Tektronix 4051 by Dave Roberts.
 *
 */

tek_interrupt = function() {

	if( tek.resetAtNextInterrupt ) {
	    tek.resetAtNextInterrupt = false;
	    tek.execute_reset();
	}

	document.getElementById('interrupts').value=tek.interruptCounter;

	//if (tek.interruptCounter%600==0) 
	//tek.println('interrupt='+tek.interruptCounter+',ticks='+this.tstatesPerInterrupt+' cpu ticks/interrupt');
	tek.interruptCounter++;

	if( tek.pinta ) {
	    tek.updateScreen(); // First run through.
	    tek.pinta = false;
	}

	if( tek.interruptCounter % tek.frameSkip == 0 ) {
	 
		tek.updateScreen()

		// Check for a key press.		
		// if( (this.PIA_U461_IRA & 0x7F) != 0 ) {

			// And generate an interrupt if so.
			this.PIA_U461_CRA |= 0x80; // Set IRQA1.
		
		// } // End if.
		
	} // End if.

	//calls superclass' interrupt() in TEKTRONIX4051 context/scope.
    //
	return tek.MC6800_interrupt.apply( tek ); 

} // End of function tek_interrupt.

function TEKTRONIX4051( window, canvas, logbuf ) {

    this.window    = window;
    this.canvas    = canvas;
    this.logbuf    = logbuf;

    //this class builds on the MC6800 class.
    this.superclass = MC6800; //superclass
    this.superclass( 0.8333 ); //initialization (0.8333 MHz processorclock)

    this.memoria = new Array(); //int[][]
    this.pinta = true;
    this.interruptCounter=0;
    var frameSkip = 0;
    this.resetAtNextInterrupt=false;
    this.pauseAtNextInterrupt=false;
    this.refreshNextInterrupt=true;

	this.PIA_U561_CRA  = 0x00;
	this.PIA_U561_CRB  = 0x00;
	this.PIA_U561_ORA  = 0x00;
	this.PIA_U561_ORB  = 0x00;
	this.PIA_U561_IRA  = 0x00;
	this.PIA_U561_IRB  = 0x00;
	this.PIA_U561_DDRA = 0x00;
	this.PIA_U561_DDRB = 0x00;
	
	this.PIA_U565_CRA  = 0x00;
	this.PIA_U565_CRB  = 0x00;
	this.PIA_U565_ORA  = 0x00;
	this.PIA_U565_ORB  = 0x00;
	this.PIA_U565_IRA  = 0x00;
	this.PIA_U565_IRB  = 0x00;
	this.PIA_U565_DDRA = 0x00;
	this.PIA_U565_DDRB = 0x00;
	
	this.PIA_U361_CRA  = 0x00;
	this.PIA_U361_CRB  = 0x00;
	this.PIA_U361_ORA  = 0x00;
	this.PIA_U361_ORB  = 0x00;
	this.PIA_U361_IRA  = 0x00;
	this.PIA_U361_IRB  = 0x00;
	this.PIA_U361_DDRA = 0x00;
	this.PIA_U361_DDRB = 0x00;

	this.PIA_U461_CRA  = 0x00;
	this.PIA_U461_CRB  = 0x00;
	this.PIA_U461_ORA  = 0x00;
	this.PIA_U461_ORB  = 0x00;
	this.PIA_U461_IRA  = 0x00;
	this.PIA_U461_IRB  = 0x00;
	this.PIA_U461_DDRA = 0x00;
	this.PIA_U461_DDRB = 0x00;

	this.PIA_U265_CRA  = 0x00;
	this.PIA_U265_CRB  = 0x00;
	this.PIA_U265_ORA  = 0x00;
	this.PIA_U265_ORB  = 0x00;
	this.PIA_U265_IRA  = 0x00;
	this.PIA_U265_IRB  = 0x00;
	this.PIA_U265_DDRA = 0x00;
	this.PIA_U265_DDRB = 0x00;
	
	this.GPIB_DATA_IN  = 0x00; // Data from GPIB to 4051.
	this.GPIB_DATA_OUT = 0x00; // Data from 4051 to GPIB.
	
	this.GPIB_TALK     = 0; // 4051 is talking on the GPIB.
	this.GPIB_LISTEN   = 0; // 4051 is listening to the GPIB.
	this.GPIB_EOI_OUT  = 0; // End Or Identify. [TALKER].
	this.GPIB_IFC_OUT  = 0; // Interface Clear. [CONTROLLER].
	this.GPIB_ATN_OUT  = 0; // Attention. [CONTROLLER].
    this.GPIB_NRFD_OUT = 0; // Not Ready For Data. [LISTENERS].
    this.GPIB_DAV_OUT  = 0; // Data Valid. [TALKER].
    this.GPIB_NDAC_OUT = 0; // Not Data Accepted. [LISTENER].
    	
    this.GPIB_EOI_IN   = 0; // End Or Identify. [TALKER].
    this.GPIB_NRFD_IN  = 0; // Not Ready For Data. [LISTENERS].
    this.GPIB_SRQ_IN   = 0; // Service Request. [CONTROLLER].
    this.GPIB_DAV_IN   = 0; // Data Valid. [TALKER].
    this.GPIB_NDAC_IN  = 1; // Not Data Accepted. [LISTENER].
    
    this.GPIB_REN_OUT  = 0; // Remote Enable. [CONTROLLER].
    
    this.sbytes = []; // An empty array initially.
    this.sbyteslength = 0; // Initially no bytes stored in the empty array!
    this.sbytesindex = -1; // Initially no program has been stored.
    
    this.GPIB_STATE = 0; // No file loaded.
    
    this.GPIB_RDTIME_STATE = 0;
    this.GPIB_RDTIME = "";
    this.GPIB_RDTIME_INDEX = 0;
    
    this.KBD_TTY_0   = 1;
    this.KBD_SHIFT_0 = 1;
    this.KBD_CTRL_0  = 1;
    
	this.BANK_SWITCH_SELECTOR = 0;
	
	this.X_DA = 0;
	this.Y_DA = 0;
	
	var beep = new Audio( "beep.mp3" );
	
	var click = new Audio( "click.mp3" );
	
    // ********************
    // ***              ***
    // ***  STD VIDEO.  ***
    // ***              ***
    // ********************

    this.FGColourIndex = 0x2;
    this.BGColourIndex = 0x0;

    // Colours are indexed [0..7] and then [Red,Green,Blue] (where Red, Green and Blue are in the range 0..255).
    //
    this.ColourTable = [
      [   0,   0,   0 ], // [0] Black.
      [   0,   0, 255 ], // [1] Blue.
      [   0, 255,   0 ], // [2] Green.
      [ 255,   0,   0 ], // [3] Red.
      [   0, 255, 255 ], // [4] Cyan.
      [ 255,   0, 255 ], // [5] Magenta.
      [ 255, 255,   0 ], // [6] Yellow.
      [ 255, 255, 255 ], // [7] White.
    ];

    // ***************
    // ***         ***
    // ***  print  ***
    // ***         ***
    // ***************

    this.print = function( str ) {

	this.logbuf.textContent += str;

    } // End of function print.

    // *****************
    // ***           ***
    // ***  println  ***
    // ***           ***
    // *****************

    this.println = function( str ) {

	this.logbuf.textContent += str + "\n";

    } // End of function println.

    // ******************
    // ***            ***
    // ***  printHex  ***
    // ***            ***
    // ******************

    this.printHex1 = function( n ) {

      var nn = n & 0x000F; // Isolate the least significant 4 bits.

      switch( nn ) {
        case 0x0 : this.print( '0' ); break;
        case 0x1 : this.print( '1' ); break;
        case 0x2 : this.print( '2' ); break;
        case 0x3 : this.print( '3' ); break;
        case 0x4 : this.print( '4' ); break;
        case 0x5 : this.print( '5' ); break;
        case 0x6 : this.print( '6' ); break;
        case 0x7 : this.print( '7' ); break;
        case 0x8 : this.print( '8' ); break;
        case 0x9 : this.print( '9' ); break;
        case 0xA : this.print( 'A' ); break;
        case 0xB : this.print( 'B' ); break;
        case 0xC : this.print( 'C' ); break;
        case 0xD : this.print( 'D' ); break;
        case 0xE : this.print( 'E' ); break;
        case 0xF : this.print( 'F' ); break;
        default  : break;
      } // End switch nn.

    } // End of function printHex1.

    this.printHex2 = function( n ) {

      this.printHex1( n >>> 4 ); // Hi nibble.
      this.printHex1( n       ); // Lo nibble.

    } // End of function printHex2.

    this.printHex4 = function( n ) {

      this.printHex2( n >>> 8 ); // Hi word.
      this.printHex2( n       ); // Lo word.

    } // End of function printHex4.

    // *********************
    // ***               ***
    // ***  HandleEvent  ***
    // ***               ***
    // *********************

    this.handleEvent = function( e ) {

		// this.println("You pressed: which="+e.which+",keyUniCode="+e.keyCode+",shift="+e.shiftKey+",charCode="+e.charCode+",tochar="+String.fromCharCode(e.which)+",type="+e.type);

        return tek.HandleKeyboardEvent.call( tek, e.keyCode, e.type=='keydown', e );

    } // End of function handleEvent.

    // *************
    // ***       ***
    // ***  Key  ***
    // ***       ***
    // *************

    this.Key = function( keyrow, bitshift, keystate ) {

      // Is this a keypress event?
      //
      if( keystate ) {

        // YES
//@@@        this.KeyboardScancode[ keyrow ] &= ((~(1 << bitshift)) & 0xFF);

      } else {

        // NO - must be a key release event then.
//@@@        this.KeyboardScancode[ keyrow ] |= ((1 << bitshift) & 0xFF);

      } // End if identify keypress or keyrelease event.

    } // End of function Key.

    // *****************************
    // ***                       ***
    // ***  HandleKeyboardEvent  ***
    // ***                       ***
    // *****************************

    this.HandleKeyboardEvent = function( i, bool, e ) {

		//@@@ this.print('Key = '); this.printHex4( i ); this.print(' bool = '); this.print( bool ); this.print(' PC=0x'); this.printHex4( this.last_PC ); this.println('');
		
		// Decide what action to take depending upon the key code in 'i'.
		//
		switch( i ) {
		
		  case 0x0010 : // Left or Right shift keys.
		                this.KBD_SHIFT_0 = bool ? 0 : 1; // Shift key down = logical '0' (negative logic sense).
		                                                 // Shift key up   = logical '1'. 
		                i = 0; break;
		                
		  case 0x0011 : // Left or Right control keys.
		                this.KBD_CTRL_0 = bool ? 0 : 1; // Control key down = logical '0' (negative logic sense).
		                                                // Control key up   = logical '1'. 
		                i = 0; break;
		                
		  case 0x0014 : // Caps lock key.
		                this.KBD_TTY_0 = bool ? 0 : 1; // Caps lock key down = logical '0' (negative logic sense).
		                                               // Caps lock key up   = logical '1'. 
		                i = 0; break;
		  
		  // Letters.
		  case 0x0041 : // 'A'
		  case 0x0042 : // 'B'
		  case 0x0043 : // 'C'
		  case 0x0044 : // 'D'
		  case 0x0045 : // 'E'
		  case 0x0046 : // 'F'
		  case 0x0047 : // 'G'
		  case 0x0048 : // 'H'
		  case 0x0049 : // 'I'
		  case 0x004A : // 'J'
		  case 0x004B : // 'K'
		  case 0x004C : // 'L'
		  case 0x004D : // 'M'
		  case 0x004E : // 'N'
		  case 0x004F : // 'O'
		  case 0x0050 : // 'P'
		  case 0x0051 : // 'Q'
		  case 0x0052 : // 'R'
		  case 0x0053 : // 'S'
		  case 0x0054 : // 'T'
		  case 0x0055 : // 'U'
		  case 0x0056 : // 'V'
		  case 0x0057 : // 'W'
		  case 0x0058 : // 'X'
		  case 0x0059 : // 'Y'
		  case 0x005A : // 'Z'
		                break; // Nothing further to do.

		  // Numbers.
		  case 0x0030 : // '0'
		  case 0x0031 : // '1'
		  case 0x0032 : // '2'
		  case 0x0033 : // '3'
		  case 0x0034 : // '4'
		  case 0x0035 : // '5'
		  case 0x0036 : // '6'
		  case 0x0037 : // '7'
		  case 0x0038 : // '8'
		  case 0x0039 : // '9'
		                break; // Nothing further to do.
		  
		  // Symbols.
		  
		  case 0x007C : // <f13> = PAGE 
		                i = 0x6A; break;  

		  case 0x00DB : // '[' 
		                i = 0x5B; break;  

		  case 0x00DE : // ''' = ':' 
		                i = 0x3A; break;  
		  
		  case 0x00BD : // '-' 
		                i = 0x3D; break;  
		  
		  case 0x00DD : // ']' 
		                i = 0x5D; break;  

		  case 0x0008 : // BK SPACE
		                break; // Nothing further to do.
		                
		  case 0x001B : // ESC
		                break; // Nothing further to do.
		                
		  // '^' missing from table.
		  
		  case 0x00C0 : // '`' = '@' 
		                i = 0x40; break;  
		  
		  case 0x000A : // LF
		                break; // Nothing further to do.
		                
		  case 0x000D : // RETURN
		                break; // Nothing further to do.
		                
		  case 0x0009 : // TAB
		                break; // Nothing further to do.
		                
		  case 0x00BA : // ';' 
		                i = 0x3B; break;  

		  case 0x00BF : // '/'
		                i = 0x3F; break;  

		  case 0x002E : // <DELETE> = RUBOUT 
		                i = 0x5F; break;  

		  case 0x00BC : // ',' 
		                i = 0x3C; break;  

		  case 0x00BE : // '.' 
		                i = 0x3E; break;  

		  case 0x00DC : // '\'
		                i = 0x5C; break;  

		  case 0x007D : // <f14> = BREAK 
		                i = 0x6B; break;  

		  case 0x0020 : // ' ' = SPACE 
		                i = 0x10; break;  

		  // Numeric keypad.
		  
		  case 0x0060 : i = 0x20; break; // KC '0'  
		  case 0x006E : i = 0x2E; break; // KC '.'  
		  case 0x0061 : i = 0x21; break; // KC '1'  
		  case 0x0062 : i = 0x22; break; // KC '2'  
		  case 0x0063 : i = 0x23; break; // KC '3'  
		  case 0x0064 : i = 0x24; break; // KC '4'  
		  case 0x0065 : i = 0x25; break; // KC '5'  
		  case 0x0066 : i = 0x26; break; // KC '6'  
		  case 0x0067 : i = 0x27; break; // KC '7'  
		  case 0x0068 : i = 0x28; break; // KC '8'  
		  case 0x0069 : i = 0x29; break; // KC '9'  
		  // KC '(', 'E', ')' and '^' missing.
		  case 0x006F : i = 0x2F; break; // KC '/'  
		  case 0x006A : i = 0x2A; break; // KC '*'  
		  case 0x006D : i = 0x2D; break; // KC '-'  
		  case 0x006B : i = 0x2B; break; // KC '+'  
		  
		  // Function keys. Note - potential error in Tek documentation 0x5X verses 0x6X
		  
		  case 0x0070 : i = 0x60; break; // FK1  
		  case 0x0071 : i = 0x61; break; // FK2
		  case 0x0072 : i = 0x62; break; // FK3  
		  case 0x0073 : i = 0x63; break; // FK4  
		  case 0x0074 : i = 0x64; break; // FK5  
		  case 0x0075 : i = 0x65; break; // FK6  
		  case 0x0076 : i = 0x66; break; // FK7  
		  case 0x0077 : i = 0x67; break; // FK8  
		  case 0x0078 : i = 0x68; break; // FK9  
		  case 0x0079 : i = 0x69; break; // FK10  
		  
		  // Line editing keys.
		  
//		  case 0x0000 : i = 0x70; break; // EXPAND
//		  case 0x0000 : i = 0x71; break; // BK SPACE
//		  case 0x0000 : i = 0x72; break; // SPACE
		  case 0x007E : i = 0x73; break; // <f15> = CLEAR
//		  case 0x0000 : i = 0x74; break; // RECALL


		  // Tape control keys.
		  
//		  case 0x0000 : i = 0x77; break; // AUTO LOAD
//		  case 0x0000 : i = 0x5C; break; // REWIND
//		  case 0x0000 : i = 0x5D; break; // MAKE COPY


		  // Program control keys.
		  
		  case 0x007F : i = 0x75; break; // <f16> = AUTO NUM
		  case 0x0080 : i = 0x76; break; // <f17> = STEP PROG

		  default : i = 0; break; // Ignore the key press if I don't know what to map it to.
		  
		} // End switch.
		
		//@@@ this.print( 'KBDDEBUG: i = 0x' ); this.printHex2( i ); this.print( ' bool = ' ); this.printHex2( bool ); this.println( '' );
		
		// Check for a key press or key release.
		if( bool ) {
		
			// Key press.
			this.PIA_U461_IRA  = ((this.KBD_TTY_0 & 0x01) << 7) | (i & 0x7F);
		
			click.play(); // This works - but not too well!
			
		} else {
		
			// Key release.
			this.PIA_U461_IRA  = (this.KBD_TTY_0 & 0x01) << 7; // Just the TTY-0 signal.
			
		} // End if.
		
      	e.returnValue = false;
      	
		return false;
		
    } // End of function HandleKeyboardEvent.

    // ****************
    // ***          ***
    // ***  SCREEN  ***
    // ***          ***
    // ****************

	this.adotpending = false;
	
	this.SCREEN = function( bufclk, adot, sot, why ) {

		X_CHAR = (this.PIA_U561_ORA >>> 5) & 0x07;
		Y_CHAR = (this.PIA_U561_ORA >>> 2) & 0x07;
		
		VECTOR_0 = (this.PIA_U565_ORA >>> 6) & 1;
		VEN_1    = (this.PIA_U565_ORA >>> 7) & 1;
			
		if( bufclk != 0 ) {
		
			var old_X = this.X_DA;
			var old_Y = this.Y_DA;
			
		    var new_X = ((this.PIA_U565_ORA & 0x03) << 8) | this.PIA_U565_ORB;
		    var new_Y = ((this.PIA_U561_ORA & 0x03) << 8) | this.PIA_U561_ORB;

			if( (VEN_1 == 1) && (VECTOR_0 == 0) )
				this.VECTOR( old_X, old_Y, new_X, new_Y );
		    
			// DER 9th August 2014 - Add debug code for drawing vectors.
			if( (VEN_1 == 1) && (VECTOR_0 == 0) && 0 ) {
			
				this.print( 'DER: TEKTRONIX4051.js - $$$' );
				this.print( ' DRAW VECTOR ' ); 
				this.print( ' old_X = '     ); this.print( old_X );
				this.print( ' old_Y = '     ); this.print( old_Y );
				this.print( ' new_X = '     ); this.print( new_X );
				this.print( ' new_Y = '     ); this.print( new_Y );
				this.print( ' VECTOR-0 = '  ); this.print( VECTOR_0 );
				this.print( ' VEN-1 = '     ); this.print( VEN_1 );
				this.println('');
				
			} // End if.
			
			this.X_DA = new_X;
			this.Y_DA = new_Y;
		
		} // End if bufclk.
		
		// DER - Debug code...
		if( 0 ) {
			this.print( 'DER: TEKTRONIX4051.js - $$$' );
			this.print( ' WHY = '      ); this.print( why   );
			this.print( ' X_DA = '     ); this.print( this.X_DA   );
			this.print( ' Y_DA = '     ); this.print( this.Y_DA   );
			this.print( ' X_CHAR = '   ); this.print( X_CHAR );
			this.print( ' Y_CHAR = '   ); this.print( Y_CHAR );
			this.print( ' VECTOR-0 = ' ); this.print( VECTOR_0 );
			this.print( ' VEN-1 = '    ); this.print( VEN_1 );
			this.println('');
		} // End if
		
		// Check for cursor dot.
		if( (sot != 0) && (VECTOR_0 == 1) && (VEN_1 == 0) ) {
		
			if( this.adotpending ) {
				this.setPixel( this.X_DA + X_CHAR, this.Y_DA + Y_CHAR, 2 ); // ADOT
			} else {
				this.setPixel( this.X_DA + X_CHAR, this.Y_DA + Y_CHAR, 1 ); // SOT
			}
			this.adotpending = false;
			
		} // End if sot.
		
		// Check for alphanumeric dot.
		if( (adot != 0) && (VECTOR_0 == 1) && (VEN_1 == 0) ) {

			this.adotpending = true;
			
		} // End if adot.
		
	} // End of function SCREEN.

    // ***************
    // ***         ***
    // ***  ERASE  ***
    // ***         ***
    // ***************

	this.ERASE = function() {
	
		for( x=0; x<1024; x++ ) {
		
			for( y=0; y<780; y++ ) {
			
				this.setPixel( x, y, 0 ); // ERASE
				
			} // End for y.
			
		} // End for x.
		
	} // End of function ERASE.
	
    // **************
    // ***        ***
    // ***  COPY  ***
    // ***        ***
    // **************

	this.COPY = function() {
	
		// TODO:
		
	} // End of function COPY.
	
    // ****************
    // ***          ***
    // ***  VECTOR  ***
    // ***          ***
    // ****************

	this.VECTOR = function( x0, y0, x1, y1 ) {

		var dx = Math.abs( x1 - x0 );
		var dy = Math.abs( y1 - y0 );
		
		var sx = -1;
		var sy = -1;
		
		if( x0 < x1 ) sx = 1;
		if( y0 < y1 ) sy = 1;
		
		var err = dx - dy;
		
		do {
		
			this.setPixel( x0, y0, 3 ); // VECTOR
			
			if( (x0 == x1) && (y0 == y1) ) break;
			
			var e2 = 2 * err;
			
			if( e2 > -dy ) {
			
				err -= dy;
				
				x0 += sx;
				
			} // End if.
			
			if( e2 < dx ) {
			
				err += dx;
				
				y0 += sy;
				
			} // End if.
			
		} while( true );
			
	} // End of function VECTOR.

    // **********************
    // ***                ***
    // ***  PROCESS_GPIB  ***
    // ***                ***
    // **********************
    
    this.PROCESS_GPIB = function() {
    
		//@@@ this.print( 'PROCESS_GPIB:' );
		//@@@ 
		//@@@ if( this.GPIB_TALK   ) this.print( ' TALK   -' );
		//@@@ if( this.GPIB_LISTEN ) this.print( ' LISTEN -' );
		//@@@ 	
		//@@@ this.print( ' IFC=' ); this.printHex1( this.GPIB_IFC_OUT );
		//@@@ this.print( ' REN=' ); this.printHex1( this.GPIB_REN_OUT );
		//@@@ this.print( ' ATN=' ); this.printHex1( this.GPIB_ATN_OUT );
		//@@@ this.print( ' SRQ=' ); this.printHex1( this.GPIB_SRQ_IN  );
		//@@@ 
		//@@@ if( this.GPIB_TALK ) {
		//@@@ 	this.print( ' DATA=0x'  ); this.printHex2( this.GPIB_DATA_OUT );
		//@@@	this.print( ' EOI='     ); this.printHex1( this.GPIB_EOI_OUT  );
		//@@@ 	this.print( ' DAV='     ); this.printHex1( this.GPIB_DAV_OUT  );
		//@@@	this.print( ' NRFD='    ); this.printHex1( this.GPIB_NRFD_IN  );
		//@@@	this.print( ' NDAC='    ); this.printHex1( this.GPIB_NDAC_IN  );
		//@@@ } // End if GPIB_TALK.
		//@@@ 
		//@@@ if( this.GPIB_LISTEN ) {
		//@@@ 	this.print( ' DATA=0x'  ); this.printHex2( this.GPIB_DATA_IN );
		//@@@	this.print( ' EOI='     ); this.printHex1( this.GPIB_EOI_IN   );
		//@@@ 	this.print( ' DAV='     ); this.printHex1( this.GPIB_DAV_IN   );
		//@@@ 	this.print( ' NRFD='    ); this.printHex1( this.GPIB_NRFD_OUT );
		//@@@ 	this.print( ' NDAC='    ); this.printHex1( this.GPIB_NDAC_OUT );
		//@@@ } // End if GPIB_LISTEN.
		
		// Process data from the 4951 to the GPIB.
		if( this.GPIB_TALK ) {
		
			if( (this.GPIB_DAV_OUT == 1) && (this.GPIB_NDAC_IN == 1) ) {
			
				// Some 'new' command data for me.
				
				if( this.GPIB_ATN_OUT == 1 ) {
				
					// Some GPIB command data for me to process.
					
					//@@@ this.print( 'PROCESS_GPIB: COMMAND=0x' ); this.printHex2( this.GPIB_DATA_OUT ); this.println('');
					
					if( this.GPIB_DATA_OUT == 0x42 ) {
						//
						this.GPIB_RDTIME_STATE = 1;
						//
					} else if( this.GPIB_DATA_OUT == 0x6D ) {
						//
						if( this.GPIB_RDTIME_STATE == 1 ) {
							//
							var d = new Date();
							//
							var m = [ "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" ];
							//
							this.GPIB_RDTIME = "";
							//
							var i;
							//
							i = d.getDate(); if( i < 10) this.GPIB_RDTIME += "0"; this.GPIB_RDTIME += ("" + i);
							this.GPIB_RDTIME += "-";
							this.GPIB_RDTIME += m[ d.getMonth() ];
							this.GPIB_RDTIME += "-";
							i = d.getFullYear() % 100; if( i < 10) this.GPIB_RDTIME += "0"; this.GPIB_RDTIME += ("" + i);
							this.GPIB_RDTIME += " ";
							i = d.getHours(); if( i < 10) this.GPIB_RDTIME += "0"; this.GPIB_RDTIME += ("" + i);
							this.GPIB_RDTIME += ":";
							i = d.getMinutes(); if( i < 10) this.GPIB_RDTIME += "0"; this.GPIB_RDTIME += ("" + i);
							this.GPIB_RDTIME += ":";
							i = d.getSeconds(); if( i < 10) this.GPIB_RDTIME += "0"; this.GPIB_RDTIME += ("" + i);
							//
							this.GPIB_RDTIME_INDEX = 0; // First character of the DATE/TIME string.
							//
							//@@@ this.println( 'PROCESS_GPIB: RDTIME = ' + this.GPIB_RDTIME );
							//
							this.GPIB_RDTIME_STATE = 2;
							//
						} // End if
						//
					} // End if
					
					// Look for the GPIB Primary Address.
					//
					if( this.GPIB_DATA_OUT == 0x41 ) { // 0x41 == 65 == GPIB Device #1 Primary Talk Address.
					
						// State '1' means that the file has been loaded and is awaiting the correct GPIB Primary Address.
						//
						if( this.GPIB_STATE == 1 ) {
						
							this.GPIB_STATE = 2; // File has been loaded and I have now seen the correct GPIB Primary Address.
							
						// State '0' means that a file has not been loaded - so I should just ignore the GPIB Primary Address.
						//
						} else if( this.GPIB_STATE == 0 ) {
						
							// Ignore as no file is loaded.
							
						// I have seen a correct GPIB Primary Address - but potentially out of context!
						//
						} else if( this.GPIB_STATE >= 2 ) {
						
							this.GPIB_STATE = 1; // Go back to the 'file loaded' state as obviously something has gone wrong with the protocol.
							
						} // End if.
						 
					} // End if correct GPIB Primary Address.

					// Look for the GPIB Secondary Address.
					//
					if( this.GPIB_DATA_OUT == 0x64 ) { // 0x64 == 100 == OLD/APPEND command.
					
						// State '2' means that the file has been loaded, I have already seen the correct GPIB Primary Address and I
						// am awaiting the correct GPIB Secondar Address.
						//
						if( this.GPIB_STATE == 2 ) { 
						
							this.GPIB_STATE = 3; // File loaded and seen the correct primary and secondary GPIB addresses.
							
							if( this.sbyteslength > 0 )
								this.sbytesindex = 0; // Start at the beginning of the desired 'program' to be loaded.

						// State '0' means that a file has not been loaded - so I should just ignore the GPIB Secondary Address.
						//
						} else if( this.GPIB_STATE == 0 ) {
						
							// Ignore as no file is loaded.
							
						// State '1' means that a file has been loaded - but is awaiting the correct GPIB Primary Address - so I should just ignore the GPIB Secondary Address.
						//
						} else if( this.GPIB_STATE == 1 ) {
						
							// Ignore as no correct GPIB Primary Address seen so far.
							
						} else if( this.GPIB_STATE >= 3 ) {
						
							this.GPIB_STATE = 1; // Go back to the 'file loaded' state as obviously something has gone wrong with the protocol.
							
						} // End if.
						 
					} // End if.
					
					// Look for a GPIB 'UNTALK' Command.
					if( this.GPIB_DATA_OUT == 0x5F ) { // 0x5F == 95 == UNTALK.
					
						if( this.GPIB_STATE == 0 ) {
						
							// Ignore as no file loaded.
							
						} else {
						
							this.GPIB_STATE = 1; // Force to 'waiting for the correct GPIB Primary Address'.
							this.GPIB_RDTIME_STATE = 0;
							
						} // End if.
						
					} // End if looking for GPIB 'UNTALK' command.
					
				} else {
				
					// Normal data for me.
					// this.print( 'PROCESS_GPIB: NORMAL =0x' ); this.printHex2( this.GPIB_DATA_OUT ); this.println('');
					
					if( this.GPIB_DATA_OUT == 0x0D )
						this.println('');
					else
						this.print( "" + String.fromCharCode( this.GPIB_DATA_OUT ) );
					
				} // End if
				
			} // End if something to process.

			this.GPIB_NDAC_IN = this.GPIB_DAV_OUT ^ 0x01; // Invert before return.				
		
		} // End if GPIB_TALK.
		
		if( this.GPIB_LISTEN ) {
		
			// TODO:
			
		} // End if GPIB_LISTEN.
		
		//@@@ this.println( '' );
		
    } // End of function PROCESS_GPIB.

    // *******************
    // ***             ***
    // ***  POLL_GPIB  ***
    // ***             ***
    // *******************
    
    this.POLL_GPIB = function() {
    
		if( this.GPIB_LISTEN ) {
		
			if( (this.GPIB_NDAC_OUT == 0) && (this.GPIB_NRFD_OUT == 1) && (this.GPIB_DAV_IN == 0) && (this.GPIB_STATE == 3) ) {
			
				// Are there any (more) characters to send?
				if( (this.sbytesindex >= 0) && (this.sbytesindex < this.sbyteslength) ) {
				
					// Send a new data byte to the 4015.
					//
					this.GPIB_DATA_IN = this.sbytes[ this.sbytesindex ];
					
					// Is this the 'last' character?
					//
					if( this.sbytesindex == (this.sbyteslength - 1) ) this.GPIB_EOI_IN = 1;
					else                                              this.GPIB_EOI_IN = 0;
				
					// Bump the index.
					//
					this.sbytesindex++;
					
					if( this.GPIB_EOI_IN == 1 ) { this.print('POLL_GPIB: EOI set with DATA=0x'); this.printHex2( this.GPIB_DATA_IN ); this.println(''); }
					
					this.GPIB_DAV_IN = 1; // Signify that you have new data.
				
					//@@@ this.println( 'GPIB: >>>>>>>>>>>>>>>>>>>>>>>>>> Send new data to the 4051 and set the DAV flag...' );
					//@@@ this.print(   'GPIB: >>>>>>>>>>>>>>>>>>>>>>>>>> Data was 0x' ); this.printHex2( this.GPIB_DATA_IN ); this.println( '.' );
				
					if( this.GPIB_EOI_IN == 1 ) {
				
						//@@@ this.println( 'GPIB: >>>>>>>>>>>>>>>>>>>>>>>>>> Setting EOI flag...' );		
					
						// Check to see if PIA U265 CRA CA1 input is set for an active high-going edge interrupt.
						//
						if( ( (this.PIA_U265_CRA >>> 0) & 0x03 ) == 0x03 ) {
				
							// YES
					
							//@@@ this.println( 'GPIB: >>>>>>>>>>>>>>>>>>>>>>>>>> Setting PIA U265 IRQA1...' );
						
							//Set IRQA1 interrupt bit in PIA U265 CRA.
					
							// this.PIA_U265_CRA |= 0x80; @@@ Disable for now...
					
						} // End if is the PIA permitted to generate an interrupt.
				
					} // End if should we set the EOI flag.
				
				} // End if any more character to send of the program.

			} else if( (this.GPIB_NDAC_OUT == 0) && (this.GPIB_NRFD_OUT == 1) && (this.GPIB_DAV_IN == 0) && (this.GPIB_RDTIME_STATE == 2) ) {

				// Send a data byte to the 4015.
				//
				this.GPIB_DATA_IN = this.GPIB_RDTIME.charCodeAt( this.GPIB_RDTIME_INDEX ) & 0x7F;
					
				// Is this the 'last' character?
				//	
				if( this.GPIB_RDTIME_INDEX == (this.GPIB_RDTIME.length - 1) ) this.GPIB_EOI_IN = 1;
				else                                                          this.GPIB_EOI_IN = 0;
				
				// Bump the index.
				//
				this.GPIB_RDTIME_INDEX++;

				//@@@ this.print('POLL_GPIB: RDTIME DATA=0x'); this.printHex2( this.GPIB_DATA_IN ); this.println('');
					
				//@@@ if( this.GPIB_EOI_IN == 1 ) { this.print('POLL_GPIB: RDTIME_EOI set with DATA=0x'); this.printHex2( this.GPIB_DATA_IN ); this.println(''); }
					
				if( this.GPIB_EOI_IN == 1 ) this.GPIB_RDTIME_STATE = 0;
				
				this.GPIB_DAV_IN = 1; // Signify that you have new data.
				
			} else if( (this.GPIB_NDAC_OUT == 1) && (this.GPIB_DAV_IN == 1) ) {

				this.GPIB_DAV_IN = 0; // Clear the data available flag.

				this.GPIB_EOI_IN = 0; // Clear the EOI flag. WARNING: This needs to be here to permit a <BREAK> to occur when commanded by the human!
				
				//@@@ this.println( 'GPIB: >>>>>>>>>>>>>>>>>>>>>>>>>> Cleared the EOI and DAV flag...' );
				
			} else {
			
				// TODO:
			
			} // End if.
			
		} // End if GPIB_LISTEN.
		
    } // End of function POLL_GPIB.
	
    // ***************
    // ***         ***
    // ***  PEEKB  ***
    // ***         ***
    // ***************

    this.peekb = function( address ) {

		// Is this a read from an I/O device?
		if( (address >= 0x8780) && (address <= 0x87FF) ) {
		
			// YES
			
			switch( address ) {
			
				// **************
				// ***  U561  ***
				// **************
				
				case 0x878C : 	if( this.PIA_U561_CRA & 0x04 ) {
									this.PIA_U561_CRA &= 0x3F; // Clear both interrupt bits.
									this.SCREEN( 0, 0, 1, 'SOT' );
									return (this.PIA_U561_IRA & (this.PIA_U561_DDRA ^ 0xFF)) | (this.PIA_U561_ORA & this.PIA_U561_DDRA);
								} else
									return this.PIA_U561_DDRA;
								break;
								
				case 0x878D : 	return this.PIA_U561_CRA;
								break;
								
				case 0x878E : 	if( this.PIA_U561_CRB & 0x04 ) {
									this.PIA_U561_CRB &= 0x3F; // Clear both interrupt bits.
									return (this.PIA_U561_IRB & (this.PIA_U561_DDRB ^ 0xFF)) | (this.PIA_U561_ORB & this.PIA_U561_DDRB);
								} else
									return this.PIA_U561_DDRB;
								break;
								
				case 0x878F : 	return this.PIA_U561_CRB;
								break;
								
				// **************
				// ***  U565  ***
				// **************
				
				case 0x8794 : 	if( this.PIA_U565_CRA & 0x04 ) {
									this.PIA_U565_CRA &= 0x3F; // Clear both interrupt bits.
									// 0x8794 must be XXXX01XX VPULSE-1 = '0', DRBUSY-0 = '1'.
									this.PIA_U565_IRA = 0x04;
									this.SCREEN( 0, 1, 0, 'ADOT' );
									return (this.PIA_U565_IRA & (this.PIA_U565_DDRA ^ 0xFF)) | (this.PIA_U565_ORA & this.PIA_U565_DDRA);
								} else
									return this.PIA_U565_DDRA;
								break;
								
				case 0x8795 : 	return this.PIA_U565_CRA;
								break;
								
				case 0x8796 : 	if( this.PIA_U565_CRB & 0x04 ) {
									this.PIA_U565_CRB &= 0x3F; // Clear both interrupt bits.
									return (this.PIA_U565_IRB & (this.PIA_U565_DDRB ^ 0xFF)) | (this.PIA_U565_ORB & this.PIA_U565_DDRB);
								} else
									return this.PIA_U565_DDRB;
								break;
								
				case 0x8797 : 	return this.PIA_U565_CRB;
								break;
								
				// **************
				// ***  U361  ***
				// **************
				
				case 0x8798 : 	if( this.PIA_U361_CRA & 0x04 ) {
									this.PIA_U361_CRA &= 0x3F; // Clear both interrupt bits.
									return (this.PIA_U361_IRA & (this.PIA_U361_DDRA ^ 0xFF)) | (this.PIA_U361_ORA & this.PIA_U361_DDRA);
								} else
									return this.PIA_U361_DDRA;
								break;
								
				case 0x8799 : 	return this.PIA_U361_CRA;
								break;
								
				case 0x879A : 	if( this.PIA_U361_CRB & 0x04 ) {
									this.PIA_U361_CRB &= 0x3F; // Clear both interrupt bits.
									return (this.PIA_U361_IRB & (this.PIA_U361_DDRB ^ 0xFF)) | (this.PIA_U361_ORB & this.PIA_U361_DDRB);
								} else
									return this.PIA_U361_DDRB;
								break;
								
				case 0x879B : 	return this.PIA_U361_CRB;
								break;
								
				// **************
				// ***  U461  ***
				// **************
				
				case 0x87A8 : 	if( this.PIA_U461_CRA & 0x04 ) {
									this.PIA_U461_CRA &= 0x3F; // Clear both interrupt bits.
									// this.print('Reading U461 DRA value='); this.printHex2( (this.PIA_U461_IRA & (this.PIA_U461_DDRA ^ 0xFF)) | (this.PIA_U461_ORA & this.PIA_U461_DDRA) ); this.println('');
									// this.PIA_U461_IRA &= 0x7F; // TTY-0 = 0
									return (this.PIA_U461_IRA & (this.PIA_U461_DDRA ^ 0xFF)) | (this.PIA_U461_ORA & this.PIA_U461_DDRA);
								} else
									return this.PIA_U461_DDRA;
								break;
								
				case 0x87A9 : 	// this.print('Reading U461 CRA value='); this.printHex2( this.PIA_U461_CRA ); this.println('');
								return this.PIA_U461_CRA;
								break;
								
				case 0x87AA : 	if( this.PIA_U461_CRB & 0x04 ) {
									this.PIA_U461_CRB &= 0x3F; // Clear both interrupt bits.
									// 0x87AA must be XXXX1XXX LOAD-0 = '1'.
									// 0x87AA must be XXXX1X11 LOAD-0 = '1', CTRL-0 = '1', SHIFT-0 = '1'.
									this.PIA_U461_IRB  = 0x08 | ((this.KBD_CTRL_0 & 0x01) << 1) | ((this.KBD_SHIFT_0 & 0x01) << 0);
									this.PIA_U461_IRB &= 0xEF; // Clear out EOI-1 input (PB4).
									this.PIA_U461_IRB |= ((this.GPIB_EOI_IN & 0x01) << 4); // Merge in EOI-1 input on PB4.
									return (this.PIA_U461_IRB & (this.PIA_U461_DDRB ^ 0xFF)) | (this.PIA_U461_ORB & this.PIA_U461_DDRB);
								} else
									return this.PIA_U461_DDRB;
								break;
								
				case 0x87AB : 	return this.PIA_U461_CRB;
								break;
								
				// **************
				// ***  U265  ***
				// **************

				case 0x87B0 : 	if( this.PIA_U265_CRA & 0x04 ) {
									this.PIA_U265_CRA &= 0x3F; // Clear both interrupt bits.
									this.PIA_U265_IRA = this.GPIB_DATA_IN;
									// this.print( 'DER: TEKTRONIX4051.js -' );
									// this.print( ' GPIB Data input = 0x' );
									// this.printHex2( (this.PIA_U265_IRA & (this.PIA_U265_DDRA ^ 0xFF)) | (this.PIA_U265_ORA & this.PIA_U265_DDRA) );
									// this.println('');
									return (this.PIA_U265_IRA & (this.PIA_U265_DDRA ^ 0xFF)) | (this.PIA_U265_ORA & this.PIA_U265_DDRA);
								} else
									return this.PIA_U265_DDRA;
								break;
								
				case 0x87B1 : 	return this.PIA_U265_CRA;
								break;
								
				case 0x87B2 : 	if( this.PIA_U265_CRB & 0x04 ) {
									this.PIA_U265_CRB &= 0x3F; // Clear both interrupt bits.
									this.POLL_GPIB();
									this.PIA_U265_IRB = ( (this.GPIB_NDAC_IN & 1) << 7 ) |
									                    ( (this.GPIB_DAV_IN  & 1) << 6 ) |
									                    ( (this.GPIB_SRQ_IN  & 1) << 5 ) |
									                    ( (this.GPIB_NRFD_IN & 1) << 4 ) ; // PB3..PB0 are outputs.
									// this.print( 'DER: TEKTRONIX4051.js -' );
									// this.print( ' GPIB Control input = 0x' );
									// this.printHex2( (this.PIA_U265_IRB & (this.PIA_U265_DDRB ^ 0xFF)) | (this.PIA_U265_ORB & this.PIA_U265_DDRB) );
									// this.println('');
									return (this.PIA_U265_IRB & (this.PIA_U265_DDRB ^ 0xFF)) | (this.PIA_U265_ORB & this.PIA_U265_DDRB);
								} else
									return this.PIA_U265_DDRB;
								break;
								
				case 0x87B3 : 	return this.PIA_U265_CRB;
								break;
								
				// **************
				// ***  ELSE  ***
				// **************
												
				default :		return 0x00;
								break;
				
			} // End of switch address.
			
			// !!! WE SHOULD NEVER GET HERE !!!
						
			if(        (address >= 0x878C) && (address <= 0x878F) ) {
			   this.print( 'DER: TEKTRONIX4051.js - Reading from PIA Y-AXIS/TAPE address ' );
          	   this.printHex4( address );
       		   this.print( ' from the instruction at address ' );
       		   this.printHex4( this.last_PC );
          	   this.println( '' );
			   return 0xFF;
			} else if( (address >= 0x8794) && (address <= 0x8797) ) { 
			   this.print( 'DER: TEKTRONIX4051.js - Reading from PIA X-AXIS address ' );
          	   this.printHex4( address );
       		   this.print( ' from the instruction at address ' );
       		   this.printHex4( this.last_PC );
          	   this.println( '' );
          	   // 0x8794 must be XXXX01XX VPULSE-1 = '0', DRBUSY-0 = '1'.
          	   if( address == 0x8794 ) return 0x04;
          	   return 0x00;
			} else if( (address >= 0x8798) && (address <= 0x879B) ) { 
			   this.print( 'DER: TEKTRONIX4051.js - Reading from PIA TAPE/JOYSTICK address ' );
          	   this.printHex4( address );
       		   this.print( ' from the instruction at address ' );
       		   this.printHex4( this.last_PC );
          	   this.println( '' );
			} else if( (address >= 0x87A8) && (address <= 0x87AB) ) { 
			   this.print( 'DER: TEKTRONIX4051.js - Reading from PIA KEYBOARD/STATUS LAMPS address ' );
          	   this.printHex4( address );
       		   this.print( ' from the instruction at address ' );
       		   this.printHex4( this.last_PC );
          	   this.println( '' );
          	   // 0x87AA must be XXXX1XXX LOAD-0 = '1'
          	   if( address == 0x87AA ) return 0x08;
          	   return 0x00;
			} else if( (address >= 0x87B0) && (address <= 0x87B3) ) {
			   this.print( 'DER: TEKTRONIX4051.js - Reading from PIA GPIB address ' );
          	   this.printHex4( address );
       		   this.print( ' from the instruction at address ' );
       		   this.printHex4( this.last_PC );
          	   this.println( '' );
			} else { 
          	   this.print( 'DER: TEKTRONIX4051.js - Reading from an unidentified I/O address ' );
          	   this.printHex4( address );
       		   this.print( ' from the instruction at address ' );
       		   this.printHex4( this.last_PC );
          	   this.println( '' );
            } // End if
            
			return 0x00; // BODGE
			
		} else {

			// NO - must be RAM or ROM or BANK SWITCHED ROM then...
			
			// Check for bank switching being enabled...
			if( (address >= 0x8800) && (address < 0xA800) && (this.BANK_SWITCH_SELECTOR > 0) ) {
			
				// BANK SWITCH ENABLED and not the default 4051 ROM set.
				
				if( (address >= 0x8800) && (address < 0x9000) && (this.BANK_SWITCH_SELECTOR == 1) ) {
				
					// U101/U1 (747)
					return this.ROM747[ address - 0x8800 ];
					
				} else if( (address >= 0x9000) && (address < 0x9800) && (this.BANK_SWITCH_SELECTOR == 1) ) {

					// U201/U11 (748)
					return this.ROM748[ address - 0x9000 ];
					
				} else if( (address >= 0x8800) && (address < 0x9000) && (this.BANK_SWITCH_SELECTOR == 4) ) {
				
					// My ROM BS(L).
					return this.DERROM[ address - 0x8800 ];
					
				} else {
				
					// Unknown bank switch mapping.
					return 0x00;
					
				} // End if.
				
				// !!! SHOULD NEVER GET HERE !!!
				
			} else {
			
				return this.memoria[0][ address ]; // No bank switch enabled at the moment.
				
			} // End if.
			
			// !!! SHOULD NEVER GET HERE !!!
			
		} // End if.
			
	} // End of function peekb.
	
    // ***************
    // ***         ***
    // ***  POKEB  ***
    // ***         ***
    // ***************

    this.pokeb = function( address, value ) {

		// Is this a write to an I/O device?
		if( (address >= 0x8780) && (address <= 0x87FF) ) {
		
			// YES

			switch( address ) {
			
				// **************
				// ***  U561  ***
				// **************
				
				case 0x878C : 	if( this.PIA_U561_CRA & 0x04 ) {
									// PIAHY
									this.PIA_U561_ORA = value;
								} else
									this.PIA_U561_DDRA = value;
								break;
								
				case 0x878D : 	this.PIA_U561_CRA = value & 0x3F;
								// this.print('U561 CRA = '); this.printHex2(value);this.println('');
								break;
								
				case 0x878E : 	if( this.PIA_U561_CRB & 0x04 ) {
									// PIALY
									this.PIA_U561_ORB = value;
								} else
									this.PIA_U561_DDRB = value;
								break;
								
				case 0x878F : 	this.PIA_U561_CRB = value & 0x3F;
								// this.print('U561 CRB = '); this.printHex2(value);this.println('');
								break;
								
				// **************
				// ***  U565  ***
				// **************
				
				case 0x8794 : 	if( this.PIA_U565_CRA & 0x04 ) {
									// PIAHX
									if( ((value & 0x10) == 0) && ((this.PIA_U565_ORA & 0x10) != 0) ) {
									 	this.println( 'DER: TEKTRONIX4051.js - ERASE CRT' ); 
									 	this.ERASE();
									} // End if
									if( ((value & 0x20) == 0) && ((this.PIA_U565_ORA & 0x20) != 0) ) {
									 	this.println( 'DER: TEKTRONIX4051.js - COPY CRT' ); 
									 	this.COPY();
									} // End if
									this.PIA_U565_ORA = value;
								} else
									this.PIA_U565_DDRA = value;
								break;
								
				case 0x8795 : 	this.PIA_U565_CRA = value & 0x3F;
								// this.print('U565 CRA = '); this.printHex2(value);this.println('');
								break;
								
				case 0x8796 : 	if( this.PIA_U565_CRB & 0x04 ) {
									// PIALX
									this.PIA_U565_ORB = value;
									this.SCREEN( 1, 0, 0, 'BUFCLK' );
								} else
									this.PIA_U565_DDRB = value;
								break;
								
				case 0x8797 : 	this.PIA_U565_CRB = value & 0x3F;
								// this.print('U565 CRB = '); this.printHex2(value);this.println('');
								break;
								
				// **************
				// ***  U361  ***
				// **************
				
				case 0x8798 : 	if( this.PIA_U361_CRA & 0x04 ) {
									this.PIA_U361_ORA = value;
								} else
									this.PIA_U361_DDRA = value;
								break;
								
				case 0x8799 : 	this.PIA_U361_CRA = value & 0x3F;
								break;
								
				case 0x879A : 	if( this.PIA_U361_CRB & 0x04 ) {
									this.PIA_U361_ORB = value;
								} else
									this.PIA_U361_DDRB = value;
								break;
								
				case 0x879B : 	this.PIA_U361_CRB = value & 0x3F;
								break;
								
				// **************
				// ***  U461  ***
				// **************
				
				case 0x87A8 : 	if( this.PIA_U461_CRA & 0x04 ) {
									this.PIA_U461_ORA = value;
								} else
									this.PIA_U461_DDRA = value;
								break;
								
				case 0x87A9 : 	this.PIA_U461_CRA = value & 0x3F;
								break;
								
				case 0x87AA : 	if( this.PIA_U461_CRB & 0x04 ) {
									var opb7 = (this.PIA_U461_ORB >>> 7) & 0x01;
									var npb7 = (value             >>> 7) & 0x01;
									if( opb7 != npb7 ) {
										//!!! beep.play(); // This works - but not too well!
									} // End if.
									this.PIA_U461_ORB = value;
									this.GPIB_EOI_OUT = (this.PIA_U461_ORB >>> 4) & 0x01;
									this.GPIB_REN_OUT = (this.PIA_U461_ORB >>> 7) & 0x01;
									//@@@ this.print( 'DER: TEKTRONIX4051.js - PC=0x' );
					       			//@@@ this.printHex4( this.last_PC );
									//@@@ this.print( ' BUSY-1 = '  ); this.print( (this.PIA_U461_ORB >>> 7) & 0x01 );
									//@@@ this.print( ' I/O-1 = '   ); this.print( (this.PIA_U461_ORB >>> 6) & 0x01 );
									//@@@ this.print( ' BREAK-1 = ' ); this.print( (this.PIA_U461_ORB >>> 5) & 0x01 );
									//@@@ this.print( ' REN-1 = '   ); this.print( (this.PIA_U461_ORB >>> 7) & 0x01 );
									//@@@ this.println('');
								} else
									this.PIA_U461_DDRB = value;
								break;
								
				case 0x87AB : 	this.PIA_U461_CRB = value & 0x3F;
								break;
								
				// **************
				// ***  U265  ***
				// **************

				case 0x87B0 : 	if( this.PIA_U265_CRA & 0x04 ) {									
									this.PIA_U265_ORA = value;
									this.GPIB_DATA_OUT = this.PIA_U265_ORA;
								} else
									this.PIA_U265_DDRA = value;
								break;
								
				case 0x87B1 : 	this.PIA_U265_CRA = value & 0x3F;
								switch( (this.PIA_U265_CRA >>> 0) & 0x03 ) { // CA1 control.
								  case 0 : /* this.println( 'U265: CA1 - Input Masked.'  ); */ break;
								  case 1 : /* this.println( 'U265: CA1 - Input Lo edge.' ); */ break;
								  case 2 : /* this.println( 'U265: CA1 - Input Masked.'  ); */ break;
								  case 3 : /* this.println( 'U265: CA1 - Input Hi edge.' ); */ break;
								} // End of switch.
								switch( (this.PIA_U265_CRA >>> 3) & 0x07 ) { // CA2 control.
								  case 0 : /* this.println( 'U265: CA2 - Input Masked.'  ); */ break;
								  case 1 : /* this.println( 'U265: CA2 - Input Lo edge.' ); */ break;
								  case 2 : /* this.println( 'U265: CA2 - Input Masked.'  ); */ break;
								  case 3 : /* this.println( 'U265: CA2 - Input Hi edge.' ); */ break;
								  case 4 : /* this.println( 'U265: CA2 - Output H/S#1.'  ); */ break;
								  case 5 : /* this.println( 'U265: CA2 - Output H/S#2.'  ); */ break;
								  case 6 : this.GPIB_DAV_OUT = 0; this.PROCESS_GPIB(); break;
								  case 7 : this.GPIB_DAV_OUT = 1; this.PROCESS_GPIB(); break;
								} // End of switch.
								break;
								
				case 0x87B2 : 	if( this.PIA_U265_CRB & 0x04 ) {
									this.PIA_U265_ORB = value;
									this.GPIB_NDAC_OUT =   (this.PIA_U265_ORB >>> 7) & 0x01;
									this.GPIB_NRFD_OUT =   (this.PIA_U265_ORB >>> 4) & 0x01;
									this.GPIB_ATN_OUT  = ( (this.PIA_U265_ORB >>> 3) & 0x01 ) ^ 0x01; // Active low signal.
									this.GPIB_IFC_OUT  =   (this.PIA_U265_ORB >>> 1) & 0x01;
									this.GPIB_EOI_OUT  =   (this.PIA_U265_ORB >>> 0) & 0x01;
									this.PROCESS_GPIB();
								} else
									this.PIA_U265_DDRB = value;
								break;
								
				case 0x87B3 : 	this.PIA_U265_CRB = value & 0x3F;
								switch( (this.PIA_U265_CRB >>> 0) & 0x03 ) { // CB1 control.
								  case 0 : /* this.println( 'U265: CB1 - Input Masked.'  ); */ break;
								  case 1 : /* this.println( 'U265: CB1 - Input Lo edge.' ); */ break;
								  case 2 : /* this.println( 'U265: CB1 - Input Masked.'  ); */ break;
								  case 3 : /* this.println( 'U265: CB1 - Input Hi edge.' ); */ break;
								} // End of switch.
								switch( (this.PIA_U265_CRB >>> 3) & 0x07 ) { // CB2 control.
								  case 0 : /* this.println( 'U265: CB2 - Input Masked.'  ); */ break;
								  case 1 : /* this.println( 'U265: CB2 - Input Lo edge.' ); */ break;
								  case 2 : /* this.println( 'U265: CB2 - Input Masked.'  ); */ break;
								  case 3 : /* this.println( 'U265: CB2 - Input Hi edge.' ); */ break;
								  case 4 : /* this.println( 'U265: CB2 - Output H/S#1.'  ); */ break;
								  case 5 : /* this.println( 'U265: CB2 - Output H/S#2.'  ); */ break;
								  case 6 : this.GPIB_TALK = 0; this.GPIB_LISTEN = 1; this.PROCESS_GPIB(); break;
								  case 7 : this.GPIB_TALK = 1; this.GPIB_LISTEN = 0; this.PROCESS_GPIB(); break;
								} // End of switch.
								break;
					
				// ******************************
				// ***  BANK SWITCH SELECTOR  ***
				// ******************************

				case 0x87C0 :	this.BANK_SWITCH_SELECTOR = (value >>> 3) & 0x07; 
								break;
							
				// **************
				// ***  ELSE  ***
				// **************
												
				default :		break;
				
			} // End of switch address.
			
			// No going passed here!
			//
			return;
			
			if(        (address >= 0x878C) && (address <= 0x878F) ) {
			    this.print( 'DER: TEKTRONIX4051.js - Writing to   PIA Y-AXIS/TAPE address ' );
         		this.printHex4( address );
     			this.print( ' of ' ); 
	       		this.printHex2( value );
     			this.print( ' from the instruction at address ' );
     			this.printHex4( this.last_PC );
          		this.println( '' );
			} else if( (address >= 0x8794) && (address <= 0x8797) ) { 
 	 		    this.print( 'DER: TEKTRONIX4051.js - Writing to   PIA X-AXIS address ' );
          		this.printHex4( address );
       			this.print( ' of ' ); 
	       		this.printHex2( value );
       			this.print( ' from the instruction at address ' );
       			this.printHex4( this.last_PC );
          		this.println( '' );
			} else if( (address >= 0x8798) && (address <= 0x879B) ) { 
			    this.print( 'DER: TEKTRONIX4051.js - Writing to   PIA TAPE/JOYSTICK address ' );
          		this.printHex4( address );
       			this.print( ' of ' ); 
	       		this.printHex2( value );
       			this.print( ' from the instruction at address ' );
       			this.printHex4( this.last_PC );
          		this.println( '' );
			} else if( (address >= 0x87A8) && (address <= 0x87AB) ) { 
			    this.print( 'DER: TEKTRONIX4051.js - Writing to   PIA KEYBOARD/STATUS LAMPS address ' );
          		this.printHex4( address );
       			this.print( ' of ' ); 
	       		this.printHex2( value );
       			this.print( ' from the instruction at address ' );
       			this.printHex4( this.last_PC );
          		this.println( '' );
			} else if( (address >= 0x87B0) && (address <= 0x87B3) ) {
			    this.print( 'DER: TEKTRONIX4051.js - Writing to   PIA GPIB address ' );
          		this.printHex4( address );
       			this.print( ' of ' ); 
	       		this.printHex2( value );
       			this.print( ' from the instruction at address ' );
       			this.printHex4( this.last_PC );
          		this.println( '' );
			} else { 
          		this.print( 'DER: TEKTRONIX4051.js - Writing to an unidentified I/O address ' );
          		this.printHex4( address );
       			this.print( ' of ' ); 
	       		this.printHex2( value );
       			this.print( ' from the instruction at address ' );
       			this.printHex4( this.last_PC );
          		this.println( '' );
            } // End if
			
		} else {
		
			// NO - must be RAM or ROM then...
			
			// Check for a stupid write to the ROM!
			if( address >= 0x8000 ) {

	        	// ********************************************
				// ***                                      ***
				// ***  DUMB - Why are you writing to ROM?  ***
				// ***                                      ***
				// ********************************************
				//
  		      	this.print( 'DER: TEKTRONIX4051.js - Writing to ROM at address ' );
        		this.printHex4( address );
        		this.print( ' of ' ); 
 		       	this.printHex2( value );
        		this.print( ' from the instruction at address ' );
        		this.printHex4( this.last_PC );
        		this.println( '' );
			
			} else {
			
				this.memoria[0][ address ] = value;
				
			} // End if.
			
		} // End if.
		
	} // End of function pokeb.
	
    // ***************
    // ***         ***
    // ***  @@@@@  ***
    // ***         ***
    // ***************
    
    this.checkIRQ = function() {

    	return	(this.PIA_U461_CRA & 0xC0) | (this.PIA_U461_CRB & 0xC0) |
    			(this.PIA_U565_CRA & 0xC0) | (this.PIA_U565_CRB & 0xC0) |
    			(this.PIA_U265_CRA & 0xC0) | (this.PIA_U265_CRB & 0xC0) ;
    			
    } // End of function checkIRQ.
    
    this.checkNMI = function() {
    
    	return	(this.PIA_U561_CRA & 0xC0) | (this.PIA_U561_CRB & 0xC0) |
    			(this.PIA_U361_CRA & 0xC0) | (this.PIA_U361_CRB & 0xC0) ;
    			
    } // End of function checkNMI.
    
    this.interrupt = tek_interrupt;

    this.interrupt_start = function() {

        this.interval = setInterval( 'tek_interrupt.apply(tek)', 17 ); // 17 == 60 intervals/sec

    } // End of function interrupt_start.

    this.interrupt_stop = function() {

        clearInterval( this.interval );

    } // End of function interrupt_stop.

    this.execute_start = function() {

		this.exec_interval = setInterval( 'tek.execute.apply(tek)', 17 ); // 17 == 60 intervals/sec

    } // End of function execute_start.

    this.execute_reset = function() {

		// Completely ZAP the memory...
		for( var i=0; i<(64*1024); i++ )
			this.m0[i] = 0x01; // MC6800 NOP instruction.

		// Load up the standard ROMS.
		//
		for( var i=0x0000; i<0x0800; i++ ) { // 'i' is the offset into the ROM.
		
			this.m0[0x8000+i] = this.ROM659[ i ];
			this.m0[0x8800+i] = this.ROM660[ i ];
			this.m0[0x9000+i] = this.ROM661[ i ];
			this.m0[0x9800+i] = this.ROM662[ i ];
			this.m0[0xA000+i] = this.ROM663[ i ];
			this.m0[0xA800+i] = this.ROM664[ i ];
			this.m0[0xB000+i] = this.ROM665[ i ];
			this.m0[0xB800+i] = this.ROM666[ i ];
			this.m0[0xC000+i] = this.ROM667[ i ];
			this.m0[0xC800+i] = this.ROM668[ i ];
			this.m0[0xD000+i] = this.ROM669[ i ];
			this.m0[0xD800+i] = this.ROM670[ i ];
			this.m0[0xE000+i] = this.ROM671[ i ];
			this.m0[0xE800+i] = this.ROM672[ i ];
			this.m0[0xF000+i] = this.ROM673[ i ];
			this.m0[0xF800+i] = this.ROM674[ i ];
			
		} // End for i.

		this.reset( 0xFFFE ); // Reset the MC6800 microprocesor.
		
    } // End of function execute_reset.

    this.execute_stop = function() {

		clearInterval( tek.exec_interval );

    } // End of function execute_stop.

	this.execute_load = function( size, bytes ) {
    
        var i;
        
        // Sanity check!
        if( size < 1 )
        	return;
        	
        // Create the correct sized array to hold the 'program'.
        
        this.sbytes = Array( size );
        
        // Store the program data bytes into global memory so that it can be
        // accessed by the GPIB software.
        
		for( i=0; i<size; i++ )
			this.sbytes[i] = bytes[i];
			
		// Store the length of the 'program'.
		this.sbyteslength = size;
			 	
		this.GPIB_STATE = 1; // File loaded.
		
		this.println( 'Selected file loaded from host operating system.' );
		
	} // End of function execute_load.
    
	this.setPixel = function( x, y, n ) {
	
		switch( n ) {
		
			case 0 :
			
				// ERASE
		 				
				this.setPixelRGB( x, y, 0, 0, 0 ); // BLACK

				break;
				
			case 1 :
			
				// SOT (cursor refresh dot).
				
				this.setPixelRGB( x, y, 0, 0, 200 ); // BLUE
				// this.setPixelRGB( x, y, 0, 255, 0 ); // GREEN
				// this.setPixelRGB( x, y, 0,   0, 0 ); // BLACK
				
				break;
				
			case 2 :
			
				// ADOT (stored character dot).
				
				this.setPixelRGB( x, y, 0, 200, 0 ); // DARK GREEN
					
				break;
				
			case 3 :
			
				// VECTOR
				
				this.setPixelRGB( x, y, 0, 200, 0 ); // DARK GREEN
					
				break;
				
			default :
			
				break;
				
		} // End switch.
							
	} // End of function setPixel.

	this.setPixelRGB = function( x, y, r, g, b ) {
	
		my = 779 - y; // Convert to 'canvas' coordinates from Tektronix coordinates.
		
		this.canvas.fillStyle = "rgb("+r+","+g+","+b+")";
		this.canvas.fillRect( x,my, 1,1 );
		
	} // End of function setPixelRGB.

    this.updateScreen = function() {
    } // End of function updateScreen.
	
    //local constructor
    //initializes local variables
    {

	this.println("Booting jsTEKTRONIX4051...");

	this.pinta = true;
	this.interruptCounter = 0;
	this.frameSkip = 1;
	this.resetAtNextInterrupt = false;
	this.pauseAtNextInterrupt = false;
	this.refreshNextInterrupt = true;

    // ********************
    // ***              ***
    // ***  ROM IMAGES  ***
    // ***              ***
    // ********************

    this.ROM659 = [
0x88,0x80,0xD1,0x0D,0x80,0x70,0x8D,0x80,
0x70,0x7F,0x80,0xC5,0x20,0x80,0xBC,0xF0,
0x81,0x74,0xB0,0x81,0x8A,0xF1,0x80,0xD1,
0xB1,0x80,0xC5,0xF2,0x80,0xBC,0xB2,0x80,
0xB0,0xF3,0x81,0x32,0xB3,0x81,0x3B,0xF4,
0x81,0xC3,0xB4,0x81,0xB3,0xF5,0x82,0x38,
0xF7,0x81,0x25,0xCE,0x80,0x00,0xC6,0x11,
0xA1,0x00,0x27,0x2E,0x08,0x08,0x08,0x5A,
0x26,0xF6,0xDE,0x62,0x26,0x0E,0xDE,0x60,
0xC6,0x20,0xE7,0x00,0x09,0x8C,0x02,0x21,
0x26,0xF8,0xDF,0x62,0x84,0x7F,0xA7,0x00,
0xBD,0xF2,0xA6,0xDE,0x62,0x08,0x9C,0x60,
0x27,0x03,0xDF,0x62,0x39,0x86,0x08,0x7E,
0xF2,0x2E,0x86,0x20,0xEE,0x01,0x6E,0x00,
0xDE,0x62,0x27,0x35,0x08,0x9C,0x60,0x27,
0x0C,0x09,0xA1,0x00,0x26,0x0B,0x08,0x9C,
0x60,0x26,0xF7,0xDE,0x62,0xDF,0x64,0x20,
0x02,0x8D,0x75,0x86,0x0D,0xA7,0x00,0xDF,
0x95,0xCE,0x02,0x21,0xDF,0x98,0xD6,0x8F,
0xCA,0x20,0xD7,0x8F,0xD6,0x6B,0xCA,0x40,
0xD7,0x6B,0xC6,0x01,0xD7,0x6A,0x7E,0x81,
0x32,0xCE,0x02,0x21,0xDF,0x64,0x20,0xDB,
0xDE,0x62,0x27,0xB0,0xA1,0x00,0x26,0x28,
0x8D,0x02,0x20,0x24,0xDE,0x62,0x27,0x02,
0xA6,0x00,0x7E,0x80,0x42,0xDE,0x62,0x27,
0x9B,0xA1,0x00,0x26,0x13,0x8D,0x22,0x20,
0x0F,0xDE,0x62,0x27,0x8F,0x8D,0x1A,0xDE,
0x62,0xA6,0x00,0xBD,0xF2,0xA6,0x20,0x1B,
0xDE,0x62,0x86,0x20,0xA1,0x00,0x27,0x3C,
0xA7,0x00,0x86,0x80,0xBD,0xF2,0xA6,0x20,
0x0A,0xDE,0x62,0x8C,0x02,0x21,0x27,0x2C,
0x09,0xDF,0x62,0x86,0x08,0x7E,0xF2,0x2E,
0xC6,0x20,0xDE,0x60,0x09,0xE1,0x00,0x26,
0x0D,0x9C,0x62,0x26,0x03,0x7F,0x00,0x62,
0x8C,0x02,0x21,0x26,0xEF,0x09,0x08,0xDF,
0x64,0x96,0x62,0x26,0x07,0x9C,0x60,0x26,
0x01,0x09,0xDF,0x62,0x39,0x96,0x6B,0x85,
0x01,0x26,0xF9,0x8A,0x08,0x97,0x6B,0xBD,
0x82,0x2F,0xDE,0x41,0xDF,0x62,0x86,0x0D,
0x7E,0xF2,0x2E,0xDE,0x62,0x26,0x0D,0xDE,
0x64,0x27,0xEF,0x8C,0x02,0x21,0x27,0xEA,
0xA7,0x00,0xA7,0x01,0xBD,0x81,0x36,0x8D,
0xAF,0xCE,0x02,0x21,0xDF,0x00,0x9C,0x64,
0x27,0x0A,0xA6,0x00,0xBD,0xF2,0xA6,0xDE,
0x00,0x08,0x20,0xF0,0xDF,0x00,0x9C,0x62,
0x27,0xBA,0x86,0x08,0xBD,0xF2,0x2E,0xDE,
0x00,0x09,0x20,0xF0,0x8D,0x29,0xDE,0x60,
0x09,0xA1,0x00,0x26,0xCF,0x09,0xE6,0x00,
0xE7,0x01,0x9C,0x62,0x26,0xF7,0xA7,0x00,
0x20,0xEC,0x8D,0x13,0xDE,0x62,0xA1,0x00,
0x26,0xBA,0xE6,0x01,0xE7,0x00,0x08,0x9C,
0x60,0x26,0xF7,0xA7,0x00,0x20,0xED,0xDE,
0x62,0x27,0x0D,0xA1,0x00,0x26,0x0B,0x08,
0x9C,0x60,0x26,0xF7,0x32,0x32,0x20,0x9C,
0x32,0x32,0x39,0x86,0x0D,0xB1,0x02,0x21,
0x27,0x3D,0x8D,0x5A,0x96,0x4B,0x26,0x37,
0x08,0x26,0x0A,0xDE,0x62,0x27,0x67,0x8D,
0x4D,0x96,0x4B,0x26,0x2A,0xDF,0x00,0x96,
0x80,0x26,0x24,0x96,0x00,0xD6,0x01,0xDE,
0x3D,0x27,0x53,0xA1,0x07,0x22,0x06,0x25,
0x08,0xE1,0x08,0x23,0x04,0xEE,0x03,0x20,
0xF0,0xDF,0x02,0xBD,0x98,0xAA,0xDE,0x41,
0xDF,0x62,0x8D,0x07,0x7E,0x81,0x4C,0x7F,
0x00,0x4B,0x39,0xDE,0x95,0x09,0x9C,0x60,
0x27,0x2C,0x08,0x86,0x20,0xC6,0x0D,0xE1,
0x00,0x26,0x02,0xA7,0x00,0xA7,0x01,0x9C,
0x60,0x27,0x1B,0x08,0x20,0xF7,0xCE,0x02,
0x21,0xDF,0x00,0xDE,0x60,0xDF,0x02,0xBD,
0xB0,0x5D,0xCE,0x05,0xB9,0xBD,0xB1,0xA2,
0xFE,0x05,0xBC,0x26,0x01,0x08,0x39,0xDE,
0x41,0xDF,0x68,0xDF,0x62,0xDF,0x64,0x39,
0x96,0x6B,0x85,0x01,0x26,0xF0,0xDE,0x68,
0x27,0x07,0xDE,0x41,0xDF,0x68,0x7E,0x81,
0x32,0xDE,0x62,0x27,0x06,0x8D,0xC7,0x96,
0x4B,0x27,0x03,0xCE,0x00,0x64,0xDF,0x66,
0x09,0xDF,0x14,0xDE,0x62,0x27,0x0A,0x96,
0x4B,0x26,0x06,0x8D,0xB6,0x96,0x4B,0x27,
0x03,0xCE,0x00,0x0A,0xDF,0x68,0x7F,0x04,
0x62,0x7F,0x00,0x4B,0xBD,0x81,0x32,0x20,
0x1D,0x7D,0x04,0x62,0x26,0x33,0xDE,0x68,
0x27,0xAC,0xDE,0x41,0xDF,0x62,0x96,0x66,
0xD6,0x67,0x97,0x14,0xD7,0x15,0xDB,0x69,
0x99,0x68,0x97,0x66,0xD7,0x67,0x96,0x14,
0xD6,0x15,0x7D,0x00,0x80,0x26,0x48,0xDE,
0x3D,0x27,0x44,0xA1,0x07,0x22,0x06,0x25,
0x0C,0xE1,0x08,0x25,0x08,0xEE,0x03,0x20,
0xF0,0x7F,0x04,0x62,0x39,0xDF,0x16,0x96,
0x66,0xD6,0x67,0xA1,0x07,0x25,0x28,0x22,
0x06,0xE1,0x08,0x25,0x22,0x27,0x3B,0xDF,
0x02,0xBD,0x98,0xAA,0xCE,0x02,0x21,0x09,
0xDF,0x00,0xA6,0x01,0xBD,0xF2,0xA6,0xDE,
0x00,0x08,0x9C,0x95,0x26,0xF2,0xBD,0x81,
0x36,0xDE,0x16,0xEE,0x03,0x26,0xCE,0xDE,
0x66,0xBD,0xB0,0xC2,0xCE,0x00,0x21,0xDF,
0x1E,0xA6,0x00,0x27,0x08,0xBD,0x80,0x33,
0xDE,0x1E,0x08,0x20,0xF2,0x86,0x20,0x7E,
0x80,0x33,0xDF,0x02,0xBD,0x98,0xAA,0xBD,
0x81,0xFB,0x7E,0x81,0x4F,0x7F,0x00,0x8F,
0x7F,0x00,0x81,0xCE,0x1B,0x21,0xDF,0x56,
0xBD,0xAE,0x22,0xBD,0xAE,0xC7,0x96,0x4B,
0x26,0x30,0xC6,0x01,0xD7,0x0E,0xBD,0xDB,
0xDD,0x96,0x4B,0x26,0x25,0x96,0x82,0x85,
0x1C,0x27,0x1F,0x88,0x0A,0x85,0x0A,0x27,
0x05,0xD6,0x0E,0x5C,0x20,0xE6,0xBD,0xAF,
0x1D,0x86,0x80,0x9A,0x55,0x97,0x55,0x86,
0x6E,0x97,0x53,0x86,0x23,0x97,0x90,0x7E,
0xE0,0x8C,0x7F,0x00,0x4B,0x7E,0xAF,0x1D,
0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x14,
0x18,0x18,0x1C,0x20,0x20,0x20,0x20,0x20,
0x20,0x20,0x20,0x20,0x25,0x25,0x25,0x25,
0x25,0x25,0x25,0x25,0x25,0x25,0x25,0x2A,
0x2A,0x2E,0x33,0x38,0x3C,0x42,0x47,0x4C,
0x50,0x50,0x54,0x50,0x5A,0x5A,0x5A,0x60,
0x67,0x67,0x6D,0x72,0x78,0x7D,0x7D,0x81,
0x8B,0x90,0x86,0x90,0x95,0x9D,0x9D,0x9D,
0x9D,0x9D,0x9D,0x9D,0xA2,0xA7,0xA7,0xA7,
0xA7,0xA7,0xA7,0xA7,0xA7,0xA7,0xA7,0xA7,
0xA7,0xA7,0xA7,0xA7,0xA7,0xA7,0xAC,0x50,
0xC6,0xCA,0x04,0x08,0x0C,0x0E,0x50,0x50,
0x96,0x4B,0x27,0x40,0x96,0x55,0x84,0xF7,
0x97,0x55,0x86,0x0D,0xBD,0xF2,0xA6,0x7F,
0x04,0x62,0xBD,0xC1,0x9F,0x96,0x4B,0x81,
0x60,0x23,0x02,0x86,0x60,0x97,0x4C,0x7F,
0x00,0x4B,0xCE,0x83,0x57,0xBD,0x04,0x51,
0x8D,0x20,0x7D,0x04,0x62,0x26,0x05,0x86,
0x0D,0xBD,0xF2,0xA6,0xDE,0x41,0xFF,0x04,
0x11,0xFF,0x04,0x13,0xFF,0x04,0x15,0xFF,
0x04,0x17,0xDF,0x4D,0x4F,0x97,0x4B,0x97,
0x4C,0x39,0x4F,0xFB,0x84,0x1D,0xB9,0x84,
0x1C,0x97,0x14,0xD7,0x15,0xDE,0x14,0xA6,
0x00,0x08,0xDF,0x14,0x81,0xF0,0x24,0x06,
0x8D,0x12,0x20,0xF1,0x85,0x6C,0xCE,0x84,
0x74,0xBD,0x04,0x4B,0xCE,0x84,0x95,0xBD,
0x04,0x5D,0x20,0xE1,0x5F,0x48,0x59,0xBB,
0x84,0x84,0xF9,0x84,0x83,0x97,0x17,0xD7,
0x16,0xDE,0x16,0xDF,0x16,0xA6,0x00,0x44,
0x44,0x8D,0x21,0xDE,0x16,0xA6,0x00,0xE6,
0x01,0x58,0x49,0x58,0x49,0x58,0x49,0x8D,
0x13,0xDE,0x16,0xA6,0x01,0x8D,0x0D,0xDE,
0x16,0x08,0x08,0x6D,0x00,0x2A,0xDC,0x86,
0x20,0x7E,0xF2,0xA6,0x84,0x1F,0x27,0x2C,
0x81,0x1B,0x27,0x0A,0x25,0x10,0x81,0x1C,
0x27,0x08,0x86,0x20,0x20,0x0A,0x86,0x2D,
0x20,0x06,0x86,0x2F,0x20,0x02,0x8B,0x40,
0x7E,0xF2,0xA6,0x86,0x37,0x96,0x15,0x36,
0x96,0x14,0x36,0xBD,0x84,0x02,0x32,0x97,
0x14,0x32,0x97,0x15,0x39,0xDE,0x4D,0x27,
0xFB,0xEE,0x07,0x27,0x07,0xDF,0x18,0xC6,
0xB3,0x7E,0x84,0x85,0xC6,0xBF,0x20,0xF9,
0xDE,0x4F,0x26,0x08,0xDE,0x4D,0x27,0xE4,
0xEE,0x03,0x27,0xE0,0xEE,0x07,0xDF,0x18,
0xC6,0xB7,0x7E,0x84,0x85,0x7F,0x00,0x18,
0xB6,0x04,0x34,0x84,0x7F,0x97,0x19,0x39,
0x4F,0x97,0x12,0x97,0x13,0xCE,0x02,0xB4,
0x96,0x35,0x2B,0x05,0x08,0x08,0xDF,0x12,
0x08,0xDF,0x0A,0xBD,0x9B,0x90,0xBD,0x81,
0xFB,0xC6,0xC3,0xD7,0x1B,0xF7,0x04,0x62,
0xB6,0x04,0x36,0x81,0x47,0x23,0x05,0x86,
0x47,0xB7,0x04,0x36,0x97,0x1A,0x80,0x0D,
0x23,0x08,0x97,0x1A,0xBD,0x84,0x85,0x7F,
0x00,0x1B,0x96,0x1A,0x27,0x0A,0x86,0x20,
0xBD,0xF2,0xA6,0x7A,0x00,0x1A,0x26,0xF6,
0x86,0x7F,0xBD,0xF2,0xA6,0xD6,0x1B,0x27,
0x0A,0x86,0x20,0xBD,0xF2,0xA6,0xC6,0xC3,
0xBD,0x84,0x85,0x4F,0xF6,0x04,0x36,0xFB,
0x85,0x37,0xB9,0x85,0x36,0x97,0x62,0xD7,
0x63,0x86,0xB3,0x7E,0x80,0x33,0x02,0x21,
0x7F,0x00,0x18,0x96,0x4C,0x97,0x19,0xC6,
0xBB,0x32,0x32,0x7E,0x84,0x85,0xDE,0x18,
0xBD,0xB0,0xC2,0xCE,0x00,0x21,0xDF,0x18,
0xA6,0x00,0xBD,0xF2,0xA6,0xDE,0x18,0x08,
0xA6,0x00,0x26,0xF2,0x86,0x20,0x7E,0xF2,
0xA6,0x32,0x32,0x39,0x00,0x13,0x28,0x33,
0x49,0xA3,0xB1,0xCC,0x6D,0x1A,0xF0,0xF5,
0x73,0xF0,0xF1,0xF7,0x5B,0x00,0xF0,0xF7,
0xF3,0xF7,0xF4,0xF7,0x3D,0x42,0x4E,0xF5,
0x6B,0x1A,0xF0,0xF5,0x12,0x1A,0xF0,0xF5,
0x28,0x7A,0xF0,0xF5,0x3D,0x0D,0x04,0xF0,
0xF5,0x3D,0x28,0x04,0xF0,0xF5,0x07,0x3D,
0xF0,0xF5,0x10,0x70,0x3D,0xF0,0xF5,0x42,
0x4E,0x65,0xF0,0xF5,0x7A,0x7F,0xF0,0xF5,
0x1F,0x28,0x68,0x65,0xF0,0xF5,0x5B,0x41,
0x69,0xF0,0xF5,0x46,0x41,0x2D,0xF0,0xF5,
0x5B,0x39,0xF1,0xF7,0x77,0x1A,0xF0,0xF5,
0x50,0x6F,0x52,0x7D,0xF0,0xF5,0x50,0x1C,
0x52,0x7D,0xF0,0xF5,0x19,0x52,0x7D,0xF2,
0xF6,0xF0,0xF5,0x70,0x41,0x79,0x44,0xF0,
0xF5,0x3D,0x42,0x4E,0xF0,0xF5,0x4B,0x24,
0x51,0x2B,0xF0,0xF5,0x4B,0x63,0x1A,0xF0,
0xF5,0x4B,0x1A,0xF0,0xF5,0x4B,0x41,0x82,
0xF0,0xF5,0x50,0x5B,0x2B,0xF0,0xF5,0x4B,
0x0A,0x65,0xF0,0xF5,0x3D,0x4B,0x26,0xF0,
0xF5,0x3D,0x55,0x52,0x03,0x53,0x24,0xF0,
0xF5,0x3D,0x40,0x55,0xF0,0xF5,0x2F,0x40,
0x1A,0xF0,0xF5,0x5E,0x26,0x1A,0xF0,0xF5,
0x22,0x58,0x38,0x5E,0x26,0xF0,0xF5,0x38,
0x42,0xF6,0xF7,0x60,0x42,0xF6,0xF7,0x48,
0x4E,0xF6,0xF7,0x38,0x35,0x42,0xF7,0x75,
0x1A,0xF7,0x1A,0xF0,0xF1,0xF5,0xF7,0x84,
0x4F,0x4A,0x85,0x10,0x00,0x85,0xC0,0x86,
0x47,0x55,0xA5,0x3A,0x80,0x8C,0x2C,0x33,
0xAE,0x05,0xA5,0x8C,0x32,0x52,0x49,0x10,
0xE5,0x8D,0xED,0x34,0x2E,0x10,0x00,0x90,
0x34,0x04,0x00,0x91,0x2D,0x15,0xD3,0x25,
0xEE,0x6E,0x75,0x0A,0x63,0x49,0x30,0x50,
0x00,0x95,0xE6,0x96,0x52,0x3E,0x40,0x97,
0x14,0x16,0x4E,0x05,0x80,0x97,0x14,0x15,
0xC4,0x14,0x80,0x99,0x25,0x30,0x80,0x99,
0x2C,0x14,0x00,0x99,0xF2,0x34,0x34,0x9A,
0xAE,0x0E,0x89,0x3D,0xC0,0x99,0xF5,0x38,
0x80,0x9A,0xAC,0x30,0x00,0x9E,0x1D,0x25,
0xD4,0x16,0x46,0x04,0x65,0x74,0x55,0x4C,
0x00,0xA5,0xAD,0x14,0x89,0x06,0x85,0xA5,
0xC0,0xA5,0xD4,0x16,0x52,0x56,0x14,0x14,
0x80,0xA5,0xD6,0x05,0x89,0x10,0x00,0xA7,
0x8F,0xA6,0x60,0xB1,0x2E,0x14,0x00,0xB1,
0xEE,0x1C,0x00,0xB4,0xAD,0x3E,0x59,0xEF,
0xAD,0x16,0x73,0x04,0xE5,0xB4,0x27,0x76,
0x81,0x40,0xA0,0xBA,0xAD,0x08,0xB2,0xB9,
0xE0,0xB9,0xF4,0xBD,0xC0,0xBE,0x05,0x38,
0x00,0xBE,0x05,0x48,0x34,0x25,0xEE,0xBE,
0xC5,0x48,0xCC,0x3E,0xE0,0xC2,0x4F,0x1E,
0x41,0x34,0x00,0xC2,0x49,0x3A,0x80,0xC2,
0x49,0x3E,0x5D,0x51,0xE0,0xC8,0xA1,0x10,
0x00,0xC8,0xB1,0x55,0x32,0x14,0x80,0xC9,
0xED,0xCC,0xA3,0x48,0xB4,0xCD,0x01,0x40,
0xA0,0xCD,0x3A,0x14,0x00,0xCE,0x51,0xCE,
0x81,0x50,0xAD,0x15,0xD4,0xCE,0x8F,0x40,
0x00,0xCF,0x2E,0x50,0x38,0xCF,0x33,0x50,
0xAD,0xD1,0xEF,0xD5,0xC4,0x14,0xC9,0x38,
0xA4,0xD5,0xC9,0x50,0x00,0xD8,0x32,0x24,
0x22,0x30,0xA0,0xDE,0x49,0x50,0xBD,0x42,
0x4F,0x50,0xA3,0x50,0xA4,0x80,0x08,0x08,
0xBD,0xFA,0xEE,0x09,0x09,0xA6,0x01,0x7E,
0xFB,0x01,0xFF,0x05,0xE0,0x96,0xF8,0x8A,
0x80,0x39,0xBD,0xDD,0x18,0xBD,0xCB,0xBF,
0x7F,0x00,0x4B,0x7E,0xD1,0x59,0x86,0x18,
0x36,0x7E,0x96,0xC2,0xBD,0xF4,0xB0,0x86,
0x0C,0xD6,0x77,0x27,0x02,0x86,0x9C,0xB7,
0x87,0xB2,0x31,0x31,0x7E,0xAF,0x5E,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
    ];
    
    this.ROM660 = [
0x40,0x51,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x45,0x58,0x45,
0x43,0x20,0x20,0xF4,0x5C,0x00,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x86,0x15,
0x36,0x86,0x80,0x97,0x35,0xDE,0x04,0xDF,
0x36,0x96,0x99,0xB7,0x04,0x36,0x7F,0x04,
0x3D,0x4F,0x97,0x30,0x97,0x31,0x97,0x0C,
0x8D,0x30,0x7C,0x00,0x31,0xDE,0x98,0xA6,
0x00,0x08,0x81,0x20,0x27,0xF9,0x09,0xDF,
0x98,0x81,0x0D,0x26,0x26,0x8D,0x1B,0x86,
0x80,0xA7,0x04,0x86,0x94,0xDE,0x04,0xA7,
0x00,0x08,0xDF,0x04,0x96,0x05,0x90,0x37,
0x97,0x03,0x96,0x04,0x92,0x36,0x97,0x02,
0x32,0x39,0xFE,0x00,0x3B,0x4F,0xA7,0x02,
0xA7,0x03,0x39,0xBD,0xAA,0x5A,0x97,0x0D,
0x81,0x41,0x25,0x13,0x81,0x5B,0x24,0x0F,
0xA6,0x01,0xBD,0xAA,0x5A,0x81,0x41,0x25,
0x18,0x81,0x5B,0x24,0x14,0x96,0x0D,0x80,
0x20,0x85,0xC0,0x26,0x09,0xCE,0x8E,0x9B,
0x48,0xBD,0x04,0x45,0x26,0x06,0x7E,0x8B,
0x15,0x7E,0x8A,0xA3,0x09,0x27,0xFA,0x44,
0x36,0xDF,0x00,0xDE,0x98,0x08,0xDF,0x08,
0x32,0x36,0xDE,0x00,0xA8,0x00,0x84,0x3F,
0x27,0x04,0x32,0x7E,0x8A,0xA3,0xA6,0x00,
0x85,0xC0,0x26,0x24,0x08,0xDF,0x00,0xE6,
0x00,0xDE,0x08,0xA6,0x00,0x08,0xDF,0x08,
0xDE,0x00,0xBD,0xAA,0x5A,0x80,0x20,0x10,
0xC4,0xC0,0x1B,0x27,0x08,0xA6,0x00,0x08,
0x2A,0xFB,0x08,0x20,0xC4,0x5D,0x27,0xDC,
0x32,0xDE,0x00,0xA6,0x00,0x08,0x2B,0x1E,
0xDF,0x00,0xE6,0x00,0xC4,0x3F,0xDE,0x08,
0xA6,0x00,0xBD,0xAA,0x5A,0x80,0x20,0x10,
0x26,0x05,0x08,0xDF,0x08,0x20,0xE2,0xDE,
0x00,0xA6,0x00,0x08,0x2A,0xFB,0xA6,0x00,
0xDE,0x08,0xDF,0x98,0x97,0x08,0x81,0xB5,
0x26,0x4B,0xDE,0x04,0x86,0xB5,0xA7,0x00,
0x08,0xDF,0x08,0x7F,0x00,0x06,0x7F,0x00,
0x07,0x08,0x08,0xDF,0x04,0xDE,0x98,0xA6,
0x00,0x08,0x81,0x22,0x27,0x14,0x81,0x0D,
0x27,0x25,0xDF,0x98,0xDE,0x04,0xA7,0x00,
0x08,0xDF,0x04,0xDE,0x06,0x08,0xDF,0x06,
0x20,0xE3,0xA6,0x00,0x81,0x22,0x27,0x12,
0xDF,0x98,0xDE,0x08,0x96,0x06,0xA7,0x00,
0x96,0x07,0xA7,0x01,0x7E,0x88,0x42,0x7E,
0x8B,0x6E,0x08,0x20,0xD5,0xBD,0x90,0xA6,
0xC5,0x40,0x26,0x1A,0xB6,0x04,0x3D,0x26,
0x03,0x7C,0x00,0x30,0x96,0x35,0x84,0xFB,
0x97,0x35,0x96,0x08,0xDE,0x04,0xA7,0x00,
0x08,0xDF,0x04,0x7E,0x88,0x42,0x96,0x08,
0x81,0x16,0x22,0x39,0x81,0x11,0x26,0x2F,
0xD6,0x35,0xC4,0xFB,0xC5,0x20,0x26,0x21,
0xB6,0x04,0x3D,0x26,0x2C,0x96,0x30,0x4C,
0x97,0x30,0x4A,0x26,0x24,0x96,0x0C,0x27,
0x20,0x86,0x8C,0xCA,0x40,0xDE,0x04,0xA7,
0x00,0x08,0xDF,0x04,0xD7,0x35,0x7E,0x88,
0x42,0x86,0x51,0xC4,0xDF,0x20,0xEE,0x81,
0x0D,0x23,0x14,0x20,0x04,0x81,0x18,0x26,
0x4B,0x96,0x31,0x81,0x02,0x26,0x9D,0x96,
0x35,0x8A,0x80,0x97,0x35,0x20,0x95,0x81,
0x0B,0x23,0xEE,0x96,0x35,0x85,0x04,0x26,
0xE8,0x4D,0x2B,0x06,0x96,0x31,0x81,0x02,
0x27,0xDF,0xDE,0x98,0xA6,0x00,0x81,0x2E,
0x27,0x08,0x81,0x30,0x25,0x0A,0x81,0x3A,
0x24,0x06,0x09,0xA6,0x00,0x7E,0x8B,0x19,
0x86,0x22,0xD6,0x08,0xC1,0x0D,0x27,0x02,
0x86,0x21,0xDE,0x04,0xA7,0x00,0x08,0xDF,
0x04,0x7E,0x88,0x42,0x81,0x86,0x22,0x0B,
0x25,0x27,0x96,0x35,0x84,0xBF,0x97,0x35,
0x7E,0x89,0x74,0x81,0x8B,0x22,0x0B,0x25,
0x18,0x96,0x35,0x8A,0x02,0x97,0x35,0x7E,
0x89,0x74,0x81,0x92,0x22,0x2A,0x25,0x1C,
0x96,0x35,0x7C,0x04,0x3D,0x85,0x04,0x26,
0x03,0x7E,0x89,0x7C,0x84,0xFB,0x97,0x35,
0xDE,0x04,0x86,0x93,0xA7,0x00,0x08,0xDF,
0x04,0x7E,0x88,0x42,0x7A,0x04,0x3D,0x96,
0x35,0x8A,0x04,0x97,0x35,0x7E,0x89,0x82,
0x81,0xA3,0x25,0xF3,0x27,0x2D,0x81,0xA7,
0x25,0x09,0x96,0x35,0x8A,0x20,0x97,0x35,
0x7E,0x89,0x7C,0x86,0xA4,0xDE,0x04,0xA7,
0x00,0x08,0xDF,0x04,0xDE,0x98,0xA6,0x00,
0x08,0xDF,0x98,0x81,0x0D,0x27,0x09,0xDE,
0x04,0xA7,0x00,0x08,0xDF,0x04,0x20,0xEC,
0x7E,0x88,0x55,0x86,0xA3,0x20,0xDE,0x32,
0x7E,0x8B,0x6E,0x96,0x0D,0xF6,0x04,0x3D,
0x26,0x03,0x7C,0x00,0x0C,0xDE,0x98,0x08,
0x97,0x06,0xA6,0x00,0x97,0x07,0x81,0x30,
0x25,0x3C,0x81,0x3A,0x24,0x38,0x08,0xDF,
0x98,0xBD,0xB0,0xD6,0xDE,0x04,0x96,0x35,
0x85,0x04,0x27,0x03,0x7E,0x8B,0x6E,0x85,
0x08,0x27,0x07,0x86,0xB4,0xA7,0x00,0x08,
0x20,0x05,0x86,0xB0,0xA7,0x00,0x08,0x96,
0x06,0xA7,0x00,0x08,0x96,0x07,0xA7,0x00,
0x08,0xDF,0x04,0x86,0xE7,0x94,0x35,0x8A,
0x04,0x97,0x35,0x7E,0x88,0x42,0x81,0x24,
0x26,0x15,0x96,0x35,0x8A,0x10,0x97,0x35,
0x08,0xDF,0x98,0xBD,0xB0,0xD6,0x86,0xB1,
0xDE,0x04,0xA7,0x00,0x08,0x20,0xD0,0x86,
0x20,0x97,0x07,0x20,0xAA,0xDE,0x98,0x96,
0x0D,0x08,0xDF,0x98,0x97,0x8D,0xD6,0x35,
0xCA,0x04,0xD7,0x35,0xBD,0xB5,0x6F,0xDE,
0x98,0x09,0xB6,0x05,0xA9,0x27,0x01,0x09,
0xDF,0x98,0xB6,0x05,0xA7,0x26,0x32,0xB6,
0x05,0xA8,0x27,0x1B,0x30,0xBD,0xB1,0xA2,
0xA6,0x03,0xE6,0x04,0xBD,0xAA,0x3A,0x35,
0xDE,0x04,0xA7,0x01,0xE7,0x02,0x86,0xB2,
0xA7,0x00,0x08,0x08,0x08,0x20,0x0D,0xDE,
0x04,0x86,0xB6,0xA7,0x00,0x08,0xBD,0xB7,
0x0F,0xBD,0xAA,0x3B,0xDF,0x04,0x7E,0x88,
0x42,0x30,0xBD,0xAA,0x3A,0x35,0x86,0x5E,
0x97,0x4B,0x96,0x99,0xB0,0x04,0x36,0xB7,
0x04,0x36,0x32,0x39,0xC8,0x92,0xC9,0x8F,
0xCA,0x0E,0xCB,0x0C,0xCC,0x90,0xCD,0x0D,
0xCF,0x0F,0x21,0x22,0xF3,0x3F,0x21,0x23,
0xF3,0x48,0x21,0x23,0x6F,0xB3,0x48,0x21,
0x2C,0xEC,0xAE,0x21,0x2E,0xE4,0x0A,0x21,
0x30,0x70,0x25,0x2E,0xA4,0x59,0x21,0x33,
0xE3,0x1E,0x21,0x33,0xEE,0x47,0x21,0x33,
0x69,0xAE,0x47,0x21,0x33,0x73,0x29,0x27,
0xAE,0x74,0x21,0xF3,0x85,0x21,0x34,0xEE,
0x49,0x21,0x34,0x21,0xEE,0x49,0x21,0x38,
0x69,0xB3,0x6D,0x29,0xE6,0x54,0x29,0x2D,
0x61,0x27,0xA5,0xA3,0x29,0x2E,0x69,0xB4,
0x62,0x29,0x2E,0x70,0x35,0xB4,0x69,0x29,
0x2E,0xF4,0x43,0x29,0x2E,0xF6,0x1B,0xC2,
0xB5,0xC3,0x4F,0xC5,0x4D,0xC6,0x17,0x23,
0x21,0x6C,0xAC,0x6A,0x23,0x21,0x73,0xA5,
0x9C,0x23,0x28,0xF2,0x1C,0x23,0x2C,0x6F,
0x33,0xA5,0x7E,0x23,0x2D,0xE4,0x83,0x23,
0x2F,0x70,0xB9,0x73,0x23,0x2F,0xF3,0x4B,
0x23,0x32,0x65,0x21,0x34,0xA5,0x6F,0x24,
0x21,0x74,0xA1,0xA5,0x24,0x25,0xE6,0xA7,
0x24,0x25,0x67,0x32,0x25,0x25,0xB3,0x96,
0x24,0x25,0x6C,0x25,0x34,0xA5,0x58,0x24,
0x25,0xF4,0xA2,0x24,0x29,0xED,0xA6,0x24,
0x29,0x72,0x25,0x23,0x34,0x2F,0x32,0xB9,
0x7D,0x24,0x32,0x61,0xB7,0x03,0x2B,0x25,
0xF9,0x9A,0x2B,0x29,0x6C,0xAC,0x71,0x2C,
0x25,0xEE,0x20,0x2C,0x25,0xF4,0x65,0x2C,
0x27,0xF4,0x44,0x2C,0x29,0x73,0xB4,0x5A,
0x2C,0x2F,0xE7,0x45,0x2D,0x21,0x72,0xAB,
0x79,0x2D,0x21,0xF4,0x66,0x2D,0x21,0xF8,
0x09,0x2D,0x25,0x6D,0x2F,0x32,0xB9,0xA1,
0x2D,0x29,0xEE,0x08,0x2D,0x2F,0x76,0xA5,
0x01,0x2D,0x30,0xF9,0x18,0x2E,0x25,0x78,
0xB4,0x5D,0x2E,0x2F,0x63,0x21,0x33,0xA5,
0x9D,0x2E,0x2F,0x6B,0x25,0xB9,0x9B,0x2E,
0x2F,0x72,0x2D,0x21,0xAC,0x99,0x2E,0x2F,
0xF4,0x23,0xDA,0x8D,0xDB,0x8E,0x1C,0xDD,
0x14,0x1C,0xDE,0x12,0xDC,0x13,0x1D,0xDE,
0x15,0x1D,0xDC,0x14,0xDD,0x11,0x1E,0xDD,
0x15,0x1E,0xDC,0x12,0xDE,0x16,0xE0,0x4E,
0x27,0x29,0xEE,0x63,0x27,0x2F,0x73,0x35,
0xA2,0x53,0x27,0xEF,0x52,0x27,0x32,0x61,
0x24,0xB3,0x95,0x2F,0x26,0xE6,0xA8,0x2F,
0xE6,0x8B,0x2F,0x2C,0xE4,0x6E,0x2F,0xEE,
0xA9,0x2F,0x30,0x65,0xAE,0x70,0x2F,0xF2,
0x0B,0x30,0x21,0x67,0xA5,0x7B,0x30,0xE9,
0x9F,0x30,0x2F,0x69,0x2E,0x34,0x25,0xB2,
0x84,0x30,0x2F,0x6C,0xAC,0x64,0x30,0x2F,
0xF3,0x07,0x30,0x32,0x69,0x2E,0xB4,0x5C,
0x32,0x21,0x64,0x29,0x21,0x2E,0xB3,0x97,
0x32,0x22,0x79,0x34,0xA5,0x77,0x32,0x25,
0x61,0xA4,0x67,0x32,0x25,0xF0,0x06,0x32,
0x25,0x6C,0x21,0x22,0x25,0xAC,0x8A,0x32,
0x25,0xED,0xA4,0x32,0x25,0x6E,0x35,0x2D,
0x22,0x25,0xB2,0x57,0x32,0x25,0x73,0x34,
0x2F,0x32,0xA5,0x56,0x32,0x25,0x74,0x35,
0x32,0xAE,0x5E,0x32,0x24,0x72,0x21,0xB7,
0x04,0x32,0x2D,0x6F,0x36,0xA5,0x02,0x32,
0x2E,0xE4,0x42,0x32,0x2F,0x74,0x21,0x34,
0xA5,0x82,0x32,0x35,0xEE,0x55,0x35,0x2E,
0x69,0xB4,0x75,0x35,0x33,0x69,0x2E,0xA7,
0x91,0x36,0x21,0xEC,0x1F,0x36,0x29,0x65,
0x37,0x30,0x2F,0x32,0xB4,0x6C,0x37,0x21,
0x69,0xB4,0x7F,0x37,0x22,0x79,0x34,0xA5,
0x78,0x37,0x29,0x6E,0x24,0x2F,0xB7,0x6B,
0x37,0x32,0x69,0x34,0xA5,0x68,0xFE,0x10,
0xDD,0x51,0xDD,0x8C,0xC8,0x93,0xDB,0xAF,
0xCB,0x21,0xCD,0x22,0x34,0xEF,0x87,0x25,
0x2E,0xE4,0x60,0x25,0x2F,0xE6,0x9E,0x25,
0x2F,0xE9,0xAD,0x25,0x38,0xF0,0x46,0x26,
0x29,0x6E,0xA4,0x76,0x26,0x2F,0xF2,0x50,
0x26,0x35,0x6C,0xAC,0xAB,0x26,0x35,0x7A,
0xBA,0x80,0x26,0x2E,0xE1,0x24,0x26,0x2E,
0xE2,0x25,0x26,0x2E,0xE3,0x26,0x26,0x2E,
0xE4,0x27,0x26,0x2E,0xE5,0x28,0x26,0x2E,
0xE6,0x29,0x26,0x2E,0xE7,0x2A,0x26,0x2E,
0xE8,0x2B,0x26,0x2E,0xE9,0x2C,0x26,0x2E,
0xEA,0x2D,0x26,0x2E,0xEB,0x2E,0x26,0x2E,
0xEC,0x2F,0x26,0x2E,0xED,0x30,0x26,0x2E,
0xEE,0x31,0x26,0x2E,0xEF,0x32,0x26,0x2E,
0xF0,0x33,0x26,0x2E,0xF1,0x34,0x26,0x2E,
0xF2,0x35,0x26,0x2E,0xF3,0x36,0x26,0x2E,
0xF4,0x37,0x26,0x2E,0xF5,0x38,0x26,0x2E,
0xF6,0x39,0x26,0x2E,0xF7,0x3A,0x26,0x2E,
0xF8,0x3B,0x26,0x2E,0xF9,0x3C,0x26,0x2E,
0xFA,0x3D,0x28,0x2F,0x6D,0xA5,0x7C,0x33,
0x21,0x76,0xA5,0x5B,0x33,0x23,0x61,0x2C,
0xA5,0x81,0x33,0x25,0x63,0x32,0x25,0xB4,
0x72,0x33,0x25,0xE7,0x05,0x33,0x25,0xF4,
0x61,0x33,0x27,0xEE,0x41,0x33,0x29,0x67,
0xAE,0x41,0x33,0x29,0xEE,0x4C,0x33,0x29,
0x7A,0xA5,0xAA,0x33,0x30,0x61,0x23,0xA5,
0xA0,0x33,0x31,0xF2,0x40,0x33,0x32,0xF1,
0xAC,0x33,0x34,0x65,0xB0,0x88,0x33,0x34,
0x6F,0xB0,0x5F,0x33,0x34,0xF2,0x1D,0x33,
0x35,0xED,0x19,0x34,0x21,0xEE,0x4A,0x34,
0x28,0x65,0xAE,0x89,0x34,0x2C,0x69,0x33,
0xB4,0x7A,0x34,0xEF,0x86,0x34,0x32,0x61,
0x23,0xA5,0x98,0x34,0x32,0xEE,0x1A,0x34,
0x39,0xF0,0x3E,0x00,0x00,0x00,0x00,0x8B,
0xE8,0x8B,0xEA,0x00,0x00,0x8B,0xEC,0x8B,
0xEE,0x00,0x00,0x8B,0x7D,0x8B,0x7F,0x8B,
0x81,0x8B,0x83,0x8B,0x85,0x8B,0x87,0x00,
0x00,0x8B,0x89,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8C,
0xA3,0x8C,0xA5,0x8C,0xA7,0x8C,0xAF,0x8C,
0xB7,0x00,0x00,0x8C,0xBF,0x8B,0x8B,0x00,
0x01,0x8B,0xF0,0x8C,0x18,0x8D,0xA0,0x8D,
0xB0,0x8C,0xC1,0x8E,0x2B,0x8B,0xCC,0x00,
0x01,0x8C,0x47,0x8C,0x50,0x8C,0x65,0x8C,
0x86,0x8C,0xD4,0x8C,0xEA,0x00,0x01,0x8D,
0x09,0x8E,0x30,0x8E,0x7C,0x8D,0x5F,0x8D,
0x6A,0x8D,0x77,0x00,0x01,0x00,0x01,0x00,
0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x8D,
0x8F,0x00,0x00,0x66,0xE9,0x48,0x03,0xAA,
0xA5,0xF1,0xC9,0xF7,0xFF,0xFF,0xFF,0xFF,
0xFF,0xFF,0xF3,0xE1,0x70,0x37,0x67,0xA8,
0xA9,0x1E,0x6F,0xCA,0x5A,0x1A,0x26,0x74,
0xBA,0x7B,0x5B,0xE6,0x3F,0xEA,0xA0,0x80,
0x9B,0x55,0x7B,0x52,0x56,0xBF,0xC8,0xFF,
0x11,0x0A,0x11,0x0A,0x11,0x0A,0x11,0x0A,
0xDD,0x13,0xDD,0x13,0xDD,0x13,0x88,0x6A,
0x88,0x6A,0x55,0x6A,0x55,0x6A,0x99,0x62,
0x99,0x62,0xAA,0x62,0xAA,0x62,0xBB,0x62,
0x77,0x62,0x77,0x62,0x77,0x62,0x77,0x62,
0x77,0x62,0x77,0x62,0xAA,0x22,0xAA,0x6A,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xCC,0x01,0xCC,0x01,0x66,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0xDD,0x11,0xDD,0x11,0xDD,0x11,0xDD,0x11,
0x22,0x80,0x22,0x80,0x22,0x80,0x11,0xC8,
0x22,0x22,0x11,0x49,0x11,0x49,0x11,0x0A,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88
    ];
    
    this.ROM661 = [
0x11,0x09,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x0A,0x11,0x88,
0x11,0x49,0x11,0x49,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x88,0x11,0x88,0x11,0x88,0x11,0x88,
0x11,0x29,0x22,0x49,0x22,0x0A,0x22,0x09,
0x22,0x69,0x22,0x88,0x22,0xE8,0x22,0x22,
0x23,0x01,0x33,0x01,0x33,0x41,0x43,0x01,
0x44,0x09,0xF3,0x61,0xF3,0x01,0x00,0x01,
0xDD,0x08,0xDD,0x08,0xDD,0x08,0xDD,0x08,
0xDD,0x08,0xDD,0x08,0xDD,0x08,0xDD,0x08,
0xDD,0x08,0x22,0x19,0xEE,0x40,0xEE,0x40,
0xEE,0x40,0xEE,0x40,0x00,0xC8,0x00,0xC0,
0x00,0x88,0x00,0x88,0x00,0x49,0x00,0x09,
0x00,0x09,0x00,0x08,0x00,0x08,0x00,0x08,
0x00,0x08,0x00,0x08,0x00,0x00,0xCE,0x8F,
0x47,0x48,0x24,0x03,0xCE,0x90,0x47,0xBD,
0x04,0x51,0x09,0xBD,0x04,0x4B,0x39,0xFF,
0x7F,0x03,0xB5,0x7F,0x03,0xB6,0x32,0x33,
0xB7,0x03,0xB1,0xF7,0x03,0xB2,0x32,0x32,
0xB7,0x03,0xB3,0x32,0xB7,0x03,0xB4,0xDE,
0x3D,0x26,0x05,0xFE,0x03,0xB1,0x6E,0x02,
0xFF,0x03,0xB5,0xA6,0x07,0xB1,0x03,0xB3,
0x25,0x12,0x22,0x0B,0xA6,0x08,0xB1,0x03,
0xB4,0x25,0x09,0x22,0x02,0x20,0x33,0xFE,
0x03,0xB1,0x6E,0x00,0xEE,0x03,0x20,0xD9,
0x32,0x33,0xB7,0x03,0xB7,0xF7,0x03,0xB8,
0x8D,0xB6,0x20,0x16,0x20,0x1C,0x32,0x33,
0xB7,0x03,0xB7,0xF7,0x03,0xB8,0x8D,0xA8,
0x20,0x10,0x7F,0x03,0xB5,0x7F,0x03,0xB6,
0x20,0x08,0xFE,0x03,0xB5,0xEE,0x05,0xFF,
0x03,0xB5,0xFE,0x03,0xB5,0xA6,0x08,0x36,
0xA6,0x07,0x36,0x86,0x0D,0x36,0xB6,0x03,
0xB6,0x36,0xB6,0x03,0xB5,0x36,0x86,0x07,
0x36,0xFE,0x03,0xB7,0x6E,0x00,0x32,0x33,
0x97,0x02,0xD7,0x03,0xDF,0x04,0xEE,0x07,
0xDF,0x06,0x96,0x07,0x36,0x96,0x06,0x36,
0x86,0x0E,0x36,0xBD,0x90,0xF8,0xBD,0xA9,
0xCC,0x32,0x32,0x33,0x97,0x08,0xD7,0x09,
0x32,0x32,0x33,0x97,0x0A,0xD7,0x0B,0xDE,
0x08,0x27,0x3A,0xDE,0x06,0x9C,0x0A,0x26,
0x27,0xDE,0x08,0xEE,0x05,0xDF,0x08,0xCE,
0x00,0x00,0xBD,0xB6,0xEB,0xCE,0x00,0x08,
0xBD,0xB6,0xEB,0xDE,0x0A,0xDF,0x06,0xDF,
0x08,0xBD,0xE6,0xAB,0xCE,0x00,0x08,0xBD,
0xB7,0x0F,0xCE,0x00,0x00,0xBD,0xB7,0x0F,
0xDE,0x04,0xEE,0x00,0x8C,0x00,0x0A,0x27,
0x5C,0xDE,0x08,0x26,0x2A,0xDE,0x04,0xEE,
0x00,0x8C,0x00,0x0A,0x27,0x4F,0xDE,0x04,
0x96,0x3D,0xA7,0x03,0x96,0x3E,0xA7,0x04,
0x4F,0xA7,0x05,0xA7,0x06,0x96,0x04,0x97,
0x3D,0xD6,0x05,0xD7,0x3E,0xEE,0x03,0x27,
0x04,0xA7,0x05,0xE7,0x06,0x20,0x2E,0xDE,
0x08,0xEE,0x03,0xDF,0x0A,0xDE,0x08,0x96,
0x04,0xA7,0x03,0x96,0x05,0xA7,0x04,0xDE,
0x0A,0x27,0x08,0x96,0x04,0xA7,0x05,0x96,
0x05,0xA7,0x06,0xDE,0x04,0x96,0x0A,0xA7,
0x03,0x96,0x0B,0xA7,0x04,0x96,0x08,0xA7,
0x05,0x96,0x09,0xA7,0x06,0xBD,0xA9,0x94,
0xDE,0x02,0x6E,0x00,0xBD,0xA9,0xE7,0x4F,
0x97,0x28,0xDE,0x41,0xDF,0x4F,0xBD,0xCE,
0x70,0x26,0x23,0x30,0xBD,0xB1,0xA2,0x26,
0x1D,0xDE,0x00,0xDF,0x06,0xBD,0xCE,0x77,
0x26,0x1E,0xDE,0x06,0x08,0xBD,0xB1,0xA2,
0x26,0x0C,0x30,0xEE,0x03,0xDF,0x1A,0x30,
0xBD,0xAA,0x3A,0x35,0x20,0x12,0x86,0x0E,
0x97,0x4B,0xBD,0xD1,0x59,0x7E,0xA9,0xFD,
0xCE,0x00,0x0A,0xDF,0x1A,0x73,0x00,0x28,
0x30,0xEE,0x03,0xDF,0x18,0x32,0x32,0xBD,
0x90,0xF8,0x30,0xEE,0x01,0xDF,0x1C,0x30,
0xBD,0xAA,0x39,0x35,0xDE,0x1C,0x27,0x06,
0xEE,0x07,0x9C,0x18,0x27,0x06,0x86,0x0F,
0x97,0x4B,0x20,0xCE,0xDF,0x06,0xDF,0x08,
0xDE,0x1C,0xEE,0x03,0x86,0x07,0xBD,0xAA,
0x0A,0xDE,0x1C,0xEE,0x05,0x26,0x03,0xCE,
0x00,0x3A,0x86,0x07,0xBD,0xAA,0x0A,0xBD,
0xE6,0xAB,0xDE,0x3D,0x86,0x06,0xBD,0xAA,
0x0A,0xDE,0x41,0xDF,0x3D,0xBD,0xD9,0xEC,
0xBD,0xAA,0x19,0x24,0xBD,0xAA,0x19,0x1C,
0xBD,0xAA,0x19,0x20,0xDE,0x20,0xEE,0x07,
0xDF,0x22,0x96,0x4B,0x27,0x05,0x97,0x4C,
0x7F,0x00,0x4B,0x86,0xFF,0x36,0x36,0x86,
0x0D,0x36,0xBD,0x90,0xF8,0x30,0xEE,0x01,
0xDF,0x26,0xEE,0x07,0xDF,0x06,0x30,0xBD,
0xAA,0x3D,0x35,0xDE,0x1A,0xDF,0x02,0xDE,
0x3D,0x27,0x15,0xEE,0x07,0x9C,0x18,0x26,
0x04,0x96,0x28,0x26,0x0B,0xDE,0x18,0xDF,
0x00,0xDE,0x41,0xDF,0x04,0xBD,0x9D,0xC8,
0xDE,0x3D,0xDF,0x06,0xDE,0x24,0xDF,0x3D,
0xDE,0x06,0xDF,0x24,0x27,0x68,0xDE,0x3D,
0x27,0x33,0xDE,0x26,0x27,0x2F,0xEE,0x07,
0xDF,0x00,0xD6,0x01,0x96,0x28,0x27,0x10,
0xDE,0x20,0x27,0x21,0x96,0x00,0x91,0x22,
0x25,0x1B,0x22,0x04,0xD1,0x23,0x25,0x15,
0x96,0x00,0xDB,0x1B,0x99,0x1A,0xD7,0x01,
0x97,0x00,0xDE,0x22,0xDF,0x04,0xDE,0x1A,
0xDF,0x02,0xBD,0x9D,0xC8,0x96,0x24,0xD6,
0x25,0xDE,0x1C,0xA7,0x03,0xE7,0x04,0xD6,
0x1D,0x96,0x1C,0x26,0x01,0x5F,0xDE,0x24,
0x27,0x04,0xA7,0x05,0xE7,0x06,0x96,0x26,
0xD6,0x27,0xDE,0x20,0x27,0x04,0xA7,0x05,
0xE7,0x06,0x96,0x20,0xD6,0x21,0xDE,0x26,
0x27,0x04,0xA7,0x03,0xE7,0x04,0x96,0x4C,
0x7F,0x00,0x4C,0x97,0x4B,0x7E,0x92,0x3A,
0x96,0x53,0x80,0x24,0x48,0xCE,0x03,0xDD,
0xBD,0x04,0x45,0x27,0x1D,0xDF,0x00,0x86,
0x15,0x36,0xBD,0xD0,0x62,0x86,0x10,0x97,
0x54,0xDE,0x00,0xDF,0x4D,0xBD,0xAA,0x35,
0xDF,0x51,0xBD,0x96,0xBC,0xBD,0xD0,0x7C,
0x32,0x39,0xBD,0xD1,0x65,0x0B,0x96,0x55,
0x85,0xC0,0x27,0x22,0xDE,0x51,0xE6,0x03,
0xC0,0x24,0x58,0x4F,0xFB,0x93,0xC3,0xB9,
0x93,0xC2,0x97,0x00,0xD7,0x01,0xDE,0x00,
0x01,0x0F,0x96,0x4D,0xA7,0x00,0x96,0x4E,
0xA7,0x01,0x0E,0x7E,0xD1,0x59,0xBD,0xD1,
0x65,0x23,0x03,0xDD,0x86,0x15,0x36,0xBD,
0xA9,0x9E,0x01,0x0F,0x96,0x6D,0x26,0x50,
0x96,0x6E,0x26,0x40,0xD6,0x55,0x2A,0x04,
0xC5,0x10,0x26,0x35,0xF6,0x04,0x11,0x27,
0x30,0xF7,0x04,0x62,0xCE,0x04,0x12,0xBD,
0xB6,0xEB,0x09,0xBD,0xB7,0x0F,0x0E,0x58,
0x58,0xD7,0x01,0x7F,0x00,0x00,0xBD,0xD0,
0x82,0x96,0x4B,0x7F,0x00,0x4B,0x4D,0x26,
0x10,0xBD,0x82,0x2F,0xBD,0xD0,0x62,0xDE,
0x00,0xDF,0x4F,0xBD,0x96,0xB3,0xBD,0xD0,
0x7C,0x0E,0x32,0x39,0xB7,0x04,0x34,0x48,
0x8B,0x10,0x5F,0x7F,0x00,0x6E,0x20,0x24,
0x4F,0xBD,0xAB,0x3B,0x96,0x6D,0x27,0xE9,
0xCE,0x94,0xC1,0x4F,0xC6,0x80,0xD5,0x6D,
0x26,0x06,0x08,0x4C,0x4C,0x54,0x20,0xF6,
0xD8,0x6D,0xD7,0x6D,0xE6,0x00,0xDA,0x55,
0xD7,0x55,0xE6,0x00,0x0E,0x36,0x37,0xC6,
0x0E,0x37,0xD6,0x55,0x2A,0x52,0xCE,0x03,
0xB9,0xBD,0x04,0x51,0x27,0x4A,0xD7,0x00,
0x5F,0xBD,0x04,0x57,0x4C,0xBD,0x04,0x51,
0x37,0xD7,0x01,0x5F,0xBD,0x04,0x57,0x96,
0x00,0x36,0x86,0x07,0x36,0xBD,0xD0,0x62,
0xDE,0x00,0xDF,0x4F,0xBD,0x96,0xB3,0xBD,
0xD0,0x7C,0x32,0x30,0xA6,0x04,0xCE,0x03,
0xB9,0x33,0xBD,0x04,0x57,0x4C,0x33,0xBD,
0x04,0x57,0x32,0x33,0x32,0x01,0x0F,0x53,
0xD4,0x55,0xCA,0x02,0xD7,0x55,0x81,0x06,
0x26,0x03,0xBD,0xF5,0xAC,0x7E,0x94,0x11,
0x81,0x04,0x27,0xE6,0x81,0x08,0x27,0xE2,
0xBD,0xC1,0x9F,0x30,0xA6,0x02,0x44,0x81,
0x08,0x23,0x02,0x86,0x08,0x8B,0x28,0x97,
0x4B,0x96,0x4C,0x27,0xCD,0x97,0x4B,0x20,
0xC9,0x00,0x00,0x00,0x08,0x00,0x00,0x00,
0x00,0xFF,0x30,0x09,0x09,0xBD,0x95,0x3C,
0xBD,0x95,0x5D,0xDE,0x41,0xDF,0x10,0xDF,
0x12,0x20,0x27,0xBD,0x95,0x38,0xBD,0x95,
0x5D,0xBD,0x95,0x68,0x20,0x1C,0xBD,0x95,
0x38,0xBD,0x95,0x68,0x30,0x08,0xDF,0x14,
0xDF,0x16,0x20,0x0E,0xBD,0x95,0x38,0xBD,
0x95,0x5D,0x30,0xBD,0xAA,0x39,0xDF,0x10,
0xDF,0x12,0xDE,0x10,0x27,0x0E,0xBD,0xB6,
0xEB,0xBD,0xAA,0x3B,0xDF,0x10,0xDE,0x12,
0x27,0x02,0xDF,0x10,0xDE,0x14,0xBD,0xB6,
0xEB,0xBD,0xAA,0x3B,0xDF,0x14,0xDE,0x16,
0x27,0x02,0xDF,0x14,0xBD,0xAA,0xE4,0xDE,
0x0C,0xBD,0xB7,0x0F,0xBD,0xAA,0x3B,0xDF,
0x0C,0x9C,0x0E,0x26,0xCD,0x7E,0xD1,0x60,
0x30,0xBD,0xAA,0x3A,0xDF,0x04,0xEE,0x0C,
0xEE,0x0B,0xBD,0xAA,0x3E,0xDF,0x0C,0xDE,
0x04,0xEE,0x0E,0xBD,0xAA,0x3E,0xDF,0x0E,
0xDE,0x04,0xEE,0x10,0xDF,0x00,0xDE,0x04,
0xEE,0x12,0xDF,0x02,0x39,0x30,0xBD,0x95,
0x76,0xDF,0x14,0xDE,0x41,0xDF,0x16,0x39,
0x30,0xBD,0xAA,0x3A,0xBD,0x95,0x76,0xDF,
0x10,0xDE,0x41,0xDF,0x12,0x39,0xDF,0x04,
0xEE,0x07,0x9C,0x00,0x26,0x12,0xDE,0x04,
0xEE,0x09,0x9C,0x02,0x26,0x0A,0xDE,0x04,
0xEE,0x03,0xEE,0x0B,0xBD,0xAA,0x3E,0x39,
0x32,0x32,0x32,0x32,0xBD,0xD1,0x6A,0x08,
0x30,0xEE,0x03,0xDF,0x00,0xA6,0x00,0xE6,
0x01,0x30,0xEE,0x08,0xDF,0x04,0x7F,0x00,
0x06,0xA1,0x00,0x22,0x0D,0x25,0x06,0xE1,
0x01,0x22,0x07,0x27,0x0C,0x7C,0x00,0x06,
0x20,0x07,0xA6,0x00,0xE6,0x01,0x7A,0x00,
0x06,0xDB,0x01,0x99,0x00,0x97,0x02,0xD7,
0x03,0x30,0xBD,0xAA,0x39,0x35,0xDE,0x00,
0x9C,0x02,0x27,0x1A,0xA6,0x02,0x08,0xDF,
0x00,0xBD,0xAA,0x52,0x16,0xDE,0x04,0xA6,
0x02,0x08,0xDF,0x04,0xBD,0xAA,0x52,0x11,
0x22,0x0E,0x25,0x10,0x20,0xE0,0x96,0x06,
0x2E,0x06,0x2D,0x08,0x86,0x01,0x20,0x06,
0x86,0x02,0x20,0x02,0x86,0x04,0x97,0x08,
0x96,0x53,0x80,0x11,0xCE,0x96,0x1A,0xBD,
0x04,0x4B,0xCE,0xE9,0x50,0x95,0x08,0x26,
0x03,0xCE,0xE9,0x26,0xBD,0xB6,0xEB,0x7E,
0x00,0x58,0x01,0x06,0x04,0x05,0x03,0x02,
0x30,0xEE,0x01,0xEE,0x0B,0xBD,0xAA,0x3E,
0xDF,0x00,0x30,0xEE,0x03,0xBD,0xAA,0x3E,
0xDF,0x02,0x30,0xBD,0xAA,0x3A,0x35,0xCE,
0xE9,0x26,0xBD,0xB6,0xEB,0xDE,0x00,0xBD,
0xB6,0xEB,0xBD,0xB2,0x45,0xDE,0x00,0xBD,
0xAA,0x3B,0xDF,0x00,0x9C,0x02,0x26,0xEF,
0x7E,0x00,0x58,0xFF,0x81,0x06,0x22,0x12,
0x7F,0x00,0x4B,0xD6,0x4C,0x26,0x08,0x97,
0x4C,0x86,0x40,0x9A,0x6D,0x97,0x6D,0x7E,
0x97,0x0B,0x86,0x18,0x9F,0x00,0xBD,0xD0,
0xE9,0x9E,0x00,0x96,0x4B,0x26,0x21,0x86,
0x80,0x95,0x54,0x26,0x2E,0x96,0x55,0x2A,
0x17,0x7F,0x00,0x54,0xBD,0x93,0xC4,0x96,
0x4B,0x26,0x0D,0x96,0x54,0x85,0x80,0x26,
0x1A,0x7F,0x00,0x54,0x96,0x55,0x2B,0x27,
0x7F,0x00,0x54,0x30,0xEE,0x01,0x8C,0xC6,
0x2A,0x27,0x0B,0x8C,0x93,0x8D,0x27,0x06,
0x7E,0xC5,0xB2,0x7F,0x00,0x54,0x32,0x39,
0x7E,0xAA,0xB6,0x86,0x80,0x9A,0x55,0x97,
0x55,0x7F,0x00,0x4C,0x86,0x18,0x36,0x7F,
0x00,0x8F,0x96,0x54,0x85,0x10,0x26,0x1D,
0x85,0x20,0x26,0x12,0xDE,0x4F,0x26,0x08,
0xDE,0x4D,0x27,0xDC,0xEE,0x03,0x27,0xD8,
0xDF,0x4D,0xEE,0x03,0xDF,0x4F,0xDE,0x4D,
0xBD,0xAA,0x3A,0xDF,0x51,0x86,0xB9,0x94,
0x54,0x97,0x54,0x9F,0x00,0x86,0x01,0xC6,
0xF4,0xDB,0x46,0x99,0x45,0xD0,0x01,0x92,
0x00,0x2A,0x7C,0x96,0x55,0x85,0x20,0x27,
0x03,0x7E,0x98,0x81,0x96,0x4B,0x27,0x03,
0x7E,0x96,0x54,0x86,0x40,0x95,0x54,0x27,
0x03,0x7E,0x96,0x6A,0xCE,0x97,0x04,0xDF,
0x59,0xCE,0x04,0x26,0xDF,0x49,0xDE,0x51,
0xA6,0x00,0x97,0x53,0x08,0xDF,0x51,0x81,
0xB6,0x22,0x6B,0x81,0x24,0x23,0x08,0x80,
0x19,0x81,0x24,0x22,0x02,0x86,0x24,0x97,
0x56,0x44,0xCE,0xBB,0xDE,0xBD,0x04,0x51,
0x25,0x04,0x54,0x54,0x54,0x54,0xC4,0x0F,
0x96,0x56,0x80,0x3C,0x25,0x0A,0xCB,0x0F,
0xCE,0xBB,0x1A,0x8D,0x0F,0x7E,0x97,0x04,
0xD7,0x57,0xBD,0xCE,0x70,0x96,0x56,0xD6,
0x57,0xCE,0xBA,0xA2,0x48,0xBD,0x04,0x45,
0xDF,0x56,0xCE,0x97,0x99,0x17,0xBD,0x04,
0x4B,0xCE,0x97,0xB0,0x7E,0x04,0x5D,0x7D,
0x00,0x55,0x2B,0x08,0xC0,0x64,0x82,0x00,
0x2A,0x02,0x4F,0x5F,0x37,0x36,0x86,0x0E,
0x36,0xBD,0xF9,0xAA,0x96,0x4B,0x26,0x03,
0x7E,0x96,0xFB,0x7E,0x96,0x6A,0xBD,0xBC,
0x30,0x68,0x03,0x09,0x00,0x16,0x1D,0x37,
0x3F,0x48,0x54,0x7D,0x88,0x97,0xA3,0xAB,
0x66,0xB7,0xBA,0xC6,0xBD,0xC0,0xC3,0xC9,
0xBD,0xCE,0x77,0x96,0x08,0x27,0x61,0x20,
0x52,0x96,0x08,0x27,0x5B,0xBD,0xCF,0x77,
0x81,0x50,0x27,0x5A,0x20,0x45,0xBD,0xCE,
0x77,0x27,0x4D,0x20,0x09,0xBD,0xCE,0x77,
0x27,0x46,0x81,0x28,0x27,0x4B,0xBD,0xCF,
0x77,0x81,0x50,0x27,0x4A,0x81,0x44,0x27,
0x43,0x81,0x54,0x27,0x45,0x20,0x24,0x96,
0x08,0x81,0x20,0x27,0x2B,0x20,0x1C,0xBD,
0xCF,0x77,0x81,0x80,0x27,0x22,0x20,0x13,
0xBD,0xCE,0x77,0xBD,0xCF,0x77,0x81,0xA8
    ];
    
    this.ROM662 = [
0x27,0x16,0x20,0x07,0xBD,0xCF,0x77,0x81,
0x50,0x27,0x0D,0x96,0x4B,0x26,0x04,0x86,
0x1B,0x97,0x4B,0x7E,0x96,0x6A,0x32,0x32,
0xBD,0xAA,0xE4,0x7E,0x97,0x04,0x7E,0x94,
0xCA,0x7E,0x95,0x98,0x7E,0x94,0xF4,0x7E,
0x94,0xE6,0x7E,0x94,0xDB,0xBD,0xCE,0x77,
0x27,0xE6,0x81,0x14,0x27,0xE2,0x20,0xD3,
0xBD,0xCE,0x77,0xBD,0xCE,0x77,0xBD,0xCF,
0x77,0x81,0xA0,0x27,0xD3,0x20,0xC4,0xBD,
0xCE,0x77,0xBD,0xCE,0x77,0x81,0x28,0x27,
0xC7,0x20,0xB8,0x96,0x08,0x81,0x10,0x27,
0xBF,0x20,0xB0,0xBD,0xCE,0x77,0xBD,0xCF,
0x77,0x81,0x54,0x27,0xB3,0x20,0xA4,0x7E,
0xAB,0x5C,0x7E,0xE8,0x6C,0x7E,0xF4,0x47,
0x7E,0xD9,0xC0,0x7E,0xE1,0x2B,0x7C,0x04,
0x3D,0x32,0x32,0x96,0x57,0x36,0x7E,0x97,
0x04,0x96,0x54,0x85,0x10,0x26,0x1F,0xDE,
0x4D,0xEE,0x07,0x27,0x19,0xBD,0xB0,0xC2,
0xCE,0x00,0x1F,0x86,0x23,0xDF,0x14,0xBD,
0xF2,0xA6,0xDE,0x14,0x08,0xA6,0x00,0x26,
0xF4,0x86,0x0D,0xBD,0xF2,0xA6,0x7E,0x97,
0x04,0xFF,0x96,0x80,0x27,0x05,0x86,0x26,
0x97,0x4B,0x39,0xDE,0x02,0xBD,0xAA,0x3C,
0xDF,0x12,0x08,0x08,0xDF,0x02,0xCE,0x03,
0xB8,0xDF,0x0A,0x4F,0x97,0x0C,0x36,0xC6,
0x94,0x37,0x36,0x97,0x04,0xBD,0xA9,0xCC,
0xDE,0x02,0xA6,0x00,0x81,0xA4,0x27,0x10,
0x81,0xA3,0x27,0x0C,0x97,0x05,0x08,0xDF,
0x02,0x81,0x94,0x26,0x21,0x7E,0x99,0x93,
0xCE,0x02,0xB4,0xDF,0x0A,0xA7,0x01,0x81,
0x94,0x27,0x0E,0x08,0xDF,0x04,0xDE,0x02,
0xA6,0x01,0x08,0xDF,0x02,0xDE,0x04,0x20,
0xEC,0x32,0x32,0x7E,0x9A,0x11,0x81,0xAF,
0x23,0x51,0x7F,0x00,0x32,0x81,0xB5,0x25,
0x08,0x27,0x0C,0xC6,0x08,0xD7,0x31,0x20,
0x1F,0xC6,0x02,0xD7,0x31,0x20,0x19,0xE6,
0x00,0xD7,0x32,0xD7,0x33,0xE6,0x01,0xD7,
0x31,0xD7,0x34,0x08,0x08,0x5D,0x27,0x11,
0x20,0x06,0x7A,0x00,0x32,0x7A,0x00,0x31,
0xE6,0x00,0x37,0x08,0x7A,0x00,0x31,0x26,
0xF7,0xD6,0x32,0x26,0xED,0xDF,0x02,0x81,
0xB5,0x26,0x06,0xD6,0x34,0x37,0xD6,0x33,
0x37,0x36,0x7C,0x00,0x04,0x96,0x04,0x36,
0x7E,0x98,0xD0,0x36,0x81,0x93,0x26,0x09,
0xD6,0x04,0xDE,0x45,0xE7,0x00,0x08,0xDF,
0x45,0xBD,0x90,0xA6,0x5D,0x2A,0x07,0x86,
0x01,0x36,0x97,0x04,0x20,0xE2,0x96,0x05,
0x81,0x8F,0x26,0x0C,0xDE,0x45,0x09,0xA6,
0x00,0xDF,0x45,0x36,0x97,0x04,0x20,0xD0,
0x96,0x04,0xC4,0x07,0x10,0x4C,0x36,0x97,
0x04,0x20,0xC5,0x32,0x96,0x35,0x84,0xFA,
0x97,0x35,0x4F,0xDE,0x45,0xA7,0x00,0x4C,
0x97,0x38,0x86,0x94,0x08,0xA7,0x00,0x08,
0xDF,0x45,0xDE,0x0A,0xA7,0x00,0x09,0xDF,
0x0A,0x32,0x81,0x9E,0x23,0x0F,0x7E,0x9A,
0xAB,0x86,0x93,0xD6,0x38,0xBD,0x9A,0x83,
0x32,0x97,0x38,0x20,0xEC,0x81,0x8F,0x27,
0xF0,0x81,0x94,0x27,0x2F,0x97,0x04,0xBD,
0x90,0xA6,0x84,0xF0,0x44,0x44,0x44,0x44,
0x97,0x05,0x96,0x0C,0x7F,0x00,0x0C,0x4D,
0x26,0x12,0xDE,0x45,0x09,0xA6,0x00,0x97,
0x06,0xBD,0x90,0xA6,0x84,0x0F,0x91,0x05,
0x24,0x48,0x20,0x25,0x84,0x0F,0x91,0x05,
0x22,0x40,0x20,0x1D,0xDE,0x45,0x09,0xA6,
0x00,0x09,0xDF,0x45,0x81,0x94,0x27,0x09,
0xDE,0x0A,0xA7,0x00,0x09,0xDF,0x0A,0x20,
0xEB,0x32,0xBD,0xA9,0x94,0xBD,0x9B,0x90,
0x39,0x96,0x04,0x81,0x93,0x26,0x27,0xDE,
0x45,0x09,0xA6,0x00,0x09,0xDF,0x45,0xDE,
0x0A,0xA7,0x00,0x09,0xDF,0x0A,0x32,0x97,
0x38,0x96,0x35,0x84,0xFB,0x97,0x35,0x7E,
0x99,0xB1,0x86,0x92,0xD6,0x38,0x8D,0x43,
0x96,0x04,0x81,0x8F,0x27,0x29,0xD6,0x38,
0xDE,0x45,0xE7,0x00,0xD7,0x08,0x08,0xA7,
0x00,0x08,0xDF,0x45,0x81,0x91,0x27,0x08,
0x81,0x4F,0x22,0x13,0x81,0x4D,0x25,0x0F,
0xDE,0x0A,0xC6,0x8D,0xE7,0x00,0x09,0xDF,
0x0A,0xD6,0x35,0xC4,0xFB,0xD7,0x35,0x33,
0xD7,0x38,0xBD,0x90,0xA6,0xC4,0x10,0x26,
0x02,0x20,0xBC,0x86,0x92,0xD6,0x08,0x8D,
0x02,0x20,0xB4,0x7F,0x00,0x0C,0xDE,0x45,
0x08,0x08,0xDF,0x45,0x09,0x09,0xE7,0x00,
0xA7,0x01,0xDE,0x0A,0xD6,0x35,0xC5,0x04,
0x27,0x05,0x86,0x90,0xA7,0x00,0x09,0x86,
0x8F,0xA7,0x00,0x09,0xDF,0x0A,0xC4,0xFB,
0xD7,0x35,0x39,0xD6,0x35,0x81,0xA2,0x23,
0x0C,0x81,0xAF,0x25,0x1B,0x26,0x06,0xC4,
0xFB,0xD7,0x35,0x20,0x13,0xC4,0x04,0x27,
0x09,0xDE,0x0A,0xC6,0x90,0xE7,0x00,0x09,
0xDF,0x0A,0xD6,0x35,0xCA,0x04,0xD7,0x35,
0x7F,0x00,0x32,0xDE,0x0A,0x81,0xAF,0x23,
0x42,0x81,0xB5,0x25,0x08,0x27,0x0C,0xC6,
0x08,0xD7,0x31,0x20,0x1A,0xC6,0x02,0xD7,
0x31,0x20,0x14,0x33,0xD7,0x32,0xD7,0x33,
0x33,0xD7,0x31,0xD7,0x34,0x27,0x11,0x20,
0x06,0x7A,0x00,0x32,0x7A,0x00,0x31,0x33,
0xE7,0x00,0x09,0x7A,0x00,0x31,0x26,0xF7,
0x7D,0x00,0x32,0x26,0xEC,0x81,0xB5,0x26,
0x0A,0xD6,0x34,0xE7,0x00,0x09,0xD6,0x33,
0xE7,0x00,0x09,0xA7,0x00,0x09,0xDF,0x0A,
0xDE,0x41,0xDF,0x0C,0x32,0x97,0x38,0x7F,
0x00,0x32,0xDE,0x45,0x09,0x09,0xE6,0x01,
0xC1,0x92,0x26,0x07,0xD6,0x0D,0x27,0x03,
0x7E,0x99,0xB1,0xA1,0x00,0x25,0x1A,0x27,
0x07,0x20,0xF5,0x09,0x09,0x7C,0x00,0x32,
0xA6,0x01,0x81,0x92,0x27,0xF5,0xBD,0x90,
0xA6,0xC5,0x20,0x27,0xE3,0x97,0x0C,0x97,
0x0D,0x96,0x35,0x84,0xFE,0xDE,0x45,0x09,
0xE6,0x00,0x09,0xDF,0x45,0xC1,0x91,0x25,
0x04,0xC1,0x93,0x23,0x0E,0xC1,0x4C,0x22,
0x08,0xC1,0x1D,0x22,0x06,0xC1,0x07,0x27,
0x02,0x84,0xFB,0x97,0x35,0xDE,0x0A,0xE7,
0x00,0x09,0xDF,0x0A,0x96,0x38,0xD6,0x32,
0x27,0xA0,0x7A,0x00,0x32,0x20,0xCA,0xFF,
0x86,0x80,0x97,0x3A,0x97,0x39,0xDE,0x91,
0x09,0xDF,0x95,0x7C,0x00,0x87,0xDE,0x0A,
0x08,0xDF,0x0A,0xDF,0x04,0x4F,0xB7,0x04,
0x36,0xDE,0x12,0x27,0x25,0xA6,0x00,0xE6,
0x01,0x37,0x36,0x86,0x0E,0x36,0xBD,0xB1,
0xFB,0x33,0x30,0xBD,0x9D,0x43,0x30,0xBD,
0xAA,0x3B,0x35,0xD6,0x4B,0x27,0x03,0x7E,
0x9C,0x7C,0xC6,0x20,0xBD,0x9C,0x6E,0x73,
0x00,0x39,0xB6,0x04,0x35,0x27,0x03,0x7A,
0x04,0x35,0xDE,0x0A,0xA6,0x00,0x08,0x97,
0x06,0xDF,0x0A,0x81,0x94,0x26,0x04,0x7F,
0x00,0x87,0x39,0x81,0xAF,0x23,0x03,0x7E,
0x9C,0x95,0xBD,0x90,0xA6,0xC4,0x08,0xD7,
0x32,0x27,0x08,0x96,0x39,0x26,0x04,0xC6,
0x20,0x8D,0x6B,0x7F,0x00,0x39,0x96,0x06,
0x4A,0x46,0x56,0x46,0x56,0xCE,0x8F,0x1B,
0xBD,0x04,0x4B,0xCB,0x40,0x25,0x04,0x46,
0x46,0x20,0xF8,0x84,0x03,0x48,0xCE,0x9D,
0x51,0xBD,0x04,0x45,0xDF,0x0E,0xA6,0x00,
0x08,0x2A,0xFB,0xA6,0x00,0x08,0x91,0x06,
0x26,0xF2,0xDE,0x0E,0xE6,0x00,0x17,0xC4,
0x3F,0xCB,0x20,0x8D,0x31,0x95,0x3A,0x26,
0x03,0x08,0x20,0xF0,0x96,0x06,0x81,0xA4,
0x27,0x11,0x81,0xA3,0x27,0x0D,0xD6,0x32,
0x27,0x06,0xC6,0x20,0x8D,0x18,0x97,0x39,
0x7E,0x9B,0xD2,0xDE,0x0A,0xE6,0x00,0xC1,
0x94,0x26,0x03,0x7E,0x9B,0xE7,0x8D,0x06,
0x08,0x20,0xF2,0xDE,0x08,0x39,0x36,0xDF,
0x08,0x17,0xBD,0xC3,0x3D,0x32,0xD6,0x4B,
0x27,0xF1,0x32,0x32,0x96,0x3A,0x2B,0x03,
0x7E,0x9B,0xE7,0x44,0x97,0x3A,0x7F,0x00,
0x4B,0xDE,0x91,0x09,0xDF,0x95,0xDE,0x04,
0xDF,0x0A,0x7E,0x9B,0xA5,0x7F,0x00,0x39,
0x81,0xB0,0x27,0x04,0x81,0xB1,0x26,0x1C,
0xDE,0x0A,0xEE,0x00,0xE6,0x02,0x8D,0xC6,
0xE6,0x03,0xC1,0x20,0x27,0x02,0x8D,0xBE,
0xDE,0x0A,0x08,0x08,0xDF,0x0A,0xC1,0x20,
0x27,0x00,0x20,0x9C,0x81,0xB4,0x25,0x2B,
0x26,0x14,0xDE,0x0A,0xE6,0x00,0x8D,0xA6,
0xE6,0x01,0xC1,0x20,0x27,0x02,0x8D,0x9E,
0x08,0x08,0xDF,0x0A,0x20,0xE4,0x81,0xB6,
0x26,0x1B,0xDE,0x0A,0x8D,0x65,0xD6,0x4B,
0x26,0x9A,0xDE,0x0A,0xBD,0xAA,0x3B,0xDF,
0x0A,0x20,0xCF,0xDE,0x0A,0x8D,0x44,0xD6,
0x4B,0x26,0x89,0x20,0xC5,0x81,0xB5,0x26,
0x37,0xDE,0x0A,0xE6,0x00,0xD7,0x33,0x08,
0xA6,0x00,0x08,0xC6,0x22,0xBD,0x9C,0x6E,
0x4D,0x27,0x12,0xE6,0x00,0xC1,0x22,0x26,
0x05,0xBD,0x9C,0x6E,0xE6,0x00,0xBD,0x9C,
0x6E,0x08,0x4A,0x26,0xEE,0xD6,0x33,0x27,
0x06,0x7A,0x00,0x33,0x4A,0x20,0xE4,0xC6,
0x22,0xBD,0x9C,0x6E,0xDF,0x0A,0x20,0x8A,
0xBD,0xBC,0x30,0xA6,0x00,0x08,0xE6,0x00,
0x08,0xDF,0x0A,0x37,0x36,0x36,0xBD,0xB1,
0xFB,0x20,0x03,0xBD,0xB6,0xEB,0xBD,0xB8,
0x53,0x86,0x01,0x97,0xED,0xBD,0xD1,0x83,
0x39,0x8B,0x7C,0x8B,0xE7,0x8C,0xA2,0x8D,
0x8E,0xFF,0xBD,0xA9,0xE7,0x9F,0x00,0x7F,
0x00,0x08,0x8D,0x2E,0x25,0x14,0x26,0x27,
0x8D,0x28,0x25,0x13,0x26,0x21,0x8D,0x22,
0x25,0x12,0x26,0x1B,0xDE,0x06,0xDF,0x00,
0x20,0x0F,0xCE,0x00,0x64,0xDF,0x00,0xCE,
0x00,0x0A,0xDF,0x02,0xCE,0x00,0x64,0xDF,
0x04,0xDE,0x3D,0x27,0x02,0x8D,0x39,0x7E,
0xA9,0xFD,0xBD,0xA9,0xE7,0xDE,0x00,0xA6,
0x01,0x81,0x18,0x26,0x07,0xDE,0x06,0xDF,
0x00,0x0D,0x20,0xEB,0xBD,0xCE,0x77,0x27,
0x06,0x86,0x11,0x97,0x4B,0x20,0x16,0x30,
0xBD,0xB1,0xA2,0x26,0xF4,0xDE,0x02,0xDF,
0x04,0xDE,0x06,0xDF,0x02,0x30,0xEE,0x03,
0xDF,0x06,0x9E,0x00,0x4F,0x0C,0x20,0xC7,
0xBD,0xA9,0xE7,0xBD,0xA9,0xCC,0xDE,0x00,
0x27,0x04,0xDE,0x02,0x26,0x0A,0x86,0x11,
0x97,0x4B,0xBD,0xA9,0x94,0x7E,0xA9,0xFD,
0xDE,0x04,0xBD,0xAA,0x08,0xBD,0x91,0x06,
0x30,0xEE,0x01,0x26,0x05,0xBD,0x9F,0x08,
0x20,0xE8,0xDF,0x08,0xBD,0x9F,0x08,0x96,
0x00,0xD6,0x01,0xD0,0x05,0x92,0x04,0x24,
0x24,0xDE,0x04,0x09,0xBD,0xAA,0x08,0xBD,
0x90,0xF8,0x30,0xEE,0x01,0x26,0x05,0xBD,
0x9F,0x08,0x20,0x2D,0xEE,0x07,0xDF,0x0C,
0xBD,0x9F,0x08,0x96,0x0C,0xD6,0x0D,0xD0,
0x01,0x92,0x00,0x24,0xB1,0x8D,0x30,0xDE,
0x08,0xEE,0x05,0x27,0x16,0xDF,0x0C,0xEE,
0x05,0xDF,0x0E,0xDE,0x0C,0xA6,0x07,0xA7,
0x05,0xA6,0x08,0xA7,0x06,0xDE,0x0E,0x20,
0xEA,0x8D,0x14,0xDE,0x08,0x96,0x00,0xD6,
0x01,0xA7,0x05,0xE7,0x06,0xEE,0x03,0x27,
0x1C,0xDB,0x03,0x99,0x02,0x20,0xF2,0xDE,
0x08,0x96,0x00,0xD6,0x01,0xEE,0x03,0x26,
0x01,0x39,0xDB,0x03,0x99,0x02,0x24,0xF5,
0x32,0x32,0x7E,0x9D,0xD6,0xDE,0x3D,0xDF,
0x00,0xA6,0x09,0x81,0xA3,0x25,0x0E,0x81,
0xA9,0x27,0x26,0x24,0x08,0xDE,0x00,0xEE,
0x03,0x26,0xEC,0x20,0x4E,0xA6,0x00,0xE6,
0x01,0xDB,0x01,0x99,0x00,0x97,0x02,0xD7,
0x03,0xDE,0x02,0x09,0x09,0xA6,0x00,0x81,
0x52,0x25,0xE2,0x86,0x5C,0xA1,0x00,0x25,
0xDC,0xDE,0x00,0xBD,0xAA,0x3A,0xA6,0x00,
0x81,0xB3,0x27,0x09,0x81,0x94,0x27,0xCD,
0xBD,0xA2,0xCC,0x20,0xF1,0xDF,0x0C,0xEE,
0x01,0xDF,0x0E,0x8D,0x37,0x25,0x10,0xDE,
0x06,0xEE,0x05,0xDF,0x0E,0xDE,0x0C,0x96,
0x0E,0xA7,0x01,0x96,0x0F,0xA7,0x02,0xDE,
0x0C,0x20,0xDD,0x7F,0x00,0x00,0x7F,0x00,
0x01,0xDE,0x3D,0xA6,0x05,0xA7,0x07,0xA6,
0x06,0xA7,0x08,0x96,0x00,0xA7,0x05,0x96,
0x01,0xA7,0x06,0xDF,0x00,0xEE,0x03,0x26,
0xEA,0x7E,0x9D,0xDA,0xDE,0x3D,0xDF,0x06,
0xEE,0x07,0x9C,0x0E,0x27,0x08,0xDE,0x06,
0xEE,0x03,0x26,0xF2,0x0D,0x39,0x0C,0x39,
0x30,0xEE,0x00,0xDF,0x0E,0x30,0xBD,0xAA,
0x3B,0x35,0xDE,0x0E,0x6E,0x00,0x30,0xA6,
0x00,0x81,0x0C,0x26,0x1B,0xBD,0xD0,0x87,
0x96,0x4B,0x26,0x11,0xDE,0x00,0x4F,0xDF,
0x4F,0x9A,0x55,0x97,0x55,0x85,0x40,0x26,
0x04,0x8A,0x80,0x97,0x55,0x7E,0x00,0x58,
0x9E,0x43,0x86,0x19,0x36,0xBD,0xBF,0x5C,
0xDE,0x3D,0x86,0x10,0x8D,0x03,0x7E,0xC5,
0xB2,0xC6,0x18,0x37,0x20,0xD9,0xBD,0xD1,
0x65,0x5B,0xBD,0xD0,0x87,0x96,0x4B,0x26,
0x04,0xDE,0x00,0xDF,0x4F,0x7E,0x00,0x58,
0xBD,0xD0,0x87,0x96,0x4B,0x26,0x1D,0xDE,
0x00,0xDF,0x4F,0x96,0x4E,0xD6,0x4D,0xDE,
0x4D,0xEE,0x07,0x26,0x02,0x4F,0x5F,0x36,
0x37,0x86,0x03,0x36,0x8D,0x03,0x7E,0xC5,
0xB2,0x86,0x18,0x36,0x7E,0x00,0x58,0x9F,
0x00,0x86,0x18,0xBD,0xD0,0xE9,0xDE,0x00,
0xBD,0xAA,0x48,0xDF,0x00,0xBD,0xCE,0x74,
0x96,0x08,0x26,0x44,0xDE,0x00,0xBD,0xAA,
0x49,0xBD,0xB1,0xA2,0x26,0x34,0xA6,0x03,
0x26,0x33,0xE6,0x04,0x27,0x2F,0x43,0x53,
0x6C,0x04,0x58,0x58,0x58,0x49,0xE0,0x04,
0x82,0x00,0xDB,0x01,0x99,0x00,0x9F,0x02,
0x97,0x04,0xD7,0x05,0xD0,0x03,0x92,0x02,
0x2B,0x13,0x9E,0x04,0xCE,0x00,0x08,0xBD,
0xB7,0x0F,0x9E,0x00,0xBD,0xB6,0xEB,0x7E,
0x00,0x58,0x7F,0x00,0x4B,0x7E,0xD1,0x60,
0xBD,0xD1,0x6A,0x13,0x9F,0x00,0x86,0x02,
0xC6,0x03,0xBD,0xD1,0x04,0xDE,0x00,0x27,
0x1F,0xA6,0x01,0x81,0x03,0x27,0x05,0xDE,
0x00,0x09,0x35,0x39,0x9E,0x00,0x32,0x32
    ];
    
    this.ROM663 = [
0x97,0x4D,0x32,0x97,0x4E,0xDE,0x4D,0x27,
0x02,0xEE,0x03,0xDF,0x4F,0x7E,0xD1,0x60,
0x7E,0xAA,0xB6,0xBD,0xD0,0x87,0x96,0x4B,
0x26,0x09,0xBD,0xEB,0xCB,0x27,0x04,0xDE,
0x00,0xDF,0x4F,0x7E,0xD1,0x60,0x96,0x55,
0x85,0xC0,0x26,0x04,0xBD,0xD1,0x6A,0x23,
0xDE,0x4D,0xEE,0x0A,0xA6,0x04,0x85,0x40,
0x27,0x28,0xBD,0xCE,0x70,0xBD,0xCE,0x77,
0x96,0x08,0x27,0x0A,0x81,0x30,0x26,0x1A,
0xCE,0xE9,0x50,0xBD,0xB6,0xEB,0xDE,0x4D,
0xA6,0x0B,0x36,0xA6,0x0A,0x36,0x96,0x4E,
0x36,0x96,0x4D,0x36,0x86,0x04,0x36,0x7E,
0x9F,0x7C,0xBD,0xD1,0x6A,0x13,0x9F,0x00,
0x30,0xEE,0x01,0xDF,0x0E,0xA6,0x04,0x88,
0x40,0x85,0xC0,0x27,0x07,0xBD,0xD1,0x6A,
0x13,0xBD,0xD1,0x1D,0x86,0x04,0xBD,0xD0,
0xE9,0xDE,0x00,0x27,0xF0,0xEE,0x04,0x9C,
0x0E,0x26,0xEE,0xBD,0xAA,0x3E,0xBD,0xB6,
0xEB,0xDE,0x00,0xBD,0xAA,0x3C,0xBD,0xB6,
0xEB,0xBD,0xB2,0x45,0xDE,0x0E,0xBD,0xAA,
0x3E,0xBD,0xB7,0x0F,0xBD,0xB6,0xEB,0xDE,
0x00,0xBD,0xAA,0x3C,0xBD,0xAA,0x3A,0xBD,
0xB6,0xEB,0xBD,0xB8,0x0D,0xDE,0x00,0xA6,
0x07,0x2B,0x06,0x96,0xC2,0x2E,0x18,0x20,
0x04,0x96,0xC2,0x2B,0x12,0xDE,0x00,0x09,
0x09,0x35,0xEE,0x04,0xDF,0x4D,0x27,0x02,
0xEE,0x03,0xDF,0x4F,0x7E,0xD1,0x60,0xBD,
0xD1,0x1D,0x9E,0x00,0x20,0xF6,0x86,0x94,
0x36,0xDE,0x04,0xDF,0x0C,0xDE,0x02,0xA6,
0x00,0x08,0xDF,0x02,0x81,0xA2,0x22,0x03,
0x7E,0xA1,0x7D,0xDE,0x04,0xA7,0x00,0x08,
0xDF,0x04,0x81,0xA3,0x27,0x23,0x81,0xA4,
0x27,0x1F,0x81,0xAF,0x23,0xDF,0x81,0xB4,
0x22,0x05,0xBD,0xA1,0x35,0x20,0xD6,0x81,
0xB5,0x27,0x37,0x86,0x04,0x97,0x06,0xBD,
0xA1,0x35,0x7A,0x00,0x06,0x26,0xF8,0x20,
0xC4,0xDE,0x02,0xA6,0x00,0x81,0x94,0x27,
0xBC,0x08,0xDF,0x02,0xDE,0x04,0xA7,0x00,
0x08,0xDF,0x04,0x20,0xEC,0xDE,0x02,0xA6,
0x00,0xE6,0x01,0x08,0x08,0xDF,0x02,0xDE,
0x04,0xA7,0x00,0xE7,0x01,0x08,0x08,0xDF,
0x04,0x39,0xDE,0x02,0xA6,0x00,0xE6,0x01,
0x08,0x08,0xDF,0x02,0xDE,0x04,0xA7,0x00,
0xE7,0x01,0x08,0x08,0xDF,0x04,0xDB,0x03,
0x99,0x02,0xD7,0x07,0x97,0x06,0xDE,0x02,
0x9C,0x06,0x26,0x03,0x7E,0xA0,0xE5,0xA6,
0x00,0x08,0xDF,0x02,0xDE,0x04,0xA7,0x00,
0x08,0xDF,0x04,0x20,0xE9,0x97,0x06,0xBD,
0x90,0xA6,0x84,0xF0,0x44,0x44,0x44,0x44,
0x97,0x07,0x32,0x36,0x97,0x08,0xBD,0x90,
0xA6,0x84,0x0F,0x97,0x09,0x91,0x07,0x24,
0x22,0x96,0x06,0x81,0x8E,0x27,0x0D,0x81,
0x93,0x26,0x12,0xDE,0x04,0xA7,0x00,0x08,
0xDF,0x04,0x20,0x09,0xDE,0x04,0x86,0xAF,
0xA7,0x00,0x08,0xDF,0x04,0x96,0x06,0x36,
0x7E,0xA0,0xE5,0x81,0x03,0x27,0x19,0x32,
0xDE,0x04,0xA7,0x00,0x08,0xDF,0x04,0x81,
0x94,0x26,0xBF,0x96,0x05,0x90,0x0D,0x97,
0x34,0x96,0x04,0x92,0x0C,0x97,0x33,0x39,
0x96,0x06,0x81,0x90,0x27,0x1F,0x81,0x8F,
0x26,0x17,0xD6,0x08,0xC1,0x93,0x26,0x09,
0xDE,0x04,0xA7,0x00,0x08,0xDF,0x04,0x20,
0x04,0xC1,0x92,0x26,0x04,0x32,0x7E,0xA0,
0xE5,0x32,0x7E,0xA1,0x8A,0x96,0x08,0x81,
0x93,0x26,0xF6,0x7E,0xA0,0xE5,0x9F,0x0E,
0x86,0x15,0x36,0x4F,0xD6,0x31,0x58,0x49,
0x58,0x49,0x58,0x49,0xCB,0x6E,0x89,0x00,
0xDB,0x46,0x99,0x45,0xD0,0x0F,0x92,0x0E,
0x2B,0x0F,0x37,0x36,0x86,0x0E,0x36,0xBD,
0xF9,0xAA,0x96,0x4B,0x27,0x03,0x7E,0xA4,
0x54,0x4F,0x97,0xD1,0x36,0xB7,0x04,0x35,
0x4C,0x97,0xC8,0xCE,0x02,0xB5,0xD6,0x35,
0x2B,0x03,0x08,0x08,0x08,0xDF,0x00,0x58,
0x2A,0x05,0xCE,0xA5,0xAB,0x20,0x4E,0xA6,
0x00,0x81,0x04,0x23,0x37,0x81,0x4F,0x23,
0x23,0x81,0x84,0x23,0x2D,0x81,0x8A,0x26,
0x0A,0xDE,0x00,0x08,0xDF,0x00,0xCE,0xA7,
0x3C,0x20,0x2F,0x81,0x94,0x26,0x05,0xCE,
0xA5,0xF5,0x20,0x29,0x81,0xA2,0x23,0x04,
0x81,0xA9,0x23,0x0C,0xD6,0x35,0x2B,0x03,
0x7E,0xA4,0x41,0xCE,0xA6,0xA9,0x20,0x15,
0x80,0x1E,0x80,0x4B,0x4A,0xDF,0x12,0x8D,
0x3B,0xDF,0x00,0x48,0xCE,0xA4,0x63,0xBD,
0x04,0x45,0x7C,0x00,0xC8,0xDF,0x08,0x7F,
0x00,0xD2,0x96,0xD2,0xDE,0x08,0xBD,0x04,
0x4B,0x48,0x25,0x03,0x7E,0xA3,0x7C,0x48,
0x25,0x03,0x44,0x20,0x6C,0x48,0x25,0x04,
0x44,0x44,0x20,0x39,0x44,0x44,0xCE,0xA2,
0xC4,0x7E,0x04,0x3E,0xA3,0x0B,0xA3,0xE7,
0xA4,0x3C,0xA3,0x1C,0xC6,0xB0,0xE1,0x00,
0x22,0x0E,0xC6,0xB5,0xE1,0x00,0x22,0x06,
0x27,0x08,0xBD,0xAA,0x3A,0x39,0x08,0x08,
0x08,0x39,0x08,0x08,0xE6,0x00,0x27,0xF8,
0x08,0x5A,0x20,0xFA,0x96,0xD2,0x4C,0xDE,
0x08,0xBD,0x04,0x45,0x39,0xCE,0xA2,0xFF,
0xBD,0x04,0x3E,0x25,0x51,0x20,0x46,0xA4,
0x26,0xA4,0x36,0xA7,0xA6,0xA6,0x6F,0xA4,
0x2E,0xA4,0x57,0x8D,0xDF,0xDF,0x0A,0x4C,
0x4C,0x97,0xD2,0xD6,0xD1,0xCA,0x80,0xD7,
0xD1,0x7E,0xA2,0xA4,0x8D,0xCE,0x7E,0xA2,
0x9D,0x44,0x7F,0x00,0xD0,0xDE,0x00,0xE6,
0x00,0xCE,0xA4,0xE3,0xE1,0x00,0x23,0x06,
0x08,0x7C,0x00,0xD0,0x20,0xF6,0x91,0xD0,
0x26,0x14,0x7C,0x00,0xC8,0xDE,0x00,0xDF,
0x12,0x8D,0x89,0xDF,0x00,0x7C,0x00,0xD2,
0x7F,0x00,0xD1,0x7E,0xA2,0xA2,0x96,0xD1,
0x2A,0x08,0x84,0x01,0x97,0xD1,0xDE,0x0A,
0x20,0xC4,0x8A,0x01,0x97,0xD1,0xDE,0x08,
0x96,0xD2,0x4C,0x97,0xD2,0xBD,0x04,0x4B,
0x81,0xE0,0x26,0x06,0x96,0xD2,0x4C,0x4C,
0x20,0xF0,0x81,0xE1,0x27,0xD5,0x81,0xE2,
0x27,0xD1,0x20,0xE4,0xD6,0x08,0x37,0xD6,
0x09,0x37,0xD6,0xC8,0x37,0xD6,0xD2,0x5C,
0x37,0xD6,0xD1,0x2A,0x1C,0xC4,0x01,0xD7,
0xD1,0xD6,0x00,0x37,0xD6,0x01,0x37,0xD6,
0x0A,0x37,0xD6,0x0B,0x37,0xC6,0x80,0x37,
0x37,0xCE,0xA3,0xAF,0xBD,0x04,0x45,0x20,
0xAF,0x5F,0x37,0xC6,0x80,0x20,0xF1,0xA5,
0x16,0xA7,0xD5,0xA5,0xC5,0xA5,0x1E,0xA5,
0x35,0xA5,0x3E,0xA5,0x41,0xA5,0x4A,0xA5,
0x54,0xA5,0x58,0xA5,0x5F,0xA5,0x25,0xA5,
0x72,0xA5,0x79,0xA7,0x8B,0xA7,0xB5,0xA7,
0xB8,0xA7,0xC0,0xA5,0x2C,0xA7,0xC7,0xA7,
0xCD,0xA7,0xDA,0xA7,0xE0,0xA7,0xF8,0xA7,
0xB5,0xA8,0x04,0xA8,0x34,0xA5,0x94,0x33,
0x5D,0x27,0x56,0x33,0x5D,0x2A,0x0E,0x33,
0xD7,0x07,0x33,0xD7,0x06,0xC6,0x01,0xD5,
0xD1,0x26,0x14,0x31,0x31,0x33,0xD7,0xD2,
0x33,0x33,0xD7,0x09,0x33,0xD7,0x08,0xD6,
0xD1,0x54,0x25,0xDB,0x7E,0xA2,0xA2,0xDE,
0x06,0x33,0xD7,0x01,0x33,0xD7,0x00,0x31,
0x33,0xD7,0xC8,0x31,0x31,0xD6,0xD1,0xC4,
0x80,0xD7,0xD1,0x7E,0xA2,0x9D,0xDE,0x12,
0xC6,0xB3,0xE7,0x00,0x0C,0x39,0xDE,0x12,
0x86,0x87,0xA7,0x00,0x0C,0x39,0xDE,0x12,
0xDF,0x00,0x0C,0x39,0x96,0xD1,0x44,0x24,
0x0B,0x86,0x5D,0x97,0x4B,0x96,0xC8,0xB7,
0x04,0x35,0x20,0x08,0xDE,0x00,0xA6,0x00,
0x81,0x94,0x26,0xED,0x9E,0x0E,0x39,0x96,
0x35,0x85,0x02,0x27,0x04,0x86,0x03,0x97,
0xD2,0x0C,0x39,0xA6,0xA1,0xA6,0xA1,0xA6,
0xA1,0xA6,0xA1,0xA6,0x77,0xA6,0x6D,0xA6,
0x89,0xA6,0x8A,0xA6,0xA4,0xA7,0x52,0xA7,
0x52,0xA7,0x42,0xA6,0x1B,0xA5,0x9B,0xA6,
0xC6,0xA7,0x58,0xA6,0xFF,0xA6,0xDD,0xA6,
0xFE,0xA6,0xFE,0xA6,0xFE,0xA7,0x5F,0xA6,
0xFE,0xA6,0x9C,0xA7,0x23,0xA6,0x6D,0xA6,
0x6D,0xA6,0xAB,0xA7,0x7D,0xA6,0xAB,0xA5,
0xD6,0xA7,0x79,0xA7,0x79,0xA5,0xCB,0xA6,
0xE6,0xA6,0x08,0xA6,0xF0,0xA6,0xBE,0xA6,
0xFD,0xA5,0xF6,0xA5,0xA7,0xA7,0x89,0xA6,
0x74,0xA7,0x33,0xA7,0x61,0xA6,0xD3,0xA6,
0xFD,0xA6,0xFD,0xA6,0xFD,0xA6,0x62,0xA5,
0xE6,0xA6,0xFE,0xA6,0x82,0xA7,0x5D,0xA6,
0xDF,0xA5,0xEC,0xA5,0xF0,0xA7,0x40,0xA7,
0x40,0xA6,0x0C,0xA6,0x49,0xA6,0x3B,0xA6,
0xE1,0xA6,0xEB,0x04,0x06,0x07,0x10,0x16,
0x17,0x18,0x19,0x1B,0x1D,0x20,0x23,0x3D,
0x4C,0x4E,0x4F,0x50,0x51,0x64,0x65,0x66,
0x84,0x85,0x86,0x88,0x89,0x8A,0x8B,0x8C,
0x8D,0x8E,0x8F,0x90,0x91,0x92,0x93,0x94,
0x9D,0x9E,0xA2,0xA9,0xAD,0xAE,0xAF,0xB0,
0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xE0,0xA5,
0x1B,0x06,0xE1,0x0C,0x9E,0xE1,0xE0,0xA5,
0x23,0x06,0xE1,0x0C,0xE1,0xE0,0xA5,0x2A,
0xB1,0xE1,0xAD,0xE1,0xE0,0xA5,0x33,0x11,
0x0B,0x9F,0xE1,0x0B,0xE1,0xE0,0xA5,0x3C,
0xA2,0xAC,0x9F,0xE1,0xAC,0xE1,0xE0,0xA5,
0x49,0x8E,0x14,0xE0,0xA5,0x48,0xA0,0x14,
0x9D,0xE1,0xE0,0xA5,0x4F,0xA9,0xE1,0xA6,
0xA2,0x1B,0x9F,0xE1,0x14,0xA0,0x14,0xE1,
0xE0,0xA5,0x5D,0x12,0xE1,0x14,0xE1,0xE0,
0xA5,0x66,0x06,0xE3,0xA5,0x71,0xE0,0xA5,
0x71,0x8F,0x1B,0xE0,0xA5,0x70,0xA0,0x14,
0x9D,0xE1,0x12,0xE0,0xA5,0x78,0xA0,0x12,
0xE1,0xE0,0xA5,0x83,0x06,0xE0,0xA5,0x93,
0xE3,0xA5,0x8B,0xE0,0xA5,0x8B,0x0C,0xE0,
0xA5,0x93,0x9E,0xAE,0xC0,0xE0,0xA5,0x93,
0xA0,0xAE,0xC0,0xE1,0xE0,0xA5,0x99,0xB2,
0xE1,0xAE,0xE1,0xE0,0xA5,0x9F,0x00,0xAE,
0xC0,0xE0,0xA5,0xA6,0xA0,0x14,0xE2,0x0C,
0x9E,0x12,0xE2,0xE0,0xA5,0xAF,0x93,0xE0,
0xA5,0xB4,0x02,0xE2,0xE0,0xA5,0xC1,0x18,
0x9C,0xE0,0xA5,0xBF,0x88,0x04,0xE2,0x14,
0xE2,0xAD,0x9C,0x0E,0xE2,0xAC,0x9C,0xAC,
0x86,0xAC,0xE1,0x05,0xE0,0xA5,0xD5,0x08,
0xE0,0xA5,0xD5,0xA0,0x08,0xE2,0x12,0xE0,
0xA5,0xDE,0xA0,0xE3,0xA5,0xE2,0xE0,0xA5,
0xF5,0x9E,0x09,0xE3,0xA5,0xD7,0x05,0xE0,
0xA5,0xEB,0x1B,0xE2,0x12,0xE3,0xA5,0xF3,
0x18,0xA0,0x18,0xA0,0xAD,0xE2,0xE0,0xA6,
0x07,0x12,0xE0,0xA5,0xFF,0xA0,0x14,0x97,
0xC4,0x12,0xE0,0xA6,0x07,0xA0,0x14,0xE2,
0x0C,0x9E,0x08,0xE2,0xE0,0xA6,0x13,0x1B,
0xE3,0xA6,0x14,0xB1,0xE0,0xA5,0xF5,0xA0,
0xE3,0xA6,0x0C,0xE0,0xA6,0x20,0xAA,0xE2,
0xE0,0xA6,0x2C,0xAE,0xC0,0xE0,0xA6,0x2B,
0xA0,0xAE,0xC0,0xE2,0xE0,0xA6,0x37,0xAC,
0xE0,0xA6,0x2B,0xA0,0xE3,0xA6,0x2C,0xAD,
0xE3,0xA6,0x30,0x8C,0xE0,0xA6,0x45,0xA2,
0xB0,0x9F,0xE3,0xA6,0x46,0xB0,0x91,0x14,
0xE2,0xE0,0xA6,0x57,0xAD,0xA3,0x14,0x9F,
0xE0,0xA6,0x48,0xA0,0xE3,0xA6,0x49,0xAC,
0xA3,0x14,0xE0,0xA6,0x4F,0xA0,0x14,0xE3,
0xA6,0x4F,0x05,0xE0,0xA6,0x6C,0x14,0xE0,
0xA6,0x6C,0xA0,0x12,0xE2,0xC3,0xE2,0x7A,
0x00,0xC8,0x0D,0x39,0x05,0x14,0xE2,0xAC,
0x9C,0x14,0x97,0x14,0xE0,0xA6,0x81,0x98,
0x14,0xE2,0x14,0xE0,0xA6,0x88,0xA0,0x14,
0xE2,0x97,0xE3,0xA6,0x8D,0xC5,0xAE,0xC0,
0xE2,0x14,0x9B,0xAE,0xC0,0xE0,0xA6,0x90,
0xA0,0xE3,0xA6,0x93,0x05,0x18,0xA0,0x18,
0xE2,0x05,0x08,0xE2,0x14,0x99,0xAE,0xC0,
0xE2,0x09,0xE2,0xE0,0xA6,0xAF,0x0A,0xE0,
0xA6,0xB6,0x18,0xE3,0xA6,0xB7,0xAD,0xE0,
0xA6,0xD2,0xA0,0xE3,0xA6,0xAF,0xE0,0xA6,
0xC3,0x0C,0xE2,0x05,0x14,0xE2,0x05,0xE0,
0xA6,0xD2,0xAE,0xC0,0xE0,0xA6,0xD2,0xA0,
0xAE,0xC0,0xE2,0x05,0x14,0xA0,0x14,0xE0,
0xA6,0xDC,0xA0,0x14,0xE2,0xAC,0xE2,0x14,
0xE2,0xE0,0xA6,0xE5,0x07,0xE2,0xE0,0xA6,
0xEA,0x03,0xE2,0x07,0x99,0xAE,0xC0,0xE2,
0x12,0xE0,0xA6,0xF6,0xA0,0x12,0x9E,0x1B,
0xA0,0x12,0xA0,0xAD,0xE2,0x05,0xE2,0xE0,
0xA7,0x03,0x0A,0xE0,0xA7,0x11,0xA1,0xE0,
0xA7,0x0E,0x12,0xE3,0xA7,0x10,0xAE,0xC0,
0x9D,0xE0,0xA5,0xF5,0x09,0xE0,0xA7,0x1C,
0xA0,0xE3,0xA7,0x11,0xE0,0xA5,0xF5,0x9E,
0xE3,0xA7,0x11,0xAC,0xA0,0xAC,0x9E,0x14,
0xE0,0xA7,0x2D,0xA0,0x14,0xE0,0xA5,0xF5,
0xE3,0xA7,0x26,0x18,0xE0,0xA5,0xF5,0xA0,
0x18,0xE3,0xA7,0x34,0x0C,0x96,0x0C,0xE2,
0xC2,0xE2,0xE0,0xA7,0x51,0x14,0xE0,0xA7,
0x51,0xA0,0x14,0xE0,0xA7,0x51,0xA0,0xAE,
0xC0,0xE2,0xE0,0xA7,0x57,0xAE,0xC0,0xE2,
0xE0,0xA7,0x5C,0x0D,0xE2,0x08,0xE2,0xA5,
0xE2,0xE0,0xA7,0x6E,0x8E,0x14,0xE0,0xA7,
0x6D,0xA0,0xE3,0xA7,0x65,0x9D,0xE0,0xA7,
0x60,0x14,0xE0,0xA7,0x60,0xA0,0xE3,0xA7,
0x6E,0x08,0xA0,0x08,0xE2,0xE0,0xA7,0x81,
0x0A,0x09,0xE0,0xA7,0x7C,0xA0,0xE3,0xA7,
0x81,0x1B,0xE2,0xE0,0xA7,0x97,0x81,0xA2,
0x12,0xA0,0x14,0xA0,0x14,0x9F,0xE1,0xE0,
0xA7,0x9F,0x89,0xA2,0x14,0x9F,0xE1,0x12,
0xE0,0xA7,0xA5,0x85,0x12,0xE1,0x86,0x94,
0xDE,0x00,0xA1,0x00,0x27,0x03,0x08,0x20,
0xF9,0x0C,0xDF,0x00,0x39,0xAC,0x10,0xE1,
0xE0,0xA7,0xBF,0x11,0x14,0x13,0x9F,0xE1,
0xE0,0xA7,0xC5,0xA2,0xE1,0xA3,0xE1,0xE0,
0xA7,0xCC,0xA0,0x14,0xE1,0xE0,0xA7,0xD2,
0x01,0xE1,0x15,0x17,0xE1,0x12,0x84,0x12,
0x17,0xE1,0xE0,0xA7,0xDE,0x8B,0x16,0xE1,
0xE0,0xA7,0xE5,0xB0,0xE1,0xE0,0xA7,0xEA,
0x1B,0xE1,0xE0,0xA7,0xF1,0x11,0x14,0x9F,
0xE1,0xE0,0xA7,0xF6,0x18,0xE1,0x19,0xE1,
0xE0,0xA7,0xFE,0x84,0x14,0xE1,0xE0,0xA8
    ];
    
    this.ROM664 = [
0x03,0x83,0x14,0xE1,0xE0,0xA8,0x0A,0x8A,
0x12,0xE1,0xE0,0xA8,0x0F,0xA7,0xE1,0xE0,
0xA8,0x1B,0x82,0x11,0x12,0xA0,0x12,0xA0,
0x14,0x9F,0xE1,0x1A,0xE0,0xA8,0x23,0x11,
0x14,0x9F,0xE1,0xE0,0xA8,0x28,0xAC,0xE1,
0xE0,0xA8,0x2D,0x1B,0xE1,0xE0,0xA8,0x32,
0xA7,0xE1,0xB0,0xE1,0xE0,0xA8,0x39,0x8D,
0xE1,0xE0,0xA8,0x3E,0x8C,0xE1,0x87,0xE1,
0x30,0xA6,0x00,0x81,0x0C,0x27,0x0B,0x81,
0x01,0x27,0x2D,0x81,0x08,0x27,0x3B,0xBD,
0xBC,0x30,0xBD,0xD0,0x87,0x96,0x4B,0x26,
0x1B,0xDE,0x00,0xA6,0x09,0x81,0xA3,0x26,
0x13,0xFF,0x05,0xDE,0xBD,0xAA,0x39,0xDF,
0xD6,0xDE,0x00,0xEE,0x00,0xBD,0xAA,0x46,
0xDF,0xE9,0x20,0x39,0xBD,0xD1,0x6A,0x33,
0xEE,0x03,0xEE,0x00,0xDF,0xE9,0xDE,0x4D,
0xFF,0x05,0xDE,0x30,0xEE,0x03,0x08,0x08,
0x20,0x1A,0xEE,0x01,0xEE,0x07,0xDF,0xE9,
0x30,0xA6,0x01,0x8A,0x80,0xB7,0x05,0xDE,
0xA6,0x02,0xB7,0x05,0xDF,0xEE,0x01,0xEE,
0x0B,0xBD,0xAA,0x3E,0xDF,0xD6,0x9F,0x00,
0xBD,0xD1,0x1D,0x9E,0x00,0x7E,0x00,0x58,
0x86,0x15,0x36,0xCE,0x02,0xB5,0xDF,0x04,
0xBD,0x88,0x26,0x96,0x4B,0x27,0x02,0x32,
0x39,0xDE,0x02,0x8C,0x00,0x01,0x27,0xF7,
0xBD,0xA2,0x06,0x96,0x4B,0x26,0xF0,0xDE,
0x02,0xBD,0xAA,0x3A,0xDF,0x02,0x96,0x02,
0xD6,0x03,0xCB,0xF4,0x89,0x01,0x7D,0x00,
0x35,0x2A,0x04,0xC0,0x64,0x82,0x00,0xDB,
0x46,0x99,0x45,0x9F,0x08,0xD0,0x09,0x92,
0x08,0x2D,0x11,0x2E,0x03,0x5D,0x27,0x0C,
0x37,0x36,0x86,0x0E,0x36,0xBD,0xF9,0xAA,
0x96,0x4B,0x26,0xBB,0xDE,0x45,0x96,0x46,
0x9B,0x03,0x01,0x0F,0x97,0x46,0x96,0x45,
0x99,0x02,0x97,0x45,0x4F,0xA7,0x03,0xA7,
0x04,0xA7,0x05,0xA7,0x06,0xA7,0x07,0xA7,
0x08,0xDF,0x0A,0x86,0x08,0xA7,0x02,0xCE,
0x02,0xB5,0xD6,0x35,0x2A,0x0C,0x96,0x8E,
0x81,0x04,0x26,0x06,0x86,0x23,0x97,0x4B,
0x32,0x39,0x5D,0x2B,0x13,0xA6,0x01,0xE6,
0x02,0x4D,0x26,0x03,0x5D,0x27,0x41,0xDE,
0x0A,0xA7,0x07,0xE7,0x08,0xCE,0x02,0xB8,
0xDF,0x02,0xDE,0x0A,0xBD,0xAA,0x3A,0xDF,
0x04,0xBD,0xA0,0xDE,0xDE,0x0A,0xD6,0x34,
0xCB,0x09,0xE7,0x01,0x96,0x33,0x89,0x00,
0xA7,0x00,0xDB,0x0B,0xD7,0x46,0x99,0x0A,
0x97,0x45,0x0E,0xDE,0x0A,0x96,0x35,0x2B,
0x05,0xBD,0x91,0x3E,0x32,0x39,0xDF,0x4D,
0x86,0x20,0x97,0x54,0x32,0x7E,0x96,0xBC,
0xDE,0x0A,0xDF,0x45,0x0E,0x86,0x07,0x97,
0x4B,0x32,0x39,0xFF,0xB6,0x05,0x0A,0x27,
0x05,0x7A,0x05,0x0A,0x26,0x18,0xB6,0x05,
0x0B,0x27,0x13,0xCE,0x05,0x0C,0xA6,0x02,
0x85,0x40,0x26,0x0B,0xBD,0xAA,0x3D,0xA6,
0x00,0x26,0xF3,0x7F,0x05,0x0B,0x39,0x01,
0x0F,0x84,0xBF,0xA7,0x02,0x84,0x3F,0xBD,
0xA9,0xD5,0xEE,0x03,0xAD,0x03,0xBD,0xA9,
0xD4,0x0E,0x20,0xD2,0x7C,0x05,0x0A,0x26,
0xE5,0xBD,0xBC,0x30,0x4F,0x16,0x07,0x36,
0x01,0x0F,0x96,0x7C,0x84,0xC0,0x1B,0xB7,
0x87,0xC0,0x97,0x7C,0x32,0x06,0x39,0xDE,
0x49,0x33,0xD7,0x5C,0x33,0xD7,0x5D,0x33,
0xE7,0x00,0x33,0xE7,0x01,0x08,0x08,0xDF,
0x49,0x30,0x7E,0x00,0x5B,0xDE,0x49,0x09,
0x09,0xDF,0x49,0xEE,0x00,0x16,0x6E,0x00,
0x86,0x0E,0xDF,0x2A,0xBD,0xA9,0xE7,0xD6,
0x2B,0x37,0xD6,0x2A,0x37,0x36,0x7E,0xA9,
0xFD,0x30,0xEE,0x00,0xDF,0x2A,0xA6,0x00,
0xCE,0x00,0x00,0x33,0x33,0x33,0x33,0xBD,
0x04,0x57,0x4C,0x33,0xBD,0x04,0x57,0xDE,
0x2A,0x6E,0x01,0x08,0x08,0x08,0x08,0x08,
0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
0x08,0x08,0x08,0x39,0x09,0x09,0x09,0x09,
0x09,0x09,0x09,0x09,0x09,0x09,0x09,0x09,
0x09,0x39,0x36,0x96,0x55,0x85,0x01,0x32,
0x26,0x0A,0x81,0x60,0x23,0x14,0x81,0x7A,
0x22,0x03,0x84,0xDF,0x39,0x81,0x7D,0x26,
0x02,0x86,0x29,0x81,0x7B,0x26,0x02,0x86,
0x28,0x39,0x81,0x5B,0x26,0x02,0x86,0x28,
0x81,0x5D,0x26,0x02,0x86,0x29,0x39,0x86,
0x15,0x36,0x07,0x97,0x2F,0x01,0x0F,0xDF,
0x2C,0x4F,0x97,0x2E,0xBD,0xA9,0xD5,0xFE,
0x88,0x00,0x8C,0x40,0x51,0x26,0x08,0xDE,
0x2C,0xEE,0x00,0x27,0x02,0xAD,0x00,0x96,
0x2E,0x81,0x30,0x25,0x0D,0x4C,0x81,0x40,
0x26,0xE0,0xBD,0xA9,0xD4,0x96,0x2F,0x06,
0x32,0x39,0x8B,0x08,0x20,0xD4,0x96,0x4B,
0x97,0x4C,0x86,0x18,0x36,0xBD,0xF7,0xAA,
0x96,0x4C,0x97,0x4B,0x7F,0x00,0x4C,0x01,
0x0F,0x9E,0x43,0x86,0x19,0x36,0xDE,0x3D,
0xDF,0x4F,0xDE,0x41,0xFF,0x04,0x11,0xFF,
0x04,0x13,0xFF,0x04,0x15,0xFF,0x04,0x17,
0x0E,0x7E,0xC5,0xB2,0xDE,0x56,0x2B,0x40,
0x32,0xB7,0x04,0x37,0x32,0xB7,0x04,0x38,
0x96,0x56,0x27,0x0B,0x8A,0x80,0x97,0x5C,
0x96,0x57,0x97,0x5D,0x7E,0x00,0x5B,0x96,
0x57,0x84,0x03,0xCE,0x04,0x69,0xBD,0x04,
0x4B,0x27,0x2A,0xBD,0xA9,0xD5,0xFE,0x88,
0x00,0x8C,0x40,0x51,0x26,0x14,0x96,0x57,
0x44,0x44,0xFE,0x88,0x0A,0xBD,0x04,0x45,
0xAD,0x00,0xBD,0xA9,0xD4,0xFE,0x04,0x37,
0x6E,0x00,0x96,0x57,0x84,0x03,0xCE,0x04,
0x69,0x5F,0xBD,0x04,0x57,0x86,0x25,0x97,
0x4B,0x20,0xEA,0x36,0xB6,0x04,0x6B,0x27,
0x12,0xBD,0xA9,0xD5,0xFE,0x88,0x00,0x8C,
0x40,0x51,0x26,0x07,0x32,0xFE,0x88,0x0A,
0x7E,0x04,0x5D,0x32,0x4D,0x27,0x04,0x86,
0x25,0x97,0x4B,0x39,0x86,0x18,0x36,0x9F,
0x00,0xBD,0xAB,0x86,0xBD,0xAB,0x67,0x86,
0x15,0x36,0x9F,0x00,0x86,0x18,0xBD,0xD0,
0xE9,0x9E,0x00,0x32,0x32,0x97,0x59,0x32,
0x97,0x5A,0x9F,0x00,0x86,0x18,0xBD,0xD0,
0xE9,0x9E,0x00,0x7E,0x00,0x58,0x86,0x0F,
0x36,0xDE,0x00,0xDF,0x88,0xBD,0xAE,0x22,
0xBD,0xAE,0xC7,0x7F,0x00,0x8A,0x96,0x8E,
0x81,0x0C,0x26,0x11,0xFE,0x05,0xDE,0x8C,
0xFF,0xFF,0x27,0x09,0x96,0x8F,0x8A,0x08,
0x97,0x8F,0xBD,0xD1,0xD7,0xDE,0x88,0xDF,
0x00,0x20,0x03,0x86,0x0F,0x36,0xBD,0xD1,
0x1D,0xDE,0x00,0xE6,0x01,0xC1,0x12,0x27,
0x0B,0xC1,0x18,0x27,0x07,0xC1,0x11,0x26,
0x06,0x37,0x20,0xEA,0x7E,0xAC,0x50,0x9F,
0x02,0x96,0x00,0xD6,0x01,0xD0,0x03,0x92,
0x02,0x37,0x36,0xC6,0x0E,0x37,0x20,0xD6,
0xC5,0x08,0x26,0x09,0xBD,0xAD,0x43,0x96,
0x8A,0x84,0xDF,0x20,0x07,0xBD,0xAD,0xBA,
0x96,0x8A,0x8A,0x20,0x97,0x8A,0xD6,0x6E,
0x26,0x08,0xD6,0x4B,0x26,0x04,0x85,0x10,
0x27,0x4E,0xD6,0x8E,0xC1,0x0C,0x27,0x0A,
0xC1,0x10,0x26,0x1D,0x96,0x90,0x81,0x21,
0x27,0x17,0x96,0x8F,0x85,0x08,0x27,0x05,
0xBD,0xD2,0x08,0x20,0x09,0x96,0x8A,0x85,
0x03,0x26,0x03,0xBD,0xC2,0x78,0xBD,0xC4,
0x04,0xBD,0xAF,0x1D,0x9F,0x00,0x86,0x0F,
0xBD,0xD0,0xE9,0x9E,0x00,0x32,0x39,0xD6,
0x8A,0xC5,0x01,0x27,0xA3,0x30,0xA6,0x00,
0x81,0x11,0x26,0x07,0xCA,0x02,0xC4,0xDF,
0xD7,0x8A,0x31,0xBD,0xAD,0xF6,0x20,0x90,
0x96,0x4B,0x26,0xAE,0xCE,0xFF,0xFF,0xFF,
0x04,0x76,0xFF,0x05,0xE0,0x30,0xA6,0x00,
0xD6,0x8A,0xC4,0x20,0x81,0x11,0x26,0x03,
0x31,0xCA,0x02,0xD7,0x8A,0x30,0xA6,0x00,
0x81,0x0F,0x27,0x8E,0x31,0x32,0x33,0x9F,
0x02,0xDB,0x03,0x99,0x02,0x97,0x02,0xD7,
0x03,0xD6,0x8A,0xDE,0x02,0x08,0xA6,0x00,
0x81,0x0C,0x27,0x0F,0x81,0x0A,0x27,0x18,
0x81,0x0B,0x27,0x0A,0x81,0x01,0x27,0x3E,
0x7E,0xAC,0xED,0x08,0x20,0x07,0xEE,0x03,
0x4F,0x97,0xF8,0x97,0xF9,0x7E,0xAD,0x3E,
0xEE,0x01,0xDF,0xF8,0xA6,0x04,0x85,0x40,
0x27,0x0A,0x85,0x80,0x27,0x04,0x86,0x40,
0xA7,0x05,0x20,0x7F,0xCA,0x01,0xD7,0x8A,
0xFF,0x04,0x76,0xEE,0x07,0xDF,0x9E,0xCE,
0x00,0x01,0xDF,0x9A,0xDF,0xA2,0xDF,0x88,
0xDE,0xF8,0xEE,0x0B,0x20,0x65,0xEE,0x03,
0x08,0x08,0xDF,0x9C,0x09,0x09,0xEE,0x00,
0xDF,0x9A,0xCA,0x08,0xD7,0x8A,0xDE,0x4D,
0xFF,0x05,0xE0,0x20,0x53,0xCA,0x08,0xD7,
0x8A,0xEE,0x01,0xDF,0xF8,0xBD,0x87,0x52,
0xB7,0x05,0xE0,0xA6,0x04,0x2B,0x1C,0xD6,
0x8F,0x2B,0x0E,0xEE,0x0B,0xA6,0x02,0x85,
0x40,0x26,0x14,0xDE,0xF8,0xEE,0x07,0x20,
0x02,0xEE,0x05,0xDF,0x9A,0xDE,0xF8,0xEE,
0x0B,0x20,0x20,0xD6,0x8F,0x2B,0x06,0x86,
0x24,0x97,0x4B,0x20,0x13,0xDE,0xF8,0x86,
0x48,0xB7,0x00,0x17,0x7F,0x00,0x16,0xBD,
0xE4,0xD4,0xDE,0xF8,0x96,0x4B,0x27,0xD9,
0x7E,0xAC,0x02,0xBD,0xAA,0x3E,0xDF,0x9C,
0x7E,0xAC,0x37,0x86,0x15,0x36,0x96,0x8F,
0x2B,0x0E,0xDE,0x9C,0xA6,0x00,0x85,0x40,
0x27,0x06,0x86,0x24,0x97,0x4B,0x20,0x09,
0xCE,0xAD,0xF2,0x96,0x8E,0x48,0xBD,0x04,
0x3E,0x31,0x96,0x8F,0x2A,0x0A,0xDE,0xF8,
0x27,0x06,0xA6,0x04,0x84,0x7F,0xA7,0x04,
0x8D,0x6D,0x25,0x45,0x96,0x8A,0x85,0x01,
0x27,0x3F,0xDE,0x9C,0xBD,0xAA,0x3B,0xDF,
0x9C,0xDE,0x9E,0x27,0x0B,0xDE,0xA2,0x9C,
0x9E,0x27,0x12,0x08,0xDF,0xA2,0x20,0x22,
0xDE,0xF8,0xEE,0x05,0x9C,0x88,0x26,0x1A,
0xBD,0xAD,0xF6,0x20,0x1C,0xBD,0xAD,0xF6,
0xCE,0x00,0x01,0xDF,0xA2,0xDE,0xF8,0xEE,
0x05,0x9C,0x9A,0x27,0x0C,0xDE,0x9A,0x08,
0xDF,0x9A,0xDE,0x88,0x08,0xDF,0x88,0x20,
0x8A,0x39,0x86,0x15,0x36,0xCE,0xAE,0x00,
0x96,0x8E,0x48,0xBD,0x04,0x3E,0x31,0x96,
0x8F,0x2A,0x2A,0xDE,0xF8,0x07,0x01,0x0F,
0xD6,0x8B,0xE7,0x07,0xD6,0x8C,0xE7,0x08,
0xEE,0x0B,0xC6,0x20,0xE7,0x02,0x06,0x0C,
0x96,0x4B,0x9A,0x6E,0x26,0x08,0x96,0x6A,
0x2A,0x0B,0x85,0x38,0x27,0x07,0x96,0x8A,
0x8A,0x10,0x97,0x8A,0x0D,0x39,0x96,0x8E,
0x81,0x0C,0x26,0x0D,0x96,0x8F,0x85,0x08,
0x26,0x07,0x86,0x15,0x36,0xBD,0xC2,0x76,
0x31,0x39,0xC2,0x7E,0xF0,0x33,0xCC,0xAA,
0xCD,0x6F,0xC2,0x7E,0xF1,0xE2,0xF2,0x0F,
0xC2,0x64,0xEF,0xF8,0xCC,0xCF,0xCD,0x3D,
0xC2,0x64,0x86,0x15,0x36,0x96,0x56,0x97,
0x8E,0xD6,0x8F,0xC4,0xCF,0xC5,0x04,0x27,
0x0E,0xF7,0x00,0x8F,0xBD,0xAF,0x94,0x96,
0x4B,0x27,0x02,0x32,0x39,0xD6,0x8F,0xC5,
0x02,0x26,0x06,0x96,0x8E,0x84,0x3F,0x97,
0x97,0xC5,0x01,0x26,0x06,0x96,0x57,0x84,
0x3F,0x97,0x90,0x5D,0x2B,0x04,0x96,0x57,
0x84,0xC0,0x1B,0x8A,0x03,0x97,0x8F,0x96,
0x90,0x81,0x21,0x26,0x05,0xBD,0xDF,0xE8,
0x20,0x08,0x81,0x20,0x27,0x0C,0x81,0x22,
0x27,0x08,0xB6,0x87,0xAA,0x8A,0x40,0xB7,
0x87,0xAA,0x86,0x01,0xB7,0x06,0x0B,0xB7,
0x04,0x67,0x86,0x0D,0xD6,0x8F,0x2A,0x20,
0xD6,0x77,0x27,0x0D,0xB6,0x04,0x66,0x97,
0x7A,0xB6,0x04,0x64,0xF6,0x04,0x65,0x20,
0x13,0xD6,0x90,0xC1,0x1F,0x27,0x09,0xF6,
0x04,0x63,0x27,0x04,0x86,0x0A,0x97,0x77,
0xC6,0xFF,0xD7,0x7A,0x97,0x78,0xD7,0x79,
0xCE,0xFF,0xFF,0xFF,0x04,0x76,0xFF,0x05,
0xE0,0x86,0xAC,0xB7,0x04,0x79,0x4F,0x97,
0xAE,0x97,0x87,0x97,0x6A,0x31,0x39,0x96,
0x90,0x81,0x25,0x24,0x04,0x80,0x1E,0x2E,
0x01,0x4F,0xCE,0xAF,0x01,0x81,0x03,0x26,
0x08,0xD6,0x81,0xC5,0x01,0x27,0x02,0x86,
0x06,0x48,0x48,0x16,0xBD,0x04,0x45,0xDF,
0x91,0xDF,0x98,0x09,0x6F,0x00,0xDF,0x95,
0x17,0x8B,0x02,0xCE,0xAF,0x01,0xBD,0x04,
0x45,0x96,0x8F,0x2A,0x01,0x08,0xDF,0x93,
0x39,0x02,0x6B,0x02,0xB2,0x02,0x21,0x02,
0x68,0x02,0x6B,0x02,0xB2,0x01,0x1F,0x02,
0x1E,0x02,0x6B,0x02,0xB2,0x02,0x6B,0x02,
0xB2,0x01,0x1F,0x01,0x9E,0x96,0x4B,0x36,
0x96,0x8E,0xD6,0x8F,0x2A,0x0F,0x81,0x0D,
0x26,0x0E,0x96,0x4B,0x9A,0x6A,0x26,0x08,
0xBD,0xF0,0xB1,0x20,0xF5,0xBD,0xF1,0xBA,
0x96,0x90,0x81,0x1E,0x22,0x12,0xBD,0xF5,
0x7F,0xBD,0xF5,0x39,0xC6,0x5F,0xBD,0xF5,
0x0E,0xC6,0x3F,0xBD,0xF5,0x0E,0x20,0x0B,
0x81,0x21,0x27,0x04,0x81,0x23,0x26,0x03,
0xBD,0xCB,0xBF,0xBD,0xF4,0xD8,0x4F,0x97,
0x8F,0x97,0xA4,0x97,0x8A,0x97,0x77,0x97,
0x87,0x43,0x97,0x90,0x86,0x0D,0x97,0x78,
0xCE,0xFF,0xFF,0xFF,0x05,0xDE,0x32,0x97,
0x4B,0x86,0x80,0xB7,0x87,0xAA,0x39,0x73,
0x00,0x77,0x32,0x97,0x59,0x32,0x97,0x5A,
0x86,0x12,0x36,0x96,0x8F,0x8A,0x04,0x97,
0x8F,0x7E,0x00,0x58,0x86,0x0F,0x36,0x9F,
0x00,0x86,0x12,0xBD,0xD0,0xE9,0xDE,0x00,
0x08,0xDF,0x00,0xBD,0xE8,0x2E,0x33,0x33,
0x30,0xEE,0x00,0x32,0x32,0x5D,0x26,0x45,
0x08,0xBD,0xB1,0xA2,0x26,0x3F,0xA6,0x03,
0x26,0x3B,0xA6,0x04,0x27,0x37,0x81,0x23,
0x27,0x33,0x97,0x90,0x96,0x8F,0x8A,0x01,
0x84,0xFB,0x97,0x8F,0x33,0xC1,0x0F,0x26,
0x01,0x39,0x33,0x30,0xEE,0x00,0x32,0x32,
0x5D,0x26,0x1A,0x08,0xBD,0xB1,0xA2,0x26,
0x14,0xA6,0x03,0x26,0x10,0xA6,0x04,0x81,
0x21,0x24,0x0A,0x97,0x97,0x96,0x8F,0x8A,
0x02,0x97,0x8F,0x33,0x39,0x86,0x42,0x97,
0x4B,0x86,0x0F,0x9F,0x00,0xBD,0xD0,0xE9
    ];
    
    this.ROM665 = [
0x9E,0x00,0x32,0x39,0x96,0x01,0x36,0x96,
0x00,0x36,0x36,0xBD,0xB1,0xFB,0xCE,0x05,
0xBA,0xBD,0xB7,0x0F,0xCE,0x05,0xBA,0xDF,
0x9C,0xCE,0x00,0x8F,0xBD,0xB6,0xEB,0x96,
0x02,0xD6,0x03,0xDE,0x04,0x09,0xDF,0x04,
0xDB,0x05,0x99,0x04,0x97,0x93,0xD7,0x94,
0xDE,0x02,0xDF,0x91,0x09,0xDF,0x95,0x6F,
0x00,0x7F,0x00,0x8F,0x86,0x01,0x97,0x87,
0x86,0x24,0x97,0x90,0x86,0x02,0x97,0x8A,
0xBD,0xC2,0x7E,0x7F,0x00,0x87,0xDE,0x95,
0xDF,0x04,0xDE,0x91,0xDF,0x02,0xCE,0x00,
0x8F,0xBD,0xB7,0x0F,0x39,0xCE,0x00,0x8F,
0xBD,0xB6,0xEB,0x86,0xA0,0x97,0x8F,0xDE,
0x00,0xDF,0x98,0xDF,0x91,0xDE,0x02,0xDF,
0x95,0xCE,0x05,0xBA,0xDF,0x9C,0x86,0x24,
0x97,0x90,0x7F,0x00,0x6A,0xBD,0xF0,0x33,
0xDE,0x98,0xDF,0x00,0xDE,0x95,0xDF,0x02,
0xCE,0x00,0x8F,0xBD,0xB7,0x0F,0x39,0x96,
0xED,0x4A,0x27,0x06,0x86,0x58,0x97,0x4B,
0x0B,0x39,0xDE,0x9C,0xBD,0xB6,0xEB,0xBD,
0xB6,0xEB,0xCE,0x05,0xBA,0xBD,0xB7,0x0F,
0xCE,0x05,0xB2,0xBD,0xB7,0x0F,0x09,0xBD,
0xB1,0xA2,0x5F,0x4D,0x27,0x02,0x0B,0x39,
0xE6,0x04,0xA6,0x03,0x27,0x01,0x0D,0xEE,
0x03,0x39,0xDF,0x00,0xCE,0x00,0x20,0xDF,
0x02,0xCE,0x00,0x08,0xDF,0x04,0xBD,0xB0,
0x04,0xDE,0x04,0x6F,0x01,0x39,0xCE,0x00,
0x3B,0xDF,0x00,0xDE,0x3B,0xD6,0x35,0xC5,
0x20,0x27,0x10,0x96,0x06,0xA7,0x02,0x96,
0x07,0xA7,0x03,0x86,0x08,0xA7,0x04,0x1B,
0x97,0x35,0x39,0x96,0x06,0xF6,0x00,0x07,
0x8C,0x00,0x00,0x27,0x28,0xA1,0x02,0x25,
0x16,0x22,0x1C,0xE1,0x03,0x25,0x10,0x22,
0x16,0xA6,0x04,0x81,0x08,0x26,0x05,0x9A,
0x35,0x97,0x35,0x39,0xDF,0x06,0x39,0xA6,
0x04,0x85,0x08,0x27,0x08,0x96,0x06,0xDF,
0x00,0xEE,0x00,0x20,0xD3,0x86,0x15,0x36,
0xBD,0xA9,0xCC,0x9F,0x0E,0xD6,0x46,0x96,
0x45,0xCB,0x9D,0x89,0x01,0xD0,0x0F,0x92,
0x0E,0x2D,0x17,0x2E,0x04,0xC1,0x00,0x27,
0x11,0x37,0x36,0x86,0x0E,0x36,0xBD,0xF9,
0xAA,0x96,0x4B,0x27,0x05,0xBD,0xA9,0x94,
0x32,0x39,0xDF,0x10,0xDE,0x0E,0xBD,0xAA,
0x45,0x35,0xDE,0x43,0xBD,0xAA,0x44,0xDF,
0x43,0x30,0x09,0x9C,0x43,0x27,0x07,0x08,
0xA6,0x0D,0xA7,0x00,0x20,0xF5,0xDE,0x00,
0x96,0x44,0x8B,0x01,0x01,0x0F,0xA7,0x01,
0x96,0x43,0x89,0x00,0xA7,0x00,0xEE,0x00,
0x96,0x11,0xA7,0x01,0x96,0x10,0xA7,0x00,
0x96,0x07,0xA7,0x03,0x96,0x06,0xA7,0x02,
0x0E,0x86,0x10,0x94,0x35,0x26,0x02,0x86,
0x40,0x8A,0x80,0xA7,0x04,0xDF,0x06,0x20,
0xAC,0xFF,0xA6,0x01,0x2B,0x41,0x80,0x04,
0x2B,0x36,0x26,0x43,0xC6,0x08,0xA6,0x03,
0xE0,0x02,0x23,0x0D,0x44,0x5A,0x26,0xFC,
0x24,0x01,0x4C,0xA7,0x04,0xE7,0x03,0x20,
0x23,0xCB,0x08,0x27,0x17,0xC1,0x09,0x24,
0x26,0x44,0x66,0x04,0x5A,0x26,0xFA,0x24,
0x07,0x6C,0x04,0x26,0x03,0x4C,0x27,0x17,
0xA7,0x03,0x20,0x08,0x69,0x05,0x20,0xEF,
0x6F,0x03,0x6F,0x04,0x96,0x4B,0x39,0xC6,
0x5F,0xD7,0x4B,0x84,0x07,0x20,0xB7,0xC6,
0x60,0xD7,0x4B,0x86,0xFF,0xA7,0x03,0xA7,
0x04,0x20,0xE9,0xBD,0xA9,0xE7,0x32,0x4F,
0x36,0x36,0x36,0x36,0xC6,0x30,0x37,0x86,
0x04,0x36,0x86,0x0C,0x36,0x7E,0xB3,0x43,
0x64,0x03,0x66,0x04,0x66,0x05,0x66,0x06,
0x66,0x07,0x66,0x08,0x4A,0x85,0x07,0x26,
0xEF,0x4D,0x27,0xC2,0xE6,0x07,0xE7,0x08,
0xE6,0x06,0xE7,0x07,0xE6,0x05,0xE7,0x06,
0xE6,0x04,0xE7,0x05,0xE6,0x03,0xE7,0x04,
0x6F,0x03,0x80,0x08,0x20,0xE4,0x30,0xA6,
0x03,0x88,0x80,0xA7,0x03,0xBD,0xA9,0xE7,
0xA6,0x01,0x97,0xD3,0x84,0x07,0xA7,0x01,
0xA6,0x03,0x27,0x24,0xA6,0x0C,0x26,0x27,
0xA6,0x08,0xA7,0x11,0xA6,0x07,0xA7,0x10,
0xA6,0x06,0xA7,0x0F,0xA6,0x05,0xA7,0x0E,
0xA6,0x04,0xA7,0x0D,0xA6,0x03,0xA7,0x0C,
0x96,0xD3,0xA7,0x0A,0xA6,0x02,0xA7,0x0B,
0xBD,0xAA,0x3A,0x35,0x7E,0xA9,0xFD,0xA6,
0x0B,0xA0,0x02,0xE6,0x0A,0xC4,0x07,0xE2,
0x01,0x2A,0x19,0x5C,0x26,0xCA,0x81,0xD1,
0x25,0xC6,0x40,0xE6,0x0A,0xC4,0x80,0xEA,
0x01,0xE7,0x0A,0xE6,0x02,0xE7,0x0B,0xBD,
0xAA,0x3A,0x20,0x06,0x26,0xD2,0x81,0x30,
0x24,0xCE,0xBD,0xB2,0x1D,0x30,0x96,0xD3,
0xA8,0x0A,0x2B,0x3F,0xA6,0x08,0xAB,0x11,
0xA7,0x11,0xA6,0x07,0xA9,0x10,0xA7,0x10,
0xA6,0x06,0xA9,0x0F,0xA7,0x0F,0xA6,0x05,
0xA9,0x0E,0xA7,0x0E,0xA6,0x04,0xA9,0x0D,
0xA7,0x0D,0xA6,0x03,0xA9,0x0C,0xA7,0x0C,
0x24,0x12,0x66,0x0C,0x66,0x0D,0x66,0x0E,
0x66,0x0F,0x66,0x10,0x66,0x11,0x6C,0x0B,
0x26,0x02,0x6C,0x0A,0xBD,0xAA,0x3A,0x35,
0x7E,0xB3,0x93,0xA6,0x11,0xA0,0x08,0xA7,
0x11,0xA6,0x10,0xA2,0x07,0xA7,0x10,0xA6,
0x0F,0xA2,0x06,0xA7,0x0F,0xA6,0x0E,0xA2,
0x05,0xA7,0x0E,0xA6,0x0D,0xA2,0x04,0xA7,
0x0D,0xA6,0x0C,0xA2,0x03,0xA7,0x0C,0x24,
0x26,0xA6,0x0A,0x88,0x80,0xA7,0x0A,0x63,
0x0C,0x63,0x0D,0x63,0x0E,0x63,0x0F,0x63,
0x10,0x60,0x11,0x26,0x12,0x6C,0x10,0x26,
0x0E,0x6C,0x0F,0x26,0x0A,0x6C,0x0E,0x26,
0x06,0x6C,0x0D,0x26,0x02,0x6C,0x0C,0xBD,
0xAA,0x3A,0x35,0x5F,0x30,0x6D,0x03,0x26,
0x3A,0xCB,0x08,0xC1,0x30,0x26,0x0F,0x6F,
0x01,0x6F,0x02,0x73,0x05,0xCB,0x7E,0xA9,
0xFD,0xBD,0xA9,0xE7,0x20,0xE5,0xA6,0x04,
0xA7,0x03,0xA6,0x05,0xA7,0x04,0xA6,0x06,
0xA7,0x05,0xA6,0x07,0xA7,0x06,0xA6,0x08,
0xA7,0x07,0x6F,0x08,0x20,0xCF,0x5C,0x68,
0x08,0x69,0x07,0x69,0x06,0x69,0x05,0x69,
0x04,0x69,0x03,0x2A,0xF1,0xF7,0x05,0xCB,
0xA6,0x02,0x10,0xA7,0x02,0xA6,0x01,0x82,
0x00,0xA7,0x01,0xA6,0x01,0x85,0x78,0x27,
0x26,0x85,0x40,0x26,0x1F,0x86,0x01,0x97,
0x4B,0x30,0xA6,0x01,0x84,0x80,0x8A,0x07,
0xA7,0x01,0x86,0xFF,0xA7,0x02,0xA7,0x03,
0xA7,0x04,0xA7,0x05,0xA7,0x06,0xA7,0x07,
0xA7,0x08,0x20,0x03,0x30,0x8D,0x03,0x7E,
0xA9,0xFD,0x6F,0x05,0x6F,0x06,0x6F,0x07,
0x6F,0x08,0x6F,0x01,0x6F,0x02,0x6F,0x03,
0x6F,0x04,0x39,0xBD,0xA9,0xE7,0xA6,0x03,
0x26,0x0A,0xBD,0xAA,0x3A,0x35,0x86,0x02,
0x97,0x4B,0x20,0xBD,0xA6,0x0C,0x26,0x06,
0xBD,0xAA,0x3A,0x35,0x20,0xD1,0xA6,0x0B,
0xA0,0x02,0xE6,0x0A,0xE2,0x01,0x8B,0x01,
0xC9,0x04,0xA7,0x0B,0xE7,0x0A,0x32,0x32,
0x32,0x32,0x97,0xC8,0x32,0x97,0xC9,0x32,
0x97,0xCA,0x32,0x97,0xCB,0x32,0x97,0xCC,
0x32,0x97,0xCD,0x30,0xA6,0x03,0x97,0xCE,
0xA6,0x04,0x97,0xCF,0xA6,0x05,0x97,0xD0,
0xA6,0x06,0x97,0xD1,0xA6,0x07,0x97,0xD2,
0xA6,0x08,0x97,0xD3,0x86,0x06,0x97,0xC0,
0x97,0xBF,0x86,0x01,0xD6,0xD3,0xD0,0xCD,
0xD7,0xD3,0xD6,0xD2,0xD2,0xCC,0xD7,0xD2,
0xD6,0xD1,0xD2,0xCB,0xD7,0xD1,0xD6,0xD0,
0xD2,0xCA,0xD7,0xD0,0xD6,0xCF,0xD2,0xC9,
0xD7,0xCF,0xD6,0xCE,0xD2,0xC8,0xD7,0xCE,
0x56,0x53,0xDA,0xBF,0x59,0x49,0x16,0x24,
0x0D,0xA7,0x03,0x08,0x86,0x01,0x7A,0x00,
0xC0,0x26,0x03,0x7E,0xB3,0x43,0x78,0x00,
0xD3,0x79,0x00,0xD2,0x79,0x00,0xD1,0x79,
0x00,0xD0,0x79,0x00,0xCF,0x79,0x00,0xCE,
0x56,0xD7,0xBF,0x25,0xAF,0xD6,0xD3,0xDB,
0xCD,0xD7,0xD3,0xD6,0xD2,0xD9,0xCC,0xD7,
0xD2,0xD6,0xD1,0xD9,0xCB,0xD7,0xD1,0xD6,
0xD0,0xD9,0xCA,0xD7,0xD0,0xD6,0xCF,0xD9,
0xC9,0xD7,0xCF,0xD6,0xCE,0xD9,0xC8,0xD7,
0xCE,0x56,0xD4,0xBF,0x20,0xAE,0xCE,0x00,
0x00,0xDF,0xC8,0xDF,0xCA,0xDF,0xCC,0xDF,
0xBF,0xBD,0xA9,0xE7,0xA6,0x0C,0x26,0x07,
0xBD,0xAA,0x3A,0x35,0x7E,0xB3,0xBC,0xA6,
0x03,0x27,0xF5,0x32,0x33,0x32,0xAB,0x0B,
0xE9,0x0A,0xC0,0x04,0xA7,0x0B,0xE7,0x0A,
0x32,0x97,0xCE,0x32,0x97,0xCF,0x32,0x97,
0xD0,0x32,0x97,0xD1,0x32,0x97,0xD2,0x32,
0x97,0xD3,0x86,0x06,0x97,0xBF,0xE6,0x11,
0x26,0x1C,0xD1,0xC0,0x27,0x57,0x96,0xCC,
0x97,0xCD,0x96,0xCB,0x97,0xCC,0x96,0xCA,
0x97,0xCB,0x96,0xC9,0x97,0xCA,0x96,0xC8,
0x97,0xC9,0xD7,0xC8,0x20,0x3F,0xD7,0xC0,
0x0D,0x56,0x24,0x24,0x96,0xCD,0x9B,0xD3,
0x97,0xCD,0x96,0xCC,0x99,0xD2,0x97,0xCC,
0x96,0xCB,0x99,0xD1,0x97,0xCB,0x96,0xCA,
0x99,0xD0,0x97,0xCA,0x96,0xC9,0x99,0xCF,
0x97,0xC9,0x96,0xC8,0x99,0xCE,0x97,0xC8,
0x76,0x00,0xC8,0x76,0x00,0xC9,0x76,0x00,
0xCA,0x76,0x00,0xCB,0x76,0x00,0xCC,0x76,
0x00,0xCD,0x54,0x26,0xC5,0x09,0x7A,0x00,
0xBF,0x26,0x9B,0x30,0x96,0xC8,0xA7,0x03,
0x96,0xC9,0xA7,0x04,0x96,0xCA,0xA7,0x05,
0x96,0xCB,0xA7,0x06,0x96,0xCC,0xA7,0x07,
0x96,0xCD,0xA7,0x08,0x7E,0xB3,0x43,0xBD,
0xA9,0xE7,0xC6,0x7B,0xF7,0x05,0xA7,0xF7,
0x05,0xA8,0x96,0x31,0x4A,0x26,0x04,0xD4,
0x35,0xD7,0x35,0xCE,0x00,0x00,0xDF,0xCE,
0xDF,0xD0,0xDF,0xD2,0xDF,0xBF,0xDF,0xC1,
0xDF,0xD4,0x7F,0x05,0xA9,0x7F,0x00,0xC3,
0x96,0x8D,0x81,0x2B,0x27,0x0B,0x81,0x2D,
0x26,0x0A,0xBD,0xB6,0xD9,0x0D,0x76,0x00,
0xC3,0xBD,0xF0,0x85,0x80,0x30,0x24,0x0F,
0x81,0xFE,0x26,0x7B,0xD6,0xBF,0x26,0x77,
0xBD,0xB6,0xD9,0x97,0xBF,0x20,0xEA,0x81,
0x0A,0x24,0x1C,0x7F,0x05,0xA7,0xD6,0xBF,
0x26,0x05,0xDE,0xC0,0x08,0xDF,0xC0,0xD6,
0xCE,0xC1,0x15,0x24,0xD4,0xBD,0xB7,0x31,
0xDE,0xC0,0x09,0xDF,0xC0,0x20,0xCA,0x81,
0x35,0x27,0x04,0x81,0x15,0x26,0x48,0x73,
0x05,0xA9,0xBD,0xF0,0x85,0x81,0x2B,0x27,
0x0B,0x81,0x20,0x27,0x07,0x81,0x2D,0x26,
0x0C,0x7C,0x00,0xC3,0xBD,0xB6,0xD9,0x7F,
0x05,0xA9,0xBD,0xF0,0x85,0x80,0x30,0x25,
0x26,0x81,0x0A,0x24,0x22,0xD6,0xD4,0x27,
0x03,0x7C,0x00,0xC2,0xD6,0xD5,0xC1,0x34,
0x25,0x03,0x7C,0x00,0xC2,0x58,0x58,0xDB,
0xD5,0x58,0x79,0x00,0xD4,0x1B,0x97,0xD5,
0x24,0xD2,0x7C,0x00,0xD4,0x20,0xCD,0x96,
0xCE,0x9A,0xCF,0x9A,0xD0,0x9A,0xD1,0x27,
0x03,0xBD,0xB6,0xD9,0xD6,0xC0,0x96,0xC3,
0x46,0x96,0xC1,0x24,0x06,0x90,0xD5,0xD2,
0xD4,0x20,0x04,0x9B,0xD5,0xD9,0xD4,0xD7,
0xC1,0x2A,0x08,0x70,0x00,0xC2,0x53,0x40,
0x26,0x01,0x4C,0xC1,0x02,0x25,0x04,0xD6,
0xC1,0xD7,0xC2,0x97,0xD5,0xD7,0xD4,0xCE,
0x00,0xCC,0xBD,0xB6,0xEB,0x30,0x86,0x30,
0xA7,0x02,0x96,0xC3,0x84,0x80,0x8A,0x04,
0xA7,0x01,0xBD,0xB3,0x59,0xD6,0xC2,0x27,
0x08,0x2B,0x03,0x7E,0xB3,0xA1,0x7E,0xB3,
0xBC,0x86,0x06,0x97,0xC2,0xCE,0xB9,0xD2,
0xDF,0xC4,0x76,0x00,0xD4,0x20,0x03,0x78,
0x00,0xD5,0x24,0x0F,0xBD,0xB6,0xEB,0x96,
0xC1,0x2A,0x05,0xBD,0xB3,0xD3,0x20,0x03,
0xBD,0xB4,0xAE,0xDE,0xC4,0xBD,0xAA,0x3B,
0xDF,0xC4,0x7A,0x00,0xC2,0x26,0xE0,0x96,
0xD5,0x27,0x13,0x8B,0x20,0x26,0x15,0xBD,
0xB6,0xEB,0x96,0xC1,0x2A,0x05,0xBD,0xB3,
0xD3,0x20,0x03,0xBD,0xB4,0xAE,0x7F,0x00,
0x4B,0x7E,0xA9,0xFD,0xBD,0xAA,0x3B,0x20,
0xE2,0x7F,0x05,0xA8,0xC6,0x01,0xD1,0x31,
0x26,0x08,0xD7,0x30,0xD6,0x35,0xCA,0x84,
0xD7,0x35,0x39,0x32,0x97,0x5C,0x32,0x97,
0x5D,0xA6,0x07,0x36,0xA6,0x06,0x36,0xA6,
0x05,0x36,0xA6,0x04,0x36,0xA6,0x03,0x36,
0xA6,0x02,0x36,0xA6,0x01,0x36,0xA6,0x00,
0x36,0x86,0x0C,0x36,0x7E,0x00,0x5B,0x32,
0x97,0x5C,0x32,0x97,0x5D,0x31,0x32,0xA7,
0x00,0x32,0xA7,0x01,0x32,0xA7,0x02,0x32,
0xA7,0x03,0x32,0xA7,0x04,0x32,0xA7,0x05,
0x32,0xA7,0x06,0x32,0xA7,0x07,0x7E,0x00,
0x5B,0x5F,0x36,0x96,0xD3,0x48,0x97,0xCD,
0x96,0xD2,0x49,0x97,0xCC,0x96,0xD1,0x49,
0x97,0xCB,0x96,0xD0,0x49,0x97,0xCA,0x96,
0xCF,0x49,0x97,0xC9,0x96,0xCE,0x49,0x97,
0xC8,0x59,0x78,0x00,0xCD,0x79,0x00,0xCC,
0x79,0x00,0xCB,0x79,0x00,0xCA,0x79,0x00,
0xC9,0x79,0x00,0xC8,0x59,0x96,0xCD,0x9B,
0xD3,0x97,0xD3,0x96,0xCC,0x99,0xD2,0x97,
0xD2,0x96,0xCB,0x99,0xD1,0x97,0xD1,0x96,
0xCA,0x99,0xD0,0x97,0xD0,0x96,0xC9,0x99,
0xCF,0x97,0xCF,0x96,0xC8,0x99,0xCE,0x97,
0xCE,0xC9,0x00,0x78,0x00,0xD3,0x79,0x00,
0xD2,0x79,0x00,0xD1,0x79,0x00,0xD0,0x79,
0x00,0xCF,0x79,0x00,0xCE,0x59,0x32,0x9B,
0xD3,0x97,0xD3,0x24,0x1A,0x7C,0x00,0xD2,
0x26,0x15,0x7C,0x00,0xD1,0x26,0x10,0x7C,
0x00,0xD0,0x26,0x0B,0x7C,0x00,0xCF,0x26,
0x06,0x7C,0x00,0xCE,0x26,0x01,0x5C,0x39,
0xDE,0xC4,0xA6,0x00,0xE6,0x01,0x30,0xA0,
0x03,0x25,0x3F,0x26,0x3A,0xE1,0x04,0x25,
0x39,0x26,0x34,0xA6,0x05,0xE6,0x06,0xDE,
0xC4,0xE0,0x03,0xA2,0x02,0x25,0x28,0x26,
0x29,0x5D,0x26,0x26,0xE6,0x05,0xA6,0x04,
0x30,0xE0,0x08,0xA2,0x07,0x25,0x1B,0x26,
0x16,0x5D,0x26,0x13,0xE6,0x0A,0xA6,0x09,
0xDE,0xC4,0xE0,0x07,0xA2,0x06,0x25,0x07
    ];
    
    this.ROM666 = [
0x26,0x08,0x5D,0x26,0x05,0x4F,0x39,0x86,
0xFF,0x39,0x86,0x01,0x39,0xBD,0xA9,0xE7,
0xA6,0x03,0x27,0x04,0xA6,0x0C,0x26,0x1E,
0xBD,0xB2,0x3E,0xCE,0x05,0xC3,0xDF,0xC4,
0x30,0xA6,0x01,0x97,0xC2,0x84,0x07,0xA7,
0x01,0xBD,0xB7,0xC0,0x2F,0x16,0x96,0xC2,
0x2A,0x1A,0x86,0xFF,0x20,0x18,0x7F,0x05,
0xCB,0xBD,0xB2,0x3E,0xB6,0x05,0xCB,0xB1,
0x05,0xC2,0x25,0x03,0x4F,0x20,0x07,0x30,
0xA6,0x01,0x2B,0xE6,0x86,0x01,0x97,0xC2,
0x7E,0xB3,0xBC,0xBD,0xA9,0xE7,0xA6,0x03,
0xB7,0x05,0xF1,0x26,0x07,0xBD,0xAA,0x3A,
0x35,0x7E,0xA9,0xFD,0xC6,0x20,0xA6,0x01,
0x2A,0x06,0xC6,0x2D,0x88,0x80,0xA7,0x01,
0xF7,0x05,0xF2,0x5F,0xD7,0xD4,0xD7,0xD5,
0xC6,0x06,0xD7,0xC2,0x81,0x04,0x2B,0x6A,
0x26,0x0A,0x6D,0x02,0x26,0x06,0x86,0xFF,
0x16,0x7E,0xB9,0x33,0xCE,0xB9,0xD2,0xDF,
0xC4,0xCE,0xBA,0x3A,0xDF,0xC6,0xBD,0xB7,
0xC0,0x4A,0x26,0x0B,0xDE,0xC6,0xBD,0xB6,
0xEB,0xBD,0xB4,0xAE,0x7C,0x00,0xD5,0x78,
0x00,0xD5,0x8D,0x36,0x7A,0x00,0xC2,0x27,
0x07,0xDE,0xC6,0xBD,0xAA,0x3B,0x20,0xDC,
0x86,0x07,0x97,0xC2,0xBD,0xB7,0xC0,0x4A,
0x27,0x07,0x8D,0x17,0x7A,0x00,0xC2,0x26,
0xF3,0xDE,0xC6,0xBD,0xB6,0xEB,0xBD,0xB4,
0xAE,0x96,0xD5,0x5F,0x48,0x48,0x59,0x9B,
0xC2,0x20,0x58,0xDE,0xC6,0xBD,0xAA,0x3B,
0xDF,0xC6,0xDE,0xC4,0xBD,0xAA,0x3B,0xDF,
0xC4,0x39,0xCE,0xB9,0xD2,0xDF,0xC6,0xCE,
0xBA,0x3A,0xDF,0xC4,0xBD,0xB7,0xC0,0x4A,
0x2A,0x0B,0xDE,0xC6,0xBD,0xB6,0xEB,0xBD,
0xB4,0xAE,0x7C,0x00,0xD5,0x8D,0xD4,0x78,
0x00,0xD5,0x7A,0x00,0xC2,0x26,0xE5,0x86,
0x07,0x97,0xC2,0xBD,0xB7,0xC0,0x2F,0x09,
0x8D,0xC1,0x7A,0x00,0xC2,0x26,0xF4,0x20,
0x08,0xDE,0xC6,0xBD,0xB6,0xEB,0xBD,0xB4,
0xAE,0x96,0xD5,0x5F,0x48,0x48,0x59,0x9B,
0xC2,0x43,0x53,0x97,0xD5,0xD7,0xD4,0xCE,
0x00,0xCC,0xBD,0xB7,0x0F,0x96,0xCD,0x40,
0xCE,0x00,0xCB,0xBD,0xB2,0x1D,0x7E,0xA9,
0xFD,0x97,0xC2,0x7F,0x00,0xBF,0xCE,0x05,
0xF1,0x86,0x28,0xBD,0xB7,0x31,0xC1,0x0A,
0x26,0x06,0x20,0x2F,0x4F,0xBD,0xB7,0x31,
0xCB,0x30,0xE7,0x00,0x09,0x7C,0x00,0xBF,
0x7A,0x00,0xC2,0x26,0xEF,0x96,0xCE,0x2B,
0x0C,0x86,0x30,0xA1,0x01,0x26,0x21,0x7A,
0x00,0xBF,0x08,0x20,0xF6,0x86,0x3A,0x6C,
0x01,0xA1,0x01,0x26,0x13,0x08,0x7A,0x00,
0xBF,0x26,0xF4,0xDE,0xD4,0x08,0xDF,0xD4,
0x7C,0x00,0xBF,0xC6,0x31,0xF7,0x05,0xF1,
0x86,0x30,0xB7,0x05,0xE5,0xC6,0x2B,0xF7,
0x05,0xE2,0xD6,0xD5,0x96,0xD4,0x2A,0x0B,
0xC6,0x2D,0xF7,0x05,0xE2,0x4F,0x5F,0xD0,
0xD5,0x92,0xD4,0xC0,0x64,0x82,0x00,0x25,
0x05,0x7C,0x05,0xE5,0x20,0xF5,0xCB,0x64,
0x86,0x2F,0x4C,0xC0,0x0A,0x24,0xFB,0xB7,
0x05,0xE4,0xCB,0x3A,0xF7,0x05,0xE3,0x96,
0xBF,0x39,0x07,0x53,0xAA,0x7E,0xEB,0xFB,
0x9D,0xF9,0x05,0xAA,0x93,0xBA,0x47,0xC9,
0x80,0xE9,0x04,0xD5,0xC2,0x78,0x1F,0x49,
0xFF,0xCF,0x04,0x6B,0x9D,0xC5,0xAD,0xA8,
0x2B,0x70,0x04,0x36,0x8E,0x1B,0xC9,0xBF,
0x04,0x00,0x04,0x1B,0xBE,0xBC,0x20,0x00,
0x00,0x00,0x04,0x18,0x98,0x96,0x80,0x00,
0x00,0x00,0x04,0x14,0xF4,0x24,0x00,0x00,
0x00,0x00,0x04,0x11,0xC3,0x50,0x00,0x00,
0x00,0x00,0x04,0x0E,0x9C,0x40,0x00,0x00,
0x00,0x00,0x04,0x0A,0xFA,0x00,0x00,0x00,
0x00,0x00,0x04,0x07,0xC8,0x00,0x00,0x00,
0x00,0x00,0x04,0x04,0xA0,0x00,0x00,0x00,
0x00,0x00,0x00,0xAE,0xC0,0x31,0x43,0x25,
0x63,0x7A,0x02,0x57,0xDD,0xD0,0x46,0x7C,
0x64,0xBC,0x03,0x2C,0xA8,0x7F,0xEA,0x27,
0xA5,0x39,0x03,0x96,0xCF,0xB1,0x1E,0xAD,
0x45,0x39,0x03,0xCB,0xE6,0x95,0x94,0xBE,
0xC4,0x4D,0x03,0xE6,0xAB,0xCC,0x77,0x11,
0x84,0x61,0x03,0xE9,0xD6,0xBF,0x94,0xD5,
0xE5,0x7A,0x03,0xED,0x86,0x37,0xBD,0x05,
0xAF,0x6C,0x03,0xF0,0xA7,0xC5,0xAC,0x47,
0x1B,0x47,0x03,0xF3,0xD1,0xB7,0x17,0x58,
0xE2,0x19,0x03,0xF7,0x83,0x12,0x6E,0x97,
0x8D,0x4F,0x03,0xFA,0xA3,0xD7,0x0A,0x3D,
0x70,0xA3,0x03,0xFD,0xCC,0xCC,0xCC,0xCC,
0xCC,0xCC,0xBC,0x30,0x00,0x38,0x00,0x40,
0x00,0x28,0x00,0x30,0x00,0x80,0x00,0x78,
0x00,0x70,0xEF,0x1B,0xEF,0x2C,0xEE,0x25,
0xEE,0x25,0xB2,0x45,0xB2,0x3E,0xB4,0xAE,
0xB3,0xD3,0xEE,0x6E,0xE9,0xEE,0xE9,0xEA,
0xE9,0xFA,0xE9,0xF6,0xE9,0xF2,0xE9,0xFE,
0x00,0x88,0x00,0x29,0x16,0x20,0x00,0x21,
0x00,0x19,0x00,0x60,0x00,0x98,0x00,0x58,
0x00,0x90,0x00,0x68,0xD1,0x58,0xEB,0xAA,
0xEE,0x0E,0x93,0x68,0xCE,0x00,0xEE,0x06,
0xEB,0xDF,0xED,0xE7,0x00,0xB8,0xEF,0x3C,
0xED,0x27,0xED,0x27,0xEC,0x4D,0x00,0xA0,
0x00,0xA8,0x00,0xB0,0xEA,0x2C,0xEA,0x2C,
0xEA,0x2C,0xAF,0x7F,0xAF,0x82,0xE8,0x70,
0x20,0x26,0x3E,0xE1,0x1F,0x52,0x1F,0x60,
0x20,0x13,0x1F,0x16,0x3F,0x65,0x9D,0x5A,
0xE6,0x1E,0x92,0x04,0x13,0x20,0x01,0x21,
0x0C,0x20,0x20,0x66,0x1F,0xE4,0x9F,0x4E,
0xAA,0xB6,0xD1,0x58,0xC1,0x43,0x00,0x50,
0xF6,0xA2,0xD1,0x58,0xD1,0x58,0x0E,0xA2,
0x0F,0x21,0x0D,0x9F,0xE2,0x86,0x00,0x18,
0x00,0x10,0x00,0x00,0x04,0xA1,0x05,0x00,
0x03,0x00,0xD9,0xDA,0x1D,0x25,0xF7,0xC8,
0x10,0x00,0x08,0x00,0x1B,0x21,0xF1,0xBF,
0xF1,0x88,0x1C,0x21,0xE2,0x07,0x16,0x20,
0x17,0x20,0x09,0x00,0xF7,0xAA,0xC2,0x50,
0x00,0xC0,0x00,0x08,0x00,0x20,0x1A,0x00,
0x00,0x48,0xAF,0x82,0xD1,0x58,0xD1,0x58,
0xD1,0x58,0xD1,0x58,0x0B,0x00,0x1F,0x87,
0x40,0x03,0xBC,0x30,0xBC,0x30,0xE3,0x54,
0xBC,0x30,0x28,0x40,0xBC,0x30,0x00,0x10,
0xC1,0x2C,0xC2,0x2D,0xC2,0x28,0xC2,0x21,
0xC2,0x32,0xC2,0x39,0xC2,0x40,0xC2,0x44,
0xC2,0x48,0xC2,0x4C,0x40,0xAC,0xE9,0x18,
0x41,0xD5,0x41,0xFB,0x00,0x11,0xD1,0x59,
0xD1,0x59,0xD1,0x59,0xE3,0x4A,0x93,0x96,
0xC1,0x07,0xC1,0x1B,0x41,0x00,0x00,0x0B,
0x41,0x00,0x41,0x00,0x00,0x14,0x00,0x11,
0x3E,0x99,0x3E,0xB1,0x3E,0x84,0x3E,0x84,
0x3E,0xC1,0x3E,0x69,0x3E,0x56,0x0A,0xAA,
0xAB,0xBC,0x44,0x44,0x44,0x44,0x45,0x55,
0x55,0x58,0xED,0x99,0x77,0x66,0x62,0x22,
0x11,0x22,0x22,0x22,0x22,0x22,0x22,0x22,
0x11,0x11,0x11,0x13,0x00,0x00,0x06,0x61,
0x00,0x00,0x00,0x00,0x00,0x11,0x10,0x00,
0x06,0x22,0x04,0x02,0x25,0x00,0x50,0x44,
0x20,0x00,0x00,0x20,0x00,0x00,0x02,0x00,
0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x07,0x70,0x00,0x00,0x00,0xFF,0x8D,0x00,
0x01,0x0F,0x9F,0x00,0xDF,0x02,0x30,0xEE,
0x00,0xDF,0x04,0x97,0x06,0xD7,0x07,0x96,
0x7C,0x8A,0x80,0x97,0x08,0x96,0x53,0x97,
0x09,0x20,0x03,0x7F,0x00,0x08,0x8E,0x00,
0x28,0x7F,0x87,0xC1,0x86,0x38,0xB7,0x87,
0xC0,0x86,0x04,0xB7,0x87,0xC1,0x7F,0x87,
0xC0,0x86,0x03,0xB7,0x87,0xC6,0xCE,0x7F,
0xF5,0x6F,0x0A,0x09,0x26,0xFB,0xFE,0xBC,
0x67,0x86,0xFF,0x6D,0x0A,0x26,0x06,0xA7,
0x0A,0xA1,0x0A,0x27,0x05,0x09,0x26,0xF3,
0x20,0xFE,0xBD,0xAA,0x38,0xDF,0x3F,0x96,
0x40,0x26,0xF5,0x96,0x3F,0x85,0x0F,0x26,
0xEF,0xBD,0xAA,0x4C,0xDF,0x3B,0x09,0xDF,
0x43,0x9E,0x43,0x86,0x19,0x36,0xCE,0xBD,
0xCF,0xDF,0x14,0xDE,0x14,0xEE,0x00,0x2B,
0x1E,0xDF,0x16,0xDE,0x14,0xA6,0x02,0x08,
0x08,0x08,0xDF,0x14,0xDE,0x14,0xE6,0x00,
0x08,0xDF,0x14,0xDE,0x16,0xE7,0x00,0x08,
0xDF,0x16,0x4A,0x26,0xEF,0x20,0xDC,0xDE,
0x41,0xBD,0xB6,0xEB,0xCE,0x04,0x1A,0xBD,
0xB7,0x0F,0xDE,0x08,0xFF,0x04,0x22,0xBD,
0xFB,0x90,0xDE,0x18,0xA6,0x00,0xE6,0x01,
0xDE,0x1A,0xA7,0x00,0xE7,0x01,0xDE,0x18,
0xA6,0x02,0xE6,0x03,0xDE,0x1A,0xA7,0x03,
0xE7,0x04,0xDE,0x18,0xA6,0x04,0xE6,0x05,
0xEE,0x00,0x36,0x86,0x04,0xA7,0x01,0x86,
0xFF,0xA7,0x00,0x32,0x6F,0x01,0xA7,0x00,
0xE7,0x01,0xDE,0x18,0xBD,0xAA,0x3D,0xDF,
0x18,0xDE,0x1A,0xBD,0xAA,0x3D,0xDF,0x1A,
0x7A,0x00,0x1C,0x26,0xBD,0x7F,0x87,0xAA,
0xBD,0xE0,0x0C,0xBD,0xFB,0xD7,0x7F,0x00,
0x4B,0xBD,0xCB,0xBF,0xBD,0xC6,0xC8,0x86,
0x08,0xB7,0x87,0xB2,0xCE,0x88,0x02,0xBD,
0xC1,0x46,0xB6,0x04,0x22,0x26,0x07,0xB6,
0x87,0xA8,0x0E,0x7E,0xC5,0xB2,0x7C,0x00,
0xA5,0xDE,0x41,0xDF,0xA8,0xBD,0xCB,0x81,
0xCE,0xBD,0xAC,0xDF,0x14,0xA6,0x00,0x27,
0x08,0xBD,0xF2,0x2E,0xDE,0x14,0x08,0x20,
0xF2,0xCE,0x04,0x1A,0xDF,0x14,0x8D,0x14,
0x8D,0x12,0x8D,0x10,0x8D,0x17,0x8D,0x15,
0x8D,0x13,0x8D,0x11,0xBD,0xF3,0x4B,0xBD,
0xC6,0xC8,0x20,0xC3,0xDE,0x14,0xA6,0x00,
0x08,0xDF,0x14,0x8D,0x0E,0xDE,0x14,0xA6,
0x00,0x08,0xDF,0x14,0x8D,0x05,0x86,0x20,
0x7E,0xF2,0x2E,0x97,0x16,0x8D,0x04,0x96,
0x16,0x20,0x04,0x44,0x44,0x44,0x44,0x84,
0x0F,0x8B,0x30,0x81,0x39,0x23,0x02,0x8B,
0x07,0x7E,0xF2,0x2E,0x53,0x59,0x53,0x54,
0x45,0x4D,0x20,0x45,0x52,0x52,0x4F,0x52,
0x20,0x2D,0x20,0x4D,0x45,0x53,0x53,0x41,
0x47,0x45,0x20,0x4E,0x55,0x4D,0x42,0x45,
0x52,0x20,0x30,0x20,0x2D,0x20,0x00,0x04,
0x3E,0x24,0xB7,0x04,0x42,0xEE,0x00,0x6E,
0x00,0xB7,0x04,0x49,0xEE,0x00,0x39,0xB7,
0x04,0x4F,0xA6,0x00,0x39,0xB7,0x04,0x55,
0xE6,0x00,0x39,0xB7,0x04,0x5B,0xE7,0x00,
0x39,0xB7,0x04,0x61,0x6E,0x00,0x00,0x58,
0x01,0x7E,0x00,0x5B,0x01,0x7E,0x00,0x45,
0x04,0x06,0x0C,0x06,0x0C,0x00,0x60,0x02,
0x02,0x69,0x00,0x18,0x05,0xBE,0x31,0x05,
0x0C,0x06,0x00,0x72,0x04,0x01,0x00,0x01,
0x00,0x00,0x49,0x02,0x04,0x26,0x04,0x24,
0x02,0xBC,0x30,0x05,0x91,0x01,0xFF,0x05,
0xDE,0x02,0xFF,0xFF,0x05,0x8F,0x01,0xF0,
0x80,0x87,0x94,0xBC,0x30,0xF3,0x2C,0x87,
0x96,0xBC,0x30,0xFF,0x2C,0x87,0xB0,0xF5,
0x91,0x00,0x37,0x87,0xB2,0xF5,0xAC,0x9F,
0x37,0x87,0xA8,0xC4,0xDE,0x00,0x27,0x87,
0xAA,0xBC,0x30,0xE4,0x2C,0xFF,0xDE,0x51,
0xBD,0xB6,0xEB,0x4F,0xC6,0x08,0xDB,0x52,
0x99,0x51,0x97,0x51,0xD7,0x52,0x7E,0x00,
0x58,0x96,0x52,0x36,0x96,0x51,0x36,0x96,
0x4E,0x36,0x96,0x4D,0x36,0x86,0x01,0x36,
0xDE,0x51,0xA6,0x00,0xE6,0x01,0xCB,0x02,
0x89,0x00,0x20,0xDA,0xDE,0x51,0xA6,0x01,
0x36,0xA6,0x00,0x36,0x86,0x0E,0x36,0x08,
0x08,0xDF,0x51,0xBD,0xB1,0xFB,0x7E,0x00,
0x58,0x36,0x36,0x36,0x36,0x36,0x36,0xDE,
0x51,0xA6,0x01,0x36,0xA6,0x00,0x36,0x86,
0x0A,0x36,0x08,0x08,0xDF,0x51,0x7E,0x00,
0x58,0x4F,0x36,0x36,0xDE,0x51,0xA6,0x01,
0x36,0xA6,0x00,0x36,0x86,0x08,0x36,0x20,
0xE9,0x9F,0x00,0x86,0x18,0xBD,0xD0,0xE9,
0xDE,0x00,0x27,0x12,0x86,0x09,0xE6,0x18,
0x37,0x09,0x4A,0x26,0xF9,0xDE,0x51,0x08,
0x08,0xDF,0x51,0x7E,0x00,0x58,0xBD,0xBC,
0x30,0x9F,0x00,0x86,0x18,0xBD,0xD0,0xE9,
0xDE,0x00,0x27,0x13,0x31,0x86,0x08,0x33,
0xE7,0x11,0x08,0x4A,0x26,0xF9,0x86,0xC0,
0x9A,0x54,0x97,0x54,0x7E,0x00,0x58,0xBD,
0xBC,0x30,0xFE,0x04,0x3B,0x27,0x33,0x4F,
0xE6,0x00,0xC1,0x94,0x27,0x41,0xC1,0xB5,
0x27,0x20,0xC1,0xB6,0x27,0x18,0xC1,0xB2,
0x27,0x03,0xBD,0xBC,0x30,0xC6,0x03,0xFB,
0x04,0x3C,0xB9,0x04,0x3B,0xB7,0x04,0x3B,
0xF7,0x04,0x3C,0xDF,0x00,0x39,0xC6,0x09,
0x20,0xED,0xC6,0x03,0xEB,0x02,0xA9,0x01,
0x20,0xE5,0xDE,0x3D,0x27,0x18,0xFF,0x04,
0x39,0xA6,0x09,0x81,0xA5,0x26,0x0B,0xBD,
0xAA,0x39,0xFF,0x04,0x3B,0x20,0xB8,0xFE,
0x04,0x39,0xEE,0x03,0x26,0xE8,0x8D,0x04,
0xBD,0xD1,0x65,0x22,0xDE,0x41,0xFF,0x04,
0x39,0xFF,0x04,0x3B,0x39,0x8D,0xF5,0x30,
0xA6,0x00,0x81,0x0C,0x26,0x16,0xBD,0xD0,
0x87,0xDE,0x00,0x27,0x0F,0x86,0xA5,0xA1,
0x09,0x26,0x0C,0xFF,0x04,0x39,0xBD,0xAA,
0x39,0xFF,0x04,0x3B,0x7E,0x00,0x58,0xBD,
0xD1,0x6A,0x22,0x9F,0x00,0x30,0xA6,0x05,
0xA1,0x0E,0x26,0x63,0xA6,0x06,0xA1,0x0F,
0x26,0x5D,0xA6,0x07,0xA1,0x10,0x26,0x57,
0xA6,0x08,0xA1,0x11,0x26,0x51,0xEE,0x01,
0xEE,0x0B,0xBD,0xAA,0x3E,0xDF,0x02,0xCE,
0x00,0x00,0xDF,0x04,0x30,0xEE,0x0A,0xA6,
0x04,0x8A,0x04,0xA7,0x04,0xEE,0x0B,0xBD,
0xAA,0x3E,0xDF,0x06,0x30,0xEE,0x0C,0xBD,
0xAA,0x3E,0xDF,0x08,0xDE,0x02,0x9C,0x06,
0x27,0x20,0xDE,0x02,0xBD,0xB6,0xEB,0xBD,
0xAA,0x3B,0xDF,0x02,0xDE,0x06,0xBD,0xB7,
0x0F,0xBD,0xAA,0x3B,0xDF,0x06,0xDE,0x04,
0x27,0x02,0xDF,0x02,0xDE,0x06,0x9C,0x08,
0x26,0xE0,0x9E,0x00,0x7E,0x00,0x58,0xBD,
0xD1,0x6A,0x15,0x30,0x08,0xDF,0x02,0xDF
    ];
    
    this.ROM667 = [
0x04,0x20,0xB1,0xBD,0xCE,0x70,0x96,0x4B,
0x26,0x3D,0xDE,0x51,0x09,0xDF,0x51,0xBD,
0xCF,0x77,0x96,0x4B,0x26,0x31,0xBD,0xD1,
0x1D,0x96,0x08,0x27,0x13,0x81,0xA0,0x27,
0x2D,0x81,0x40,0x27,0xD6,0x81,0x50,0x26,
0x03,0x7E,0xBF,0x8D,0xBD,0xD1,0x6A,0x15,
0x30,0xA6,0x09,0x81,0x0B,0x27,0x13,0xEE,
0x0A,0xA6,0x04,0x84,0x7F,0xA7,0x04,0xBD,
0xAA,0x3E,0xBD,0xB7,0x0F,0x9E,0x00,0x7E,
0x00,0x58,0xEE,0x0C,0x20,0xF4,0x30,0xEE,
0x03,0xA6,0x00,0xE6,0x01,0xDF,0x02,0x30,
0xEE,0x06,0xA1,0x05,0x22,0xCE,0x25,0x04,
0xE1,0x06,0x22,0xC8,0x01,0x0F,0x97,0x08,
0xD7,0x09,0xDF,0x0A,0x6F,0x07,0x6F,0x08,
0xEB,0x0C,0xA9,0x0B,0x97,0x06,0xD7,0x07,
0xEE,0x0B,0xDF,0x04,0x0E,0x9C,0x06,0x27,
0x12,0xDE,0x02,0xE6,0x02,0x08,0xDF,0x02,
0xDE,0x04,0xE7,0x05,0x08,0xDF,0x04,0x9C,
0x06,0x26,0xEE,0xDE,0x0A,0x96,0x08,0xD6,
0x09,0x01,0x0F,0xA7,0x07,0xE7,0x08,0x86,
0x10,0xA7,0x04,0xEE,0x0B,0x86,0x20,0xA7,
0x02,0x0E,0x20,0x99,0x30,0xBD,0xB1,0xA2,
0x26,0x48,0x30,0xA6,0x03,0x26,0x43,0xE6,
0x04,0xC1,0x09,0x22,0x3D,0xCB,0x08,0x4F,
0x58,0xFB,0xC0,0xFF,0xB9,0xC0,0xFE,0x97,
0x0E,0xD7,0x0F,0xDE,0x0E,0x96,0x54,0x85,
0x02,0x26,0x0A,0x01,0x0F,0x6F,0x00,0x6F,
0x01,0x0E,0x7E,0x00,0x58,0xDE,0x51,0xEE,
0x01,0xDF,0x00,0xBD,0xD0,0x82,0x96,0x4B,
0x26,0x0D,0xDE,0x0E,0x01,0x0F,0x96,0x00,
0xA7,0x00,0x96,0x01,0xA7,0x01,0x0E,0x7E,
0xD1,0x60,0xBD,0xD1,0x6A,0x14,0x03,0xB9,
0xD6,0x53,0xC0,0xAA,0x5C,0x20,0xB8,0xDE,
0x51,0xA6,0x00,0x81,0x94,0x26,0x0B,0xCE,
0x03,0xB9,0x86,0x1E,0x6F,0x00,0x08,0x4A,
0x26,0xFA,0x39,0x96,0x55,0x85,0xC0,0x27,
0x07,0x96,0x54,0x8A,0x02,0x97,0x54,0x39,
0xBD,0xD1,0x65,0x23,0xBD,0xD1,0x59,0x96,
0x55,0x2A,0x01,0x39,0x30,0xA6,0x02,0x81,
0x18,0x27,0xF8,0xCE,0x0C,0x20,0xDF,0x56,
0x7E,0xAB,0x5C,0xCE,0x88,0x04,0x86,0x18,
0x36,0xBD,0xAA,0x7F,0xBD,0xC2,0x21,0xBD,
0xEF,0xA3,0xCE,0xBA,0x4A,0xBD,0xB6,0xEB,
0xCE,0x05,0xC3,0xBD,0xB7,0x0F,0x86,0x28,
0xB7,0x05,0xC2,0xDE,0x3B,0xA6,0x04,0x84,
0x10,0x26,0x02,0x8A,0x40,0x8A,0x80,0xA7,
0x04,0xEE,0x00,0x26,0xF0,0x07,0x01,0x0F,
0xCE,0x03,0xB9,0x6F,0x00,0x08,0x8C,0x04,
0x1A,0x26,0xF8,0xD6,0x55,0xC4,0xD6,0xCA,
0x10,0xD7,0x55,0x7F,0x00,0x6D,0x06,0x7F,
0x00,0x7B,0xBD,0xBF,0x5C,0xBD,0xF7,0xAA,
0xBD,0xF5,0x5A,0x7F,0x00,0x4B,0x32,0x96,
0x6D,0x84,0xEF,0x97,0x6D,0x39,0x01,0x0F,
0x32,0x97,0x59,0x32,0x97,0x5A,0xDE,0x47,
0xDF,0x45,0xDE,0x3B,0x6F,0x00,0x6F,0x01,
0x09,0xDF,0x43,0x9E,0x43,0x86,0x19,0x36,
0xDE,0x41,0xDF,0x3D,0xDF,0x4D,0xDF,0x4F,
0x7F,0x00,0x80,0x0E,0xCE,0x88,0x06,0xBD,
0xC1,0x46,0x7E,0x00,0x58,0xCE,0xE9,0x26,
0xBD,0xB6,0xEB,0xDE,0x3D,0x27,0x11,0xDF,
0x00,0xCE,0xC1,0xF3,0xBD,0xB6,0xEB,0xBD,
0xB2,0x45,0xDE,0x00,0xEE,0x03,0x26,0xEF,
0x7E,0x00,0x58,0x04,0x07,0x90,0x00,0x00,
0x00,0x00,0x00,0x4F,0x36,0x36,0x86,0x0E,
0x36,0xBD,0xF9,0xAA,0x9F,0x00,0x96,0x00,
0xD6,0x01,0xD0,0x46,0x92,0x45,0xC0,0xF4,
0x82,0x01,0x2A,0x02,0x4F,0x5F,0x37,0x36,
0x86,0x0E,0x36,0xBD,0xB1,0xFB,0x7E,0x00,
0x58,0xCE,0xE9,0xDA,0xFF,0x05,0xCC,0x39,
0xCE,0xE9,0xBA,0x20,0xF7,0xCE,0xE9,0xCA,
0x20,0xF2,0x86,0x20,0x9A,0x55,0x97,0x55,
0x39,0x86,0xDF,0x94,0x55,0x97,0x55,0x39,
0x86,0xEF,0x20,0xF7,0x86,0x10,0x20,0xEC,
0x86,0xFE,0x20,0xEF,0x86,0x01,0x20,0xE4,
0x86,0x15,0x36,0x96,0x55,0x84,0xFD,0x97,
0x55,0xBD,0x93,0xC4,0x96,0x55,0x85,0x02,
0x27,0xF7,0x32,0x39,0x86,0x15,0x36,0xDE,
0x9C,0xDF,0xD8,0xDE,0x9A,0xDF,0xEB,0x86,
0x02,0x97,0xED,0x7E,0xC2,0xCD,0x8D,0x00,
0x86,0x0D,0xBD,0xC3,0x3D,0x39,0x86,0x15,
0x36,0x86,0x01,0x97,0xED,0x7E,0xC2,0xCD,
0x86,0x15,0x36,0x96,0x8A,0x85,0x02,0x27,
0x06,0x85,0x20,0x27,0x2C,0x32,0x39,0xDE,
0x95,0xE6,0x00,0xD1,0x78,0x26,0x05,0xBD,
0xC3,0x74,0x20,0x1D,0xB6,0x06,0x0B,0x4A,
0x27,0x17,0x81,0x37,0x24,0x10,0x80,0x12,
0x22,0xFC,0x27,0x0D,0x36,0x86,0x20,0xBD,
0xC3,0x3D,0x32,0x4C,0x20,0xF4,0xBD,0xC2,
0x78,0x96,0xED,0x4A,0x26,0x05,0x86,0x20,
0xBD,0xC3,0x3D,0x32,0x39,0x96,0x90,0xD6,
0x97,0x81,0x20,0x27,0x25,0x81,0x21,0x27,
0x26,0x81,0x25,0x27,0x18,0x96,0xED,0x4A,
0x26,0x08,0xDE,0x9C,0xBD,0xB6,0xEB,0xBD,
0xB8,0x53,0x96,0x8F,0x85,0x08,0x27,0x24,
0xBD,0xD1,0x78,0x31,0x39,0xCE,0xC3,0x36,
0x20,0x08,0xCE,0xC3,0x1C,0x20,0x03,0xCE,
0xC3,0x29,0xA6,0x00,0x2B,0xD7,0x11,0x27,
0x05,0x08,0x08,0x08,0x20,0xF4,0xEE,0x01,
0xAD,0x00,0x31,0x39,0xBD,0xC2,0x88,0xBD,
0xD1,0x83,0x31,0x39,0x14,0xF3,0xAE,0x15,
0xF3,0xA8,0x12,0xF4,0x2B,0x1A,0xF4,0x34,
0xFF,0x07,0xDA,0x64,0x1C,0xDA,0xA0,0x1B,
0xDB,0xD1,0x00,0xDF,0xBB,0xFF,0x00,0xC4,
0x79,0x1A,0xC4,0x9D,0xFF,0xF6,0x04,0x35,
0x27,0x03,0x7C,0x04,0x36,0xDE,0x95,0x9C,
0x93,0x27,0x06,0xE6,0x00,0xD1,0x78,0x26,
0x04,0x8D,0x21,0xDE,0x95,0xD6,0x4B,0x26,
0x1A,0x08,0xDF,0x95,0xA7,0x00,0x81,0x0D,
0x26,0x05,0xC6,0x01,0xF7,0x06,0x0B,0x81,
0x1F,0x23,0x08,0x7C,0x06,0x0B,0x26,0x03,
0x7C,0x06,0x0B,0x39,0x36,0x86,0x13,0x36,
0x96,0x87,0x85,0x01,0x27,0x07,0xC6,0x31,
0xD7,0x4B,0x31,0x32,0x39,0xDE,0x98,0x09,
0x9C,0x95,0x27,0xF6,0x85,0x02,0x27,0x3D,
0xCE,0x02,0xB7,0xFF,0x02,0xB5,0xDE,0x98,
0xA6,0x00,0xFE,0x02,0xB5,0x85,0xE0,0x26,
0x10,0x81,0x0D,0x27,0x0C,0x8A,0x40,0xA7,
0x00,0x08,0x86,0x08,0xA7,0x00,0x08,0x86,
0x5F,0xA7,0x00,0x08,0xFF,0x02,0xB5,0xDE,
0x98,0x9C,0x95,0x27,0x05,0x08,0xDF,0x98,
0x20,0xD6,0xCE,0x02,0xB7,0xDF,0x98,0xFE,
0x02,0xB5,0x09,0xDF,0x95,0x96,0x8E,0x81,
0x01,0x26,0x07,0x96,0x7B,0x27,0x03,0xBD,
0xC4,0xCD,0x96,0x8E,0x81,0x0F,0x27,0x14,
0xDE,0x95,0xA6,0x00,0x81,0x0D,0x26,0x0C,
0xB6,0x04,0x63,0x27,0x07,0x86,0x0A,0x08,
0xA7,0x00,0xDF,0x95,0xCE,0xC4,0x1E,0xBD,
0xC4,0x31,0xDE,0x91,0xDF,0x98,0x09,0xDF,
0x95,0x7E,0xC3,0x82,0xD6,0x97,0xC1,0x1D,
0x26,0x0B,0x96,0x90,0x81,0x25,0x26,0x05,
0x86,0xFF,0x97,0x7B,0x39,0x96,0x87,0x8A,
0x80,0x97,0x87,0x7E,0xC3,0x74,0xE8,0x68,
0xF5,0xC0,0xC4,0x2C,0xF2,0xEA,0xC4,0x30,
0xC4,0x2C,0xDD,0x98,0x86,0x43,0x97,0x4B,
0x39,0x86,0x15,0x36,0x96,0x90,0x27,0x0B,
0x81,0x24,0x24,0x0C,0x80,0x1E,0x2E,0x01,
0x4F,0x4C,0x48,0xBD,0x04,0x3E,0x32,0x39,
0xCE,0x05,0x0C,0x6D,0x00,0x26,0x05,0xBD,
0xC4,0x2C,0x31,0x39,0xA1,0x05,0x27,0x05,
0xBD,0xAA,0x3D,0x20,0xEE,0x96,0x7C,0x36,
0x36,0x86,0x0E,0x36,0xA6,0x02,0x84,0x3F,
0xBD,0xA9,0xD5,0xEE,0x03,0xAD,0x06,0x31,
0x31,0x32,0x84,0x3F,0xBD,0xA9,0xD5,0x31,
0x39,0xBD,0xB0,0x8F,0x96,0xA4,0x27,0x11,
0x4A,0x27,0x13,0x4A,0x26,0x16,0xF7,0x04,
0x66,0x96,0x8A,0x8A,0x12,0x97,0x8A,0x20,
0x08,0xF7,0x04,0x64,0x20,0x03,0xF7,0x04,
0x65,0x7C,0x00,0xA4,0x39,0xBD,0xB0,0x8F,
0xF7,0x04,0x63,0x86,0x12,0xB7,0x00,0x8A,
0x39,0xD6,0x80,0x27,0x1D,0xD6,0x8E,0xC1,
0x04,0x26,0x17,0xB8,0x04,0x79,0x36,0xB6,
0x04,0x79,0x48,0xD6,0x8F,0x2A,0x05,0x33,
0x37,0x1B,0x20,0x02,0xAB,0x00,0xB7,0x04,
0x79,0x32,0x84,0x7F,0x39,0xDE,0x98,0xA6,
0x00,0x8D,0xE0,0xA7,0x00,0x9C,0x95,0x27,
0x03,0x08,0x20,0xF3,0x39,0xFF,0xBD,0xF7,
0xDC,0x96,0x76,0x81,0x80,0x26,0x01,0x39,
0x4D,0x2B,0x03,0x7E,0xC6,0x9A,0x81,0xEA,
0x26,0x04,0x86,0x0C,0x20,0x06,0x81,0xCA,
0x26,0x05,0x86,0x1E,0x7E,0xCB,0xEE,0x81,
0xED,0x26,0x1B,0x7E,0xF3,0x4B,0xB6,0x87,
0xAA,0x36,0x8A,0x40,0xB7,0x87,0xAA,0xBD,
0xE0,0x0C,0x5F,0xBD,0xDB,0xDD,0x7F,0x00,
0x4B,0x32,0xB7,0x87,0xAA,0x39,0x81,0xEA,
0x22,0x1D,0x81,0xC0,0x25,0x19,0x80,0xBF,
0x16,0x84,0x0F,0xC5,0x20,0x26,0x02,0x8B,
0x0A,0xCE,0x04,0x10,0x08,0x6D,0x00,0x26,
0xFB,0xA7,0x00,0x7F,0x04,0x19,0x39,0x81,
0xEB,0x27,0x03,0x7E,0xC6,0x9A,0xDE,0x74,
0xDF,0x72,0xD6,0x6B,0xC5,0x02,0x27,0x10,
0xD6,0x6D,0x2B,0x0D,0xCA,0x80,0xD7,0x6D,
0xF6,0x87,0xAA,0xCA,0x20,0xF7,0x87,0xAA,
0x39,0x7D,0x05,0x0A,0x26,0xFA,0x4F,0x97,
0x8F,0x97,0x6D,0xBD,0xF5,0x5A,0x86,0xF0,
0xB7,0x05,0x8F,0xDE,0x41,0xDF,0x62,0xDF,
0x64,0x86,0x5C,0x36,0x30,0x6F,0x03,0xB6,
0xC5,0x8E,0xA7,0x09,0xB6,0xC5,0x8F,0xA7,
0x0A,0x96,0x90,0x7E,0xAF,0x50,0xAA,0xB6,
0x4D,0x2B,0x06,0x7F,0x00,0xAE,0x7E,0x80,
0x33,0x81,0xEC,0x26,0x06,0xBD,0xC5,0x06,
0x7E,0xCB,0xBF,0x81,0xF6,0x26,0xEC,0xD6,
0x6B,0xC5,0x01,0x26,0x04,0xCA,0x20,0xD7,
0x6B,0x39,0x86,0x03,0xB7,0x05,0xA5,0xCE,
0x04,0x26,0xDF,0x49,0x7F,0x00,0xAE,0xBD,
0x93,0xC4,0x7F,0x00,0x6B,0xBD,0x83,0xB8,
0x4F,0x97,0x6B,0x97,0x54,0x97,0x8F,0x96,
0x55,0x84,0x3F,0x97,0x55,0xB6,0x04,0x72,
0x27,0x0D,0xBD,0xA9,0xD5,0xFE,0x04,0x70,
0xAD,0x00,0xBD,0xA9,0xD4,0x20,0xCB,0xCE,
0x0D,0x9F,0xDF,0x56,0xBD,0xAE,0x22,0x7F,
0x87,0xAA,0x86,0x98,0xB7,0x87,0xB2,0xBD,
0xAE,0xC7,0xDE,0x93,0x09,0xDF,0x93,0x01,
0x86,0x80,0x97,0xB2,0x96,0x6D,0x84,0x7F,
0x97,0x6D,0xBD,0x82,0x79,0xBD,0xC6,0x5C,
0x7F,0x00,0xAE,0x86,0x80,0xB7,0x87,0xAA,
0x86,0x9C,0xB7,0x87,0xB2,0x96,0x6B,0x8A,
0x02,0x97,0x6B,0x85,0x40,0x27,0x05,0xBD,
0xA8,0xB0,0x20,0x86,0x85,0x20,0x27,0x0E,
0xD6,0x55,0xCA,0x40,0xD7,0x55,0xBD,0x82,
0x2F,0xBD,0x96,0xBC,0x20,0xEC,0x85,0x08,
0x27,0xE8,0xBD,0x82,0x2F,0xBD,0x83,0x0D,
0x20,0xE0,0xDE,0x74,0x8C,0x01,0x1C,0x26,
0x05,0xCE,0x01,0x00,0x20,0x01,0x08,0xA6,
0x00,0xDF,0x74,0x39,0xD6,0x6B,0xC5,0x68,
0x27,0x01,0x39,0x96,0x6D,0x9A,0x6E,0xCE,
0x04,0x11,0xAA,0x00,0x27,0x19,0xC5,0x01,
0x27,0xF0,0x96,0x55,0x85,0x10,0x26,0x0F,
0xA6,0x00,0x27,0x0B,0x86,0x0D,0xBD,0xC5,
0x90,0x86,0x08,0x97,0x6A,0x20,0xDB,0xDE,
0x72,0x9C,0x74,0x27,0x08,0xBD,0xC6,0x4A,
0xBD,0xC5,0x90,0x20,0xC7,0xBD,0xF3,0x6F,
0x20,0xC2,0xDE,0x72,0x8C,0x01,0x1C,0x26,
0x05,0xCE,0x01,0x00,0x20,0x01,0x08,0x9C,
0x74,0x26,0x03,0x7E,0xC7,0x27,0xA7,0x00,
0xDF,0x72,0x39,0xFF,0xF6,0x87,0x94,0xC8,
0x04,0xC5,0x0C,0x26,0xF7,0x39,0x86,0xEC,
0xB7,0x87,0x94,0x8A,0x10,0xB7,0x87,0x94,
0xCE,0x02,0xFF,0xDF,0xA8,0x01,0x07,0x97,
0xA5,0xBD,0xCB,0xA1,0x7F,0x00,0xB0,0x01,
0x01,0x86,0xFC,0xB7,0x87,0x94,0x4F,0xB7,
0x87,0x96,0x97,0xAA,0x97,0xAB,0x86,0xFA,
0x7E,0xFE,0xB0,0x81,0x0A,0x26,0x26,0x96,
0xA8,0x26,0x09,0xD6,0xA9,0xC0,0x16,0x24,
0x10,0xD7,0xB0,0x39,0xD6,0xA9,0xC0,0x16,
0x29,0x02,0x82,0x00,0x97,0xA8,0xB7,0x87,
0x8C,0xD7,0xA9,0xF7,0x87,0x8E,0xB6,0x87,
0x96,0xB7,0x87,0x96,0x39,0x81,0x1E,0x26,
0x03,0x7E,0xC6,0xC8,0x81,0x0C,0x26,0x03,
0x7E,0xC6,0xBE,0x81,0x07,0x26,0x24,0xB6,
0x87,0xAA,0x36,0x5F,0x86,0x46,0x36,0x08,
0x09,0x4A,0x26,0xFB,0xB6,0x87,0xAA,0x88,
0x80,0xB7,0x87,0xAA,0x2B,0x04,0x5C,0x17,
0x20,0xED,0x32,0x4A,0x26,0xE8,0x32,0xB7,
0x87,0xAA,0x39,0x81,0x08,0x26,0x19,0xD6,
0xAB,0x96,0xAA,0x26,0x04,0xC1,0x0E,0x25,
0x06,0xC0,0x0E,0x82,0x00,0x20,0x46,0x86,
0x03,0xC6,0xE2,0x8D,0x40,0x7E,0xC6,0xE6,
0x81,0x0B,0x26,0x1D,0x96,0xA8,0xD6,0xA9,
0x81,0x03,0x27,0x10,0x4F,0xCB,0x16,0x99,
0xA8,0x81,0x03,0x27,0x04,0x7E,0xC7,0x04,
0x39,0x5D,0x27,0xF9,0xC6,0x14,0x4F,0x20,
0xF4,0x81,0x09,0x26,0xF3,0xD6,0xAB,0x96,
0xAA,0x27,0x0C,0x4A,0x27,0x18,0x4A,0x26,
0x27,0xC1,0xF4,0x25,0x15,0x20,0x21,0xC1,
0xFC,0x24,0x05,0xC6,0xFC,0x7E,0xC9,0x2B,
0x86,0x01,0xC6,0xF8,0x20,0xF7,0xC1,0xF8,
0x25,0xF6,0x86,0x02,0xC6,0xF4,0x20,0xED,
0x81,0x0D,0x27,0x04,0x81,0x1F,0x26,0x05,
0xBD,0xC6,0xEF,0x86,0x0D,0x81,0x80,0x26,
0x06,0xCE,0xCB,0x10,0x7E,0xC8,0x88,0xD6,
0xB3,0x27,0x5A,0xC1,0x05,0x22,0x56,0xCE,
0xCB,0x1C,0x81,0x40,0x27,0x3B,0x09,0xC1,
0x05,0x26,0x0C,0x81,0x23,0x27,0xE2,0x8D,
0x25,0x8D,0x23,0x8D,0x21,0x20,0x3E,0xCE,
0xCB,0x2D,0x8D,0x1A,0x8D,0x18,0xC1,0x03,
0x27,0x33,0x22,0x28,0x8D,0x10,0x8D,0x0E
    ];
    
    this.ROM668 = [
0x8D,0x0C,0x8D,0x0A,0xC1,0x02,0x27,0x13,
0x8D,0x04,0x8D,0x02,0x20,0x1F,0xBD,0xAA,
0x3D,0xA1,0x00,0x27,0x01,0x39,0x31,0x31,
0x08,0x20,0x6D,0xCE,0xCB,0x5D,0x8D,0xEE,
0x8D,0xEC,0x20,0x09,0xCE,0xCB,0x69,0x8D,
0xE5,0x8D,0xE3,0x8D,0xE1,0x81,0x1F,0x22,
0x0D,0x81,0x0D,0x27,0x03,0x7E,0xC6,0xEB,
0x7E,0xC6,0xD7,0x7E,0xC9,0x20,0x97,0xA5,
0x96,0xAE,0x2B,0x15,0x96,0xAB,0xD6,0xAA,
0xC1,0x03,0x2B,0x0D,0xB1,0x05,0x8F,0x25,
0x08,0xBD,0xC7,0xC0,0x96,0xB0,0x27,0x01,
0x39,0x96,0xA5,0x81,0x20,0x27,0xDC,0x5F,
0x48,0x59,0x48,0x59,0x9B,0xA5,0xC9,0x00,
0xBB,0xCB,0x1B,0x97,0xA7,0xF9,0xCB,0x1A,
0xD7,0xA6,0xB6,0x04,0x75,0x27,0x0F,0xD6,
0x7C,0x37,0xBD,0xA9,0xD5,0xFE,0x04,0x73,
0xAD,0x00,0x32,0xBD,0xA9,0xD5,0xDE,0xA6,
0xC6,0x05,0xD7,0xAC,0xF6,0x87,0x94,0xC4,
0x7F,0xD7,0xAF,0x7F,0x87,0x8C,0xA6,0x00,
0x26,0x14,0xF6,0x87,0x8C,0xCB,0x20,0xF7,
0x87,0x8C,0x08,0x7A,0x00,0xAC,0x26,0xEE,
0x7E,0xC9,0x1B,0xF7,0x87,0x8C,0x48,0x24,
0x15,0xD6,0xAF,0xF7,0x87,0x94,0x7D,0x00,
0xAE,0x2B,0x03,0x7D,0x87,0x94,0x7D,0x87,
0x8C,0xCA,0x80,0xF7,0x87,0x94,0xF6,0x87,
0x8C,0xCB,0x04,0xC5,0x1C,0x26,0xDC,0x7A,
0x00,0xAC,0x27,0x47,0x08,0xF6,0x87,0x8C,
0xCB,0x20,0xF7,0x87,0x8C,0xA6,0x00,0x26,
0x06,0x7E,0xC8,0xCF,0xF7,0x87,0x8C,0x44,
0x24,0x15,0xD6,0xAF,0xF7,0x87,0x94,0x7D,
0x00,0xAE,0x2B,0x03,0x7D,0x87,0x94,0x7D,
0x87,0x8C,0xCA,0x80,0xF7,0x87,0x94,0xF6,
0x87,0x8C,0xC5,0x1C,0x27,0x04,0xC0,0x04,
0x20,0xDA,0x08,0xF6,0x87,0x8C,0xCB,0x20,
0xF7,0x87,0x8C,0x7A,0x00,0xAC,0x27,0x03,
0x7E,0xC8,0x96,0x96,0xA8,0xB7,0x87,0x8C,
0xD6,0xAE,0x2B,0x10,0xD6,0xAB,0x4F,0xCB,
0x0E,0x99,0xAA,0x97,0xAA,0x8A,0xFC,0xD7,
0xAB,0xBD,0xCB,0xB8,0x39,0x00,0x00,0x5F,
0x00,0x00,0x00,0x07,0x00,0x07,0x00,0x14,
0x7F,0x14,0x7F,0x14,0x24,0x2A,0x7F,0x2A,
0x12,0x23,0x13,0x08,0x64,0x62,0x36,0x49,
0x56,0x20,0x50,0x00,0x04,0x02,0x01,0x00,
0x00,0x1C,0x22,0x41,0x00,0x00,0x41,0x22,
0x1C,0x00,0x22,0x14,0x7F,0x14,0x22,0x08,
0x08,0x3E,0x08,0x08,0x00,0xB0,0x70,0x00,
0x00,0x08,0x08,0x08,0x08,0x08,0x00,0x60,
0x60,0x00,0x00,0x20,0x10,0x08,0x04,0x02,
0x3E,0x51,0x49,0x45,0x3E,0x00,0x42,0x7F,
0x40,0x00,0x62,0x51,0x49,0x49,0x46,0x21,
0x41,0x49,0x4D,0x33,0x18,0x14,0x12,0x7F,
0x10,0x27,0x45,0x45,0x45,0x39,0x3C,0x4A,
0x49,0x49,0x31,0x01,0x71,0x09,0x05,0x03,
0x36,0x49,0x49,0x49,0x36,0x46,0x49,0x49,
0x29,0x1E,0x00,0x66,0x66,0x00,0x00,0x00,
0xB6,0x76,0x00,0x00,0x08,0x14,0x22,0x41,
0x00,0x14,0x14,0x14,0x14,0x14,0x00,0x41,
0x22,0x14,0x08,0x02,0x01,0x59,0x05,0x02,
0x3E,0x41,0x5D,0x59,0x0E,0x7C,0x12,0x11,
0x12,0x7C,0x7F,0x49,0x49,0x49,0x36,0x3E,
0x41,0x41,0x41,0x22,0x7F,0x41,0x41,0x22,
0x1C,0x7F,0x49,0x49,0x49,0x41,0x7F,0x09,
0x09,0x09,0x01,0x3E,0x41,0x41,0x51,0x71,
0x7F,0x08,0x08,0x08,0x7F,0x00,0x41,0x7F,
0x41,0x00,0x20,0x40,0x40,0x40,0x3F,0x7F,
0x08,0x14,0x22,0x41,0x7F,0x40,0x40,0x40,
0x40,0x7F,0x02,0x0C,0x02,0x7F,0x7F,0x04,
0x08,0x10,0x7F,0x3E,0x41,0x41,0x41,0x3E,
0x7F,0x09,0x09,0x09,0x06,0x3E,0x41,0x51,
0x21,0x5E,0x7F,0x09,0x19,0x29,0x46,0x26,
0x49,0x49,0x49,0x32,0x01,0x01,0x7F,0x01,
0x01,0x3F,0x40,0x40,0x40,0x3F,0x1F,0x20,
0x40,0x20,0x1F,0x7F,0x20,0x18,0x20,0x7F,
0x63,0x14,0x08,0x14,0x63,0x03,0x04,0x78,
0x04,0x03,0x61,0x51,0x49,0x45,0x43,0x7F,
0x7F,0x41,0x41,0x41,0x02,0x04,0x08,0x10,
0x20,0x41,0x41,0x41,0x7F,0x7F,0x04,0x02,
0x7F,0x02,0x04,0x80,0x80,0x80,0x80,0x80,
0x00,0x01,0x02,0x04,0x00,0x38,0x44,0x44,
0x7C,0x40,0x7F,0x28,0x44,0x44,0x38,0x38,
0x44,0x44,0x44,0x00,0x38,0x44,0x44,0x28,
0x7F,0x38,0x54,0x54,0x54,0x4C,0x00,0x08,
0x7E,0x09,0x09,0x98,0xA4,0xA4,0x98,0x7C,
0x7F,0x04,0x04,0x04,0x78,0x00,0x44,0x7D,
0x40,0x00,0x40,0x80,0x80,0x7D,0x00,0x7F,
0x10,0x10,0x28,0x44,0x00,0x01,0x7F,0x00,
0x00,0x7C,0x04,0x18,0x04,0x7C,0x04,0x7C,
0x04,0x04,0x78,0x38,0x44,0x44,0x44,0x38,
0xFC,0x28,0x44,0x44,0x38,0x38,0x44,0x44,
0x28,0xFC,0x7C,0x08,0x04,0x04,0x08,0x48,
0x54,0x54,0x54,0x24,0x00,0x04,0x3F,0x44,
0x44,0x3C,0x40,0x40,0x7C,0x40,0x0C,0x30,
0x40,0x30,0x0C,0x3C,0x40,0x30,0x40,0x3C,
0x44,0x28,0x10,0x28,0x44,0x9C,0xA0,0xA0,
0xA0,0x7C,0x44,0x64,0x54,0x4C,0x44,0x08,
0x08,0x36,0x41,0x41,0x00,0x00,0x7F,0x00,
0x00,0x41,0x41,0x36,0x08,0x08,0x08,0x04,
0x08,0x10,0x08,0x20,0x40,0xFE,0x40,0x20,
0xFF,0xFF,0xFF,0xFF,0xFF,0xF8,0xF0,0xE0,
0xD0,0x88,0xC8,0x90,0x4A,0x55,0x55,0x29,
0x00,0x7B,0x08,0x1C,0x2A,0x08,0x08,0x7C,
0xF8,0xF0,0xE0,0xD0,0x88,0x7D,0x08,0x08,
0x2A,0x1C,0x08,0x23,0x48,0x7E,0x49,0x49,
0x42,0x24,0x5A,0x24,0x24,0x24,0x5A,0x5B,
0x38,0x46,0x44,0x7E,0x40,0x7B,0x78,0x25,
0x22,0x25,0x78,0x5C,0x38,0x46,0x44,0x46,
0x38,0x7C,0x3C,0x43,0x42,0x43,0x3C,0x5D,
0x38,0x44,0x45,0x7C,0x40,0x7D,0x78,0x14,
0x13,0x14,0x78,0x5D,0x3A,0x40,0x40,0x40,
0x3A,0x7D,0x3D,0x40,0x40,0x40,0x3D,0x5B,
0x00,0x00,0x7D,0x00,0x00,0x5C,0x7D,0x09,
0x11,0x21,0x7D,0x5D,0x30,0x48,0x45,0x40,
0x20,0x86,0x3C,0xB7,0x87,0x95,0xBD,0xC6,
0xB4,0xB6,0x87,0x94,0x84,0x7F,0xB7,0x87,
0x94,0xD6,0xA5,0x26,0x0C,0x85,0x40,0x27,
0x08,0x84,0x3F,0xB7,0x87,0x94,0xBD,0xC6,
0xE6,0x96,0xA8,0xB7,0x87,0x8C,0x96,0xA9,
0xB7,0x87,0x8E,0x96,0xAA,0xD6,0xAB,0x7D,
0x00,0xA5,0x27,0x02,0x8A,0x40,0x8A,0xBC,
0xB7,0x87,0x94,0xF7,0x87,0x96,0x39,0x7F,
0x87,0x8D,0x7F,0x87,0x8F,0x86,0xFF,0xB7,
0x87,0x8C,0xB7,0x87,0x8E,0x86,0x2C,0xB7,
0x87,0x8D,0xB7,0x87,0x8F,0x96,0xA8,0xB7,
0x87,0x8C,0x96,0xA9,0xB7,0x87,0x8E,0x96,
0xAA,0x8A,0xFC,0xD6,0xAB,0x8D,0xD1,0xBD,
0xCC,0x2C,0xBD,0xA9,0x94,0x39,0xD6,0x71,
0x2A,0x0F,0x81,0x0C,0x26,0x02,0xCA,0x20,
0x81,0x1E,0x26,0x02,0xCA,0x40,0xD7,0x71,
0x39,0xCA,0x80,0xD7,0x71,0xBD,0xC6,0xB4,
0xF6,0x87,0x94,0xCA,0xFC,0xF7,0x87,0x94,
0xC6,0x2C,0xF7,0x87,0x95,0xD6,0xAE,0x26,
0x10,0xF6,0x87,0x97,0x2A,0x0B,0xF6,0x87,
0x96,0xF7,0x87,0x96,0x36,0xBD,0xC6,0xE6,
0x32,0xBD,0xC7,0xB8,0xD6,0x71,0xC5,0x08,
0x27,0x03,0xBD,0xF3,0x54,0xD6,0x71,0xC5,
0x20,0x27,0x05,0xBD,0xC6,0xBE,0x20,0x07,
0xC5,0x40,0x27,0x03,0xBD,0xC6,0xC8,0xD6,
0x71,0x7F,0x00,0x71,0xC5,0x10,0x27,0xB0,
0xBD,0xC5,0x06,0x7E,0xCB,0xBF,0xDF,0x98,
0xDF,0x93,0xBD,0xCC,0xB8,0x96,0x4B,0x39,
0xCE,0x00,0x00,0x8D,0xF1,0x9A,0x6E,0x26,
0x29,0xD6,0x00,0xC1,0xFF,0x27,0x10,0xCE,
0x00,0x01,0x8D,0xE2,0x26,0x1C,0x4F,0xDE,
0x00,0x8C,0x20,0x08,0x26,0x17,0x39,0x86,
0x01,0xD6,0x90,0xC1,0x23,0x26,0x19,0xDE,
0x85,0x09,0xDF,0x85,0xD6,0x6E,0xCA,0x80,
0xD7,0x6E,0x86,0xFD,0x39,0x4C,0xD6,0x00,
0xC8,0x40,0xD7,0x00,0xC5,0xE0,0x27,0x09,
0x43,0x81,0xFD,0x27,0x04,0xC6,0x3F,0xD7,
0x4B,0x39,0x8D,0xB4,0x4D,0x26,0xF2,0xDE,
0x9C,0xDF,0x98,0xBD,0xAA,0x3C,0xDF,0x93,
0xCE,0xCC,0xBF,0xBD,0xC4,0x31,0x39,0xE8,
0x68,0xF6,0x0F,0xC4,0x2C,0xC4,0x2C,0xC4,
0x2C,0xCD,0x9E,0xDF,0x35,0xC4,0x2C,0x8D,
0x8F,0x81,0x01,0x26,0xCC,0xDE,0x00,0x27,
0x63,0xDE,0x9A,0x96,0x00,0xD6,0x01,0xD0,
0x9B,0x92,0x9A,0xD7,0x9B,0x97,0x9A,0x2B,
0x02,0xDF,0x00,0xDE,0x9C,0xDF,0x98,0x96,
0x9C,0xD6,0x9D,0xDB,0x01,0x99,0x00,0xD7,
0x94,0x97,0x93,0xDE,0x00,0xDF,0x8B,0x8D,
0xB7,0x96,0x90,0x81,0x22,0x27,0x35,0xDE,
0x9A,0x27,0x31,0x2B,0x2F,0x09,0xDF,0x9A,
0x96,0x9A,0x27,0x14,0x4A,0x97,0x9A,0xCE,
0x02,0xB5,0xDF,0x98,0xCE,0x03,0xB4,0xDF,
0x93,0xBD,0xCC,0xB8,0x96,0x9A,0x20,0xEA,
0x96,0x9B,0xCE,0x02,0xB5,0xDF,0x98,0x9B,
0x99,0x97,0x94,0x96,0x98,0x89,0x00,0x97,
0x93,0xBD,0xCC,0xB8,0x39,0x96,0x9C,0xD6,
0x9D,0xDB,0x9B,0x99,0x9A,0x97,0x00,0xD7,
0x01,0x96,0x9A,0x85,0xA0,0x26,0x1B,0x8B,
0x40,0x97,0x9A,0xCE,0x00,0x9A,0xDF,0x98,
0x08,0xDF,0x95,0xBD,0xCD,0x87,0xDE,0x00,
0xDF,0x95,0xDE,0x9C,0xDF,0x98,0xBD,0xCD,
0x87,0x39,0x86,0x40,0x97,0x4B,0x39,0xCE,
0xCD,0xE6,0xDF,0x98,0x08,0xDF,0x95,0xBD,
0xCD,0x87,0xDE,0x9C,0xDF,0x98,0xBD,0xAA,
0x3C,0xDF,0x95,0xBD,0xCD,0x87,0x39,0xCE,
0xCD,0x8E,0xBD,0xC4,0x31,0x39,0xE8,0x68,
0xF5,0xC0,0xC4,0x2C,0xC4,0x2C,0xC4,0x2C,
0xC4,0x2C,0xDD,0x98,0xC4,0x2C,0x96,0xA4,
0x81,0x01,0x22,0x44,0x4C,0x97,0xA4,0x4A,
0x26,0x3B,0xBD,0xBF,0x02,0x96,0x4B,0x26,
0x34,0xDE,0x00,0xA6,0x00,0x81,0xB2,0x26,
0x12,0xA6,0x02,0x36,0xA6,0x01,0x36,0x36,
0xBD,0xB1,0xFB,0xCE,0x00,0x06,0xBD,0xB7,
0x0F,0x20,0x05,0x81,0xB6,0x26,0x09,0x08,
0xDF,0x02,0xCE,0x20,0x08,0xDF,0x00,0x39,
0xA6,0x02,0x97,0x01,0x86,0x40,0x97,0x00,
0x08,0x08,0x08,0xDF,0x02,0x39,0x20,0x08,
0x7F,0x00,0xA4,0xDE,0x02,0xA6,0x00,0x08,
0xDF,0x02,0xDE,0x98,0xA7,0x00,0x9C,0x93,
0x27,0x05,0x08,0xDF,0x98,0x20,0xEC,0x39,
0xBD,0xA9,0xE7,0x30,0xBD,0xB1,0xA2,0xEE,
0x03,0xDF,0x00,0x30,0xBD,0xAA,0x3A,0x35,
0x5F,0x96,0x4B,0x26,0x3C,0x96,0x00,0x26,
0x38,0x96,0x01,0x27,0x05,0xBD,0xE8,0x74,
0x20,0x36,0x5F,0x96,0x82,0x85,0x20,0x27,
0x28,0x96,0x70,0x85,0x10,0x26,0x0D,0xBD,
0xE0,0x0C,0xBD,0xDF,0x43,0xBD,0xCB,0xBF,
0x96,0x4B,0x26,0x1C,0x5F,0x96,0x82,0x85,
0x04,0x26,0x18,0x85,0x08,0x27,0x0A,0x5C,
0xDE,0x85,0xA6,0x00,0x81,0xFF,0x27,0x01,
0x5C,0x37,0x5F,0x37,0x37,0xBD,0xB1,0xFB,
0x7E,0xA9,0xFD,0xC6,0x03,0xDE,0x85,0xA6,
0x00,0x84,0xE0,0x81,0x20,0x27,0xEA,0x5C,
0x81,0x40,0x27,0xE5,0xC6,0x01,0x20,0xE1,
0x30,0x08,0xDF,0x00,0x7F,0x00,0x08,0x74,
0x00,0x08,0x74,0x00,0x08,0xDE,0x00,0xA6,
0x01,0x81,0x0C,0x27,0x17,0x81,0x0A,0x27,
0x1D,0x81,0x0B,0x27,0x46,0x81,0x08,0x27,
0x70,0x81,0x01,0x27,0x04,0x81,0x09,0x26,
0x61,0x7E,0xCF,0x33,0xDE,0x00,0xBD,0xAA,
0x3A,0xDF,0x00,0x96,0x08,0x39,0xEE,0x02,
0xA6,0x04,0x85,0x20,0x26,0x46,0x85,0x80,
0x26,0x44,0xDF,0x02,0xDE,0x00,0xDF,0x04,
0x86,0x08,0xDE,0x02,0xE6,0x05,0x08,0xDF,
0x02,0xDE,0x04,0xE7,0x02,0x08,0xDF,0x04,
0x4A,0x26,0xEF,0x86,0x0C,0xDE,0x00,0xA7,
0x01,0x20,0xC9,0xDF,0x04,0xEE,0x04,0xDF,
0x02,0xA6,0x00,0x85,0x40,0x26,0x17,0x86,
0x08,0xDE,0x02,0xE6,0x00,0x08,0xDF,0x02,
0xDE,0x04,0xE7,0x02,0x08,0xDF,0x04,0x4A,
0x26,0xEF,0x20,0xD7,0x20,0x4F,0x86,0x24,
0x97,0x4B,0x86,0x30,0x9A,0x08,0x97,0x08,
0x39,0xEE,0x02,0x86,0x80,0xA5,0x04,0x26,
0xED,0xEE,0x0B,0xA6,0x02,0x85,0x40,0x26,
0xE5,0xDE,0x00,0xEE,0x02,0xA6,0x0B,0xE6,
0x0C,0xCB,0x03,0x89,0x00,0xEE,0x07,0xDF,
0x02,0xDE,0x00,0xA7,0x04,0xE7,0x05,0xEE,
0x04,0x96,0x02,0xA7,0x00,0x96,0x03,0xA7,
0x01,0xDE,0x00,0x86,0x09,0xA7,0x01,0xDE,
0x00,0xBD,0xAA,0x3E,0xDF,0x00,0x86,0x20,
0x9A,0x08,0x97,0x08,0x39,0x86,0x10,0x9A,
0x08,0x97,0x08,0xDF,0x02,0xBD,0xCF,0xFA,
0xDF,0x04,0xDE,0x02,0xA6,0x04,0x85,0x04,
0x27,0x03,0x7E,0xCE,0x9C,0xEE,0x0B,0xC6,
0x40,0xE5,0x05,0x26,0x0F,0xBD,0xAA,0x3B,
0x9C,0x04,0x26,0xF5,0xDE,0x02,0x8A,0x04,
0xA7,0x04,0x20,0xE6,0x7E,0xCE,0xF6,0x86,
0x15,0x36,0xDE,0x00,0xA6,0x01,0x81,0x0B,
0x27,0x25,0x81,0x0A,0x27,0x0C,0x81,0x08,
0x27,0x2C,0x32,0x86,0xC0,0x9A,0x08,0x97,
0x08,0x39,0xEE,0x02,0xA6,0x04,0x85,0x40,
0x26,0x0D,0x85,0x20,0x27,0xEC,0x86,0x40,
0x9A,0x08,0x97,0x08,0xBD,0xCF,0xFA,0xDE,
0x51,0x86,0x8C,0xA1,0x00,0x26,0xDB,0x08,
0xDF,0x51,0x32,0x96,0x08,0x39,0x86,0x80,
0x9A,0x08,0x97,0x08,0xEE,0x02,0xA6,0x04,
0x85,0x80,0x27,0xE3,0xBD,0xA9,0xCC,0xDE,
0x00,0xEE,0x02,0x86,0x90,0xA7,0x04,0x6F,
0x05,0x86,0x48,0xA7,0x06,0x6F,0x07,0x6F,
0x08,0x96,0x45,0xD6,0x46,0xA7,0x0B,0xE7,
0x0C,0xCB,0x4D,0x89,0x00,0x97,0x45,0xD7,
0x46,0xEE,0x0B,0x6F,0x00,0x86,0x4D,0xA7,
0x01,0x86,0x60,0xA7,0x02,0xBD,0xA9,0x94,
0x20,0xAD,0xDF,0xC4,0xA6,0x05,0xE6,0x06
    ];
    
    this.ROM669 = [
0xDE,0x00,0xA7,0x06,0xE7,0x07,0xDE,0xC4,
0xA6,0x07,0xE6,0x08,0xDE,0x00,0xA7,0x08,
0xE7,0x09,0xEE,0x08,0x26,0x04,0xDE,0x00,
0x6C,0x09,0xDE,0xC4,0xEE,0x05,0xDF,0xBF,
0xDE,0xC4,0xA6,0x07,0x26,0x34,0xE6,0x08,
0x26,0x08,0x96,0xBF,0xD6,0xC0,0x58,0x49,
0x20,0x15,0xD7,0xC1,0x4F,0x5F,0x0D,0x79,
0x00,0xC1,0x24,0x04,0xDB,0xC0,0x99,0xBF,
0x58,0x49,0x78,0x00,0xC1,0x26,0xF3,0x58,
0x49,0x58,0x49,0xDE,0xC4,0xEB,0x0C,0xA9,
0x0B,0xDE,0x00,0xA7,0x04,0xE7,0x05,0xEE,
0x04,0x39,0xEE,0x07,0xD6,0xC0,0xDF,0xBF,
0x20,0xD0,0x32,0x97,0x5C,0x32,0x97,0x5D,
0xCE,0x00,0x54,0xA6,0x00,0x36,0x6F,0x00,
0x09,0x8C,0x00,0x4C,0x26,0xF5,0x86,0x02,
0x36,0x7E,0x00,0x5B,0xCE,0x00,0x4D,0x7E,
0xB7,0x0F,0xBD,0xA9,0xE7,0x20,0x12,0xBD,
0xA9,0xE7,0x30,0xBD,0xB1,0xA2,0x26,0x4E,
0xEE,0x03,0xDF,0x00,0x30,0xBD,0xAA,0x3A,
0x35,0x96,0x55,0x2A,0x04,0xDE,0x4D,0x26,
0x04,0xDE,0x3D,0x27,0x39,0x96,0x00,0xD6,
0x01,0xA1,0x07,0x22,0x0A,0x25,0x1A,0xE1,
0x08,0x22,0x04,0x25,0x14,0x20,0x22,0xEE,
0x03,0x27,0x23,0xA1,0x07,0x22,0xF8,0x25,
0x1D,0xE1,0x08,0x22,0xF2,0x25,0x17,0x20,
0x10,0xEE,0x05,0x27,0x11,0xA1,0x07,0x22,
0x0D,0x25,0xF6,0xE1,0x08,0x22,0x07,0x25,
0xF0,0xDF,0x00,0x7E,0xA9,0xFD,0x86,0x33,
0x97,0x4B,0xDE,0x41,0xDF,0x00,0x7E,0xA9,
0xFD,0x97,0x02,0xDE,0x00,0xA6,0x01,0x91,
0x02,0x27,0x0E,0x81,0x19,0x27,0x05,0xBD,
0xD1,0x1D,0x20,0xEF,0xDE,0x41,0xDF,0x00,
0x39,0x86,0x01,0x39,0x97,0x02,0xD7,0x03,
0xDE,0x00,0xA6,0x01,0x91,0x02,0x25,0x08,
0x91,0x03,0x23,0xED,0x81,0x19,0x27,0xE4,
0xBD,0xD1,0x1D,0x20,0xEB,0xDE,0x00,0xA6,
0x01,0x81,0x19,0x22,0x16,0xCE,0xD1,0x3E,
0xBD,0x04,0x51,0x2A,0x04,0xDE,0x00,0xE6,
0x02,0x4F,0xDB,0x01,0x99,0x00,0x97,0x00,
0xD7,0x01,0x39,0xBD,0xBC,0x30,0x01,0x05,
0x09,0x03,0x17,0x03,0x03,0x03,0x05,0x05,
0x09,0x09,0x09,0x03,0x03,0x03,0x01,0x01,
0x01,0x04,0x01,0x03,0x80,0x05,0x03,0x00,
0x39,0x86,0x40,0x9A,0x54,0x97,0x54,0x39,
0x8D,0xF7,0x7E,0x00,0x58,0xCE,0xD1,0x76,
0xDF,0x59,0x30,0xEE,0x00,0xA6,0x00,0x97,
0x4B,0x32,0x32,0x7E,0x00,0x58,0x39,0xFF,
0x7F,0x06,0x0A,0x86,0x15,0x36,0xBD,0xD1,
0xF7,0x32,0x39,0x86,0xFF,0xB7,0x06,0x0A,
0x86,0x15,0x36,0x96,0xED,0x81,0x01,0x27,
0x12,0xCE,0xD1,0xCE,0xDF,0xD6,0xCE,0x00,
0x02,0xDF,0xE9,0xBD,0xD1,0xD7,0xBD,0xD1,
0xF7,0x32,0x39,0xB6,0x05,0xF1,0x27,0x0C,
0xD6,0xD5,0x96,0xD4,0x2B,0x10,0xC0,0x07,
0x82,0x00,0x24,0x10,0xCE,0xD1,0xD2,0xDF,
0xD6,0xCE,0x00,0x05,0x20,0xDB,0xCB,0x03,
0x89,0x00,0x25,0xF0,0xCE,0xD1,0xD0,0xDF,
0xD6,0xCE,0x00,0x02,0x20,0xCB,0x46,0x41,
0x46,0x45,0x46,0x44,0x2E,0x46,0x44,0x4F,
0x97,0xE8,0x97,0xEE,0x97,0xEF,0x97,0xF0,
0x97,0xF1,0x97,0xF2,0x97,0xF3,0x97,0xF4,
0x97,0xF5,0x97,0xE6,0x97,0xE7,0xDE,0xE9,
0x26,0x04,0x86,0x47,0x97,0x4B,0x39,0x86,
0x15,0x36,0x5F,0xD7,0xF6,0xBD,0xD3,0x1E,
0x25,0x35,0x86,0x48,0x97,0x4B,0x32,0x39,
0x96,0x4B,0x27,0x01,0x39,0x86,0x15,0x36,
0x86,0xFF,0x97,0xF6,0xBD,0xD3,0x1E,0x25,
0x1E,0xBD,0xD3,0x3B,0xC5,0x7F,0x27,0x0C,
0xC5,0x3F,0x27,0x04,0x86,0x4D,0x20,0xDC,
0x86,0x4C,0x20,0xFA,0x96,0xEF,0x26,0xD6,
0x86,0x0D,0xBD,0xD3,0x2C,0x20,0xCF,0xCE,
0xD2,0x5F,0xC6,0x14,0xA1,0x00,0x26,0x04,
0xEE,0x01,0x6E,0x00,0x5A,0x27,0x05,0x08,
0x08,0x08,0x20,0xF0,0xBD,0xD2,0xEF,0x25,
0x04,0x86,0x49,0x20,0xAF,0xBD,0xD2,0x9B,
0xB6,0x00,0xF6,0x26,0xB7,0x20,0x9E,0x41,
0xD4,0xC6,0x44,0xD6,0x5B,0x45,0xD5,0x4C,
0x22,0xD4,0x89,0x58,0xD3,0xC5,0x2F,0xD3,
0xE0,0x4C,0xD3,0xDC,0x53,0xD4,0x00,0x28,
0xD4,0x0F,0x29,0xD4,0x4F,0x50,0xD3,0xD8,
0x2C,0xD3,0xBA,0x46,0xD3,0x78,0x2B,0xD3,
0x84,0x2D,0xD3,0x8F,0x2E,0xD3,0x9A,0x24,
0xD3,0x9F,0x43,0xD3,0xAB,0x20,0xD2,0x58,
0x54,0xD3,0xE4,0xD6,0xF0,0x26,0x30,0xD6,
0xF1,0x26,0x28,0x97,0xF0,0x8D,0x56,0x24,
0x26,0x8D,0x44,0x24,0x19,0x8D,0x27,0xBD,
0xD3,0x26,0x8D,0x49,0x24,0x19,0x8D,0x37,
0x24,0x0C,0x8D,0x1A,0x8D,0x68,0x8D,0x3D,
0x24,0x0D,0x8D,0x2B,0x25,0x09,0x96,0xF0,
0x27,0x05,0x39,0x86,0x4D,0x20,0x02,0x86,
0x4A,0x33,0x33,0x7E,0xD2,0x04,0xD6,0xF0,
0x58,0x25,0x10,0x58,0x25,0x0D,0xDB,0xF0,
0x25,0x09,0x58,0x25,0x06,0x1B,0x25,0x03,
0x97,0xF0,0x39,0x33,0x33,0x20,0xE0,0x81,
0x30,0x24,0x02,0x0C,0x39,0x81,0x3B,0x24,
0xFB,0x80,0x30,0x0D,0x39,0xDE,0xE6,0x9C,
0xE9,0x26,0x02,0x0C,0x39,0x96,0xE6,0xD6,
0xE7,0xDB,0xD7,0x99,0xD6,0x97,0x00,0xD7,
0x01,0xDE,0x00,0xA6,0x00,0xD6,0xEE,0x26,
0x03,0xBD,0xAA,0x5A,0x0D,0x39,0x8D,0xDD,
0x25,0x01,0x39,0x8D,0x01,0x39,0xDE,0xE6,
0x08,0xDF,0xE6,0x39,0xC6,0x15,0x37,0xBD,
0xC3,0x3D,0x96,0x4B,0x27,0x03,0x32,0x32,
0x32,0x32,0x39,0x5F,0x96,0xEF,0x8D,0x1F,
0x96,0xE8,0x8D,0x1B,0x96,0xF0,0x8D,0x17,
0x96,0xF1,0x8D,0x13,0x96,0xF2,0x8D,0x0F,
0x96,0xF3,0x8D,0x0B,0x96,0xF4,0x8D,0x07,
0x96,0xF5,0x8D,0x03,0xD7,0xF7,0x39,0x26,
0x02,0x58,0x39,0x58,0x5C,0x39,0x8D,0xD3,
0xC5,0x1F,0x27,0x04,0x33,0x33,0x20,0x46,
0xD6,0xF0,0x26,0x03,0x5C,0xD7,0xF0,0x39,
0x8D,0xC1,0xC5,0x30,0x26,0x38,0x73,0x00,
0xF1,0x7E,0xD2,0x58,0x8D,0xB5,0xC5,0x3D,
0x26,0x2C,0x73,0x00,0xF2,0x20,0xF2,0x8D,
0xAA,0xC5,0x3D,0x26,0x21,0x73,0x00,0xF3,
0x20,0xE7,0x86,0x50,0x7E,0xD2,0x04,0x8D,
0x9A,0xC5,0x3F,0x26,0x11,0x86,0x24,0x97,
0xF4,0x20,0xD6,0x8D,0x8E,0xC5,0x31,0x26,
0x05,0x73,0x00,0xF5,0x20,0xCB,0x86,0x4D,
0x20,0xE2,0xBD,0xD3,0x3B,0xC5,0x3F,0x27,
0xC0,0x86,0x4F,0x20,0xD7,0x86,0x20,0x97,
0x00,0x8D,0x9B,0x96,0x00,0xBD,0xD3,0x2C,
0x7A,0x00,0xF0,0x26,0xF6,0x7E,0xD2,0x58,
0x86,0x0C,0x20,0xEB,0x86,0x0A,0x20,0xE7,
0x86,0x0D,0x20,0xE3,0xBD,0xD3,0x66,0x86,
0x20,0x97,0x00,0x96,0xF0,0xB0,0x06,0x0B,
0x25,0x0A,0x26,0x04,0x97,0xF0,0x20,0xDD,
0x97,0xF0,0x20,0xCF,0x86,0x52,0x20,0x4C,
0xBD,0xD2,0xFD,0x25,0x06,0x86,0x01,0x97,
0xEF,0x20,0xCA,0x86,0x4E,0x20,0x3D,0xBD,
0xD3,0x66,0xBD,0xD2,0xFD,0x25,0x04,0x86,
0x48,0x20,0x31,0x86,0x04,0x90,0xE8,0x27,
0x29,0x7C,0x00,0xE8,0xDE,0xE1,0xDF,0xE4,
0xDE,0xDE,0xDF,0xE1,0xDE,0xDB,0xDF,0xDE,
0xDE,0xE6,0xDF,0xDB,0x96,0xE0,0x97,0xE3,
0x96,0xDD,0x97,0xE0,0x96,0xDA,0x97,0xDD,
0x96,0xF0,0x97,0xDA,0x7F,0x00,0xF0,0x7E,
0xD2,0x58,0x86,0x4C,0x7E,0xD2,0x04,0xBD,
0xD3,0x3B,0xC5,0x3F,0x26,0x30,0xC5,0x40,
0x27,0xF0,0x7A,0x00,0xDA,0x27,0x06,0xDE,
0xDB,0xDF,0xE6,0x20,0xE2,0x7A,0x00,0xE8,
0x26,0x02,0x20,0xDB,0xDE,0xDE,0xDF,0xDB,
0xDE,0xE1,0xDF,0xDE,0xDE,0xE4,0xDF,0xE1,
0x96,0xDD,0x97,0xDA,0x96,0xE0,0x97,0xDD,
0x96,0xE3,0x97,0xE0,0x20,0xC1,0x7E,0xD3,
0xB6,0xBD,0xD3,0x66,0xDE,0xE6,0xDF,0x04,
0x86,0xFF,0x97,0xEE,0xBD,0xD3,0x1E,0x25,
0x05,0x86,0x46,0x7E,0xD2,0x04,0x81,0x22,
0x27,0x05,0xBD,0xD3,0x2C,0x20,0xED,0xBD,
0xD2,0xFD,0x24,0x09,0x81,0x22,0x26,0x05,
0xBD,0xD3,0x1E,0x20,0xED,0x7A,0x00,0xF0,
0x26,0x06,0x7F,0x00,0xEE,0x7E,0xD2,0x58,
0xDE,0x04,0xDF,0xE6,0x20,0xCE,0x96,0xF6,
0x27,0x05,0x86,0x4B,0x7E,0xD2,0x04,0x96,
0xED,0x81,0x02,0x27,0x04,0x86,0x51,0x20,
0xF3,0xBD,0xD3,0x3B,0xC5,0x0F,0x27,0x04,
0x86,0x4D,0x20,0xE8,0xDE,0xEB,0xDF,0x02,
0xC5,0x10,0x27,0x09,0xDE,0xEB,0x27,0x35,
0x7F,0x00,0x04,0x20,0x13,0x96,0xEB,0x27,
0x04,0x86,0x57,0x20,0xCF,0x96,0xF0,0x26,
0x01,0x4C,0x90,0xEC,0x25,0xF3,0x97,0x04,
0x7F,0x00,0x00,0x7F,0x00,0x01,0x8D,0x1C,
0x24,0x05,0xBD,0xD3,0x2C,0x20,0xF7,0x96,
0x04,0x27,0x0A,0x86,0x20,0xBD,0xD3,0x2C,
0x7A,0x00,0x04,0x20,0xF4,0x4F,0x97,0xF1,
0x97,0xF0,0x32,0x39,0xDE,0x02,0x26,0x02,
0x0C,0x39,0x09,0xDF,0x02,0x96,0x00,0xD6,
0x01,0xDB,0xD9,0x99,0xD8,0x97,0x05,0xD7,
0x06,0xDE,0x05,0xA6,0x00,0x0D,0xDE,0x00,
0x08,0xDF,0x00,0x39,0x96,0xF6,0x27,0x05,
0x86,0x4B,0x7E,0xD2,0x04,0x96,0xED,0x81,
0x01,0x27,0x04,0x86,0x51,0x20,0xF3,0xBD,
0xD3,0x3B,0xC5,0x07,0x27,0x04,0x86,0x4D,
0x20,0xE8,0x96,0xF1,0x27,0x0E,0x86,0x0A,
0xBD,0xD6,0x07,0x16,0x81,0x02,0x24,0x25,
0xC6,0x02,0x20,0x21,0xD6,0xF0,0x26,0x09,
0x86,0x02,0xBD,0xD6,0x07,0xC6,0x02,0x20,
0x14,0x7F,0x00,0xF0,0x86,0x0B,0x11,0x24,
0x04,0x86,0x4A,0x20,0xBD,0x5C,0xD7,0x06,
0x17,0x8D,0x6C,0xD6,0x06,0x7F,0x00,0xF2,
0x7F,0x00,0xF1,0xD7,0x06,0xB6,0x05,0xF2,
0x81,0x2D,0x27,0x05,0xF6,0x06,0x0A,0x26,
0x03,0xBD,0xD3,0x2C,0xB6,0x05,0xF1,0xBD,
0xD3,0x2C,0x86,0x2E,0xBD,0xD3,0x2C,0x7A,
0x00,0x06,0xCE,0x05,0xF0,0xDF,0x00,0xA6,
0x00,0xBD,0xD3,0x2C,0x7A,0x00,0x06,0x27,
0x05,0xDE,0x00,0x09,0x20,0xEF,0x86,0x45,
0xBD,0xD3,0x2C,0xB6,0x05,0xE2,0xBD,0xD3,
0x2C,0xB6,0x05,0xE5,0xF6,0x06,0x0A,0x27,
0x0D,0x81,0x30,0x26,0x09,0xB6,0x05,0xE4,
0x81,0x30,0x26,0x08,0x20,0x09,0xBD,0xD3,
0x2C,0xB6,0x05,0xE4,0xBD,0xD3,0x2C,0xB6,
0x05,0xE3,0xBD,0xD3,0x2C,0x32,0x39,0xF6,
0x05,0xF1,0x26,0x21,0xD7,0xD4,0xD7,0xD5,
0xF7,0x05,0xF6,0x86,0x0F,0xC6,0x30,0xCE,
0x05,0xE3,0xE7,0x00,0x4A,0x27,0x03,0x08,
0x20,0xF8,0xC6,0x20,0xF7,0x05,0xF2,0xC6,
0x2B,0xF7,0x05,0xE2,0x39,0xBD,0xB9,0x49,
0xB7,0x05,0xF6,0x86,0x0C,0xB0,0x05,0xF6,
0x27,0x10,0xCE,0x05,0xE6,0xC6,0x30,0xE7,
0x00,0x4A,0x27,0x03,0x08,0x20,0xF8,0xB6,
0x05,0xF6,0xD6,0xF2,0x27,0x0C,0xF6,0x05,
0xF2,0xC1,0x20,0x26,0x05,0xC6,0x2B,0xF7,
0x05,0xF2,0x39,0x96,0xF6,0x27,0x05,0x86,
0x4B,0x7E,0xD2,0x04,0x96,0xED,0x81,0x01,
0x27,0x04,0x86,0x51,0x20,0xF3,0x4F,0xB7,
0x05,0xF3,0xB7,0x05,0xF6,0xB7,0x05,0xF4,
0xB7,0x05,0xF5,0xB7,0x05,0xF7,0xB7,0x05,
0xF8,0x96,0xF0,0x27,0x08,0xB7,0x05,0xF4,
0x7F,0x00,0xF0,0x20,0x11,0x96,0xF1,0x27,
0x08,0xB7,0x05,0xF7,0x7F,0x00,0xF1,0x20,
0x05,0x86,0x01,0xB7,0x05,0xF4,0xBD,0xD2,
0xFD,0x24,0x63,0x81,0x2E,0x26,0x5F,0xB7,
0x05,0xF3,0xBD,0xD3,0x26,0xBD,0xD2,0xFD,
0x24,0x54,0x16,0xC1,0x44,0x26,0x0A,0x86,
0x01,0xB7,0x05,0xF5,0xBD,0xD3,0x26,0x20,
0x45,0xBD,0xD2,0xEF,0x24,0x1D,0xBD,0xD3,
0x1E,0xBD,0xD2,0xEF,0xBD,0xD2,0x9B,0xBD,
0xD2,0xFD,0x81,0x44,0x26,0x30,0xBD,0xD3,
0x26,0x96,0xF0,0xB7,0x05,0xF5,0x7F,0x00,
0xF0,0x20,0x23,0x81,0x46,0x26,0x1F,0xBD,
0xD3,0x26,0xBD,0xD2,0xFD,0x25,0x05,0x86,
0x4D,0x7E,0xD2,0x04,0x81,0x44,0x27,0x06,
0x86,0xFF,0x97,0xF1,0x20,0x08,0xBD,0xD3,
0x26,0x86,0xFF,0xB7,0x05,0xF8,0xB6,0x05,
0xF8,0x27,0x0D,0x86,0x0C,0xBD,0xD6,0x07,
0xBD,0xD9,0x73,0x24,0x09,0x7E,0xD9,0x93,
0xB6,0x05,0xF5,0xBD,0xD9,0x17,0xBD,0xD8,
0xEB,0xBD,0xD9,0x98,0xB6,0x05,0xF6,0x26,
0x0E,0x86,0x01,0xB7,0x05,0xFC,0xCE,0xD7,
0x36,0xFF,0x05,0xFD,0x20,0x26,0x30,0xB6,
0x05,0xFA,0x26,0xED,0x7C,0x05,0xF9,0x8D,
0x4E,0x2B,0x0D,0xB6,0x05,0xF6,0xB7,0x05,
0xFC,0x96,0x00,0xB7,0x06,0x00,0x20,0x06,
0xB6,0x05,0xF9,0xB7,0x05,0xFC,0xCE,0x05,
0xF1,0xFF,0x05,0xFD,0xB6,0x05,0xF7,0x26,
0x37,0xB6,0x05,0xFC,0xF6,0x06,0x00,0x1B,
0xF6,0x06,0x09,0x1B,0x97,0x00,0x96,0xF3,
0x26,0x0C,0x96,0xF2,0x26,0x08,0xB6,0x05,
0xF2,0x27,0x03,0x7C,0x00,0x00,0xB6,0x05,
0xF4,0x90,0x00,0x24,0x05,0x86,0x57,0x7E,
0xD2,0x04,0xB7,0x05,0xFB,0x20,0x09,0xB6,
0x05,0xF9,0xB0,0x05,0xF6,0x97,0x00,0x39,
0xBD,0xD9,0x73,0x24,0x03,0x7E,0xD9,0x93,
0xB6,0x05,0xF6,0x26,0x0F,0xB6,0x05,0xF8,
0x27,0x02,0x20,0x5D,0xB6,0x05,0xF5,0xB7,
0x06,0x01,0x20,0x55,0xB6,0x05,0xFA,0x26,
0x26,0x7C,0x05,0xF9,0x8D,0xD1,0x24,0xEC,
0x40,0xB7,0x06,0x02,0x97,0x00,0xF6,0x05,
0xF9,0xCE,0x05,0xF1,0x09,0x5A,0x26,0xFC,
0xFF,0x06,0x03,0xF6,0x05,0xF5,0x27,0x31,
0xD0,0x00,0xF7,0x06,0x05,0x20,0x2A,0xB6,
0x05,0xF9,0x4A,0xB7,0x06,0x01,0xB6,0x05,
0xF6,0xB7,0x06,0x02,0xCE,0x05,0xF1,0xFF,
0x06,0x03,0xB6,0x05,0xF8,0x26,0x12,0xB6,
0x05,0xF5,0x27,0x0D,0xB0,0x06,0x01,0x2B
    ];
    
    this.ROM670 = [
0x08,0xB0,0x05,0xF6,0x2B,0x03,0xB7,0x06,
0x05,0xB6,0x05,0xFB,0x27,0x0A,0x86,0x20,
0xBD,0xD3,0x2C,0x7A,0x05,0xFB,0x26,0xF6,
0x96,0xF4,0x27,0x03,0xBD,0xD3,0x2C,0xB6,
0x05,0xF2,0x27,0x03,0xBD,0xD3,0x2C,0xB6,
0x05,0xFF,0x26,0x1E,0xFE,0x05,0xFD,0xA6,
0x00,0xBD,0xD3,0x2C,0x7A,0x05,0xFC,0x27,
0x09,0xFE,0x05,0xFD,0x09,0xFF,0x05,0xFD,
0x20,0xED,0xB6,0x06,0x00,0xBD,0xD8,0xCE,
0x20,0x47,0xFE,0x05,0xFD,0xA6,0x00,0x09,
0xFF,0x05,0xFD,0xBD,0xD3,0x2C,0x7A,0x05,
0xFC,0x27,0x11,0x7A,0x05,0xFF,0x26,0xEA,
0x86,0x2C,0xBD,0xD3,0x2C,0x86,0x03,0xB7,
0x05,0xFF,0x20,0xDE,0xB6,0x06,0x00,0x27,
0x20,0x7A,0x05,0xFF,0x27,0x0F,0x86,0x30,
0xBD,0xD3,0x2C,0x7A,0x06,0x00,0x27,0x11,
0x7A,0x05,0xFF,0x26,0xF1,0x86,0x2C,0xBD,
0xD3,0x2C,0x86,0x03,0xB7,0x05,0xFF,0x20,
0xE5,0xB6,0x05,0xF3,0x27,0x2D,0xF6,0x06,
0x0A,0x27,0x05,0xF6,0x06,0x02,0x27,0x23,
0xBD,0xD3,0x2C,0xB6,0x06,0x01,0x8D,0x26,
0xB6,0x06,0x02,0x27,0x16,0xFE,0x06,0x03,
0xA6,0x00,0x09,0xFF,0x06,0x03,0xBD,0xD3,
0x2C,0x7A,0x06,0x02,0x26,0xEF,0xB6,0x06,
0x05,0x8D,0x0B,0x4F,0x97,0xF2,0x97,0xF3,
0x97,0xF4,0x97,0xF5,0x32,0x39,0x26,0x01,
0x39,0xB7,0x06,0x06,0x32,0xB7,0x06,0x07,
0x32,0xB7,0x06,0x08,0x86,0x30,0xBD,0xD3,
0x2C,0x7A,0x06,0x06,0x26,0xF6,0xFE,0x06,
0x07,0x6E,0x00,0x4F,0xB7,0x05,0xFB,0xB7,
0x05,0xFC,0xB7,0x05,0xFF,0xB7,0x06,0x09,
0xB7,0x06,0x02,0xB7,0x06,0x00,0xB7,0x06,
0x01,0xB7,0x06,0x05,0x96,0xF2,0x26,0x0E,
0x96,0xF3,0x26,0x0A,0xB6,0x05,0xF2,0x81,
0x2D,0x27,0x03,0x7F,0x05,0xF2,0x39,0x97,
0x06,0xF6,0x05,0xF1,0x26,0x0A,0xBD,0xD6,
0x07,0xB7,0x05,0xF9,0xB7,0x05,0xFA,0x39,
0xBD,0xD9,0x73,0x24,0x03,0x7E,0xD9,0x91,
0x96,0xD4,0xD6,0xD5,0xCB,0x01,0x89,0x00,
0xDB,0x06,0x89,0x00,0x2B,0x1E,0x26,0x26,
0x5D,0x26,0x1E,0x96,0xCE,0x2A,0x15,0x86,
0x01,0xB7,0x05,0xF6,0x86,0x31,0xB7,0x05,
0xF1,0xDE,0xD4,0x08,0xDF,0xD4,0x8D,0x1B,
0xBD,0xD6,0x4A,0x39,0x7F,0x05,0xF1,0x20,
0xBD,0x86,0x0C,0x10,0x24,0x04,0x86,0x0C,
0x20,0x01,0x17,0xBD,0xD6,0x07,0x8D,0x03,
0x25,0x1F,0x39,0x7F,0x05,0xFA,0xD6,0xD5,
0x96,0xD4,0x2A,0x09,0xB7,0x05,0xFA,0x43,
0x53,0xCB,0x01,0x89,0x00,0x26,0x08,0x5D,
0x2B,0x05,0xF7,0x05,0xF9,0x0C,0x39,0x0D,
0x39,0x32,0x32,0x86,0x53,0x7E,0xD2,0x04,
0x96,0xF5,0x26,0x01,0x39,0xB6,0x05,0xFA,
0x2A,0x01,0x39,0xF6,0x05,0xF9,0x4F,0x5C,
0x5A,0x5A,0x5A,0x27,0x05,0x2B,0x03,0x4C,
0x20,0xF6,0x4D,0x26,0x01,0x39,0x5C,0x5C,
0x5C,0xF7,0x05,0xFF,0xB7,0x06,0x09,0x39,
0x86,0x18,0x36,0x9F,0x00,0xBD,0xAE,0x22,
0xBD,0xAE,0xC7,0x9F,0x00,0x86,0x10,0x97,
0x8E,0xBD,0xAB,0xB3,0xBD,0xAF,0x1D,0xBD,
0xAB,0x67,0xCE,0x07,0x21,0xDF,0x56,0x30,
0x08,0xDF,0x00,0xBD,0xCE,0x74,0x4D,0x27,
0xD7,0x7E,0xE8,0x6C,0x86,0x18,0x36,0x30,
0xBD,0xAA,0x38,0xDF,0x00,0xCE,0x04,0xA1,
0xDF,0x56,0xBD,0xE1,0x2F,0x31,0x39,0x86,
0x08,0x16,0xDB,0x92,0xD7,0x03,0xD6,0x91,
0xC9,0x00,0xD7,0x02,0xE6,0x00,0x08,0xDF,
0x00,0xDE,0x02,0xE7,0x00,0x08,0xDF,0x02,
0xDE,0x00,0x4A,0x26,0xEF,0x39,0x4E,0x45,
0x57,0x20,0x20,0x20,0x20,0x20,0x42,0x49,
0x4E,0x41,0x52,0x59,0x20,0x20,0x41,0x53,
0x43,0x49,0x49,0x20,0x20,0x20,0x4C,0x41,
0x53,0x54,0x20,0x20,0x20,0x20,0x20,0x20,
0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
0x50,0x52,0x4F,0x47,0x52,0x41,0x4D,0x20,
0x20,0x20,0x44,0x41,0x54,0x41,0x20,0x20,
0x20,0x20,0x20,0x20,0x53,0x45,0x43,0x52,
0x45,0x54,0x20,0x20,0xBD,0xDB,0xD1,0x96,
0x4B,0x26,0x10,0x96,0x81,0x85,0x04,0x26,
0x0A,0x96,0x82,0x85,0x1C,0x26,0x05,0x86,
0x34,0x97,0x4B,0x39,0xBD,0xDB,0xC8,0xCE,
0xDA,0x1E,0xBD,0xD9,0xFF,0xCE,0xDA,0x3E,
0x86,0x0A,0xC6,0x10,0xBD,0xDA,0x02,0xCE,
0xDA,0x3E,0x86,0x08,0xC6,0x1A,0xBD,0xDA,
0x02,0xBD,0xDD,0x53,0x7F,0x00,0x82,0x39,
0x96,0xA4,0x26,0x13,0x43,0x97,0xA4,0xBD,
0xB0,0x8F,0x29,0x06,0x25,0x04,0xDF,0x10,
0x26,0x04,0xC6,0x1B,0xD7,0x4B,0x39,0xB6,
0x87,0x98,0x85,0x10,0x27,0x04,0xC6,0x38,
0x20,0xF2,0x96,0x82,0x85,0x40,0x27,0x04,
0xC6,0x3D,0x20,0xE8,0xDE,0x9C,0xBD,0xB6,
0xEB,0x30,0xC6,0xF9,0x96,0x81,0x85,0x01,
0x26,0x01,0x5A,0xEB,0x02,0xE7,0x02,0xA6,
0x01,0x8A,0x80,0x25,0x01,0x4A,0xA7,0x01,
0xBD,0xEF,0x3C,0xBD,0xEE,0x06,0x30,0xBD,
0xB1,0xA2,0xA6,0x03,0xE6,0x04,0xEE,0x03,
0x4D,0x26,0x08,0xC0,0x03,0x24,0x04,0x08,
0x5C,0x26,0xFC,0xDF,0x12,0x30,0xBD,0xAA,
0x3A,0x35,0x96,0x4B,0x26,0xA4,0x96,0xBC,
0x26,0x07,0xC6,0x80,0xD7,0x70,0x7C,0x00,
0xBC,0xD6,0x70,0x2B,0x03,0xBD,0xDB,0xC8,
0x96,0x81,0x85,0x04,0x27,0x06,0x4F,0xBD,
0xDD,0x0C,0x20,0x37,0x86,0x20,0xBD,0xDD,
0x0C,0xCE,0xDA,0x1E,0xBD,0xD9,0xFF,0xDE,
0x91,0xDF,0x02,0xCE,0x00,0x08,0xDF,0x04,
0xDE,0xBB,0xDF,0x00,0xBD,0xB0,0x04,0xCE,
0x01,0x41,0xDF,0x02,0xCE,0x00,0x08,0xDF,
0x04,0xDE,0x12,0xDF,0x00,0xBD,0xB0,0x04,
0xCE,0x01,0x49,0x86,0x0D,0xA7,0x00,0x86,
0x13,0xA7,0x01,0x86,0xFF,0xD6,0x11,0x26,
0x0D,0x4F,0xD6,0x81,0xC5,0x04,0x26,0x06,
0xCE,0xDA,0x36,0xBD,0xD9,0xFF,0x97,0x14,
0xDE,0x91,0xDF,0x85,0x86,0x1A,0x97,0xB4,
0xBD,0xDE,0x21,0x96,0x81,0x85,0x02,0x26,
0x03,0xE7,0x01,0x08,0xDF,0x83,0xDE,0x12,
0x96,0x81,0x85,0x04,0x26,0x01,0x08,0xDF,
0xBD,0xBD,0xFC,0x73,0x96,0x4B,0x26,0x09,
0x7C,0x00,0xBC,0x26,0x07,0x86,0x1B,0x97,
0x4B,0x7E,0xFB,0xD7,0x96,0x11,0x27,0x0D,
0x4A,0x97,0x11,0x26,0x05,0xCE,0x00,0x03,
0xDF,0x12,0x7E,0xDB,0x20,0x7A,0x00,0xBC,
0x8D,0x06,0xBD,0xDD,0x18,0x7E,0xFB,0x90,
0xBD,0xFE,0x81,0xBD,0xFF,0x8C,0x20,0xF8,
0x39,0xBD,0xB0,0x8F,0x29,0x02,0x24,0x05,
0x86,0x1B,0x97,0x4B,0x39,0xD7,0xA1,0xBD,
0xDD,0x18,0x96,0xA1,0x97,0xBE,0xD6,0xBC,
0x27,0x19,0x81,0x01,0x27,0x03,0x4D,0x26,
0x19,0x36,0xBD,0xFB,0xD7,0x32,0xD6,0x4B,
0x26,0x04,0x97,0xBC,0x26,0x13,0x39,0xBD,
0xFB,0xD7,0x39,0x4D,0x27,0xFC,0x81,0x01,
0x27,0x07,0xBD,0xFE,0xCC,0x96,0x4B,0x26,
0xF1,0xD6,0x81,0xC5,0x04,0x27,0x11,0xCE,
0xFF,0xFF,0xDF,0xB5,0x96,0xBE,0x97,0xBC,
0x86,0x80,0x97,0x70,0x86,0x08,0x20,0x70,
0xBD,0xDC,0xA8,0x96,0x4B,0x26,0xD3,0xDE,
0x91,0xDF,0x00,0xDE,0x93,0xDF,0x02,0xBD,
0xB0,0x5D,0xCE,0x05,0xB9,0xBD,0xB1,0xA2,
0x27,0x06,0x86,0x3A,0x97,0x4B,0x20,0xB7,
0xE6,0x04,0xD1,0xBE,0x26,0xF4,0xD7,0xBC,
0xDE,0x91,0xBD,0xAA,0x35,0xDF,0x00,0xDE,
0x93,0xDF,0x02,0xBD,0xB0,0x5D,0xCE,0x05,
0xB9,0xBD,0xB1,0xA2,0x26,0xDC,0xEE,0x03,
0xDF,0xB5,0xDE,0x91,0xE6,0x08,0xC1,0x41,
0x27,0x10,0xC1,0x42,0x27,0x08,0xC1,0x4E,
0x26,0x0C,0x86,0x10,0x20,0x06,0x86,0x04,
0x20,0x02,0x86,0x08,0x9A,0x82,0xE6,0x10,
0xC1,0x50,0x27,0x02,0x20,0x02,0x8A,0x02,
0xE6,0x1A,0xC1,0x53,0x26,0x02,0x8A,0x01,
0xD6,0x81,0xC5,0x01,0x27,0x02,0x8A,0x80,
0x8A,0x20,0x97,0x82,0xBD,0xDD,0xF5,0x39,
0x86,0x0A,0x36,0x7F,0x05,0x90,0x20,0x07,
0x36,0xBD,0xFE,0x81,0x7C,0x05,0x90,0xDE,
0x91,0xDF,0x85,0xDE,0x93,0x08,0x08,0xDF,
0x83,0xBD,0xFD,0x54,0xBD,0xE0,0x57,0xBD,
0xFB,0x90,0x86,0x0A,0xBD,0xFE,0xB0,0xD6,
0x70,0xC4,0x7F,0xD7,0x70,0xB6,0x87,0x8F,
0x85,0x40,0x27,0x0B,0xDE,0x41,0xDF,0xB5,
0xCA,0x80,0xD7,0x70,0x7C,0x00,0xBC,0x96,
0x81,0x85,0x02,0x27,0x09,0xDE,0x85,0x09,
0x9C,0x93,0x27,0x16,0x20,0x07,0xBD,0xDE,
0x21,0xE1,0x01,0x27,0x0D,0x96,0x4B,0x26,
0x09,0x32,0x4A,0x26,0xAB,0x86,0x35,0x97,
0x4B,0x39,0x32,0x39,0xDE,0x91,0xA7,0x00,
0x08,0x9C,0x93,0x26,0xF9,0xA7,0x00,0x39,
0x96,0x70,0x85,0x02,0x27,0x17,0x96,0x6E,
0x36,0xDE,0x85,0x86,0xFF,0xA7,0x00,0xBD,
0xDD,0xFE,0xBD,0xDD,0x3F,0x30,0xBD,0xAA,
0x3A,0x35,0x32,0x97,0x6E,0x96,0x70,0x84,
0x80,0x97,0x70,0x7F,0x00,0x82,0x39,0xDE,
0xB5,0x09,0xDF,0xB5,0x26,0x0D,0xDE,0x93,
0x86,0xFF,0xA7,0x00,0x7F,0x00,0x70,0x86,
0x01,0x97,0xB9,0xB6,0x87,0x98,0x85,0x10,
0x27,0x05,0x86,0x38,0x97,0x4B,0x39,0xBD,
0xDE,0x21,0x96,0x81,0x85,0x02,0x26,0x03,
0x08,0xE7,0x00,0xDF,0x83,0x96,0x70,0x85,
0x80,0x27,0x07,0x4F,0x97,0x70,0x86,0x02,
0x97,0xB9,0xDE,0x91,0xDF,0x85,0xBD,0xFD,
0x25,0xBD,0xE0,0x57,0xBD,0xFE,0x4E,0xBD,
0xFB,0x90,0x8D,0x69,0xDE,0xB5,0x26,0x07,
0x86,0x80,0x97,0x70,0x7C,0x00,0xBC,0x39,
0x96,0x70,0x85,0x10,0x27,0x19,0x84,0xEF,
0x97,0x70,0xBD,0xFE,0x81,0xDE,0xB5,0x08,
0xDF,0xB5,0xB6,0x87,0x8E,0x85,0x40,0x27,
0x06,0x96,0x70,0x8A,0x80,0x97,0x70,0xDE,
0xB5,0x26,0x05,0x86,0x80,0x97,0x6E,0x39,
0x96,0x70,0x8A,0x02,0x97,0x70,0xDE,0x98,
0xA6,0x00,0xDE,0x85,0x08,0xDF,0x85,0x09,
0xA7,0x00,0x9C,0x83,0x26,0x13,0x8D,0x26,
0xBD,0xDD,0x3F,0xCE,0x00,0x8F,0xBD,0xB7,
0x0F,0x96,0x4B,0x26,0x0A,0x96,0x6E,0x2B,
0x06,0xDE,0x98,0x9C,0x95,0x26,0x01,0x39,
0x08,0xDF,0x98,0x20,0xC2,0xDE,0x91,0xDF,
0x85,0xDE,0x93,0xDF,0x83,0x39,0x32,0x97,
0x0B,0x32,0x97,0x0C,0xCE,0x00,0x8F,0xBD,
0xB6,0xEB,0xCE,0x01,0x1F,0xDF,0x91,0x96,
0x82,0x2A,0x05,0xCE,0x01,0x9E,0x20,0x03,
0xCE,0x02,0x1E,0xDF,0x93,0xDE,0x0B,0x6E,
0x00,0xC6,0x0F,0xDE,0x91,0xEB,0x00,0xC9,
0x00,0x9C,0x93,0x27,0x03,0x08,0x20,0xF5,
0x39,0x5F,0x96,0x82,0x85,0x20,0x27,0x7C,
0x85,0x1C,0x27,0x78,0x5C,0x85,0x10,0x26,
0x06,0x5C,0x85,0x08,0x26,0x01,0x5C,0x58,
0x4F,0x30,0xEB,0x01,0xE7,0x01,0xA9,0x00,
0xA7,0x00,0x39,0xBD,0xDE,0x31,0x20,0x26,
0x20,0x5A,0x20,0x0A,0x96,0x70,0x85,0x02,
0x26,0x52,0xC6,0x04,0x20,0x0E,0x96,0x70,
0x85,0x02,0x26,0x48,0x96,0x82,0x85,0x01,
0x26,0x42,0xC6,0x02,0x4F,0x30,0xEB,0x01,
0xE7,0x01,0xA9,0x00,0xA7,0x00,0x39,0xBD,
0xDE,0x31,0x20,0x72,0x20,0x08,0x20,0x2C,
0x96,0x82,0x85,0x40,0x26,0x68,0x96,0x82,
0x84,0xEF,0x8A,0x04,0x97,0x82,0xCE,0xDA,
0x26,0x20,0x42,0xBD,0xDE,0x31,0x20,0x0C,
0x20,0x12,0x20,0x02,0x20,0x0E,0x96,0x70,
0x85,0x02,0x26,0x08,0x39,0xBD,0xDE,0x53,
0x20,0x06,0x20,0x04,0x86,0x37,0x97,0x4B,
0x39,0xBD,0xDE,0x53,0x20,0x02,0x20,0xF4,
0x39,0xBD,0xDE,0x31,0x20,0x30,0x20,0x0A,
0x20,0x02,0x20,0xE8,0x96,0x82,0x85,0x40,
0x26,0x24,0x96,0x82,0x84,0xEF,0x8A,0x08,
0x97,0x82,0xCE,0xDA,0x2E,0xBD,0xD9,0xFF,
0xCE,0xDA,0x52,0x86,0x0A,0xC6,0x10,0xBD,
0xDA,0x02,0x96,0x81,0x85,0x04,0x26,0x06,
0xBD,0xDB,0xC8,0xBD,0xDD,0x53,0x39,0xBD,
0xDE,0x31,0x20,0xFA,0x20,0x0E,0x20,0x02,
0x20,0xB2,0x96,0x82,0x85,0x40,0x26,0xEE,
0x85,0x01,0x26,0xA8,0x96,0x80,0x26,0xA4,
0xCE,0xDA,0x2E,0xBD,0xD9,0xFF,0xCE,0xDA,
0x48,0x86,0x0A,0xC6,0x10,0xBD,0xDA,0x02,
0xD6,0x82,0xC4,0xEF,0xCA,0x0A,0xD7,0x82,
0x96,0x7B,0x27,0xBE,0xCE,0xDA,0x5C,0x86,
0x08,0xC6,0x1A,0x20,0xB2,0x96,0x70,0x85,
0x10,0x26,0x2A,0xBD,0xDF,0x43,0x96,0x4B,
0x27,0x23,0x39,0xDE,0xB5,0x27,0x64,0x09,
0xDF,0xB5,0xBD,0xDD,0xFE,0xBD,0xDC,0xA8,
0xBD,0xDD,0xF5,0xCE,0x00,0x8F,0xBD,0xB7,
0x0F,0x96,0x4B,0x27,0x01,0x39,0x96,0x70,
0x8A,0x10,0x97,0x70,0x39,0xDE,0x85,0xA6,
0x00,0xD6,0x8E,0xC1,0x0E,0x27,0x07,0x91,
0x79,0x27,0x38,0xBD,0xC4,0xA9,0x08,0xDF,
0x85,0x09,0x9C,0x83,0x26,0x06,0xD6,0x70,
0xC4,0xEF,0xD7,0x70,0xD6,0x7A,0x2B,0x03,
0x11,0x27,0xAA,0xDE,0x98,0xA7,0x00,0xD6,
0x8E,0xC1,0x0E,0x27,0x04,0x91,0x78,0x27,
0x09,0x9C,0x93,0x27,0x0A,0x08,0xDF,0x98,
0x20,0x93,0x86,0x01,0x97,0x6A,0x09,0x08,
0xDF,0x95,0x39,0x86,0x20,0x97,0x6A,0xDE,
0x98,0x86,0x0D,0xA7,0x00,0x86,0x80,0x97,
0x6E,0x20,0xEB,0xBD,0xB0,0x8F,0x96,0xA4,
0x26,0x04,0x7F,0x00,0x81,0x4C,0x48,0x97,
0xA4,0x44,0x5D,0x26,0x05,0x43,0x94,0x81,
0x20,0x02,0x9A,0x81,0x97,0x81,0xD6,0xA4,
0xC1,0x04,0x23,0x04,0x86,0x12,0x97,0x8A,
0x39,0x96,0x82,0x8A,0x01,0x97,0x82,0x39,
0x8D,0x22,0x96,0x4B,0x26,0x0A,0xBD,0xAE,
0xC7,0x96,0x97,0xCE,0xDF,0xF9,0x8D,0x78,
0x39,0x0C,0xDE,0xC1,0x04,0xDE,0x9B,0x0E
    ];
    
    this.ROM671 = [
0xDE,0xB9,0x0D,0xDE,0xAD,0x0F,0xDE,0x7F,
0x01,0xDE,0xF7,0x00,0xBD,0xC6,0xB4,0xD6,
0x71,0x2B,0x3B,0xBD,0xA9,0xCC,0xC6,0x80,
0xD7,0x71,0xC6,0x86,0xF7,0x87,0x8E,0xC6,
0x01,0xF7,0x87,0x8C,0xC6,0xFE,0xF7,0x87,
0x94,0x7F,0x87,0x96,0xBD,0xFB,0x90,0xB6,
0x87,0xAA,0x85,0x08,0x27,0x18,0xB6,0x87,
0x98,0x88,0x02,0xB7,0x87,0x98,0x88,0x02,
0xB7,0x87,0x98,0xB6,0x87,0xAA,0x85,0x08,
0x27,0x05,0x86,0x39,0x97,0x4B,0x39,0x5F,
0xD7,0x70,0xD7,0x82,0x7E,0xFB,0xD7,0x96,
0x6F,0x27,0x14,0xB6,0x87,0xAA,0x85,0x08,
0x27,0x04,0x86,0x39,0x20,0x07,0xB6,0x87,
0x9B,0x2A,0xEC,0x86,0x36,0x97,0x4B,0x39,
0x6D,0x00,0x27,0x17,0xA1,0x00,0x27,0x05,
0x08,0x08,0x08,0x20,0xF3,0xEE,0x01,0xAD,
0x00,0x86,0x23,0x97,0x90,0x96,0x82,0x8A,
0x40,0x97,0x82,0x39,0xBD,0xC1,0xA6,0x86,
0x59,0x97,0x53,0xCE,0x04,0xA1,0xDF,0x56,
0x36,0x36,0x86,0x18,0x36,0xBD,0xE1,0x2B,
0x32,0x32,0x32,0x96,0x4B,0x26,0x0B,0x96,
0x55,0x2A,0x07,0xDE,0x3D,0xDF,0x4F,0xBD,
0x96,0xB3,0x7E,0xC5,0xB2,0x32,0x97,0x5C,
0x32,0x97,0x5D,0x32,0x81,0x18,0x26,0x09,
0x36,0xCE,0x00,0x00,0xC6,0xFF,0x7E,0x00,
0x5B,0x32,0x30,0xEE,0x00,0x31,0x31,0x4D,
0x26,0xEF,0x08,0xBD,0xB1,0xA2,0xEE,0x03,
0x5F,0x7E,0x00,0x5B,0x96,0x53,0x81,0x6E,
0x27,0xAA,0x7F,0x00,0xA4,0x96,0x82,0x85,
0x01,0x26,0x04,0x96,0x7B,0x27,0x04,0x86,
0xFF,0x97,0x80,0xBD,0xF0,0xB1,0x96,0x4B,
0x26,0x26,0x96,0x6A,0x26,0x06,0x86,0x32,
0x97,0x4B,0x20,0x19,0xDE,0x95,0x86,0x0D,
0xA7,0x00,0xBD,0xA8,0xB0,0x96,0x6A,0x85,
0x3C,0x26,0x0A,0x96,0x4B,0x26,0x11,0x86,
0x01,0x97,0xA4,0x20,0xD6,0x7F,0x00,0x6E,
0x96,0xA4,0x26,0x04,0x86,0x3B,0x97,0x4B,
0x7E,0xE1,0xBF,0x30,0x08,0xDF,0x00,0x86,
0x18,0x36,0xBD,0xE8,0x2E,0x30,0xA6,0x00,
0x81,0x18,0x27,0x0B,0xA6,0x01,0x81,0x20,
0x26,0x05,0x86,0x0C,0xBD,0xE8,0x76,0x96,
0x4B,0x26,0xDD,0xBD,0xAE,0x22,0x96,0x90,
0x36,0x86,0x1F,0x97,0x90,0xBD,0xAE,0xC7,
0x32,0x97,0x90,0xBD,0xE0,0xB5,0xD7,0x1E,
0xDF,0x14,0xBD,0xE0,0xB5,0x26,0x04,0xDF,
0x16,0x20,0x0D,0xDE,0x14,0xDF,0x16,0x96,
0x1E,0x27,0x05,0xCE,0xFF,0xFF,0xDF,0x16,
0x96,0x4B,0x26,0xAC,0x96,0x8E,0x81,0x04,
0x26,0x03,0x7E,0xE0,0xDC,0xD6,0x15,0x37,
0xD6,0x14,0x37,0x37,0xBD,0x91,0x06,0x33,
0x30,0xEE,0x00,0x31,0x31,0xDF,0x02,0x26,
0x02,0x20,0x1E,0x33,0x30,0xEE,0x00,0xDF,
0x14,0x31,0x31,0x96,0x14,0xD6,0x15,0x91,
0x16,0x22,0x0E,0x25,0x2D,0xD1,0x17,0x22,
0x08,0x25,0x27,0xDE,0x41,0xDF,0x16,0x20,
0x21,0xBD,0xC2,0x78,0xBD,0xC4,0x04,0xDE,
0x41,0xDF,0x64,0xDF,0x62,0x96,0x90,0x81,
0x23,0x26,0x09,0x96,0x97,0x81,0x01,0x26,
0x03,0xBD,0xDD,0x18,0xBD,0xAF,0x1D,0xBD,
0xAB,0x67,0xBD,0x98,0xAA,0x96,0x4B,0x26,
0xDE,0x5F,0x96,0x8E,0x81,0x13,0x26,0x02,
0xCA,0x02,0xD7,0x87,0xBD,0xC2,0x78,0xBD,
0xC3,0x74,0x7F,0x00,0x87,0x96,0x6E,0x9A,
0x4B,0x26,0xC4,0xDE,0x16,0x27,0xBA,0xDE,
0x14,0x08,0xDF,0x14,0x7E,0xE1,0x85,0x86,
0x18,0x36,0xCE,0x13,0x20,0xDF,0x56,0xBD,
0xAE,0x22,0xC6,0x01,0x96,0x81,0x85,0x04,
0x26,0x5C,0xD7,0x9A,0x96,0x90,0x36,0x86,
0x21,0x97,0x90,0xBD,0xAE,0xC7,0xBD,0xE0,
0x0C,0xD6,0x9A,0xBD,0xDB,0xDD,0xBD,0xCB,
0xBF,0x32,0x97,0x90,0x96,0x4B,0x26,0x3E,
0xCE,0x01,0x40,0xDF,0x00,0xCE,0x01,0x4A,
0xDF,0x02,0xBD,0xB0,0x5D,0xFE,0x05,0xBA,
0xBD,0xAA,0x3C,0x96,0x82,0x2B,0x01,0x08,
0xFF,0x05,0xBA,0xCE,0x01,0x40,0xDF,0x02,
0xCE,0x00,0x14,0xDF,0x04,0xBD,0xB0,0x14,
0xCE,0x01,0x1F,0xDF,0x98,0xCE,0x01,0x4A,
0xDF,0x95,0xBD,0xC4,0x04,0xD6,0x9A,0x5C,
0x96,0x82,0x85,0x1C,0x26,0xA4,0x96,0x4B,
0x36,0xBD,0xC5,0x06,0xBD,0xCB,0xBF,0x32,
0x97,0x4B,0xBD,0xAB,0x67,0xFF,0x30,0x08,
0xDF,0x00,0x86,0x18,0xBD,0xD0,0xE9,0xBD,
0xAA,0x4C,0xDF,0x0A,0xDF,0x00,0xBD,0xCE,
0x74,0x81,0x20,0x26,0x34,0xDE,0x0A,0xEE,
0x04,0xA6,0x00,0x26,0x06,0xE6,0x01,0xC1,
0x07,0x25,0x02,0xC6,0x06,0x08,0xDF,0x0C,
0xCE,0x20,0x20,0xDF,0x02,0xDF,0x04,0xDF,
0x06,0x17,0x27,0x1A,0xDE,0x0C,0xBD,0x04,
0x4B,0xBD,0xAA,0x5A,0x36,0x17,0x33,0xCE,
0x00,0x01,0xBD,0x04,0x57,0x16,0x5A,0x20,
0xE8,0x86,0x20,0x97,0x4B,0x39,0xDE,0x0A,
0x32,0xA7,0x04,0x32,0xA7,0x05,0x07,0xA7,
0x03,0x96,0x7C,0xA7,0x02,0x86,0x17,0xA7,
0x01,0x4F,0x97,0x08,0xBD,0xA9,0xD5,0xFE,
0x88,0x00,0x8C,0x40,0x51,0x26,0x36,0xCE,
0x88,0x0D,0xDF,0x0A,0xA6,0x00,0x27,0x2D,
0xEE,0x00,0x9C,0x02,0x26,0x3C,0xDE,0x0A,
0xEE,0x02,0x9C,0x04,0x26,0x34,0xDE,0x0A,
0xEE,0x04,0x9C,0x06,0x26,0x2C,0xDE,0x0A,
0xEE,0x06,0xAD,0x00,0x9F,0x00,0x86,0x17,
0xBD,0xD0,0xE9,0x9E,0x00,0x32,0x32,0xBD,
0xA9,0xD5,0x32,0x06,0x39,0x96,0x08,0x81,
0x30,0x25,0x0B,0x4C,0x81,0x40,0x26,0xB2,
0x86,0x20,0x97,0x4B,0x20,0xDE,0x8B,0x08,
0x20,0xA8,0xDE,0x0A,0xBD,0xAA,0x3B,0x20,
0xB1,0xFF,0x7F,0x04,0x3D,0x86,0x04,0x9A,
0x54,0x97,0x54,0x39,0xBD,0xA9,0xE7,0x7A,
0x04,0x3D,0xB6,0x04,0x3D,0x27,0x06,0x86,
0xFF,0x97,0x22,0x20,0x09,0x96,0x54,0x85,
0x04,0x27,0xF4,0x7F,0x00,0x22,0xBD,0xE4,
0xFD,0x24,0x03,0x7E,0xA9,0xFD,0x30,0xA6,
0x00,0xEE,0x01,0xDF,0x1C,0xD6,0x22,0x27,
0x72,0xA6,0x04,0x85,0x20,0x26,0x06,0xBD,
0xE5,0xEB,0x7E,0xA9,0xFD,0xA6,0x05,0xE6,
0x06,0xD0,0x17,0x92,0x16,0x25,0xF0,0xDE,
0x16,0x09,0xDF,0x16,0xDE,0x1C,0x96,0x1A,
0x26,0x0B,0xEE,0x07,0x26,0xE1,0x08,0xDF,
0x18,0x4F,0x5F,0x20,0x1F,0xEE,0x07,0x27,
0xD6,0xDE,0x1C,0xA6,0x07,0xE6,0x08,0xD0,
0x19,0x92,0x18,0x25,0xCA,0xDE,0x18,0x09,
0xDF,0x18,0xDE,0x1C,0x96,0x18,0xD6,0x19,
0xEE,0x07,0xDF,0x18,0xBD,0xE5,0xC4,0x32,
0x32,0x32,0x32,0x32,0x32,0x32,0x96,0x1C,
0xD6,0x1D,0x37,0x36,0xDE,0x1C,0xA6,0x0B,
0xE6,0x0C,0xDB,0x0F,0x99,0x0E,0x37,0x36,
0xA6,0x0B,0xE6,0x0C,0x37,0x36,0x86,0x0B,
0x36,0x20,0x97,0x81,0x08,0x26,0x03,0x7E,
0xE4,0x89,0x86,0x09,0xBD,0xE5,0xE1,0xDE,
0x1C,0xA6,0x04,0x81,0x40,0x26,0x05,0xBD,
0xE5,0xEB,0x20,0x30,0x96,0x1A,0x26,0x05,
0xCE,0x00,0x01,0xDF,0x18,0x4F,0x5F,0xBD,
0xE5,0xC4,0x25,0x20,0xDE,0x0E,0xDF,0x1E,
0xDE,0x1C,0xA6,0x04,0x85,0x20,0x27,0x17,
0xA6,0x09,0xE6,0x0A,0xD0,0x1F,0x92,0x1E,
0x25,0xD5,0x01,0x0F,0xBD,0xE5,0x79,0x86,
0x20,0xA7,0x04,0x0E,0x7E,0xA9,0xFD,0xBD,
0xE5,0x44,0x25,0xF8,0x4F,0xC6,0x05,0xDB,
0x46,0x99,0x45,0x97,0x0E,0xD7,0x0F,0x01,
0x0F,0xBD,0xA9,0xCC,0xDE,0x1C,0x86,0x20,
0xA7,0x04,0xBD,0xE5,0x79,0x96,0x1E,0xA7,
0x09,0x96,0x1F,0xA7,0x0A,0xBD,0xE5,0xA9,
0x86,0x10,0xA7,0x02,0x0E,0xFE,0x00,0x0E,
0x86,0x40,0xA7,0x00,0xDE,0x20,0x09,0x27,
0x0B,0xDF,0x20,0xDE,0x0E,0xBD,0xAA,0x3B,
0xDF,0x0E,0x20,0xEE,0xBD,0xA9,0x94,0x20,
0xB3,0x86,0x05,0xBD,0xE5,0xE1,0xDE,0x1C,
0xA6,0x04,0x85,0x80,0x27,0x04,0x8D,0x3C,
0x20,0x28,0x96,0x16,0xD6,0x17,0xCB,0x04,
0x89,0x00,0x25,0x2B,0xEE,0x0B,0xE0,0x01,
0xA2,0x00,0x24,0x23,0xBD,0xA9,0xCC,0x96,
0x16,0xD6,0x17,0xDE,0x1C,0xA7,0x05,0xE7,
0x06,0xE0,0x08,0xA2,0x07,0x25,0x06,0xBD,
0xA9,0x94,0x7E,0xA9,0xFD,0xA6,0x05,0xE6,
0x06,0xA7,0x07,0xE7,0x08,0x20,0xF0,0xBD,
0xE5,0xEB,0x20,0xEE,0x86,0x15,0x36,0xDF,
0x1C,0xBD,0xA9,0xCC,0xBD,0xE5,0x94,0x25,
0x17,0xBD,0xE5,0x44,0x25,0x12,0xDE,0x1C,
0x86,0x10,0xA7,0x04,0x4F,0xA7,0x07,0xA7,
0x08,0xBD,0xE5,0xA9,0x86,0x60,0xA7,0x02,
0xBD,0xA9,0x94,0x32,0x39,0xBD,0xA9,0xE7,
0x7F,0x00,0x1A,0x9F,0x00,0x7F,0x00,0x08,
0x8D,0x18,0x25,0x13,0x32,0x81,0x10,0x27,
0x0D,0x36,0xDE,0x16,0xDF,0x18,0x8D,0x0A,
0x25,0x05,0x73,0x00,0x1A,0x32,0x0C,0x7E,
0xA9,0xFD,0xBD,0xA9,0xE7,0xBD,0xCE,0x77,
0x26,0x06,0x30,0xBD,0xB1,0xA2,0x27,0x06,
0xBD,0xE5,0xEB,0x7E,0xA9,0xFD,0xEE,0x03,
0x27,0xF6,0xDF,0x16,0x86,0x09,0xBD,0xE5,
0xE1,0x0C,0x20,0xEF,0x86,0x15,0x36,0x96,
0x1E,0xD6,0x1F,0xCB,0xF4,0x89,0x01,0x25,
0x22,0xDB,0x46,0x99,0x45,0x25,0x1C,0x9F,
0x0E,0xD0,0x0F,0x92,0x0E,0x25,0x0E,0x2B,
0x12,0x37,0x36,0x86,0x0E,0x36,0xBD,0xF9,
0xAA,0x96,0x4B,0x26,0x03,0x0C,0x32,0x39,
0x0D,0x32,0x39,0x86,0x27,0x97,0x4B,0x20,
0xF7,0x96,0x16,0xA7,0x05,0x96,0x17,0xA7,
0x06,0x96,0x1A,0x26,0x06,0x4F,0xA7,0x07,
0xA7,0x08,0x39,0x96,0x18,0xA7,0x07,0x96,
0x19,0xA7,0x08,0x39,0x96,0x16,0xD6,0x17,
0xA7,0x05,0xE7,0x06,0xCB,0x05,0x89,0x00,
0x97,0x1E,0xD7,0x1F,0x24,0x02,0x8D,0x43,
0x39,0x96,0x45,0xA7,0x0B,0xD6,0x46,0xE7,
0x0C,0xDB,0x1F,0x99,0x1E,0xDE,0x45,0x97,
0x45,0xD7,0x46,0x96,0x1E,0xD6,0x1F,0xA7,
0x00,0xE7,0x01,0x39,0xDE,0x16,0xDF,0x0E,
0xDE,0x18,0x8D,0x2F,0x26,0x10,0xDE,0x0E,
0xDF,0x20,0xCE,0x00,0x08,0x4F,0xC6,0x05,
0x8D,0x21,0x26,0x02,0x0C,0x39,0x8D,0x0B,
0x39,0xBD,0xA9,0xE7,0x33,0x4A,0x26,0xFC,
0x7E,0xA9,0xFD,0x96,0x22,0x26,0x06,0x86,
0x09,0x97,0x4B,0x0D,0x39,0x86,0x0A,0x20,
0xF8,0x4F,0x5F,0xDF,0x0C,0xCE,0x00,0x10,
0x76,0x00,0x0E,0x76,0x00,0x0F,0x24,0x04,
0xDB,0x0D,0x99,0x0C,0x46,0x56,0x76,0x00,
0x0E,0x76,0x00,0x0F,0x09,0x26,0xEF,0x97,
0x0C,0xD7,0x0D,0xDE,0x0C,0x39,0xBD,0xA9,
0xE7,0x96,0x54,0x8A,0x40,0x97,0x54,0x86,
0x0C,0x30,0xA1,0x00,0x27,0x15,0x86,0x14,
0xA1,0x00,0x27,0x71,0x7E,0xE6,0x83,0x27,
0x09,0x80,0x5F,0x26,0x05,0xA7,0x03,0x4C,
0xA7,0x04,0x39,0x30,0xBD,0xB1,0xA2,0x8D,
0xEE,0xEE,0x03,0xDF,0x08,0x30,0xBD,0xAA,
0x3A,0x35,0x86,0x0C,0xA1,0x00,0x26,0x13,
0xBD,0xB1,0xA2,0x8D,0xDA,0x7F,0x00,0x4B,
0xEE,0x03,0xDF,0x06,0x30,0xBD,0xAA,0x3A,
0x35,0x20,0x04,0xDE,0x08,0xDF,0x06,0x7F,
0x00,0x4B,0xBD,0xE6,0xAB,0x7E,0xA9,0xFD,
0xEE,0x01,0x86,0xC0,0xA7,0x04,0x30,0xBD,
0xAA,0x3A,0x35,0x86,0x0A,0xA1,0x00,0x27,
0xEF,0x86,0x08,0xA1,0x00,0x27,0x09,0x86,
0x18,0xA1,0x00,0x27,0xE0,0xBD,0xBC,0x30,
0xEE,0x01,0x86,0x90,0xA7,0x04,0x30,0xBD,
0xAA,0x3E,0x35,0x20,0xDE,0xBD,0xC1,0xA6,
0x7E,0xC5,0xB2,0x86,0x15,0x36,0xDE,0x06,
0x27,0x01,0x09,0xDF,0x10,0x96,0x11,0x36,
0x91,0x09,0x96,0x10,0x36,0x92,0x08,0x25,
0x04,0x32,0x32,0x32,0x39,0x86,0x0D,0x36,
0xBD,0x90,0xF8,0x30,0xEE,0x01,0x26,0x09,
0x30,0x4F,0xA7,0x04,0xA7,0x05,0xCE,0x00,
0x3A,0xDF,0x0A,0x30,0xEE,0x04,0xDF,0x06,
0x30,0xBD,0xAA,0x3D,0x35,0xDE,0x08,0x8C,
0xFF,0xFF,0x27,0x1A,0x08,0x86,0x0D,0xBD,
0xAA,0x0A,0xBD,0x91,0x06,0x30,0xEE,0x01,
0xDF,0x0C,0x30,0xEE,0x04,0xDF,0x08,0x30,
0xBD,0xAA,0x3D,0x35,0x20,0x04,0xDE,0x41,
0xDF,0x0C,0x01,0x0F,0xDE,0x0A,0x96,0x0C,
0xD6,0x0D,0xA7,0x03,0xE7,0x04,0xDE,0x0C,
0x27,0x0B,0xD6,0x0B,0x96,0x0A,0x26,0x01,
0x5F,0xA7,0x05,0xE7,0x06,0x0E,0xFE,0x04,
0x39,0x27,0x0C,0x8D,0x59,0x20,0x08,0xDE,
0x41,0xFF,0x04,0x3B,0xFF,0x04,0x39,0xDE,
0x4D,0x27,0x0C,0x8D,0x49,0x20,0x08,0x96,
0x0A,0x97,0x4D,0x96,0x0B,0x97,0x4E,0xDE,
0x4F,0x27,0x0C,0x8D,0x39,0x20,0x08,0x96,
0x0C,0x97,0x4F,0x96,0x0D,0x97,0x50,0x9F,
0x00,0x86,0x01,0xC6,0x05,0xBD,0xD1,0x04,
0x27,0x22,0xEE,0x02,0x8D,0x20,0x20,0x73,
0xA6,0x01,0x81,0x01,0x27,0x64,0x81,0x02,
0x26,0x06,0xE6,0x09,0xCA,0x40,0xE7,0x09,
0xDE,0x00,0x96,0x0A,0xA7,0x02,0x96,0x0B,
0xA7,0x03,0x20,0x57,0x20,0x6F,0x96,0x0A,
0x26,0x04,0x96,0x0B,0x27,0x22,0x96,0x0C,
0x26,0x04,0x96,0x0D,0x27,0x0C,0xA6,0x07,
0x91,0x08,0x22,0x33,0xA6,0x08,0x91,0x09,
0x24,0x2D,0xA6,0x07,0x91,0x06,0x25,0x27,
0xA6,0x08,0x91,0x07,0x23,0x21,0x20,0x14,
0x96,0x0C,0x26,0x04,0x96,0x0D,0x27,0x17,
0xA6,0x07,0x91,0x08,0x22,0x11,0xA6,0x08,
0x91,0x09,0x24,0x0B,0x30,0xA6,0x01,0x8B,
0x02,0xA7,0x01,0x24,0x02,0x6C,0x00,0xDE,
0x00,0x39,0x4F,0xA7,0x02,0xA7,0x03,0xA7,
0x04,0xA7,0x05,0x86,0x02,0xA1,0x01,0x26,
0x0E,0xEE,0x04,0x8D,0xA1,0x20,0x08,0x96,
0x0C,0xA7,0x04,0x96,0x0D,0xA7,0x05,0xBD,
0xD1,0x1D,0x7E,0xE7,0x59,0x9F,0x00,0x86,
0x07,0xBD,0xD0,0xE9,0x27,0x10,0xEE,0x02
    ];
    
    this.ROM672 = [
0x8D,0x84,0x20,0x05,0x5F,0xE7,0x02,0xE7,
0x03,0xBD,0xD1,0x1D,0x20,0xE9,0xCE,0x03,
0xB9,0xDF,0x0E,0xC6,0x29,0xEE,0x00,0xBD,
0xE7,0x86,0x20,0x07,0xDE,0x0E,0x4F,0xA7,
0x00,0xA7,0x01,0xDE,0x0E,0x08,0x08,0xDF,
0x0E,0x5A,0x26,0xE9,0x32,0x39,0x32,0x97,
0x5C,0x32,0x97,0x5D,0xDE,0x00,0xA6,0x01,
0x81,0x18,0x26,0x03,0x7E,0x00,0x5B,0x81,
0x12,0x27,0xF9,0x81,0x0F,0x26,0x04,0x08,
0x08,0x20,0x05,0x81,0x11,0x26,0x06,0x36,
0x08,0xDF,0x00,0x20,0xE1,0x96,0x01,0x36,
0x96,0x00,0x36,0xBD,0xCE,0x74,0x36,0xC6,
0x13,0x37,0x81,0x30,0x27,0xD6,0x20,0xCC,
0x86,0x06,0x20,0x0A,0x86,0x03,0x20,0x06,
0x86,0x09,0x20,0x02,0x86,0x0F,0x7E,0xAB,
0x3B,0x00,0xDE,0x00,0xEE,0x00,0xBD,0xB6,
0xEB,0xDE,0x00,0x08,0x08,0xDF,0x00,0x20,
0x35,0x01,0xDE,0x00,0xEE,0x00,0x20,0xEC,
0xDE,0x00,0x6E,0x00,0xDE,0x00,0xEE,0x00,
0xBD,0xB7,0x0F,0x20,0xE4,0x96,0x20,0x56,
0xDE,0x00,0xEE,0x00,0x20,0xF0,0x30,0x08,
0x20,0x05,0x20,0x64,0xCE,0x05,0xAA,0xBD,
0xB6,0xEB,0x20,0x0A,0xDE,0x00,0xBD,0xB6,
0xEB,0xBD,0xAA,0x3B,0xDF,0x00,0x96,0xC2,
0x49,0x24,0x21,0x49,0x24,0x0C,0x2B,0x05,
0xBD,0xB4,0xAE,0x20,0x17,0xBD,0xB3,0xD3,
0x20,0x12,0x2B,0x05,0xBD,0xB2,0x45,0x20,
0x0B,0xBD,0xB2,0x3E,0x20,0x06,0x32,0x97,
0x00,0x32,0x97,0x01,0xDE,0x00,0xA6,0x00,
0x08,0xDF,0x00,0x97,0xC2,0x48,0x84,0x3E,
0xCE,0xE8,0x7A,0x7E,0x04,0x5D,0xCE,0x05,
0xBA,0xBD,0xB7,0x0F,0xCE,0x05,0xB2,0xBD,
0xB7,0x0F,0xCE,0x05,0xBA,0xBD,0xB6,0xEB,
0xCE,0x05,0xB2,0xBD,0xB6,0xEB,0x20,0xAE,
0xCE,0x05,0xAA,0xBD,0xB7,0x0F,0x20,0xA6,
0xCE,0xE9,0x1E,0x7E,0xB6,0xEB,0x04,0x02,
0xC9,0x0F,0xDA,0xA2,0x21,0x68,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x02,
0x80,0x00,0x00,0x00,0x00,0x00,0x04,0x01,
0xB8,0xAA,0x3B,0x29,0x5C,0x17,0x04,0x00,
0xB1,0x72,0x17,0xF7,0xD1,0xCF,0x03,0xFF,
0xDE,0x5B,0xD8,0xA9,0x37,0x28,0xE9,0x50,
0x04,0x01,0x80,0x00,0x00,0x00,0x00,0x00,
0x85,0xAA,0xC3,0x67,0xCC,0x48,0x8B,0x95,
0xC1,0xE3,0xEA,0x8B,0x91,0xC3,0xD3,0x73,
0xAB,0x11,0x98,0x37,0xF0,0x51,0x8D,0xB8,
0x9E,0xF5,0x32,0x60,0x91,0xA1,0xA5,0xFE,
0xD6,0xA9,0xB1,0x51,0xAD,0x58,0x3E,0xEA,
0x42,0xA1,0xB5,0x04,0xF3,0x33,0xF9,0xDE,
0xBD,0x08,0xA3,0x9F,0x58,0x0C,0xC5,0x67,
0x2A,0x11,0x55,0x06,0xCE,0x24,0x8C,0x15,
0x1F,0x84,0xD7,0x44,0xFC,0xCA,0xD6,0x9D,
0xE0,0xCC,0xDE,0xEC,0x2A,0x94,0xEA,0xC0,
0xC6,0xE7,0xDD,0x24,0xF5,0x25,0x7D,0x15,
0x24,0x86,0x03,0xFF,0x89,0x30,0xA2,0xF4,
0xF6,0x6A,0x03,0xFB,0xB6,0x0B,0x60,0xB6,
0x0B,0x61,0x04,0x06,0xB4,0x00,0x00,0x00,
0x00,0x00,0x03,0xFB,0xA3,0xD7,0x0A,0x3D,
0x70,0xA4,0x04,0x06,0xC8,0x00,0x00,0x00,
0x00,0x00,0x04,0x01,0xA2,0xF9,0x83,0x6E,
0x4E,0x44,0x04,0x00,0xC9,0x0F,0xDA,0xA2,
0x21,0x68,0x86,0x05,0x20,0x12,0x86,0x02,
0x20,0x0E,0x86,0x06,0x20,0x0A,0x86,0x03,
0x20,0x06,0x86,0x01,0x20,0x02,0x86,0x04,
0x97,0x02,0xBD,0xA9,0xE7,0xBD,0xB8,0x0D,
0x96,0x02,0xD6,0xC2,0x26,0x04,0x85,0x02,
0x20,0x08,0x2A,0x04,0x85,0x01,0x20,0x02,
0x85,0x04,0x27,0x0D,0x30,0x86,0x04,0xA7,
0x01,0x86,0x01,0xA7,0x02,0x86,0x80,0xA7,
0x03,0x7E,0xA9,0xFD,0xBD,0xA9,0xE7,0xFE,
0x05,0xCC,0xBD,0xB6,0xEB,0xBD,0xB4,0xAE,
0x30,0xA6,0x01,0x97,0x03,0x84,0x7F,0xA7,
0x01,0x5F,0x81,0x04,0x27,0x09,0x25,0x19,
0x86,0x05,0x97,0x4B,0x7E,0xB3,0xBC,0xA6,
0x02,0x81,0x14,0x24,0xF3,0x6F,0x02,0xBD,
0xEB,0xB6,0xD7,0xC2,0xBD,0xB3,0x59,0xD6,
0xC2,0x96,0x53,0x80,0x4B,0x25,0x06,0x26,
0x0D,0xCB,0x02,0x20,0x0F,0xC5,0x02,0x27,
0x0B,0x73,0x00,0x03,0x20,0x06,0x96,0x03,
0x2A,0x02,0xCB,0x04,0x56,0xD7,0x02,0x24,
0x12,0xC8,0x01,0xD7,0x02,0xCE,0xE9,0x50,
0xBD,0xB6,0xEB,0xBD,0xB2,0x3E,0x30,0x69,
0x01,0x64,0x01,0x96,0x53,0x80,0x4A,0x26,
0x03,0x7E,0xEB,0x3D,0x76,0x00,0x02,0x24,
0x56,0xBD,0xE8,0xDE,0xD6,0x16,0x18,0xDD,
0x03,0xDF,0xF9,0xE9,0xB9,0x10,0xF9,0x98,
0x9D,0x83,0xE7,0xD3,0x64,0xD9,0x40,0x73,
0x95,0xD9,0x9D,0x03,0xEE,0xF0,0xFA,0x7D,
0xA4,0x77,0x42,0xD9,0x9D,0x83,0xF5,0xAA,
0xE9,0xE3,0xED,0xF5,0xEF,0xD9,0x9D,0x03,
0xFB,0x81,0xE0,0xF8,0x40,0xD8,0x35,0xD9,
0x9D,0x83,0xFF,0x9D,0xE9,0xE6,0x4D,0xF2,
0x2B,0xD9,0x9D,0x04,0x00,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFE,0x0B,0x76,0x00,0x02,0x24,
0x03,0xBD,0xEB,0xAA,0x7E,0xA9,0xFD,0xBD,
0xE8,0xDE,0x16,0xD6,0x16,0x18,0xDD,0x83,
0xE3,0xEE,0x6D,0xD1,0x8B,0x7C,0x19,0x9D,
0x03,0xEB,0xA8,0x38,0x40,0xEE,0xAA,0x05,
0xD9,0x9D,0x83,0xF2,0x99,0x69,0x61,0xD5,
0xDD,0x2C,0xD9,0x9D,0x03,0xF8,0xA3,0x35,
0xE3,0x36,0x4C,0x12,0xD9,0x9D,0x83,0xFD,
0xA5,0x5D,0xE7,0x31,0x28,0x91,0xD9,0x9D,
0x04,0x00,0xC9,0x0F,0xDA,0xA2,0x21,0x62,
0xDC,0x0B,0x7E,0xEA,0xEC,0xBD,0xE8,0xDE,
0x16,0xD6,0x16,0x18,0xDD,0x84,0x06,0xB7,
0x58,0xB3,0xDC,0x18,0x0C,0x9D,0x04,0x0E,
0xDF,0x80,0xCD,0x5D,0x74,0x51,0xD9,0x9D,
0x84,0x14,0xDC,0xCB,0x6C,0x36,0x92,0xA1,
0xD9,0x9D,0x04,0x18,0xA7,0xC8,0xA6,0x93,
0xBF,0x9F,0xDC,0x19,0x9D,0x84,0x0A,0xFF,
0x54,0x52,0x6E,0x06,0x58,0xD9,0x9D,0x04,
0x12,0x85,0x64,0xC3,0x6C,0x58,0xA1,0xD9,
0x9D,0x84,0x16,0xF5,0xFB,0xDF,0xF7,0x89,
0x05,0xD9,0x9D,0x04,0x18,0xD5,0xA1,0x03,
0x6E,0xFB,0x72,0x0B,0x76,0x00,0x02,0x24,
0x05,0xBD,0xE8,0xDE,0x12,0x0B,0xBD,0xB3,
0xD3,0x7F,0x00,0x4B,0x79,0x00,0x03,0x7E,
0xEA,0xEF,0x30,0xA6,0x05,0x27,0x06,0xA6,
0x03,0x88,0x80,0xA7,0x03,0x39,0x5F,0x4D,
0x27,0x10,0x68,0x08,0x69,0x07,0x69,0x06,
0x69,0x05,0x69,0x04,0x69,0x03,0x59,0x4A,
0x26,0xF0,0x39,0xBD,0xA9,0xE7,0xA6,0x03,
0x27,0x04,0xA6,0x01,0x46,0x46,0xBD,0xAA,
0x3A,0x35,0x84,0x01,0x7E,0xA9,0xFD,0x30,
0xA6,0x05,0x26,0x01,0x39,0xA6,0x03,0x2A,
0x08,0xC6,0x06,0xD7,0x4B,0x84,0x7F,0xA7,
0x03,0xBD,0xA9,0xE7,0xBD,0xE8,0xDE,0x18,
0x19,0x0B,0x30,0xE6,0x01,0xCB,0x04,0x54,
0x66,0x02,0x24,0x09,0x64,0x03,0x66,0x04,
0x6C,0x02,0x26,0x01,0x5C,0xE7,0x01,0xC6,
0x07,0xD7,0xBF,0x5F,0x4F,0x0D,0x59,0x68,
0x04,0x69,0x03,0x49,0x68,0x04,0x69,0x03,
0x49,0x10,0x24,0x03,0x1B,0x5A,0x5A,0x5C,
0x7A,0x00,0xBF,0x26,0xE8,0x5C,0xE7,0x03,
0x86,0x03,0x97,0x02,0xBD,0xE8,0xDE,0x16,
0x19,0xF2,0x9C,0x0B,0x30,0xA6,0x02,0x26,
0x02,0x6A,0x01,0x6A,0x02,0x7A,0x00,0x02,
0x26,0xEA,0x7E,0xA9,0xFD,0xBD,0xA9,0xE7,
0xCE,0xE9,0x36,0xBD,0xB6,0xEB,0xBD,0xB4,
0xAE,0xCE,0x00,0x01,0xDF,0x06,0xDF,0x02,
0x30,0xA6,0x01,0x97,0x03,0x84,0x07,0xA7,
0x01,0x81,0x04,0x27,0x10,0x2B,0x38,0x86,
0x04,0x97,0x4B,0x96,0x03,0x2A,0x03,0x7E,
0xB3,0xBC,0x7E,0xB3,0xA1,0xA6,0x02,0x81,
0x09,0x25,0x0D,0x80,0x08,0x81,0x03,0x24,
0xE6,0xBD,0xEB,0xB6,0xD7,0x06,0x86,0x08,
0xBD,0xEB,0xB6,0x5C,0x26,0x09,0x96,0x06,
0x4C,0x97,0x06,0x85,0x04,0x26,0xD0,0xD7,
0x07,0x86,0x04,0x20,0x11,0xA7,0x01,0xE6,
0x02,0xCB,0x04,0xE7,0x02,0x89,0x00,0x85,
0x04,0x27,0x0D,0xA7,0x01,0x17,0x6F,0x02,
0xBD,0xEB,0xB6,0xD7,0x02,0xBD,0xB3,0x59,
0xBD,0xE8,0xDE,0x16,0xD6,0x16,0x9D,0x04,
0x0D,0xA6,0x82,0x9A,0x79,0x1B,0xF1,0x18,
0xDD,0x03,0xF8,0xEC,0x96,0xF0,0xD6,0x09,
0xD7,0x9D,0x04,0x07,0xE6,0xD5,0x05,0x1A,
0xE5,0x36,0xDC,0x16,0x99,0x12,0xB9,0x0B,
0xBD,0xEB,0xAA,0x96,0x03,0x2A,0x05,0xBD,
0xE8,0xDE,0x12,0x0B,0xBD,0xB3,0xD3,0x96,
0x02,0x48,0x9B,0x02,0x48,0x5F,0xBB,0xE9,
0x4F,0xF9,0xE9,0x4E,0x97,0xC5,0xD7,0xC4,
0xDE,0xC4,0xBD,0xB6,0xEB,0x96,0x06,0xD6,
0x07,0x30,0xE7,0x02,0x88,0x04,0xA7,0x01,
0x96,0x03,0x2A,0x05,0xBD,0xB3,0xD3,0x20,
0x03,0xBD,0xB4,0xAE,0x7E,0xA9,0xFD,0xBD,
0xA9,0xE7,0xE6,0x03,0x27,0x04,0xA6,0x01,
0x2A,0x07,0x86,0x17,0x97,0x4B,0x7E,0xA9,
0xFD,0x80,0x04,0x97,0x06,0xA6,0x02,0x97,
0x07,0x6F,0x02,0x86,0x04,0xA7,0x01,0xA6,
0x04,0x80,0x05,0xC2,0xB5,0x24,0x07,0x6C,
0x02,0xDE,0x06,0x09,0xDF,0x06,0xBD,0xE8,
0xDE,0x16,0xA0,0xE9,0x50,0x12,0x80,0xE9,
0x50,0xFC,0x16,0xD6,0x16,0x18,0xDD,0x03,
0xFE,0x97,0xA6,0xC5,0xF2,0xA6,0xE5,0x9D,
0x03,0xFE,0x9C,0xCC,0xDF,0x89,0xB6,0xE9,
0xD9,0x9D,0x03,0xFE,0xBA,0x33,0x4C,0xA9,
0x09,0x6F,0xD9,0x9D,0x03,0xFE,0xE3,0x8E,
0x28,0x00,0x20,0xFD,0xD9,0x9D,0x03,0xFF,
0x92,0x49,0x24,0xA2,0x8A,0x04,0xD9,0x9D,
0x03,0xFF,0xCC,0xCC,0xCC,0xCC,0xBD,0x75,
0xD9,0x9D,0x04,0x00,0xAA,0xAA,0xAA,0xAA,
0xAA,0xC6,0xD9,0x80,0xE9,0x2E,0xDC,0x0B,
0xD6,0x07,0x96,0x06,0x2A,0x05,0x43,0x50,
0x26,0x01,0x4C,0x37,0x36,0x36,0xBD,0xB1,
0xFB,0x30,0x69,0x01,0x79,0x00,0x06,0x66,
0x01,0xCE,0xE9,0x3E,0xBD,0xB6,0xEB,0xBD,
0xB4,0xAE,0xBD,0xB2,0x45,0x96,0x53,0x81,
0x44,0x26,0x09,0xCE,0xE9,0x46,0xBD,0xB6,
0xEB,0xBD,0xB4,0xAE,0x7E,0xA9,0xFD,0x30,
0xA6,0x05,0x26,0x01,0x39,0xBD,0xA9,0xE7,
0xE6,0x01,0xBD,0xAA,0x3A,0x35,0xCE,0xE9,
0x50,0xBD,0xB6,0xEB,0xC4,0x80,0xCA,0x04,
0x30,0xE7,0x01,0x7E,0xA9,0xFD,0x30,0xA6,
0x03,0x84,0x7F,0xA7,0x03,0x39,0xBD,0xA9,
0xE7,0xBD,0xEB,0xCB,0x4A,0x16,0xCE,0xE9,
0x50,0xBD,0xB6,0xEB,0x5D,0x26,0x03,0x7E,
0xB3,0xBC,0x7E,0xA9,0xFD,0xBD,0xA9,0xE7,
0xBD,0xEB,0xCB,0x97,0xC2,0xBD,0xEB,0xCB,
0xD6,0x53,0xC1,0x0B,0x27,0x04,0x94,0xC2,
0x20,0xDB,0x9A,0xC2,0x20,0xD7,0xBD,0xA9,
0xE7,0xA6,0x01,0x2A,0x0E,0xE6,0x0A,0x2A,
0x1A,0xBD,0xAA,0x39,0xDF,0xC4,0xBD,0xB7,
0xC0,0x20,0x0D,0xA6,0x0A,0x2B,0x10,0xBD,
0xAA,0x39,0xDF,0xC4,0xBD,0xB7,0xC0,0x40,
0x7E,0xA9,0xFD,0x86,0x01,0x20,0xF9,0x86,
0xFF,0x20,0xF5,0x4F,0x20,0xF2,0xBD,0xA9,
0xE7,0xA6,0x0C,0x26,0x0E,0xA6,0x03,0x26,
0x04,0x86,0x03,0x97,0x4B,0xBD,0xAA,0x3A,
0x35,0x20,0x75,0x30,0xE6,0x03,0x26,0x0F,
0xBD,0xAA,0x3A,0xBD,0xAA,0x3A,0x35,0xCE,
0xE9,0x50,0xBD,0xB6,0xEB,0x20,0xEA,0xA6,
0x01,0x97,0x06,0x84,0x07,0x81,0x04,0x26,
0x5A,0x86,0x08,0xA0,0x02,0x25,0x54,0x27,
0x06,0x54,0x25,0x4F,0x4A,0x26,0xFA,0xD7,
0x02,0xA6,0x04,0xAA,0x05,0xAA,0x06,0xAA,
0x07,0xAA,0x08,0x26,0x3E,0x97,0x03,0xA6,
0x01,0x97,0x06,0xBD,0xAA,0x3A,0x35,0x74,
0x00,0x02,0x27,0x14,0x24,0x08,0x7C,0x00,
0x03,0x30,0x08,0xBD,0xB6,0xEB,0x30,0x08,
0xBD,0xB6,0xEB,0xBD,0xB4,0xAE,0x20,0xE7,
0x96,0x03,0x27,0x08,0xBD,0xB4,0xAE,0x7A,
0x00,0x03,0x26,0xF8,0x96,0x06,0x2A,0x08,
0xBD,0xE8,0xDE,0x00,0xE9,0x50,0xF2,0x0B,
0x7E,0xA9,0xFD,0xA6,0x0A,0x2A,0x06,0x86,
0x16,0x97,0x4B,0x20,0xF3,0xBD,0xE8,0xDE,
0x12,0x0B,0xBD,0xED,0x27,0xBD,0xB4,0xAE,
0xBD,0xEC,0x4D,0x96,0x4B,0x27,0xE1,0x86,
0x03,0x20,0xE6,0xBD,0xA9,0xE7,0xBD,0xEE,
0x3E,0x2F,0x11,0x30,0xBD,0xAA,0x39,0xBD,
0xB7,0x0F,0x20,0x0D,0xBD,0xA9,0xE7,0xBD,
0xEE,0x3E,0x2B,0xEF,0x30,0xBD,0xAA,0x3A,
0x35,0x7E,0xA9,0xFD,0xBD,0xA9,0xE7,0xA6,
0x01,0x85,0x04,0x26,0x16,0x4D,0x2B,0x03,
0x7E,0xB3,0xBC,0xBD,0xAA,0x3A,0x35,0xCE,
0xE9,0x50,0xBD,0xB6,0xEB,0xBD,0xEB,0xAA,
0x7E,0xA9,0xFD,0x85,0x03,0x26,0xF9,0xE6,
0x02,0x27,0xE2,0x50,0xCB,0x30,0x24,0xF0,
0x4F,0x97,0x02,0xC0,0x08,0x2B,0x0C,0xA1,
0x08,0x27,0x05,0xA7,0x08,0x7C,0x00,0x02,
0x09,0x20,0xF0,0xCB,0x08,0x27,0x10,0x43,
0x48,0x5A,0x26,0xFC,0xA4,0x08,0xA1,0x08,
0x27,0x05,0x7C,0x00,0x02,0xA7,0x08,0x30,
0xA6,0x01,0x2A,0xC4,0x96,0x02,0x27,0xC0,
0xCE,0xE9,0x50,0xBD,0xB6,0xEB,0xBD,0xB2,
0x3E,0x20,0xB5,0xCE,0xEF,0xB0,0xBD,0xB6,
0xEB,0xCE,0x05,0xD6,0xBD,0xB7,0x0F,0x39,
0x04,0x00,0x87,0x65,0xAB,0xCD,0x11,0x11,
0x96,0x7C,0x36,0xCE,0x05,0x0C,0xDF,0x5E,
0xA6,0x02,0x27,0x05,0x84,0x3F,0xBD,0xA9,
0xD5,0xEE,0x00,0x27,0x21,0xE6,0x01,0xDE,
0x5E,0xA6,0x02,0x2B,0x12,0xC8,0xDF,0xC5,
0x81,0x27,0x0F,0xC5,0x68,0x27,0x0B,0x08,
0x08,0x08,0x08,0x08,0x08,0x20,0xD7,0x5D,
0x2A,0xF5,0xEE,0x03,0xAD,0x00,0x32,0x84,
0x3F,0xBD,0xA9,0xD5,0x3B,0x26,0xFF,0xFF,
0x7F,0x00,0x9E,0x96,0x9D,0x9B,0x9B,0x97
    ];
    
    this.ROM673 = [
0xA3,0x96,0x9C,0x99,0x9A,0x97,0xA2,0xBD,
0xF0,0x85,0xD6,0x6A,0x2B,0x14,0xD6,0x9E,
0x26,0xF5,0xDE,0x9C,0xA7,0x00,0x08,0xDF,
0x9C,0x9C,0xA2,0x26,0xEA,0x7C,0x00,0x9E,
0x20,0xE5,0x96,0x9B,0xD6,0x9A,0x90,0xA3,
0xD2,0xA2,0x9B,0x9D,0xD9,0x9C,0x97,0x8C,
0xD7,0x8B,0x39,0x96,0x90,0x81,0x20,0x26,
0x08,0x7E,0xF1,0x1F,0x30,0xBD,0xAA,0x3A,
0x35,0xBD,0xF0,0x85,0xD6,0x6A,0x2A,0x05,
0xC5,0x38,0x27,0xF5,0x39,0xCE,0xF0,0x77,
0xE6,0x00,0x27,0xED,0x11,0x27,0x03,0x08,
0x20,0xF6,0xBD,0xB5,0x6F,0xDE,0x98,0x09,
0xB6,0x05,0xA9,0x27,0x05,0x09,0x86,0x45,
0xA7,0x00,0xDF,0x98,0x7D,0x05,0xA7,0x26,
0xCB,0xDE,0x9C,0xBD,0xB7,0x0F,0x39,0x30,
0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,
0x39,0x2B,0x2D,0x2E,0x00,0xD6,0x8F,0xC5,
0x20,0x27,0x0E,0xDE,0x98,0x9C,0x95,0x27,
0x0E,0xA6,0x00,0x97,0x8D,0x08,0xDF,0x98,
0x39,0xBD,0xF0,0xB1,0x4D,0x27,0xEC,0x96,
0x6A,0x27,0xF6,0x8A,0x80,0x97,0x6A,0xD6,
0x8F,0xC4,0xDF,0xD7,0x8F,0x86,0x0D,0x20,
0xE2,0xBD,0xF0,0xEF,0xD6,0x77,0x27,0x36,
0xDE,0x98,0xC6,0x0D,0xE1,0x00,0x27,0x07,
0x9C,0x95,0x27,0x2A,0x08,0x20,0xF5,0xDF,
0x95,0xD6,0x6A,0x26,0x21,0xCE,0x00,0x8F,
0xBD,0xB6,0xEB,0xCE,0x02,0xB5,0xDF,0x91,
0xCE,0x03,0xB5,0xDF,0x93,0xBD,0xF0,0xEF,
0xCE,0x00,0x8F,0xBD,0xB7,0x0F,0xDE,0x91,
0xDF,0x98,0x96,0x4B,0x20,0xDB,0x39,0x7F,
0x00,0x6A,0xDE,0x91,0xDF,0x98,0xCE,0xF1,
0x0F,0xBD,0xC4,0x31,0xDE,0x91,0xDF,0x98,
0x96,0x4B,0x27,0x04,0xC6,0x88,0xD7,0x6A,
0xD6,0x8F,0xCA,0x20,0xD7,0x8F,0x39,0xE8,
0x68,0xF6,0x0F,0xF1,0x63,0xC4,0x2C,0xC4,
0x2C,0xC4,0x2C,0xDF,0x35,0xC4,0x2C,0x96,
0x97,0x81,0x18,0x27,0x1A,0x81,0x0D,0x27,
0x03,0x7E,0xC4,0x2C,0x96,0xA4,0x26,0x04,
0x86,0x82,0x20,0x02,0x86,0x64,0x36,0x4F,
0x36,0x36,0xBD,0xB1,0xFB,0x20,0x1B,0x96,
0xA4,0x26,0x06,0x96,0xAA,0xD6,0xAB,0x20,
0x04,0x96,0xA8,0xD6,0xA9,0x37,0x36,0x36,
0xBD,0xB1,0xFB,0xBD,0xE8,0xDE,0xE0,0xF4,
0x23,0x0B,0xDE,0x9C,0xBD,0xB7,0x0F,0x73,
0x00,0xA4,0x39,0x96,0x97,0x81,0x0D,0x27,
0x04,0xBD,0xC4,0x2C,0x39,0x86,0x3F,0x97,
0xB2,0x86,0x03,0x97,0x6B,0xBD,0xC6,0x5C,
0xD6,0x6B,0xC5,0x40,0x27,0xEF,0xC6,0x02,
0xD7,0x6B,0x86,0x80,0x97,0xB2,0x39,0xFF,
0x86,0x18,0x36,0x9F,0x00,0xB6,0x87,0xAA,
0x8A,0x40,0xB7,0x87,0xAA,0x86,0x11,0x97,
0x8E,0x96,0x8F,0x85,0x04,0x27,0x13,0xBD,
0xF5,0x7F,0x86,0x12,0xBD,0xD0,0xE9,0x96,
0x77,0x97,0x28,0xBD,0xAB,0xB3,0x96,0x28,
0x26,0x05,0x9F,0x00,0xBD,0xAB,0xB3,0xBD,
0xAB,0x67,0x81,0x11,0x27,0x21,0x39,0x86,
0x18,0x36,0x9F,0x00,0xB6,0x87,0xAA,0x8A,
0x40,0xB7,0x87,0xAA,0xBD,0xF4,0xB0,0x86,
0x12,0x97,0x8E,0x96,0x8F,0x8A,0x80,0x97,
0x8F,0xBD,0xAB,0xB3,0xBD,0xAB,0x67,0x7E,
0x87,0x6C,0xBD,0xF5,0x39,0xDE,0x9C,0xBD,
0xB6,0xEB,0x30,0xBD,0xB1,0xA2,0x64,0x03,
0x26,0x13,0x25,0x03,0x4D,0x27,0x06,0x7F,
0x00,0x4B,0xBD,0xF5,0x6D,0xE6,0x04,0xBD,
0xF5,0x0E,0x20,0x05,0xFF,0x86,0x0D,0x97,
0x4B,0x30,0xBD,0xAA,0x3A,0x35,0x39,0xBD,
0xF4,0xE3,0x37,0x96,0x6A,0x27,0x05,0x4F,
0x5D,0x26,0x01,0x4C,0x36,0x36,0xBD,0xB1,
0xFB,0x96,0x6A,0x27,0x03,0xBD,0xEB,0xAA,
0xDE,0x9C,0xBD,0xB7,0x0F,0x39,0xF6,0x04,
0x6F,0x27,0x0B,0x36,0x17,0xBD,0xA9,0xD5,
0x32,0xFE,0x04,0x6D,0xAD,0x00,0x36,0xBD,
0xCB,0xEE,0x7D,0x00,0xB0,0x27,0x5D,0x96,
0x6B,0x88,0x02,0x85,0x03,0x26,0x4C,0xB6,
0x05,0x8E,0x27,0x14,0x4A,0x27,0x0A,0x4A,
0x27,0x03,0xBD,0xF3,0x4B,0x86,0x0C,0x20,
0x02,0x86,0x1E,0xBD,0xCB,0xEE,0x20,0x36,
0xFE,0x03,0xBD,0x27,0x2E,0x96,0x8E,0x84,
0x3F,0x81,0x14,0x27,0x26,0x81,0x15,0x27,
0x22,0x86,0x13,0x36,0xB6,0x04,0x6C,0x27,
0x17,0xD6,0x7C,0x37,0x37,0xC6,0x0E,0x37,
0xBD,0xA9,0xD5,0xFE,0x88,0x0A,0xEE,0x00,
0xAD,0x00,0x31,0x31,0x32,0xBD,0xA9,0xD5,
0x31,0x20,0x03,0xBD,0xF2,0xC2,0x32,0x81,
0x0D,0x26,0x8B,0x39,0x32,0x39,0xC6,0x15,
0x37,0x85,0xE0,0x26,0x10,0x81,0x0D,0x27,
0x0C,0x8A,0x40,0xBD,0xF2,0x2E,0x86,0x08,
0xBD,0xF2,0x2E,0x86,0x5F,0xBD,0xF2,0x2E,
0x31,0x39,0x96,0xB2,0x36,0x86,0x46,0x97,
0xB2,0x7C,0x00,0xA5,0xCE,0x00,0x00,0xDF,
0xAA,0xCE,0x03,0x20,0xDF,0xA8,0xBD,0xCB,
0x81,0xBD,0xF3,0x6F,0x96,0xB0,0x26,0xF9,
0x32,0x97,0xB2,0x7F,0x00,0xA5,0x7F,0x00,
0xAE,0x39,0x86,0x15,0x36,0x96,0x97,0x81,
0x09,0x27,0x29,0x81,0x0C,0x27,0x25,0x81,
0x13,0x27,0x21,0x81,0x16,0x27,0x0E,0x81,
0x17,0x27,0x0E,0x81,0x0A,0x27,0x27,0x81,
0x18,0x27,0x28,0x31,0x39,0x86,0x0C,0x20,
0x02,0x86,0x1E,0xBD,0xCB,0xEE,0x86,0x10,
0x97,0x8A,0x31,0x39,0xDE,0x98,0xA6,0x00,
0xBD,0xF2,0x2E,0xDE,0x98,0x9C,0x95,0x27,
0xF1,0x08,0xDF,0x98,0x20,0xF0,0xBD,0xF3,
0x4B,0x20,0xE3,0xDE,0x98,0xA6,0x00,0x97,
0xB2,0x73,0x00,0xAE,0x86,0x15,0x36,0xBD,
0xF3,0x74,0x32,0x4A,0x26,0xF8,0x7F,0x00,
0xAE,0x20,0xCB,0x96,0x71,0x2A,0x0B,0x8A,
0x08,0x97,0x71,0x39,0xBD,0xF3,0x5A,0x7E,
0xC6,0xB4,0xC6,0x3C,0xF7,0x87,0x95,0xF6,
0x87,0x94,0xC4,0xDF,0xF7,0x87,0x94,0xCA,
0x20,0xF7,0x87,0x94,0x7E,0xC6,0xE6,0x7A,
0x00,0xB1,0x27,0x11,0xB6,0x00,0xAE,0x27,
0x06,0x96,0xB2,0xBD,0xCB,0xEE,0x39,0x86,
0x55,0x4A,0x26,0xFD,0x39,0x73,0x00,0xAE,
0x26,0x07,0x86,0x90,0xB7,0x00,0xB1,0x20,
0xE3,0xFE,0x05,0x8C,0x27,0x0B,0x09,0xFF,
0x05,0x8C,0x26,0x05,0x86,0x0C,0xBD,0xCB,
0xEE,0x86,0x15,0xB7,0x00,0xB1,0x20,0xCC,
0x86,0x01,0x97,0xA5,0x20,0x03,0x7F,0x00,
0xA5,0xDE,0x9C,0xBD,0xB6,0xEB,0x30,0x6D,
0x01,0x2B,0x5D,0xE6,0x02,0xA6,0x01,0xCB,
0x07,0x89,0x00,0x84,0x07,0xA7,0x01,0xE7,
0x02,0xBD,0xB1,0xA2,0x26,0x4A,0xA6,0x03,
0xE6,0x04,0x44,0x56,0x44,0x56,0xE0,0x03,
0x82,0x00,0x64,0x03,0xE0,0x03,0x82,0x00,
0x44,0x56,0x44,0x56,0xA7,0x03,0xE7,0x04,
0x85,0xFC,0x26,0x2C,0xEE,0x03,0x7D,0x00,
0xA4,0x2B,0x06,0xDF,0xA6,0x86,0x80,0x20,
0x17,0x88,0x03,0x26,0x04,0xC1,0x13,0x22,
0x17,0x96,0xA4,0x47,0x25,0x09,0xDF,0xA8,
0xDE,0xA6,0xDF,0xAA,0xBD,0xCB,0x81,0x4F,
0x97,0xA4,0x30,0xBD,0xAA,0x3A,0x35,0x39,
0x7F,0x00,0x4B,0x96,0xA4,0x2B,0xF0,0x86,
0x81,0x20,0xED,0x04,0x03,0xFA,0x00,0x00,
0x00,0x00,0x00,0xBD,0xB0,0x8F,0xD7,0xB3,
0x7F,0x00,0x4B,0x39,0xBD,0xB0,0x8F,0x29,
0x0A,0x4D,0x26,0x07,0xC1,0x03,0x22,0x03,
0xF7,0x05,0x8E,0x7F,0x00,0x4B,0x39,0x86,
0x15,0x36,0xBD,0xAE,0x22,0xBD,0xAE,0xC7,
0xBD,0xC2,0x78,0xBD,0xC4,0x04,0xBD,0xAF,
0x1D,0x31,0x39,0xFF,0x30,0xA6,0x02,0x81,
0x08,0x27,0x05,0x86,0x44,0x97,0x4B,0x39,
0xEE,0x03,0xDF,0x02,0xA6,0x04,0x81,0x10,
0x26,0xF1,0xEE,0x07,0x27,0xED,0xDF,0x04,
0xDE,0x02,0xEE,0x0B,0xBD,0xAA,0x3E,0xDF,
0x06,0xCE,0x02,0xB5,0xDF,0x00,0x8D,0x18,
0x27,0xD9,0x16,0x8D,0x13,0x58,0x58,0x58,
0x58,0x1B,0xDE,0x00,0xA7,0x00,0x08,0xDF,
0x00,0xDE,0x04,0x26,0xE9,0x7E,0x02,0xB5,
0xDE,0x06,0xA6,0x00,0x08,0xDF,0x06,0x80,
0x30,0xDE,0x04,0x09,0xDF,0x04,0x39,0xFF,
0x07,0x0F,0xCE,0x87,0xB2,0xE6,0x00,0xC4,
0x6C,0xE7,0x00,0xC6,0x30,0xF7,0x87,0xB1,
0xE7,0x01,0xC6,0x9F,0xE7,0x00,0x7F,0x87,
0xB0,0xC6,0x37,0xF7,0x87,0xB1,0xE7,0x01,
0x06,0x39,0x84,0x7F,0xB7,0x87,0xB2,0x39,
0x8D,0xD6,0xC6,0x0C,0xF7,0x87,0xB2,0x7E,
0xF5,0xAC,0xFF,0xB6,0x87,0xB2,0x8A,0x10,
0xB7,0x87,0xB2,0xF6,0x87,0xB2,0xC5,0x40,
0x27,0xF9,0xC4,0xEF,0xF7,0x87,0xB2,0x37,
0xBD,0xF5,0x91,0x97,0x6A,0xD7,0x8D,0x32,
0x8A,0x80,0xB7,0x87,0xB2,0xB6,0x87,0xB2,
0x85,0x40,0x26,0xF9,0x20,0xC4,0x86,0x34,
0xB7,0x87,0xB1,0xB6,0x87,0xB2,0x84,0x90,
0x27,0x1A,0xF7,0x87,0xB0,0xB6,0x87,0xB2,
0x84,0x10,0x26,0xF9,0x86,0x3C,0xB7,0x87,
0xB1,0xF6,0x87,0xB2,0x2B,0xFB,0x86,0x34,
0xB7,0x87,0xB1,0x39,0x86,0x45,0x97,0x4B,
0x39,0x07,0x0F,0xC6,0x30,0xF7,0x87,0xB1,
0xF7,0x87,0xB3,0xC6,0x0F,0xF7,0x87,0xB2,
0xC6,0xFF,0xF7,0x87,0xB0,0xC6,0x34,0xF7,
0x87,0xB1,0xC6,0x3F,0xF7,0x87,0xB3,0x06,
0x20,0x1C,0xBD,0xF4,0xD8,0xCA,0x02,0xF7,
0x87,0xB2,0x86,0x18,0x4A,0x26,0xFD,0xC4,
0xFD,0xF7,0x87,0xB2,0x39,0xB6,0x87,0xB2,
0x8A,0x01,0x20,0x07,0xFF,0xFF,0xB6,0x87,
0xB2,0x84,0xFE,0xB7,0x87,0xB2,0x39,0xB6,
0x87,0xB2,0x85,0x40,0x26,0xF9,0x84,0xF7,
0x20,0xF1,0xB6,0x87,0xB2,0x8A,0x08,0x20,
0xEA,0x4F,0xF6,0x87,0xAA,0xC5,0x10,0x27,
0x08,0xD6,0x6D,0xCA,0x08,0xD7,0x6D,0x86,
0x04,0xF6,0x87,0xB0,0x39,0xFF,0xFF,0xFF,
0xFF,0xFF,0xFF,0xFF,0xB6,0x87,0xB2,0x85,
0x20,0x27,0x0C,0x96,0x55,0x85,0x08,0x26,
0x06,0x96,0x6D,0x8A,0x10,0x97,0x6D,0x39,
0x96,0x8F,0x85,0x10,0x26,0x24,0x8A,0x10,
0x97,0x8F,0xBD,0xF5,0x7F,0xBD,0xF5,0x39,
0xD6,0x90,0xCA,0x20,0xBD,0xF5,0x0E,0xD6,
0x97,0xC1,0x20,0x27,0x05,0xCA,0x60,0xBD,
0xF5,0x0E,0xBD,0xF5,0x8A,0x96,0x4B,0x27,
0x01,0x39,0xDE,0x98,0x9C,0x95,0x26,0x0B,
0x96,0x87,0x85,0x80,0x27,0x05,0xBD,0xF5,
0x6D,0x26,0x00,0xE6,0x00,0xBD,0xF5,0x0E,
0x96,0x4B,0x26,0x07,0x9C,0x95,0x27,0x03,
0x08,0x20,0xE1,0xBD,0xF5,0x76,0x39,0x96,
0x8F,0x85,0x10,0x26,0x29,0x8A,0x10,0x97,
0x8F,0xBD,0xF5,0x7F,0xBD,0xF5,0x39,0xD6,
0x90,0xCA,0x40,0xBD,0xF5,0x0E,0xD6,0x97,
0xC1,0x20,0x27,0x05,0xCA,0x60,0xBD,0xF5,
0x0E,0xBD,0xF4,0xB0,0x86,0x0C,0xB7,0x87,
0xB2,0x96,0x4B,0x27,0x01,0x39,0xDE,0x98,
0xBD,0xF4,0xE3,0x96,0x8D,0xD6,0x8E,0xC1,
0x0E,0x27,0x1C,0x91,0x79,0x26,0x07,0xC6,
0x10,0xD7,0x6A,0x09,0x20,0x25,0xBD,0xC4,
0xA9,0xD6,0x7A,0x2B,0x06,0x11,0x26,0x03,
0x09,0x20,0x06,0x91,0x78,0x27,0x0D,0xA7,
0x00,0xD6,0x6A,0x26,0x0E,0x9C,0x93,0x27,
0x0A,0x08,0x20,0xCC,0x09,0xD6,0x6A,0xCA,
0x01,0xD7,0x6A,0x08,0xDF,0x95,0x39,0x32,
0x97,0x59,0x32,0x97,0x5A,0x32,0x32,0x4D,
0x26,0x11,0x30,0xEE,0x00,0x08,0xBD,0xB1,
0xA2,0x26,0x08,0xE6,0x03,0x26,0x04,0xE6,
0x04,0x20,0x02,0xC6,0x80,0x31,0x31,0x7E,
0x00,0x58,0xB6,0x87,0xAA,0x8A,0x40,0xB7,
0x87,0xAA,0x30,0x08,0xDF,0x00,0xDF,0x16,
0x86,0x18,0x36,0xBD,0xD0,0xE9,0xD6,0x00,
0x96,0x01,0x80,0x12,0x97,0xA3,0xC2,0x00,
0xD7,0xA2,0xDE,0xA2,0xBD,0xAA,0x3A,0xDF,
0x9E,0xDE,0xA2,0x86,0x18,0xA7,0x00,0xDE,
0x16,0xDF,0x00,0xBD,0xE8,0x2E,0x4F,0x97,
0x8D,0x97,0x02,0xBD,0xF7,0x98,0x96,0x4B,
0x26,0x4E,0x8D,0x9B,0xC1,0x1F,0x22,0x3F,
0xCA,0x40,0xBD,0xF5,0x0E,0x30,0xA6,0x00,
0x81,0x18,0x27,0x0F,0x81,0x11,0x27,0x0B,
0x8D,0x85,0xC1,0x20,0x22,0x29,0xCA,0x60,
0xBD,0xF5,0x0E,0x7C,0x00,0x02,0x96,0x4B,
0x26,0x21,0xBD,0xF4,0xB0,0xBD,0xF5,0x8A,
0xBD,0xF4,0xE3,0xBD,0xF5,0x7F,0xBD,0xF5,
0x39,0x96,0x8D,0x85,0x40,0x26,0x11,0x32,
0x81,0x18,0x26,0xBE,0x36,0x20,0x04,0x86,
0x42,0x97,0x4B,0x4F,0x97,0x02,0x97,0x8D,
0xC6,0x5F,0xBD,0xF5,0x0E,0xC6,0x19,0xBD,
0xF5,0x0E,0xBD,0xF4,0xD8,0xDE,0x9E,0xD6,
0x02,0x8D,0x2C,0xDE,0xA2,0xD6,0x8D,0x8D,
0x26,0x9F,0x00,0x86,0x18,0xBD,0xD0,0xE9,
0xDE,0x00,0x08,0x08,0x08,0xDF,0x00,0x86,
0x18,0xBD,0xD0,0xE9,0xDE,0x00,0x86,0x11,
0xA7,0x01,0x9F,0x00,0xB6,0x87,0xAA,0x84,
0xBF,0xB7,0x87,0xAA,0xBD,0xAB,0x67,0xEE,
0x02,0x07,0x36,0x01,0x0F,0xA6,0x04,0x85,
0x40,0x27,0x18,0x84,0x7F,0xA7,0x04,0xBD,
0xAA,0x3E,0xDF,0x00,0x37,0x5F,0x37,0x37,
0xBD,0xB1,0xFB,0xDE,0x00,0xBD,0xB7,0x0F,
0x32,0x06,0x39,0x86,0x1B,0x97,0x4B,0x39,
0xBD,0xF5,0x39,0xBD,0xF5,0x7F,0xC6,0x3F,
0xBD,0xF5,0x0E,0xC6,0x18,0xBD,0xF5,0x0E,
0x39,0xFF,0x01,0xCE,0x88,0x08,0xBD,0xAA,
0x7F,0x96,0x53,0x81,0x6E,0x27,0x23,0x30,
0xA6,0x02,0x81,0x0C,0x27,0x06,0xBD,0xE0,
0x0C,0x7E,0x87,0x5A,0xBD,0xD1,0x65,0x42,
0xCE,0x0A,0x20,0xDF,0x56,0x30,0xA6,0x02,
0x81,0x18,0x27,0x03,0x7E,0xE8,0x6C,0xBD,
0xF3,0x4B,0x39,0xFF,0xCE,0x33,0x4B,0xFF,
0x05,0x8C,0xB6,0x87,0xA8,0x16,0x84,0x7F,
0x26,0x19,0x7F,0x05,0x97,0x7F,0x05,0x98,
0x86,0xFF,0xB7,0x05,0x91,0x7F,0x05,0xA6,
0x7F,0x05,0x92,0x7F,0x05,0x93,0x7F,0x05
    ];
    
    this.ROM674 = [
0x94,0x20,0x4E,0x97,0x76,0x7D,0x05,0x97,
0x26,0x26,0x5D,0x2B,0x05,0xC6,0x20,0xF7,
0x05,0x92,0xB6,0x87,0xAA,0x88,0x03,0x85,
0x03,0x27,0x15,0xC6,0x10,0xF7,0x05,0x93,
0x58,0xF7,0x05,0x94,0xF7,0x05,0x92,0x85,
0x02,0x27,0x05,0xC6,0x1F,0xF7,0x05,0x91,
0xCE,0x05,0x9C,0x96,0x76,0xF6,0x05,0x97,
0x27,0x09,0xA1,0x00,0x27,0x18,0x08,0x08,
0x5A,0x20,0xF5,0xF6,0x05,0x97,0xC1,0x03,
0x2C,0x07,0x7C,0x05,0x97,0xA7,0x00,0x6F,
0x01,0x86,0x80,0x97,0x76,0x39,0xA6,0x01,
0x81,0x02,0x24,0x09,0x6C,0x01,0x7D,0x05,
0x9D,0x27,0xEE,0x20,0x1F,0xFF,0x05,0x9A,
0xCE,0x05,0x9C,0xBC,0x05,0x9A,0x27,0x14,
0x7A,0x05,0x97,0xC6,0x06,0xA6,0x02,0xA7,
0x00,0x08,0x5A,0x26,0xF8,0xFE,0x05,0x9A,
0x09,0x09,0x20,0xE1,0xCE,0x05,0x9C,0xB6,
0x05,0x97,0xE6,0x00,0x4A,0x27,0x08,0x08,
0x08,0x6D,0x01,0x26,0xF5,0x20,0xF5,0xCE,
0x05,0x9C,0xD7,0x76,0x6F,0x01,0x6F,0x03,
0x6F,0x05,0xF1,0x05,0xA6,0x27,0x0C,0x86,
0x06,0xB7,0x05,0xA4,0x86,0x37,0xB7,0x05,
0x98,0x20,0x2A,0x7A,0x05,0x99,0x26,0x99,
0xD6,0x76,0xC1,0x6B,0x27,0x93,0xB6,0x05,
0x98,0x81,0x37,0x26,0x05,0x86,0x1A,0xB7,
0x05,0x98,0xB1,0x05,0xA5,0x23,0x0E,0xB0,
0x05,0xA4,0xB7,0x05,0x98,0x7A,0x05,0xA4,
0x26,0x03,0x7C,0x05,0xA4,0xB7,0x05,0x99,
0x96,0x76,0xB7,0x05,0xA6,0x16,0xC4,0x0F,
0xF7,0x05,0x95,0x47,0x47,0x47,0x47,0xB7,
0x05,0x96,0xD6,0x76,0xCE,0xF9,0x08,0xB6,
0x05,0x96,0x27,0x05,0x08,0x08,0x4A,0x20,
0xF9,0xEE,0x00,0xAD,0x00,0xD7,0x76,0x39,
0xF9,0x18,0xF9,0x1B,0xF9,0x23,0xF9,0x2D,
0xF9,0x40,0xF9,0x56,0xF9,0x76,0xF9,0x83,
0xCB,0x80,0x39,0x7D,0x05,0x95,0x26,0x02,
0xCB,0x10,0x39,0xB6,0x05,0x95,0x81,0x0A,
0x24,0x02,0xCB,0x10,0x39,0xB6,0x05,0x95,
0x27,0x0D,0x81,0x0B,0x22,0x04,0xF0,0x05,
0x93,0x39,0xC0,0x10,0xFB,0x05,0x93,0x39,
0xB6,0x05,0x95,0x26,0x08,0xFB,0x05,0x94,
0xF4,0x05,0x91,0x20,0x08,0xCB,0x20,0xF0,
0x05,0x92,0xF4,0x05,0x91,0x39,0xB6,0x05,
0x95,0x81,0x0A,0x22,0x07,0xCB,0x20,0xF0,
0x05,0x92,0x20,0x0E,0x81,0x0E,0x22,0x05,
0xFB,0x05,0x94,0x20,0x05,0xCB,0x20,0xF0,
0x05,0x94,0xF4,0x05,0x91,0x39,0xCB,0x80,
0xB6,0x05,0x95,0x81,0x0A,0x22,0x03,0xF0,
0x05,0x94,0x39,0xB6,0x05,0x95,0x80,0x0B,
0x22,0x0F,0xCB,0x80,0x8B,0x06,0x2B,0x01,
0x39,0x17,0xF6,0x05,0x94,0x58,0x10,0x16,
0x39,0xCE,0xF9,0xA5,0x4A,0x27,0x03,0x08,
0x20,0xFA,0xE6,0x00,0x39,0x5E,0x28,0x29,
0x45,0xFF,0x86,0x15,0x36,0xBD,0xA9,0xCC,
0xDE,0x3B,0x27,0x20,0xA6,0x04,0x2B,0x18,
0x85,0x30,0x27,0x14,0xDF,0x00,0xEE,0x0B,
0xA6,0x02,0x8A,0x80,0xA7,0x02,0x96,0x00,
0xA7,0x03,0x96,0x01,0xA7,0x04,0xDE,0x00,
0xEE,0x00,0x20,0xDE,0x9F,0x00,0xDE,0x4D,
0x27,0x06,0xA6,0x02,0x8A,0x80,0xA7,0x02,
0xDE,0x3D,0x27,0x0A,0xA6,0x02,0x8A,0x80,
0xA7,0x02,0xEE,0x03,0x20,0xF4,0x86,0x06,
0xBD,0xD0,0xE9,0x27,0x0B,0xDF,0x04,0xBD,
0xD1,0x1D,0xDE,0x04,0xEE,0x02,0x20,0xE2,
0xDE,0x47,0x01,0xDF,0x08,0xDF,0x04,0x9C,
0x45,0x26,0x1A,0x7E,0xFB,0x4E,0xA6,0x02,
0x2A,0x07,0x84,0x7F,0xBD,0xFB,0x7E,0x20,
0xF5,0xDF,0x08,0xBD,0xFB,0x7E,0x9C,0x45,
0x26,0x03,0x7E,0xFB,0x4E,0xE6,0x02,0x2A,
0xF2,0xC4,0x7F,0xE7,0x02,0xDE,0x04,0xDF,
0x06,0xDE,0x08,0xA6,0x01,0x9B,0x05,0x97,
0x05,0xA6,0x00,0x99,0x04,0x97,0x04,0x9C,
0x06,0x26,0x03,0x7E,0xFA,0xE3,0x9F,0x00,
0xDF,0x0C,0xDE,0x06,0xDF,0x0A,0xDE,0x0C,
0xA6,0x00,0x08,0xDF,0x0C,0xDE,0x0A,0xA7,
0x00,0x08,0x9C,0x04,0x26,0xEE,0xC5,0x08,
0x26,0x03,0x7E,0xFB,0x1A,0xDE,0x06,0xEE,
0x03,0x27,0x08,0x96,0x06,0xA7,0x05,0x96,
0x07,0xA7,0x06,0xDE,0x06,0xEE,0x05,0x26,
0x09,0xDE,0x3D,0x9C,0x08,0x26,0x0B,0xCE,
0x00,0x3A,0x96,0x06,0xA7,0x03,0x96,0x07,
0xA7,0x04,0x96,0x06,0xD6,0x07,0xDE,0x08,
0xBC,0x04,0x39,0x26,0x0D,0xB7,0x04,0x39,
0xF7,0x04,0x3A,0xCE,0x04,0x37,0x8D,0x4E,
0x20,0xE8,0x9C,0x4D,0x26,0x09,0x97,0x4D,
0xD7,0x4E,0xCE,0x00,0x4D,0x8D,0x3F,0x9C,
0x4F,0x26,0x04,0x97,0x4F,0xD7,0x50,0x86,
0x01,0xC6,0x07,0xBD,0xD1,0x04,0x9C,0x41,
0x27,0x25,0xA6,0x01,0xEE,0x02,0x9C,0x08,
0x26,0x37,0xDE,0x00,0xD6,0x06,0xE7,0x02,
0xD6,0x07,0xE7,0x03,0x81,0x02,0x27,0x08,
0x81,0x01,0x26,0x25,0x8D,0x10,0x20,0x21,
0x7E,0x87,0x46,0xDE,0x04,0x20,0x02,0xDE,
0x0C,0xDF,0x08,0x7E,0xFA,0x1E,0xA6,0x04,
0xE6,0x05,0xD0,0x09,0x92,0x08,0xDB,0x07,
0x99,0x06,0xA7,0x04,0xE7,0x05,0xA6,0x01,
0x39,0xDE,0x00,0x81,0x02,0x26,0x0E,0xEE,
0x04,0x9C,0x08,0x26,0x08,0x96,0x06,0xA7,
0x04,0x96,0x07,0xA7,0x05,0xBD,0xD1,0x1D,
0x20,0x9D,0xDE,0x06,0xEE,0x03,0x96,0x06,
0xA7,0x0B,0x96,0x07,0xA7,0x0C,0xC5,0x60,
0x26,0xBD,0x86,0x0B,0x9F,0x00,0x20,0x03,
0xBD,0xD1,0x1D,0xBD,0xD0,0xE9,0x9C,0x41,
0x27,0xAD,0xEE,0x02,0x9C,0x08,0x26,0xF0,
0xDE,0x00,0x96,0x07,0xA7,0x03,0x96,0x06,
0xA7,0x02,0x8D,0xA2,0x20,0xE2,0x30,0xA6,
0x04,0xE6,0x05,0xD0,0x46,0x92,0x45,0xDB,
0x05,0x99,0x04,0xDE,0x04,0xDF,0x45,0x4D,
0x2D,0x0A,0x2E,0x04,0xC1,0x00,0x27,0x04,
0x86,0x27,0x97,0x4B,0x30,0xEE,0x01,0x32,
0x32,0x32,0x32,0x32,0x32,0xDF,0x00,0xBD,
0xA9,0x94,0xDE,0x00,0x6E,0x00,0xDF,0x08,
0x96,0x09,0xAB,0x01,0x97,0x09,0x96,0x08,
0xA9,0x00,0x97,0x08,0xDE,0x08,0x39,0xFF,
0x86,0x16,0xB7,0x87,0x9B,0x86,0x29,0xB7,
0x87,0x9A,0x97,0xBA,0x86,0x12,0xB7,0x87,
0x9B,0xB7,0x87,0x8F,0xB7,0x87,0x99,0x86,
0xFF,0xB7,0x87,0x9A,0xB7,0x87,0x8E,0x86,
0x02,0xB7,0x87,0x98,0x86,0x16,0xB7,0x87,
0x8F,0xB7,0x87,0x9B,0xB7,0x87,0x99,0x7F,
0x87,0x8D,0x7F,0x87,0x8C,0x86,0x2E,0xB7,
0x87,0x8D,0x7F,0x00,0xB9,0x7F,0x00,0x6F,
0x39,0xBD,0xFC,0x33,0x7E,0xFF,0xA6,0xBD,
0xFB,0x90,0x7F,0x00,0x70,0xC6,0x27,0xF7,
0x87,0x9A,0xBD,0xFC,0x24,0xF6,0x87,0x9A,
0xC6,0x29,0xF7,0x87,0x9A,0xC8,0x08,0xBD,
0xFF,0xA0,0xF7,0x87,0x9A,0xF6,0x87,0x9A,
0xB6,0x87,0xAA,0x85,0x08,0x26,0xD2,0xB6,
0x87,0x9B,0x85,0xC0,0x27,0xF2,0xCE,0x02,
0x09,0x09,0x26,0xFD,0xB6,0x87,0x9B,0x2B,
0xE4,0xC6,0x29,0xF7,0x87,0x9A,0xBD,0xFF,
0xA6,0xF6,0x87,0x9A,0xCE,0x00,0x00,0xDF,
0xBB,0x7F,0x00,0xB7,0xB6,0x87,0xAA,0x85,
0x08,0x26,0x08,0xB6,0x87,0x98,0x85,0x08,
0x27,0xF2,0x39,0x86,0x39,0x97,0x4B,0xBD,
0xFB,0x90,0x39,0xB6,0x87,0x8E,0xB6,0x87,
0x98,0x8D,0x24,0xB6,0x87,0x99,0x2B,0x2A,
0xB6,0x87,0xAA,0x85,0x08,0x26,0xE4,0xB6,
0x87,0x9B,0x2A,0xEF,0xB6,0x87,0x9B,0x85,
0xC0,0x27,0x0B,0x85,0x40,0x27,0x03,0x7C,
0x00,0xB7,0x86,0x36,0x97,0x4B,0x39,0x96,
0xBA,0x16,0x84,0xDF,0xB7,0x87,0x9A,0xF7,
0x87,0x9A,0x39,0xDE,0xBD,0x09,0xDF,0xBD,
0x86,0x0A,0x97,0xB9,0xDE,0x85,0xDF,0xA5,
0xBD,0xFD,0x7C,0x96,0xB9,0x84,0xFD,0x97,
0xB9,0xDE,0xA5,0xDF,0x85,0xB6,0x00,0xB7,
0x26,0x09,0xDE,0xBD,0x09,0xDF,0xBD,0x27,
0x02,0x20,0xE5,0x96,0xB9,0x8A,0x01,0x97,
0xB9,0xBD,0xFD,0x7C,0xB6,0x00,0xB7,0x26,
0x09,0xB6,0x00,0x14,0x27,0x04,0x7F,0x00,
0x6F,0x39,0xBD,0xFF,0xA0,0xBD,0xFF,0xCB,
0x86,0xFF,0xC6,0x1E,0xBD,0xFE,0xB2,0xBD,
0xFF,0xC7,0xBD,0xFF,0xA6,0xBD,0xFC,0x24,
0xBD,0xFF,0xF3,0xB6,0x00,0xB7,0x27,0xDE,
0xBD,0xFB,0xD7,0x20,0xD9,0x86,0x20,0xB7,
0x87,0x9A,0x96,0xB9,0x85,0x01,0x26,0x04,
0xC6,0x04,0x20,0x02,0xC6,0x08,0x86,0x14,
0xB7,0x87,0x8F,0xB6,0x87,0x8E,0xB6,0x87,
0xAA,0x85,0x08,0x26,0x2D,0xB6,0x87,0x8F,
0x2A,0xF4,0xB6,0x87,0x8C,0x86,0x30,0xB7,
0x87,0x9A,0x5A,0x2C,0x06,0x86,0x20,0xB7,
0x87,0x9A,0x39,0xB6,0x87,0x98,0x85,0x01,
0x26,0xF9,0x20,0xE6,0x96,0xB9,0x85,0x02,
0x26,0x04,0xC6,0x04,0x20,0xDC,0xC6,0x08,
0x20,0xD8,0x7E,0xFC,0x33,0x96,0xB9,0x8A,
0x80,0x84,0xF7,0x97,0xB9,0x8D,0x38,0xBD,
0xFF,0xDB,0xBD,0xFC,0x67,0x96,0xB9,0x85,
0x0A,0x27,0x0E,0x84,0xFD,0x97,0xB9,0xB6,
0x87,0x8E,0x86,0x1E,0xB7,0x87,0x8F,0x20,
0x08,0xB6,0x87,0x98,0x86,0x17,0xB7,0x87,
0x99,0x7E,0xFF,0xA6,0x8D,0x11,0xBD,0xFF,
0xDF,0xBD,0xFC,0x67,0xB6,0x87,0x8C,0x86,
0x2F,0xB7,0x87,0x8D,0x7E,0xFF,0xA6,0xBD,
0xFF,0xA0,0xBD,0xFF,0xE7,0x7C,0x00,0x6F,
0xB6,0x87,0x8E,0xB6,0x87,0x8C,0xB6,0x87,
0x9A,0xDE,0x85,0x39,0x8D,0xE9,0xA6,0x00,
0xB7,0x87,0x8E,0xBD,0xFF,0xCB,0xBD,0xFF,
0xB5,0xBD,0xFD,0x14,0x8D,0x16,0xB6,0x87,
0x8C,0xB6,0x87,0xAA,0x85,0x08,0x26,0x09,
0x96,0xBA,0x85,0x08,0x27,0xF3,0x7E,0xFF,
0xA6,0x7E,0xFB,0xD1,0xB6,0x87,0x8C,0xB6,
0x87,0x98,0x85,0x01,0x26,0xF9,0xB6,0x87,
0x8E,0x86,0x17,0xB7,0x87,0x8F,0x86,0x60,
0xB7,0x87,0x9A,0x39,0xB6,0x87,0x8D,0xF6,
0x87,0x8C,0x4D,0x2B,0x14,0x96,0xB9,0x2B,
0x3B,0xB6,0x87,0x8F,0x2B,0x5B,0xB6,0x87,
0x99,0x2B,0x1D,0xBD,0xFB,0x90,0xBD,0xBC,
0x30,0xDE,0x85,0x9C,0x83,0x27,0x05,0xE7,
0x00,0x08,0xDF,0x85,0xB6,0x87,0x98,0x86,
0x17,0xB7,0x87,0x99,0xB6,0x87,0x8E,0x3B,
0xB6,0x87,0x98,0x86,0x16,0xB7,0x87,0x99,
0x86,0x2E,0xB7,0x87,0x8D,0xBD,0xFF,0xE3,
0x7F,0x00,0x6F,0x3B,0xB6,0x87,0x8C,0xB6,
0x87,0x98,0x85,0x01,0x26,0xF9,0x86,0x20,
0xB7,0x87,0x9A,0x96,0xB9,0x84,0x7F,0x97,
0xB9,0xDE,0x85,0xA6,0x00,0xB7,0x87,0x8E,
0xBD,0xFD,0xA4,0x86,0x16,0xB7,0x87,0x99,
0x3B,0xDE,0x85,0x9C,0x83,0x27,0x0C,0x08,
0xDF,0x85,0xA6,0x00,0xB7,0x87,0x8E,0xB6,
0x87,0x8E,0x3B,0xBD,0xFC,0xD5,0x96,0xB9,
0x85,0x08,0x26,0x1C,0xBD,0xFF,0xC7,0x7F,
0x00,0x6F,0x3B,0x8D,0x01,0x3B,0xBD,0xFC,
0x24,0xBD,0xFC,0x54,0x86,0x16,0xB7,0x87,
0x9B,0xBD,0xFF,0xF3,0x7F,0x00,0x6F,0x39,
0x36,0x96,0xB4,0xBD,0xFE,0xB0,0xBD,0xFF,
0xC7,0x32,0x85,0x01,0x27,0xDD,0xC6,0x01,
0xBD,0xFC,0x24,0xBD,0xFF,0xCB,0xBD,0xFF,
0xB5,0xBD,0xFF,0xC7,0x5A,0x27,0xF1,0x20,
0xCA,0xBD,0xFF,0xEB,0xBD,0xFF,0xDF,0xBD,
0xFF,0xB5,0xB6,0x87,0x8C,0xB6,0x87,0xAA,
0x85,0x08,0x26,0x19,0xB6,0x87,0x8D,0x2A,
0xF4,0xBD,0xFC,0x3B,0xD6,0xBA,0xC5,0x04,
0x27,0x04,0x86,0x80,0x8D,0x0A,0xBD,0xFF,
0xE3,0xBD,0xFC,0x24,0x39,0x7E,0xFC,0x33,
0xC6,0x01,0x37,0x36,0xF6,0x87,0x8C,0xF6,
0x87,0x98,0xC5,0x01,0x26,0xF9,0x4A,0x26,
0xF3,0x32,0x33,0x5A,0x26,0xEC,0x39,0xBD,
0xFC,0x33,0x33,0x39,0x86,0x04,0x36,0x32,
0x4A,0x27,0x28,0x36,0x96,0xBC,0x26,0x05,
0x7C,0x00,0xBC,0x20,0x1D,0xD6,0x70,0x2B,
0x19,0x91,0xBE,0x24,0x0A,0xBD,0xFE,0x81,
0xBD,0xFF,0x8C,0x20,0xE2,0x20,0x0B,0xBD,
0xFE,0x84,0xBD,0xFF,0x8C,0x20,0xD8,0x7C,
0x00,0xBC,0x31,0x86,0x80,0xBD,0xFF,0xCF,
0xB6,0x87,0x9A,0xD6,0xBE,0xD0,0xBC,0x26,
0x05,0x96,0x70,0x2A,0x01,0x39,0x22,0x0B,
0x50,0x96,0x70,0x2B,0x01,0x5C,0xBD,0xFF,
0xEB,0x20,0x03,0xBD,0xFF,0xE7,0x37,0xBD,
0xFF,0xEF,0xBD,0xFF,0xDF,0xBD,0xFF,0xB5,
0xB6,0x87,0x98,0xB6,0x87,0x9B,0x2B,0x7A,
0xB6,0x87,0xAA,0x85,0x08,0x26,0x90,0xB6,
0x87,0x99,0x85,0x40,0x27,0xED,0x33,0x5A,
0x37,0xC6,0x05,0x37,0x86,0xFA,0xBD,0xFE,
0xB0,0x33,0xB6,0x87,0x98,0x85,0x40,0x27,
0x05,0x5A,0x27,0x2F,0x20,0xED,0x33,0x5D,
0x37,0x26,0xCD,0xBD,0xFF,0xE3,0x96,0xBE,
0x97,0xBC,0xBD,0xFC,0x24,0x96,0xBA,0x88,
0x04,0x84,0xFD,0xBD,0xFF,0xD1,0x96,0xBA,
0x85,0x04,0x26,0x05,0xBD,0xFE,0x84,0x20,
0x03,0xBD,0xFE,0x81,0xBD,0xFF,0x8C,0x20,
0xED,0x33,0x39,0x86,0x34,0x97,0x4B,0xBD,
0xFB,0xD7,0x20,0xF5,0x30,0xEE,0x00,0x31,
0x31,0xB6,0x87,0x8F,0x85,0x40,0x27,0x06,
0x86,0x80,0x97,0x70,0x08,0x08,0x6E,0x00,
0x07,0x97,0xB8,0x01,0x0F,0x39,0x96,0xB8,
0x06,0x39,0xBD,0xFF,0xE3,0xBD,0xFC,0x54,
0xBD,0xFC,0x24,0x33,0x39,0xB6,0x87,0xAA,
0x85,0x08,0x26,0x08,0xB6,0x87,0x98,0x85,
0x04,0x27,0xF2,0x39,0x7E,0xFC,0x33,0x86,
0x28,0x20,0x06,0x86,0x20,0x20,0x02,0x9A,
0xBA,0x97,0xBA,0xB7,0x87,0x9A,0x39,0x94,
0xBA,0x20,0xF6,0x86,0x21,0x20,0xF2,0x86,
0xF7,0x20,0xF4,0x86,0x08,0x20,0xE8,0x86,
0xFB,0x20,0xEC,0x86,0x04,0x20,0xE0,0x86,
0x02,0x20,0xDC,0x86,0x01,0x20,0xD8,0xFF,
0xEF,0xB8,0xBC,0x2E,0xFD,0xBC,0xBC,0x4B
    ];

    this.ROM747 = [
0x40,0x51,0x88,0x40,0x8F,0xF8,0x8F,0xF8,
0x00,0x00,0x88,0x0E,0x00,0x00,0x88,0x4E,
0x91,0x6C,0x91,0x26,0x90,0xC2,0x90,0x4F,
0x92,0x0B,0x92,0x12,0x92,0x19,0x92,0x20,
0x8F,0x14,0x92,0x27,0x8E,0xCE,0x8E,0xE6,
0x8E,0xB6,0x8B,0xDC,0x8D,0xBB,0x8C,0x84,
0x8D,0x48,0x8B,0x71,0x8B,0xA0,0x95,0xDE,
0x95,0xDE,0x95,0xFE,0x96,0xF9,0x97,0xA2,
0x96,0x7C,0xB7,0x04,0x69,0x7E,0x8F,0xF8,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xBD,0xA9,
0xE7,0xCE,0x02,0xB5,0xDF,0x20,0xCE,0x02,
0xBD,0xDF,0x22,0xCE,0x02,0xC5,0xDF,0x24,
0xCE,0x02,0xCD,0xDF,0x26,0xCE,0x14,0x20,
0xDF,0x56,0xBD,0xAE,0x22,0xBD,0xAE,0xC7,
0x7F,0x00,0x2C,0x30,0xA6,0x00,0x81,0x18,
0x26,0x06,0xBD,0x89,0xEC,0x7E,0x89,0x24,
0x81,0x12,0x27,0xF6,0xBD,0xCE,0x70,0xBD,
0xCE,0x77,0x26,0x20,0xA6,0x01,0x81,0x18,
0x26,0x0E,0x30,0x08,0xDF,0x1E,0xBD,0xAA,
0x3A,0xDF,0x1C,0xBD,0x89,0xEC,0x20,0x26,
0x81,0x12,0x27,0xEE,0xBD,0xCE,0x74,0xBD,
0xCE,0x77,0x27,0x07,0x86,0x1B,0x97,0x4B,
0x7E,0x89,0xE0,0x30,0x08,0xDF,0x1A,0xBD,
0xAA,0x3A,0xDF,0x18,0xBD,0xAA,0x3A,0xDF,
0x1E,0xBD,0xAA,0x3A,0xDF,0x1C,0x96,0x2C,
0x8A,0x11,0x97,0x2C,0xDE,0x1C,0xA6,0x00,
0x84,0x7F,0xA7,0x00,0xEE,0x00,0x26,0x06,
0x86,0xEF,0x94,0x2C,0x97,0x2C,0xDE,0x1E,
0xA6,0x00,0x84,0x7F,0xA7,0x00,0xEE,0x00,
0x26,0x06,0x86,0xFE,0x94,0x2C,0x97,0x2C,
0xBD,0xE8,0xDE,0x00,0x04,0x92,0xA0,0x04,
0x8A,0xC0,0x8B,0x3D,0x16,0x18,0x88,0x00,
0x1A,0x13,0x00,0x20,0x08,0x00,0x1A,0xB9,
0x13,0x00,0x22,0x00,0x04,0x82,0xA0,0x04,
0x7A,0xC0,0x8B,0x3D,0x16,0x18,0x88,0x00,
0x18,0x13,0x00,0x24,0x08,0x00,0x18,0xB9,
0x13,0x00,0x26,0x0B,0xBD,0x8A,0x27,0x96,
0x2C,0x85,0x10,0x27,0x0F,0xCE,0x04,0x82,
0xBD,0xB6,0xEB,0xCE,0x02,0xD5,0xBD,0xB7,
0x0F,0xBD,0x8A,0x53,0xCE,0x04,0x82,0xBD,
0xB6,0xEB,0xCE,0x04,0xF2,0xBD,0xB7,0x0F,
0xBD,0x8B,0x25,0xBD,0x8A,0x27,0x96,0x2C,
0x85,0x10,0x27,0x1D,0x96,0x2C,0x8A,0x80,
0x97,0x2C,0xCE,0x04,0x7A,0xBD,0xB6,0xEB,
0xCE,0x02,0xD5,0xBD,0xB7,0x0F,0xDE,0x1C,
0xA6,0x00,0x8A,0x80,0xA7,0x00,0xBD,0x8A,
0x53,0xCE,0x04,0x7A,0xBD,0xB6,0xEB,0xCE,
0x04,0xF2,0xBD,0xB7,0x0F,0xBD,0x8B,0x25,
0xBD,0x8A,0x27,0x96,0x2C,0x84,0x11,0x97,
0x2C,0x96,0x2C,0x47,0x24,0x0F,0xCE,0x04,
0x92,0xBD,0xB6,0xEB,0xCE,0x02,0xD5,0xBD,
0xB7,0x0F,0xBD,0x8A,0xBC,0xCE,0x04,0x92,
0xBD,0xB6,0xEB,0xCE,0x05,0x02,0xBD,0xB7,
0x0F,0xBD,0x8B,0x25,0xBD,0x8A,0x27,0x96,
0x2C,0x47,0x24,0x1D,0x96,0x2C,0x8A,0x80,
0x97,0x2C,0xCE,0x04,0x8A,0xBD,0xB6,0xEB,
0xCE,0x02,0xD5,0xBD,0xB7,0x0F,0xDE,0x1E,
0xA6,0x00,0x8A,0x80,0xA7,0x00,0xBD,0x8A,
0xBC,0xCE,0x04,0x8A,0xBD,0xB6,0xEB,0xCE,
0x05,0x02,0xBD,0xB7,0x0F,0xBD,0x8B,0x25,
0xBD,0x8A,0x27,0xBD,0xAF,0x1D,0xBD,0xD1,
0x59,0x7E,0xA9,0xFD,0xFE,0x04,0x7A,0x27,
0x09,0x2A,0x0C,0xFE,0x04,0x82,0x27,0x0C,
0x2B,0x05,0xCE,0xE9,0x26,0x20,0x08,0xCE,
0x04,0x7A,0x20,0x03,0xCE,0x04,0x82,0xDF,
0x18,0xFE,0x04,0x8A,0x27,0x09,0x2A,0x0C,
0xFE,0x04,0x92,0x27,0x0C,0x2B,0x05,0xCE,
0xE9,0x26,0x20,0x08,0xCE,0x04,0x8A,0x20,
0x03,0xCE,0x04,0x92,0xDF,0x1A,0x39,0xBD,
0xE8,0xDE,0x08,0x00,0x18,0x16,0x0D,0x04,
0xF2,0x08,0x00,0x1A,0x16,0x0D,0x05,0x02,
0x0B,0x96,0x97,0x36,0x86,0x15,0xBD,0x95,
0x02,0xBD,0x8B,0x25,0x32,0xBD,0x95,0x02,
0xBD,0xE8,0xDE,0x0D,0x05,0x02,0x0D,0x04,
0xF2,0x0B,0x39,0xBD,0xE8,0xDE,0x00,0x04,
0xF2,0x88,0x00,0x1C,0x16,0x0D,0x04,0xF2,
0xA0,0x02,0xD5,0x0D,0x02,0xDD,0x0B,0xCE,
0x02,0xDD,0x96,0x2C,0x48,0x25,0x43,0xA6,
0x00,0xEE,0x00,0x27,0x03,0x48,0x24,0x43,
0xDE,0x1A,0xBD,0xB6,0xEB,0xCE,0x05,0x02,
0xBD,0xB7,0x0F,0xBD,0x8B,0x25,0xDE,0x20,
0xBD,0xB6,0xEB,0xCE,0x05,0x02,0xBD,0xB7,
0x0F,0xBD,0x8B,0x25,0xDE,0x22,0xBD,0xB6,
0xEB,0xCE,0x05,0x02,0xBD,0xB7,0x0F,0xBD,
0x8B,0x25,0xDE,0x1A,0xBD,0xB6,0xEB,0xCE,
0x05,0x02,0xBD,0xB7,0x0F,0xBD,0x8B,0x25,
0x20,0xA1,0xA6,0x00,0xEE,0x00,0x27,0xC0,
0x48,0x24,0xBD,0x39,0xBD,0xE8,0xDE,0x00,
0x05,0x02,0x88,0x00,0x1E,0x16,0x0D,0x05,
0x02,0xA0,0x02,0xD5,0x0D,0x02,0xDD,0x0B,
0xCE,0x02,0xDD,0x96,0x2C,0x48,0x25,0x43,
0xA6,0x00,0xEE,0x00,0x27,0x03,0x48,0x24,
0xDA,0xDE,0x18,0xBD,0xB6,0xEB,0xCE,0x04,
0xF2,0xBD,0xB7,0x0F,0xBD,0x8B,0x25,0xDE,
0x24,0xBD,0xB6,0xEB,0xCE,0x04,0xF2,0xBD,
0xB7,0x0F,0xBD,0x8B,0x25,0xDE,0x26,0xBD,
0xB6,0xEB,0xCE,0x04,0xF2,0xBD,0xB7,0x0F,
0xBD,0x8B,0x25,0xDE,0x18,0xBD,0xB6,0xEB,
0xCE,0x04,0xF2,0xBD,0xB7,0x0F,0xBD,0x8B,
0x25,0x20,0xA1,0xA6,0x00,0xEE,0x00,0x27,
0xC0,0x48,0x24,0xBD,0x39,0xBD,0xE8,0xDE,
0x00,0x04,0xF2,0x00,0x05,0x02,0x0B,0xBD,
0x94,0x5F,0xBD,0xE8,0xDE,0x0D,0x04,0xD2,
0x0D,0x04,0xC2,0x0B,0x39,0x03,0xFA,0xA3,
0xD7,0x0A,0x3D,0x70,0xA2,0xFF,0x30,0xEE,
0x00,0xA6,0x00,0x97,0x4B,0x32,0x32,0x7E,
0xA9,0xFD,0xDF,0x14,0x7E,0xA9,0xE7,0x96,
0x16,0xD6,0x17,0x01,0x0F,0xDE,0x14,0xA7,
0x07,0xE7,0x08,0x86,0x10,0xA7,0x04,0xEE,
0x0B,0x86,0x20,0xA7,0x02,0x0E,0x7E,0xA9,
0xFD,0xBD,0xA9,0xE7,0x30,0xEE,0x03,0xA6,
0x00,0xE6,0x01,0x08,0x08,0xDF,0x00,0xDB,
0x01,0x99,0x00,0x97,0x02,0xD7,0x03,0x30,
0xBD,0xAA,0x3E,0x35,0xBD,0xB0,0x5D,0x96,
0x4B,0x26,0x09,0xCE,0x05,0xBA,0xBD,0xB6,
0xEB,0x7E,0xA9,0xFD,0xBD,0x8B,0x46,0x1D,
0x30,0xEE,0x0C,0xBD,0x8B,0x52,0xCE,0x05,
0xBA,0xBD,0xB7,0x0F,0x30,0xEE,0x01,0xEE,
0x0B,0xBD,0xAA,0x3E,0xDF,0x02,0x30,0xEE,
0x01,0xEE,0x05,0xDF,0x04,0xBD,0xB0,0x14,
0x96,0x4B,0x26,0x14,0x96,0x04,0xD6,0x05,
0xD0,0x03,0x92,0x02,0xCB,0x01,0x89,0x00,
0x30,0xBD,0xAA,0x3E,0x35,0x7E,0x8B,0x5B,
0xBD,0x8B,0x46,0x15,0xBD,0xA9,0xE7,0xBD,
0xB1,0xA2,0x81,0x5F,0x27,0x0A,0xA6,0x03,
0xE6,0x04,0xC0,0x01,0x82,0x00,0x24,0x02,
0x4F,0x5F,0xBD,0xAA,0x3A,0x35,0xEB,0x09,
0xA9,0x08,0x25,0x66,0x97,0x02,0xD7,0x03,
0xA6,0x03,0xE6,0x04,0x97,0x08,0xD7,0x09,
0x97,0x0A,0xD7,0x0B,0xA6,0x08,0xE6,0x09,
0xEE,0x08,0xEB,0x01,0xA9,0x00,0x30,0xEE,
0x03,0xE0,0x01,0xA2,0x00,0x97,0x06,0xD7,
0x07,0xD0,0x03,0x92,0x02,0x25,0x3B,0xA6,
0x00,0xE6,0x01,0xDB,0x09,0x99,0x08,0x97,
0x0C,0xD7,0x0D,0xDE,0x02,0xDF,0x04,0xA6,
0x02,0xBD,0xAA,0x52,0x16,0xDE,0x0A,0x9C,
0x0C,0x27,0x24,0xA6,0x02,0x08,0xDF,0x0A,
0xBD,0xAA,0x52,0x11,0x26,0x05,0xDE,0x04,
0x08,0x20,0xE2,0xDE,0x08,0xDF,0x0A,0xDE,
0x02,0x9C,0x06,0x27,0x05,0x08,0xDF,0x02,
0x20,0xD3,0x4F,0x5F,0x30,0x20,0x0D,0x96,
0x02,0xD6,0x03,0x30,0xE0,0x09,0xA2,0x08,
0xCB,0x01,0x89,0x00,0xBD,0xAA,0x39,0x35,
0x37,0x36,0x36,0xBD,0xB1,0xFB,0x7F,0x00,
0x4B,0x7E,0xA9,0xFD,0x30,0xEE,0x1A,0xBD,
0x8B,0x52,0xBD,0xB1,0xA2,0xE6,0x03,0xD7,
0x16,0xE6,0x04,0xD7,0x17,0x4D,0x27,0x0B,
0x5F,0xD7,0x4B,0x81,0x5F,0x26,0x04,0xD7,
0x16,0xD7,0x17,0xBD,0xAA,0x3A,0xBD,0xB1,
0xA2,0x7F,0x00,0x4B,0x81,0x5F,0x27,0x0A,
0xA6,0x03,0xE6,0x04,0xC0,0x01,0x82,0x00,
0x24,0x02,0x4F,0x5F,0x97,0x02,0xD7,0x03,
0xBD,0xAA,0x3A,0x35,0xDB,0x17,0x99,0x16,
0x24,0x02,0x86,0xFF,0x97,0x04,0xD7,0x05,
0xEE,0x03,0xA6,0x00,0xE6,0x01,0x0D,0xD2,
0x03,0x92,0x02,0x24,0x0A,0x30,0xBD,0xAA,
0x39,0x35,0x4F,0x5F,0x7E,0x8B,0x5B,0xA6,
0x00,0xE6,0x01,0xD1,0x05,0x92,0x04,0x24,
0x0E,0xA6,0x00,0x97,0x04,0xD7,0x05,0xD0,
0x03,0x92,0x02,0x97,0x16,0xD7,0x17,0x30,
0xEE,0x06,0xA6,0x05,0xE6,0x06,0xD0,0x17,
0x92,0x16,0x24,0x04,0xBD,0x8B,0x46,0x15,
0xEE,0x0B,0xDF,0x06,0x30,0xA6,0x03,0xE6,
0x04,0xDB,0x03,0x99,0x02,0x97,0x02,0xD7,
0x03,0xA6,0x03,0xE6,0x04,0xDB,0x05,0x99,
0x04,0x97,0x04,0xD7,0x05,0xBD,0xAA,0x39,
0x35,0xDE,0x02,0x9C,0x04,0x27,0x0E,0xA6,
0x02,0x08,0xDF,0x02,0xDE,0x06,0xA7,0x05,
0x08,0xDF,0x06,0x20,0xEC,0x7E,0x8B,0x57,
0x30,0xEE,0x0D,0xBD,0x8B,0x52,0xEE,0x03,
0xA6,0x00,0xE6,0x01,0x30,0xEE,0x08,0xEB,
0x01,0xA9,0x00,0x30,0x36,0xEE,0x0B,0xE1,
0x06,0xA2,0x05,0x25,0x09,0x26,0x04,0xE1,
0x06,0x27,0x03,0x32,0x20,0x9E,0x32,0x97,
0x16,0xD7,0x17,0xEB,0x0C,0xA9,0x0B,0x97,
0x02,0xD7,0x03,0x4F,0x97,0x08,0x30,0xA6,
0x03,0xE6,0x04,0x97,0x04,0xD7,0x05,0xBD,
0xAA,0x3E,0x35,0xDE,0x04,0xEB,0x01,0xA9,
0x00,0x97,0x06,0xD7,0x07,0xDE,0x06,0x9C,
0x04,0x27,0x0E,0x09,0xA6,0x02,0xDF,0x06,
0xDE,0x02,0x09,0xA7,0x05,0xDF,0x02,0x20,
0xEC,0xB6,0x00,0x08,0x26,0x05,0x7C,0x00,
0x08,0x20,0xCB,0x30,0xBD,0xAA,0x3E,0x35,
0x7E,0x8B,0x57,0x30,0xEE,0x1A,0xBD,0x8B,
0x52,0xBD,0xB1,0xA2,0x26,0x78,0xA6,0x03,
0x2B,0x74,0xE6,0x04,0x97,0x04,0xD7,0x05,
0xBD,0xAA,0x3A,0xBD,0xB1,0xA2,0x26,0x66,
0xA6,0x03,0x2B,0x62,0xE6,0x04,0xC0,0x01,
0x82,0x00,0x25,0x5A,0x97,0x0A,0xD7,0x0B,
0xDB,0x05,0x99,0x04,0x97,0x06,0xD7,0x07,
0xBD,0xAA,0x3A,0x35,0xEE,0x03,0xA6,0x00,
0xE6,0x01,0xDB,0x0B,0x99,0x0A,0x97,0x02,
0xD7,0x03,0xA6,0x00,0xE6,0x01,0xD0,0x05,
0x92,0x04,0x97,0x04,0xD7,0x05,0x97,0x0E,
0x09,0x09,0x09,0xDF,0x0C,0x30,0xBD,0xAA,
0x3E,0x35,0xEE,0x01,0xA6,0x07,0xE6,0x08,
0xD1,0x07,0x92,0x06,0x25,0x18,0xA6,0x07,
0x97,0x08,0xD7,0x09,0xDB,0x05,0x99,0x04,
0x97,0x04,0xD7,0x05,0xA6,0x05,0xE6,0x06,
0xD0,0x05,0x92,0x04,0x24,0x04,0xBD,0x8B,
0x46,0x1C,0x96,0x04,0xD6,0x05,0x97,0x16,
0xD7,0x17,0xEE,0x0B,0xDF,0x10,0xCE,0x00,
0x02,0xA6,0x00,0xE6,0x01,0xDB,0x11,0x99,
0x10,0xA7,0x00,0xE7,0x01,0x08,0x08,0x8C,
0x00,0x0C,0x26,0xED,0x96,0x0E,0x2B,0x14,
0xDE,0x08,0x9C,0x06,0x27,0x26,0x09,0xDF,
0x08,0xA6,0x05,0xDE,0x04,0x09,0xA7,0x05,
0xDF,0x04,0x20,0xEC,0xDE,0x02,0xDF,0x0E,
0xDE,0x06,0x9C,0x08,0x27,0x0E,0xA6,0x05,
0x08,0xDF,0x06,0xDE,0x0E,0xA7,0x05,0x08,
0xDF,0x0E,0x20,0xEC,0xDE,0x0C,0x9C,0x06,
0x26,0x02,0xDE,0x02,0xA6,0x05,0x08,0xDF,
0x0C,0xDE,0x0A,0x9C,0x02,0x27,0x07,0xA7,
0x05,0x08,0xDF,0x0A,0x20,0xE6,0x30,0xBD,
0xAA,0x3E,0x35,0x7E,0x8B,0x57,0xBD,0xA9,
0xE7,0x30,0xEE,0x03,0x32,0x32,0x32,0x32,
0x32,0xA6,0x00,0xE6,0x01,0x37,0x36,0x36,
0xBD,0xB1,0xFB,0x7E,0xA9,0xFD,0xBD,0xA9,
0xE7,0x30,0xEE,0x03,0xDF,0x00,0x32,0x32,
0x32,0x32,0x32,0xEE,0x00,0x27,0x1A,0xDE,
0x00,0xE6,0x02,0x4F,0x20,0xDF,0x30,0xEE,
0x0C,0xBD,0x8B,0x52,0x30,0xBD,0xB1,0xA2,
0x96,0x4B,0x26,0x05,0x30,0xA6,0x03,0x27,
0x04,0xBD,0x8B,0x46,0x19,0xA6,0x04,0x85,
0x80,0x26,0xF6,0xEE,0x0A,0xEE,0x0B,0xA7,
0x05,0x30,0xBD,0xAA,0x35,0x35,0x4F,0xC6,
0x01,0x7E,0x8B,0x5B,0x86,0x18,0x36,0x30,
0xEE,0x04,0xA6,0x04,0x81,0x10,0x27,0x10,
0x7F,0x00,0x16,0x86,0x48,0x97,0x17,0xBD,
0xE4,0xD4,0x96,0x4B,0x27,0x02,0x32,0x39,
0x8D,0x02,0x20,0x77,0xCE,0x02,0x00,0xDF,
0xAA,0xCE,0x01,0x86,0xDF,0xA8,0xB6,0x87,
0x94,0x8A,0x40,0xB7,0x87,0x94,0x86,0x81,
0x97,0xB2,0xDE,0xA8,0xBD,0x8F,0xD5,0x85,
0x80,0x26,0x10,0x8C,0x00,0x00,0x27,0x19,
0x09,0xDF,0xA8,0x8D,0x78,0x85,0x80,0x27,
0xF2,0x20,0x0E,0x8C,0x03,0x0C,0x27,0x09,
0x08,0xDF,0xA8,0x8D,0x68,0x85,0x80,0x26,
0xF2,0xDE,0xAA,0x85,0x20,0x27,0x10,0x8C,
0x00,0x00,0x27,0x19,0x09,0xDF,0xAA,0x8D,
0x54,0x85,0x20,0x26,0xF2,0x20,0x0E,0x8C,
0x03,0xFF,0x27,0x09,0x08,0xDF,0xAA,0x8D,
0x44,0x85,0x20,0x27,0xF2,0xBD,0xF3,0x6F,
0xDE,0x72,0x9C,0x74,0x27,0xAC,0xBD,0xC6,
0x4A,0x81,0x9B,0x22,0xF3,0x84,0x7F,0x7F,
0x00,0xAE,0x39,0x81,0x0D,0x26,0x03,0x5F,
0x20,0x02,0xC6,0x01,0x30,0xEE,0x04,0x01,
0x0F,0x6F,0x07,0xE7,0x08,0xEE,0x0B,0xA7,
0x05,0x86,0x20,0xA7,0x02,0x0E,0xCE,0x18,
0xA0,0xDF,0x56,0xBD,0xAE,0x22,0x30,0xBD,
0xAA,0x3C,0x7E,0x92,0xE3,0x96,0xA8,0xB7,
0x87,0x8C,0x96,0xA9,0xB7,0x87,0x8E,0xB6,
0x87,0x94,0x84,0xFC,0x9B,0xAA,0xB7,0x87,
0x94,0x96,0xAB,0xB7,0x87,0x96,0xC6,0x06,
0x5A,0x26,0xFD,0xB6,0x87,0x98,0x39,0xFF,
0xBD,0xE8,0xDE,0x00,0xE9,0x26,0x16,0x0D
    ];
    
    this.ROM748 = [
0x04,0xC2,0x16,0x0D,0x04,0xD2,0x16,0x0D,
0x04,0xDA,0x16,0x0D,0x04,0x7A,0x16,0x0D,
0x04,0x8A,0x16,0x0D,0x04,0xA2,0x0D,0x04,
0xB2,0x00,0xE9,0x50,0x0D,0x04,0xE2,0x00,
0x90,0x47,0x16,0x0D,0x04,0x82,0x0D,0x04,
0x9A,0x00,0x90,0x3F,0x16,0x0D,0x04,0x92,
0x0D,0x04,0xAA,0x00,0xE9,0x50,0x16,0x0D,
0x04,0xBA,0x0D,0x04,0xCA,0x0B,0x39,0x04,
0x07,0xC8,0x00,0x00,0x00,0x00,0x00,0x04,
0x08,0x82,0x00,0x00,0x00,0x00,0x00,0x32,
0x97,0x59,0x32,0x97,0x5A,0xBD,0xCE,0x70,
0x4D,0x27,0x03,0x7E,0x93,0x7D,0xBD,0xE8,
0xDE,0x16,0x0B,0x86,0x4C,0x97,0x53,0xBD,
0xEA,0x2C,0xCE,0x04,0xDA,0xBD,0xB7,0x0F,
0x86,0x4B,0x97,0x53,0xBD,0xEA,0x2C,0xCE,
0x04,0xE2,0xBD,0xB7,0x0F,0x7E,0x00,0x58,
0xBD,0xE8,0xDE,0x00,0x04,0x82,0xA0,0x04,
0x7A,0xE0,0x04,0x9A,0x16,0x00,0x04,0xC2,
0xA0,0x04,0xEA,0xE0,0x04,0xBA,0xDC,0x80,
0x04,0x7A,0x0D,0x04,0xC2,0x0D,0x04,0xBA,
0x00,0x04,0x92,0xA0,0x04,0x8A,0xE0,0x04,
0xAA,0x16,0x00,0x04,0xD2,0xA0,0x04,0xFA,
0xE0,0x04,0xCA,0xDC,0x80,0x04,0x8A,0x0D,
0x04,0xD2,0x0D,0x04,0xCA,0x0B,0x7F,0x00,
0x4B,0x39,0x32,0x97,0x59,0x32,0x97,0x5A,
0x8D,0x4A,0x26,0x3F,0xBD,0xEE,0x3E,0x40,
0x2B,0x3C,0xBD,0xE8,0xDE,0x00,0x04,0x7A,
0x0D,0x04,0xEA,0x00,0x04,0x8A,0x0D,0x04,
0xFA,0x0D,0x04,0xF2,0x0D,0x05,0x02,0x0B,
0x8D,0x2A,0x26,0x1F,0xBD,0xEE,0x3E,0x40,
0x2B,0x1C,0xBD,0xE8,0xDE,0x0D,0x04,0x82,
0x0D,0x04,0x7A,0x00,0x04,0xF2,0x0D,0x04,
0x92,0x00,0x05,0x02,0x0D,0x04,0x8A,0x0B,
0xBD,0x90,0x80,0x7E,0x00,0x58,0x86,0x1B,
0x97,0x4B,0x20,0xF7,0x30,0x08,0xDF,0x00,
0xBD,0xCE,0x74,0xBD,0xCE,0x77,0x4D,0x27,
0x04,0x86,0x1B,0x97,0x4B,0x39,0x32,0x97,
0x59,0x32,0x97,0x5A,0x8D,0xE6,0x26,0x39,
0xBD,0xE8,0xDE,0x00,0x04,0x7A,0x0D,0x04,
0xEA,0x00,0x04,0x8A,0x0D,0x04,0xFA,0x0D,
0x04,0xF2,0x0D,0x04,0xB2,0x00,0x04,0xF2,
0xA0,0x04,0xB2,0x0D,0x04,0xAA,0x0B,0x8D,
0xC3,0x26,0x16,0xBD,0xE8,0xDE,0x0D,0x04,
0xF2,0x0D,0x04,0xA2,0x00,0x04,0xF2,0xA0,
0x04,0xA2,0x0D,0x04,0x9A,0x0B,0xBD,0x90,
0x80,0x7E,0x00,0x58,0xBD,0xA9,0xE7,0x8D,
0xA3,0x27,0x03,0x7E,0xA9,0xFD,0xCE,0x04,
0x8A,0xDF,0x28,0x8D,0x11,0xCE,0x04,0x7A,
0xDF,0x28,0x8D,0x0A,0x7E,0xA9,0xFD,0x86,
0x1B,0x97,0x4B,0x7E,0xA9,0xFD,0xBD,0xA9,
0xE7,0xCE,0x04,0xF2,0xBD,0xB7,0x0F,0xEE,
0x00,0x2B,0xEC,0x27,0xEA,0x86,0x48,0x8D,
0x5A,0xBD,0xB6,0xEB,0xCE,0xE9,0x26,0xBD,
0xB6,0xEB,0xDE,0x2A,0xBD,0xB7,0x0F,0xDE,
0x28,0xBD,0xB6,0xEB,0xBD,0xB2,0x3E,0x86,
0x40,0x8D,0x40,0xBD,0xB6,0xEB,0xBD,0xB3,
0xD3,0xCE,0x04,0xF2,0xBD,0xB6,0xEB,0xDE,
0x2A,0xBD,0xB7,0x0F,0x30,0xA6,0x01,0x88,
0x80,0xA7,0x01,0xBD,0xE8,0xDE,0xC0,0x04,
0xF2,0x16,0x13,0x00,0x28,0x0B,0x86,0x20,
0x8D,0x19,0xBD,0xB6,0xEB,0xCE,0x04,0xF2,
0xBD,0xB6,0xEB,0xBD,0xB4,0xAE,0xBD,0xB2,
0x45,0x86,0x08,0x8D,0x06,0xBD,0xB7,0x0F,
0x7E,0xA9,0xFD,0xDE,0x28,0xDF,0x2A,0x9B,
0x2B,0x97,0x2B,0x24,0x03,0x7C,0x00,0x2A,
0xDE,0x2A,0x39,0xCE,0x14,0x20,0xDF,0x56,
0x20,0x1C,0xCE,0x94,0x20,0xDF,0x56,0x20,
0x15,0xCE,0x15,0x20,0xDF,0x56,0x20,0x0E,
0xCE,0x95,0x20,0xDF,0x56,0x20,0x07,0xCE,
0x18,0xA0,0xDF,0x56,0x20,0x00,0x86,0x18,
0x36,0xBD,0xAE,0x22,0xBD,0xAE,0xC7,0x30,
0x08,0x08,0xDF,0x00,0x96,0x8E,0x81,0x18,
0x26,0x03,0x7E,0x92,0xE3,0xBD,0xE8,0x2E,
0xBD,0x93,0x48,0xDE,0x00,0xDF,0xA0,0xDE,
0x02,0xDF,0x9A,0xBD,0x93,0x48,0xDE,0x00,
0xDF,0x9E,0xDE,0x02,0xDF,0xA2,0x96,0x4B,
0x26,0x34,0xCE,0x92,0xA3,0x96,0x8E,0x81,
0x18,0x27,0x40,0x25,0x05,0x84,0x3F,0xCE,
0x92,0xA7,0x80,0x14,0x27,0x02,0x08,0x08,
0xEE,0x00,0xAD,0x00,0x96,0x4B,0x26,0x16,
0xDE,0xA0,0xBD,0xAA,0x3B,0xDF,0xA0,0x9C,
0x9A,0x27,0x0B,0xDE,0x9E,0xBD,0xAA,0x3B,
0xDF,0x9E,0x9C,0xA2,0x26,0xCC,0x96,0x8F,
0x2B,0x03,0xBD,0xC4,0x04,0xBD,0xAF,0x1D,
0xBD,0xAB,0x67,0x94,0x4F,0x93,0x84,0x93,
0xDA,0x94,0x0D,0xCE,0x05,0xAA,0xDF,0x9C,
0xBD,0xF0,0x33,0xCE,0x05,0xB2,0xDF,0x9C,
0xBD,0xF0,0x33,0x96,0x4B,0x26,0xD7,0xBD,
0xE8,0xDE,0x00,0x05,0xB2,0xA0,0x04,0xB2,
0xC0,0x04,0xCA,0x80,0x04,0x8A,0x13,0x00,
0x9E,0x00,0x05,0xAA,0xA0,0x04,0xA2,0xC0,
0x04,0xBA,0x80,0x04,0x7A,0x13,0x00,0xA0,
0x0B,0x20,0x99,0xDF,0x02,0x8D,0x1A,0xDE,
0x28,0xDF,0x9E,0xDE,0x2A,0xDF,0xA2,0xDE,
0x02,0xBD,0xAA,0x3A,0x8D,0x0B,0xDE,0x28,
0xDF,0xA0,0xDE,0x2A,0xDF,0x9A,0x7E,0x92,
0x62,0x32,0x97,0x59,0x32,0x97,0x5A,0xDF,
0x00,0xA6,0x01,0x81,0x0B,0x26,0x04,0xEE,
0x04,0x20,0x2B,0xEE,0x02,0xE6,0x04,0xC5,
0x40,0x26,0x13,0xDF,0x2A,0xEE,0x0B,0xBD,
0xAA,0x3E,0xDF,0x28,0xDE,0x2A,0xBD,0xCF,
0xFA,0xBD,0xAA,0x3E,0x20,0x15,0x07,0x01,
0x0F,0xC4,0x7F,0xE7,0x04,0xF6,0x00,0x40,
0xE7,0x05,0x06,0xBD,0xAA,0x3E,0xDF,0x28,
0xBD,0xAA,0x3B,0xDF,0x2A,0x7E,0x00,0x58,
0x32,0x97,0x59,0x32,0x97,0x5A,0x31,0x32,
0x30,0xEE,0x00,0x31,0x31,0x4D,0x26,0x0B,
0x08,0x08,0xDF,0x00,0xBD,0xAA,0x3B,0xDF,
0x02,0x20,0x1E,0x81,0x10,0x26,0x16,0xDF,
0x04,0xEE,0x02,0xEE,0x0B,0xBD,0xAA,0x3E,
0xDF,0x00,0xDE,0x04,0xEE,0x04,0xBD,0xAA,
0x3E,0xDF,0x02,0x20,0x04,0x86,0x24,0x97,
0x4B,0x7E,0x00,0x58,0xBD,0xE8,0xDE,0x08,
0x00,0xA0,0x16,0x0D,0x04,0xEA,0x0D,0x04,
0xC2,0x08,0x00,0x9E,0x16,0x0D,0x04,0xFA,
0x0D,0x04,0xD2,0x0B,0xBD,0x93,0xA0,0x39,
0xBD,0xE8,0xDE,0x00,0x04,0xEA,0xA0,0x04,
0x7A,0xE0,0x04,0xBA,0x80,0x04,0xA2,0x18,
0x0B,0x8D,0x17,0xBD,0xE8,0xDE,0x00,0x04,
0xFA,0xA0,0x04,0x8A,0xE0,0x04,0xCA,0x80,
0x04,0xB2,0x18,0x0B,0x8D,0x04,0xBD,0xC2,
0x78,0x39,0x7F,0x00,0x4B,0xCE,0x05,0xAA,
0xDF,0x9C,0x86,0x02,0x97,0x8A,0xBD,0xC2,
0x7E,0x39,0xBD,0x94,0x24,0xBD,0xE8,0xDE,
0x00,0x04,0xEA,0x0D,0x05,0xB2,0x00,0x04,
0xFA,0x0D,0x05,0xBA,0x00,0x04,0xEA,0x0D,
0x04,0xF2,0x00,0x04,0xFA,0x0D,0x05,0x02,
0x0B,0xBD,0x94,0x5F,0xBD,0xE8,0xDE,0x00,
0x05,0xB2,0x0D,0x04,0xC2,0x00,0x05,0xBA,
0x0D,0x04,0xD2,0x0B,0x39,0xBD,0x94,0x24,
0xBD,0xE8,0xDE,0x00,0x04,0xEA,0x0D,0x04,
0xC2,0x00,0x04,0xFA,0x0D,0x04,0xD2,0x0B,
0xBD,0x93,0xA0,0x39,0xBD,0xE8,0xDE,0x08,
0x00,0xA0,0xC0,0x04,0xE2,0x00,0x04,0xDA,
0xC8,0x00,0x9E,0xBC,0x80,0x04,0xC2,0x0D,
0x04,0xEA,0x08,0x00,0xA0,0xC0,0x04,0xDA,
0x08,0x00,0x9E,0xC0,0x04,0xE2,0x9C,0x80,
0x04,0xD2,0x0D,0x04,0xFA,0x0B,0x39,0xBD,
0xE8,0xDE,0x08,0x00,0x9E,0x0D,0x05,0x02,
0x08,0x00,0xA0,0x0D,0x04,0xF2,0x0B,0xCE,
0x04,0xC2,0xDF,0x02,0xBD,0x95,0x0B,0xD7,
0x15,0xCE,0x04,0xF2,0xDF,0x02,0xBD,0x95,
0x0B,0xD7,0x14,0x26,0x04,0x96,0x15,0x27,
0x65,0xD4,0x15,0x26,0x74,0xD6,0x14,0x27,
0x19,0xCE,0x04,0xF2,0xDF,0x02,0xCE,0x04,
0xC2,0xDF,0x04,0x86,0x02,0x36,0xBD,0x95,
0x6A,0x32,0x5D,0x27,0x05,0x4A,0x26,0xF5,
0x20,0x57,0x97,0x00,0xD6,0x15,0x27,0x3E,
0xCE,0x04,0xF2,0xDF,0x04,0xCE,0x04,0xC2,
0xDF,0x02,0x86,0x02,0x36,0xBD,0x95,0x6A,
0x32,0x5D,0x27,0x05,0x4A,0x26,0xF5,0x20,
0x38,0x81,0x02,0x27,0x04,0x91,0x00,0x27,
0x30,0x96,0x97,0x36,0x86,0x15,0x8D,0x3A,
0xBD,0xE8,0xDE,0x00,0x04,0xC2,0x0D,0x04,
0xEA,0x00,0x04,0xD2,0x0D,0x04,0xFA,0x0B,
0xBD,0x93,0xA0,0x32,0x8D,0x24,0xBD,0xE8,
0xDE,0x00,0x04,0xF2,0x0D,0x04,0xEA,0x00,
0x05,0x02,0x0D,0x04,0xFA,0x0B,0xBD,0x93,
0xA0,0xBD,0xE8,0xDE,0x08,0x00,0xA0,0x0D,
0x04,0xC2,0x08,0x00,0x9E,0x0D,0x04,0xD2,
0x0B,0x39,0xBD,0xC3,0x74,0x97,0x97,0x7F,
0x00,0x8F,0x39,0x9F,0x16,0xDE,0x02,0xBD,
0xB6,0xEB,0xCE,0x04,0x7A,0xBD,0xB6,0xEB,
0xBD,0xEE,0x3E,0x5F,0x4D,0x2C,0x04,0xC8,
0x01,0x20,0x15,0x30,0xBD,0xAA,0x3A,0x35,
0xCE,0x04,0x82,0xBD,0xB6,0xEB,0x32,0x37,
0xBD,0xEE,0x3E,0x33,0x2F,0x02,0xC8,0x02,
0xDE,0x02,0xBD,0xAA,0x33,0xBD,0xB6,0xEB,
0xCE,0x04,0x8A,0xBD,0xB6,0xEB,0x32,0x37,
0xBD,0xEE,0x3E,0x33,0x2C,0x04,0xC8,0x04,
0x20,0x15,0x30,0xBD,0xAA,0x3B,0x35,0xCE,
0x04,0x92,0xBD,0xB6,0xEB,0x32,0x37,0xBD,
0xEE,0x3E,0x33,0x2F,0x02,0xC8,0x08,0x9E,
0x16,0x39,0xC5,0x01,0x27,0x05,0xCE,0x04,
0x7A,0x20,0x07,0xC5,0x02,0x27,0x19,0xCE,
0x04,0x82,0xDF,0x0E,0xDE,0x02,0xDF,0x08,
0xBD,0xAA,0x33,0xDF,0x06,0xDE,0x04,0xDF,
0x0C,0xBD,0xAA,0x33,0xDF,0x0A,0x20,0x25,
0xC5,0x04,0x27,0x05,0xCE,0x04,0x8A,0x20,
0x08,0xC5,0x08,0x26,0x01,0x39,0xCE,0x04,
0x92,0xDF,0x0E,0xDE,0x02,0xDF,0x06,0xBD,
0xAA,0x33,0xDF,0x08,0xDE,0x04,0xDF,0x0A,
0xBD,0xAA,0x33,0xDF,0x0C,0xBD,0xE8,0xDE,
0x08,0x00,0x08,0xA8,0x00,0x0E,0x08,0x00,
0x08,0xA8,0x00,0x0C,0xFC,0x08,0x00,0x0A,
0xA8,0x00,0x06,0xDC,0x88,0x00,0x06,0x13,
0x00,0x06,0x08,0x00,0x0E,0x13,0x00,0x08,
0x0B,0xBD,0x95,0x0B,0x39,0xFF,0xBD,0xA9,
0xE7,0xBD,0xE8,0xDE,0x16,0xD6,0x00,0xE9,
0x50,0xB2,0x0B,0xBD,0xEB,0xDF,0x96,0x4B,
0x27,0x07,0x86,0x18,0x97,0x4B,0x7E,0xB3,
0xBC,0xBD,0xB3,0xD3,0x20,0x03,0xBD,0xA9,
0xE7,0x7F,0x00,0x02,0x30,0xA6,0x01,0x97,
0x03,0x84,0x07,0xA7,0x01,0x81,0x04,0x27,
0x10,0x2B,0x12,0xBD,0xE8,0xDE,0x00,0xE9,
0x50,0xF2,0x0B,0x86,0x02,0x97,0x02,0x20,
0x04,0xA6,0x02,0x26,0xEE,0xCE,0xE9,0xB2,
0xDF,0xC4,0xBD,0xB7,0xC0,0x2B,0x1A,0x7C,
0x00,0x02,0xBD,0xE8,0xDE,0x16,0x18,0xDD,
0x04,0x01,0xDD,0xB3,0xD7,0x42,0xC2,0x65,
0xA0,0xE9,0x50,0x00,0x96,0x38,0x99,0xFC,
0x0B,0xBD,0xE8,0xDE,0x16,0xD6,0x18,0x1D,
0x84,0x01,0x88,0x37,0x30,0x3B,0x60,0xEC,
0x1D,0x84,0x04,0xDD,0x6E,0x91,0xF2,0xAD,
0x24,0x1D,0x84,0x01,0x9C,0x12,0xB7,0xE6,
0xEA,0xE5,0x1D,0x83,0xFD,0xA2,0xD4,0xE2,
0xE0,0x3F,0xDA,0x1D,0x04,0x01,0xA2,0x28,
0xC4,0xB4,0xBA,0xE8,0x99,0xFC,0x99,0x9D,
0x04,0x02,0x89,0x34,0x26,0x8F,0xBF,0xDE,
0xFC,0x99,0x9D,0x04,0x03,0xD1,0x85,0xB9,
0x73,0x80,0x87,0xFC,0x99,0x9D,0x04,0x03,
0x9A,0x85,0xE0,0x82,0x48,0x41,0xFC,0xD9,
0x9D,0x04,0x01,0xA2,0xF9,0x83,0x6E,0x4E,
0x44,0xDC,0x0B,0x76,0x00,0x02,0x24,0x0D,
0xBD,0xE8,0xDE,0x9D,0x04,0x00,0xAA,0xAA,
0xAA,0xAA,0xAA,0xAA,0x0B,0x76,0x00,0x02,
0x24,0x0C,0xBD,0xEB,0xAA,0xCE,0xE9,0x2E,
0xBD,0xB6,0xEB,0xBD,0xB2,0x45,0x96,0x03,
0x2A,0x03,0xBD,0xEB,0xAA,0x96,0x53,0x81,
0x48,0x26,0x0C,0xBD,0xEB,0xAA,0xCE,0xE9,
0x2E,0xBD,0xB6,0xEB,0xBD,0xB2,0x45,0xFE,
0x05,0xCC,0xBD,0xAA,0x3B,0xBD,0xB6,0xEB,
0xBD,0xB4,0xAE,0x7F,0x00,0x4B,0x7E,0xA9,
0xFD,0xBD,0xA9,0xE7,0xA6,0x03,0x27,0x40,
0xA6,0x01,0x2A,0x40,0x85,0x04,0x27,0x19,
0x85,0x03,0x26,0x15,0xE6,0x02,0x27,0x11,
0xFE,0x05,0xDB,0x86,0x3F,0x08,0xB5,0x87,
0xA8,0x26,0xFA,0xFF,0x05,0xDB,0x30,0x20,
0x23,0xA6,0x02,0x40,0x81,0x30,0x24,0x18,
0xBD,0xB2,0x1D,0xCE,0x05,0xD6,0xBD,0xB7,
0x0F,0x76,0x05,0xDD,0x0D,0x79,0x05,0xDD,
0xCE,0x04,0x00,0xFF,0x05,0xD6,0x20,0x08,
0xBD,0xEF,0xA3,0x30,0xBD,0xAA,0x3A,0x35,
0xCE,0x00,0x08,0xBD,0xB6,0xEB,0x86,0x03,
0x97,0xBF,0xCE,0x3D,0x09,0xDF,0xC0,0xDE,
0xC0,0xDF,0x0E,0xFE,0x05,0xDC,0xBD,0xE5,
0xF9,0xDE,0x0E,0xFF,0x05,0xDC,0xDE,0xC0,
0xDF,0x0E,0xFE,0x05,0xDA,0xBD,0xE5,0xFB,
0xDE,0x0E,0xFF,0x05,0xDA,0xDE,0xC0,0xDF,
0x0E,0xFE,0x05,0xD8,0xBD,0xE5,0xFB,0xDE,
0x0E,0xFF,0x05,0xD8,0x96,0xBF,0x4A,0x27,
0x0A,0x97,0xBF,0x4A,0x26,0xC9,0xCE,0x0C,
0x35,0x20,0xC2,0xCE,0x00,0x08,0xBD,0xB7,
0x0F,0xCE,0x05,0xD6,0xBD,0xB6,0xEB,0x7E,
0xB3,0x43,0xBD,0xA9,0xE7,0xBD,0xCE,0x70,
0xBD,0xCE,0x77,0x27,0x15,0x81,0x30,0x27,
0x0B,0x86,0x1B,0x97,0x4B,0x7E,0xA9,0xFD,
0x86,0x10,0x20,0xF7,0xCE,0xE9,0x26,0xBD,
0xB6,0xEB,0x30,0xA6,0x01,0x2B,0xF1,0xCE,
0x05,0xC3,0xBD,0xB7,0x0F,0x30,0xA6,0x01,
0x2B,0xE6,0xCE,0x97,0xF8,0xBD,0xB6,0xEB,
0xBD,0xB4,0xAE,0x30,0xBD,0xB1,0xA2,0xA6,
0x03,0x27,0x04,0x86,0xFF,0x20,0x04,0xA6,
0x04,0x27,0xCD,0xB7,0x05,0xC2,0xBD,0xAA,
0x3A,0x35,0x7F,0x00,0x4B,0x7E,0xA9,0xFD,
0x04,0x02,0xD4,0x9A,0x78,0x4B,0xEC,0x11
    ];

    // 8800 to 8FFF (0800 in length)
    this.DERROM = [
0x40,0x51,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x44,0x45,0x52, // DER1
0x31,0x20,0x20,0x88,0x20,0x00,0x00,0x00, // @8820
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x39,0x00,0x00,0x00,0x00,0x00,0x00,0x00, // 8820 : 39 = RTS
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
    ];
    
	this.println("Creating RAM/ROM...");
	this.memoria = new Array(1);
	this.m0 = new Array(64*1024);
	this.memoria[0] = this.m0;

	this.execute_reset();

	this.canvas.fillStyle = "rgb("+this.ColourTable[0][0]+","+this.ColourTable[0][1]+","+this.ColourTable[0][2]+")";
	this.canvas.fillRect( 0, 0, 1024, 780 );
	// this.imagedata = this.canvas.getImageData( 0, 0, 1024, 780 );
	this.ERASE();
	
	this.println('interrupt='+this.interruptCounter+',ticks='+Math.floor(this.tstatesPerInterrupt)+' cpu ticks/interrupt, cpu clock=0.8333 MHz');
	this.println('TEKTRONIX4051 ready to go - just hit [start].');

    } // End of local constructor.

} // End of function TEKTRONIX4051.

// *********************
// ***               ***
// ***  END OF FILE  ***
// ***               ***
// *********************
